/**
 * @file html format
 */
// 支持的标签
var supportTags = { 'p': 1, 'img': 1, 'br': 1, 'audio': 1, 'a': 1, 'center': 1, 'ol': 1, 'ul': 1, 'li': 1, 'h2': 1, 'h3': 1, 'strong': 1 };
var supportStyle = { 'text-align': 1 };
// 支持的class
var supportClasss = '';
// 只需要文本的标签
var onlyTextTag = { 'table': 1 };
// 不处理的标签的样式
var notHandleClass = { 'custom_file': 1, 'custom_checkbox': 1, 'list-checkbox': 1, 'audio-container': 1, 'block-layout': 1, 'link-card-layout': 1, 'img-layout': 1 };
// 内联标签
var inLineTag = { 'span': 1, 'a': 1, 'b': 1, 'br': 1, 'i': 1, 'q': 1, 'strike': 1, 'u': 1, 'abbr': 1, 'acronym': 1, 'bdo': 1, 'big': 1, 'cite': 1, 'code': 1, 'dfn': 1, 'em': 1, 'font': 1, 'input': 1, 'kbd': 1, 'label': 1, 'samp': 1, 'select': 1, 'small': 1, 'strong': 1, 'sub': 1, 'sup': 1, 'textarea': 1, 'tt': 1 };

// 根据指定规则格式化标签
function htmlFormat (html, isNode) {
    function handleTagAttrs (nodeEle, tagName) {
        if ('p' === tagName) {
            var textAlign = nodeEle.getStyle('text-align');
            clearAttrs(nodeEle, '');
            if (textAlign) {
                nodeEle.setStyle('text-align', textAlign);
            }
        } else if ('a' === tagName) {
            clearAttrs(nodeEle, '');
        } else if ('img' === tagName) {
            clearAttrs(nodeEle, 'src,data-imgwidth,data-imgheight,data-rotate,source-path');
        } else if ('audio' === tagName) {
            clearAttrs(nodeEle, 'src');
        } else {
            nodeEle.setAttr();
        }
    }

    function clearAttrs (node, notAttrs) {
        var attrs = notAttrs.split(',');
        var attrMap = {};
        attrs.forEach(function (attr, index) {
            if (attr.length > 0) {
                var nodeAttr = node.getAttr(attr);
                if (nodeAttr && nodeAttr.length > 0) {
                    attrMap[attr] = nodeAttr;
                }
            }
        });
        node.setAttr();
        for (var key in attrMap) {
            node.setAttr(key, attrMap[key]);
        }
    }

    function isContains (classOrTag, name) {
        return classOrTag[name];
    }

    // 是否包含支持的class
    function hasClass (nodeEle, clazzs) {
        var clazz = nodeEle.getAttr('class');
        if (!clazz) {
            return false;
        }
        var clas = clazz.split(' ');
        for (var i = 0; i < clas.length; i++) {
            if (clazzs[clas[i]]) {
                return true;
            }
        }
        return false;
    }

    // 去除标签外壳,包上p标签
    function shellingNodeToP (nodeEle) {
        var p = UE.uNode.createElement('p');
        var style = nodeEle.getAttr('style');
        p.setAttr('style', style);
        var parent = nodeEle.parentNode;
        if (parent) {
            if (nodeEle.children && nodeEle.children.length > 0) {
                while (child = nodeEle.firstChild()) {
                    if (!isInLine(child)) {
                        if (p.children.length > 0) {
                            parent.insertBefore(p, nodeEle);
                            p = UE.uNode.createElement('p');
                        }
                        parent.insertBefore(child, nodeEle);
                    } else {
                        p.appendChild(child);
                        if (nodeEle.children.length === 0) {
                            parent.insertBefore(p, nodeEle);
                        }
                    }
                }
            }
            parent.removeChild(nodeEle);
        }
    }

    // 是否是内联节点
    function isInLine (node) {
        if (node.type === 'text' || (node.type === 'element' && isContains(inLineTag, node.tagName.toLowerCase()))) {
            return true;
        }
        return false;
    }

    // 去除标签外壳
    function shellingNode (nodeEle) {
        var parent = nodeEle.parentNode;
        if (parent) {
            parent.removeChild(nodeEle, true);
        }
    }

    // 是否是文本节点
    function isTextNode (node) {
        if (node && node.type === 'text') {
            return true;
        }
        return false;
    }

    // 根据指定规则格式化标签
    function formaterHtml (root) {
        if (!root.children || root.children.length === 0) {
            return;
        }
        var node = root.children[0];
        var next;
        while (node) {
            next = node.nextSibling();
            if (node.type === 'element') {
                var nodeEle = node;
                var tagName = nodeEle.tagName.toLowerCase();
                if (hasClass(nodeEle, notHandleClass)) {
                    if (hasClass(nodeEle, { 'audio-container': 1, 'link-card-layout': 1 })) {
                        nodeEle.setAttr('contentEditable', 'false');
                    }
                    nodeEle.removeAttr('style');
                    nodeEle.traversal(function (childNode) {
                        if (hasClass(childNode, { 'audio-container': 1, 'link-card-layout': 1 })) {
                            childNode.setAttr('contentEditable', 'false');
                        }
                        if (!hasClass(childNode, { 'link-card-img': 1 })) {
                            childNode.removeAttr('style');
                        }
                    });
                } else if (isContains(onlyTextTag, tagName)) {
                    // 外部拷贝的标签
                    var p = UE.uNode.createElement('p');
                    p.innerText(nodeEle.innerText());
                    nodeEle.parentNode.replaceChild(p, nodeEle);
                } else {
                    formaterHtml(nodeEle);

                    handleTagAttrs(nodeEle, tagName);

                    if (!isContains(supportTags, tagName) && !hasClass(nodeEle, supportClasss)) {
                        if (isInLine(nodeEle)) {
                            shellingNode(nodeEle);
                        } else {
                            shellingNodeToP(nodeEle);
                        }
                    } else if ('a' === tagName) {
                        shellingNode(nodeEle);
                    } else if ('p' === tagName) {
                        // 如果包含块标签，则拆分
                        shellingNodeToP(nodeEle);
                    }
                }
            }
            node = next;
        }
    }

    var root;
    if (isNode) {
        root = html;
    } else {
        root = UE.htmlparser(html, true);
    }
    formaterHtml(root);
    return root;
}
