/**
 * @file umeditor-android
 */
document.write('<script type="text/javascript" charset="utf-8" src="html-format-plugin.js"></script>');
document.write('<script type="text/javascript" charset="utf-8" src="umeditor-android-utils.js"></script>');
var undoList = window.undoList || [];
var canUndo = window.canUndo || false;
window.Umeditor = window.Umeditor || {};
var ue = {};
var UE = window.UE || {};
var nativeCallbackHandler = window.nativeCallbackHandler || {};
var timeOutEvent = 0;
var initEditorHeight = 0;
var isContentChange = false;
//  初始存文本内容
var initContentTxt = '';

/**
 * 比较两个数组的差集
 * @param {*} arr1 
 * @param {*} arr2 
 * @param {*} typeName 
 */
function getDifferenceSet (arr1, arr2, typeName) {
    return Object.values(arr1.concat(arr2).reduce(function (acc, cur) {
        if (acc[cur[typeName]] && acc[cur[typeName]][typeName] === cur[typeName]) {
            delete acc[cur[typeName]];
        } else {
            acc[cur[typeName]] = cur;
        }
        return acc;
    }, {}));
}


Umeditor.init = function () {
    var config = {
        autoFloatEnabled: false,
        wordCount: false,
        elementPathEnabled: false,
        allowDivTransToP: false,
        enableContextMenu: false,
        removeFormatAttributes: ['lang,width,height,align,hspace,valign'],
        minFrameHeight: initEditorHeight,
        initialFrameHeight: initEditorHeight,
        focus: false,
        insertunorderedlist: {
            // 自定的样式
            'checkbox': 'checkbox'
        }
    };
    ue = UE.getEditor('editor', config);
    registerTodoList(ue);
    ue.addListener('ready', function (editor) {
        Umeditor.doHtmlReady();
    });

    ue.addListener('focus', function (editor) {
        Umeditor.editorFocusChange(true);
    });
    ue.addListener('blur', function (editor) {
        Umeditor.editorFocusChange(false);
    });

    ue.addListener('selectionchange', function (editor) {
        var rng = ue.selection.getRange();
        var customFile = null;
        customFile = UE.dom.domUtils.findParent(rng.startContainer, function (node) {
            return $(node).hasClass('custom_file');
        }, true);
        if (!ue.queryCommandState('insercustomfile', rng.startContainer)) {
            // 判断它的兄弟节点是否是自定义文件
            if (rng.startContainer) {
                var startRootDom = rng.startContainer;
                while (startRootDom && !UE.dom.domUtils.isBody(startRootDom.parentNode)) {
                    startRootDom = UE.dom.domUtils.findParent(startRootDom.parentNode, function (node) {
                        return !UE.dom.domUtils.isBody(startRootDom.parentNode);
                    }, true);
                }
                var $prev = $(startRootDom).prev();
                if ($prev && $prev.hasClass('custom_file')) {
                    customFile = $prev[0];
                }
            }
        }

        if (!customFile) {
            var body = UE.dom.domUtils.findParent(rng.startContainer, function (node) {
                return node.tagName == 'BODY';
            }, true);
            $(body).find('[data-path]').removeClass('select');
        }
        Umeditor.getFontStatus();
    });

    ue.addListener('beforeclick', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            UE.dom.domUtils.preventDefault(e);
            return false;
        }
    });
    ue.addListener('click', function (type, e) {
        var dom = e.target;
        var isCustomFile = ue.queryCommandState('insercustomfile', dom);
        if (isCustomFile) {
            var range = ue.selection.getRange();
            var start = range.startContainer;
            var startRootDom = start;
            // 一直往上找，找到当前选区根节点
            while (startRootDom.nodeType == 3 ||
                (startRootDom && startRootDom.className != undefined && startRootDom.className.indexOf('custom_file') < 0)) {
                startRootDom = UE.dom.domUtils.findParent(startRootDom);
            }
            if (startRootDom.dataset && startRootDom.dataset.fsid != undefined) {
                Umeditor.customfileClick(startRootDom.dataset.fsid, startRootDom.dataset.path);
            }
            // range.setStartAfter(startRootDom).setCursor();
        }
        if (e.target.tagName.toLowerCase() == 'a' && ue.body.contentEditable != 'false') {
            var href = e.target.href;
            Umeditor.aClick(href);
        }
    });

    ImagePlugin.init();
    AudioPlugin.init();
    LinkCardPlugin.init();

    ue.addListener('beforepaste', function (type, html) {
        //  转义 &amp; 为 &,去除&quot;特殊字符
        html.html = UE.utils.specialtml(html.html);
        var root = htmlFormat(html.html);
        // LinkCardPlugin.doBeforepaste(type, root);
        ImagePlugin.doBeforepaste(type, root);
        AudioPlugin.doBeforepaste(type, root);
        html.html = root.toHtml();
        // 如果光标在li标签内，则拆分ol或ul标签，再在其中插入html
        var rng = ue.selection.getRange();
        var li = UE.dom.domUtils.findParentByTagName(rng.startContainer, 'li', true);
        if (li) {
            var ul = $(li).parent()[0];
            var index = $(li).index();
            var length = $(ul).children().length;
            if (index >= (length - 1)) {
                // 控制光标在ul后，再插入图片
                rng.setStartAfter(ul).setCursor();
            } else {
                var left = $(ul).clone()[0];
                $(left).empty();
                for (var i = 0; i <= index; i++) {
                    var child = $(ul).children().eq(i).clone();
                    $(left).append(child);
                }
                $(left).insertBefore(ul);
                var right = $(ul).clone()[0];
                $(right).empty();
                for (var i = index + 1; i < length; i++) {
                    var child = $(ul).children().eq(i).clone();
                    $(right).append(child);
                }
                $(right).insertAfter(ul);
                $(ul).remove();
                $('<p><br></p>').insertBefore(right);
                // 控制光标
                var pBar = $(right).prev()[0];
                rng.setStart(pBar, 0).setCursor();
            }
        } else {
            ImagePlugin.handleBlockLayoutBeforeOrAfterInput();
        }

        isContentChange = true;
    });

    ue.addListener('beforekeydown', function (type, evt) {
        var keyCode = evt.keyCode;
        if (keyCode === 8) {
            return;
        }

        // 控制光标在图片布局中的回车处理
        if (keyCode === 13 && ImagePlugin.handleBlockLayoutBeforeOrAfterInput()) {
            UE.dom.domUtils.preventDefault(evt);
            Umeditor.scrollToSection();
            return false;
        }

        ImagePlugin.handleBlockLayoutBeforeOrAfterInput();
    });

    ue.addListener('keydown', function (type, evt) {
        // isContentChange = true;
    });

    ue.addListener('aftersetcontent', function (type, evt) {
        initContentTxt = ue.getContent();
        Umeditor.reloadImgCard();
    });

    ue.addListener('contentchange', function (type, evt) {
        // Umeditor.getFontStatus();
        Umeditor.onNoteContentChange(ue.getContent());
    });
};

Umeditor.aClick = function (href) {
    Umeditor.callback("callback-editor-a-click", href);
};

Umeditor.onKeyboardChange = function (isShow) {
    isKeyboardShow = isShow;
};

/**
 * 插入文本
 * @param text
 */
Umeditor.insertText = function (text) {
    Umeditor.setHtml(text);
};

/**
 * 插入todolist
 */
Umeditor.insertChecklist = function (notReverse) {
    // 插入todolist时，如果光标在音频div内，则往内容的最后插入
    var rng = ue.selection.getRange();
    var div = UE.dom.domUtils.findParentByTagName(rng.startContainer, 'div', true);
    var status = Umeditor.getEditorFontStatus();
    if (status.isCenter && notReverse !== true) {
        Umeditor.setFontCenter(true);
        rng = ue.selection.getRange();
    }
    // 先标题后无序：清除标题格式，转为无序
    if (status.isHead && notReverse !== true) {
        Umeditor.setFontHead(true);
        rng = ue.selection.getRange();
    }
    if (status.isList && notReverse !== true) {
        Umeditor.setFontList(true);
        rng = ue.selection.getRange();
    }
    if (div && $(div).hasClass('play-state')) {
        ue.focus(true);
    }

    // 如果光标不再li内，并且如果该行存在img，则在其后插入
    var li = UE.dom.domUtils.findParentByTagName(rng.startContainer, 'li', true);
    if (!li) {
        var nodeType = rng.startContainer.nodeType;
        var parent = rng.startContainer;
        if (nodeType === 3) {
            parent = $(rng.startContainer).parent()[0];
        }
        var imgs = $(parent).find('img');
        var card = $(parent).find('.link-card-layout');
        if ((imgs && imgs.length > 0) || (card && card.length > 0)) {
            var p = $('<p><br></p>')[0];
            $(p).insertAfter(parent);
            rng.setStartAtFirst(p);
            rng.setCursor();
        }
    }
    ue.execCommand('insertunorderedlist', 'checkbox');
    // 移动到光标位置
    Umeditor.scrollToSection();
    isContentChange = true;
};

Umeditor.setContent = function (content, isAppendTo) {
    ue.setContent(content, isAppendTo);
    Umeditor.updateAudioStyle();

    // 控制光标
    if (content.length === 0) {
        setTimeout(function () {
            var rng = ue.selection.getRange();
            rng.setStart(ue.body.firstChild, 0).setCursor(false, true);
        }, 100);
    }
};

/**
 *  获取要分享webview内容
 *
 */
Umeditor.fetchShareContent = function () {
    Umeditor.callback('callback-share-content', ue.getContent());
};

/**
 *  请求更新笔记内容
 *
 */
Umeditor.requestUpdateNote = function (needUpload) {
    // 针对4.4以下部分机型，接收不到点击事件问题，通过存文本内容比较
    var contentTxt = ue.getContent();
    if (!compareBodyText(initContentTxt, contentTxt)) {
        isContentChange = true;
    }
    Umeditor.callback('callback-response-update-note',
        isContentChange + '##' + needUpload + '##' + contentTxt);
    return contentTxt;
};

/**
 *  插入html代码
 *
 * @param text
 */
Umeditor.setHtml = function (html) {
    var rng = ue.selection.getRange();
    // 如果焦点在语音布局中，焦点设置在最后
    var div = UE.dom.domUtils.findParentByTagName(rng.startContainer, 'div', true);
    if (div && $(div).hasClass('play-state')) {
        ue.focus(true);
    }

    ImagePlugin.handleBlockLayoutBeforeOrAfterInput();

    ue.execCommand('inserthtml', html);
    isContentChange = true;
};

/**
 *   滚动编辑器到光标所在位置
*/
Umeditor.scrollToSectionDelay = function (time) {
    setTimeout(function () {
        var range = ue.selection.getRange();
        range.scrollToView(ue.autoHeightEnabled, ue.autoHeightEnabled ? UE.dom.domUtils.getXY(ue.iframe).y : 0);
    }, time);
};

/**
 *   滚动编辑器到光标所在位置
*/
Umeditor.scrollToSection = function () {
    setTimeout(function () {
        var range = ue.selection.getRange();
        range.scrollToView(ue.autoHeightEnabled, ue.autoHeightEnabled ? UE.dom.domUtils.getXY(ue.iframe).y : 0);
    }, 100);
};

// 根据延迟时间，获取焦点
Umeditor.focusDelay = function (toEnd, time) {
    setTimeout(function () {
        ue.focus(toEnd);
    }, time);
};


/**
 *   根据延迟时间，取消焦点
*/
Umeditor.blurDelay = function (time) {
    setTimeout(function () {
        ue.blur();
    }, time);
};

/**
 *  取消焦点
*/
Umeditor.blur = function () {
    ue.blur();
};

/**
 *   获取焦点
*/
Umeditor.focus = function (toEnd) {
    ue.focus(toEnd);
};

Umeditor.setDisable = function () {
    ue.setDisabled();
};

Umeditor.setInitialFrameHeight = function (height) {
    ue.options.initialFrameHeight = height;
    ue.options.minFrameHeight = height;
    ue.fireEvent('contentchange');
};

/**
 * 传入icon、文件名称、文件大小Bytes
 */
Umeditor.insertFile = function (fileName, isDir, size, fsId) {
    ue.execCommand('insercustomfile', fileName, isDir, size, fsId);
}

Umeditor.customfileClick = function (fsId, path) {
    var result = [fsId, path];
    Umeditor.callback('callback-editor-customfile-click', JSON.stringify(result));
}

Umeditor.customfileDelete = function (fsId, path) {
    var result = [fsId || '0', path];
    console.log(JSON.stringify(result));
    Umeditor.callback('callback-editor-customfile-delete', JSON.stringify(result));
}


// --------------------------------------------ImagePlugin --------------------------------------------------------------------

var ImagePlugin = {};
// 记录长按的图片节点，用于删除
var lastLongPressImgNote = {};
//  要ocr遮罩的图片元素
var ocrMaskImage = {};

// 根据class查询父类
function findParentByClass (node, clazz, includeSelf) {
    return UE.dom.domUtils.findParent(node, function (node) {
        return $(node).hasClass(clazz);
    }, includeSelf);
};

// 处理block-layout前后的输入
ImagePlugin.handleBlockLayoutBeforeOrAfterInput = function () {
    var rng = ue.selection.getRange();
    if (!rng.collapsed) {
        return;
    }
    var node = findParentByClass(rng.startContainer, 'block-layout', true);
    if (node) {
        var pBr = $('<p><br></p>')[0];
        if (rng.startOffset === 0) {
            $(node).before(pBr);
        } else {
            $(node).after(pBr);
        }
        rng.setStart(pBr, 0).setCursor(false, true);
        return node;
    }
    return;
};

ImagePlugin.init = function () {
    ue.addListener('click', function (type, e) {
        if (e.target.tagName.toLowerCase() === 'img' && $(e.target).hasClass('img-layout')) {
            var src = $(e.target).attr('src');
            Umeditor.imgClick(src, ue.getContent());
            // 移除光标
            ue.blur();
        }
    });

    ue.addListener('touchstart', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        if (e.target.tagName.toLowerCase() === 'img' && $(e.target).hasClass('img-layout')) {
            timeOutEvent = setTimeout(function () {
                timeOutEvent = 0;
                var x = e.touches[0].screenX;
                var y = e.touches[0].screenY;
                var src = $(e.target).attr('src');
                lastLongPressImgNote = e.target;
                ocrMaskImage = e.target;
                var width = $(e.target).attr('data-imgwidth');
                var height = $(e.target).attr('data-imgheight');
                var rotate = $(e.target).attr('data-rotate');
                Umeditor.imgLongPress(src, x, y, width, height, rotate, ue.getContent());
            }, 500);

        }
    });

    ue.addListener('touchmove', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        if (e.target.tagName.toLowerCase() === 'img' && $(e.target).hasClass('img-layout')) {
            clearTimeout(timeOutEvent);
            timeOutEvent = 0;
        }
    });

    ue.addListener('touchend', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        if (e.target.tagName.toLowerCase() === 'img' && $(e.target).hasClass('img-layout')) {
            clearTimeout(timeOutEvent);
        }
    });
};

// 插入图片html代码
Umeditor.setImgHtml = function (html) {
    var rng = ue.selection.getRange();
    // 如果光标在li标签内，则拆分ol或ul标签，再在其中插入图片
    var li = UE.dom.domUtils.findParentByTagName(rng.startContainer, 'li', true);
    if (li) {
        var ul = $(li).parent()[0];
        var index = $(li).index();
        var length = $(ul).children().length;
        if (index >= (length - 1)) {
            // 控制光标在ul后，再插入图片
            // 插入图片
            var pBar = $('<p><br></p>')[0];
            $(pBar).insertAfter(ul);
            rng.setStart(pBar, 0).setCursor(false, true);
            Umeditor.setHtml(html);

        } else {
            var left = $(ul).clone()[0];
            $(left).empty();
            for (var i = 0; i <= index; i++) {
                var child = $(ul).children().eq(i).clone();
                $(left).append(child);
            }
            $(left).insertBefore(ul);
            var right = $(ul).clone()[0];
            $(right).empty();
            for (var i = index + 1; i < length; i++) {
                var child = $(ul).children().eq(i).clone();
                $(right).append(child);
            }
            $(right).insertAfter(ul);
            $(ul).remove();
            // 插入图片
            $(html + '<p><br></p>').insertBefore(right);
            // 控制光标
            var pBar = $(right).prev()[0];
            rng.setStart(pBar, 0).setCursor(false, true);
            ue.fireEvent('contentchange');
            ue.fireEvent('saveScene');
        }
    } else {
        Umeditor.setHtml(html);
    }
    isContentChange = true;
    Umeditor.scrollToSectionDelay(500);
};

// 处理图片复制操作
ImagePlugin.doBeforepaste = function (type, root) {
    inEditorTransformImage(root);
};

// 异步加载图片，替换图片地址
Umeditor.replaceImgSrc = function (key, src) {
    $(ue.body).find('.img-layout').each(function (i, image) {
        if ($(image).attr('data-url') == key) {
            var img = new Image();
            img.src = src;
            // 判断图片是否在缓存中
            if (img.complete) {
                if (typeof ($(image).attr('data-imgwidth')) == 'undefined') {
                    $(image).attr('data-imgwidth', img.width);
                }
                if (typeof ($(image).attr('data-imgheight')) == 'undefined') {
                    $(image).attr('data-imgheight', img.height);
                }
                $(image).removeAttr('style');
                $(image).attr('src', src);
                // 加载成功后，移除data-url,防止复制时出现多个相同的data-url
                $(image).removeAttr('data-url');
                $(image).removeAttr('width');
                $(image).removeAttr('height');
                if (typeof ($(image).attr('data-rotate')) != "undefined") {
                    rotateImg(i, $(image).attr('data-rotate'));
                }
                ue.fireEvent('contentchange');
            } else {
                img.onload = function () {
                    $(image).removeAttr('width');
                    $(image).removeAttr('height');
                    if (typeof ($(image).attr('data-imgwidth')) == 'undefined') {
                        $(image).attr('data-imgwidth', img.width);
                    }
                    if (typeof ($(image).attr('data-imgheight')) == 'undefined') {
                        $(image).attr('data-imgheight', img.height);
                    }
                    $(image).removeAttr('style');
                    $(image).attr('src', src);
                    // 加载成功后，移除data-url,防止复制时出现多个相同的data-url
                    $(image).removeAttr('data-url');
                    if (typeof ($(image).attr('data-rotate')) != "undefined") {
                        rotateImg(i, $(image).attr('data-rotate'));
                    }
                    ue.fireEvent('contentchange');
                };
                img.onerror = function () {
                    // 加载失败也展示默认图
                };
            }
        }
    });
};

// 外部調用，图片旋转
Umeditor.rotateImage = function (index, degree) {
    rotateImg(index, degree);
    isContentChange = true;
};

/**
* 旋转图片
*
* @param {Object} $img 图片的jQuery对象
* @param {number} degree 角度
* @param {number} maxWidth 最大宽度
*/
function rotateImg (index, degree) {
    var img = $(ue.body).find('.img-layout')[index];
    var width = $(img).width();
    var height = $(img).height();
    var webviewWidth = $(ue.body).width();
    var maxWidth = webviewWidth;
    var realWidth = $(img).data('imgwidth');
    var realHeight = $(img).data('imgheight');
    var zoom = $(img).data('zoom');
    if (!zoom) {
        zoom = (width < maxWidth ? width : maxWidth) / realWidth;
        $(img).data('zoom', zoom);
    }
    var viewWidth = realWidth * zoom;
    var viewHeight = realHeight * zoom;
    var transform = 'rotate(' + degree + 'deg)';
    if (degree < 0) {
        degree += 360;
    }
    if (degree == 90 || degree == 270) {
        var scaleX = 1;
        var scaleY = 1;
        if (realHeight > maxWidth) {
            width = maxWidth;
            height = realWidth * width / realHeight;
            scaleX = (height / viewWidth) * (viewWidth / maxWidth);
            scaleY = (viewHeight / width) * (maxWidth / viewWidth);
        } else {
            width = realHeight;
            height = realWidth * width / realHeight;
            scaleX = realWidth / width;
            scaleY = realHeight / height;
        }
        $(img).width(width);
        $(img).height(height);
        scaleX = Math.floor(scaleX * 100) / 100;
        scaleY = Math.floor(scaleY * 100) / 100;
        scaleX = scaleX - 0.01;
        scaleY = scaleY - 0.01;
        transform += ' scale(' + scaleX + ', ' + scaleY + ')';
    } else {
        width = viewWidth;
        height = viewHeight;
        $(img).width(width);
        $(img).height('auto');
        transform += ' scale(0.99)';
    }

    $(img).css({
        '-webkit-transform': transform,
        '-moz-transform': transform,
        '-ms-transform': transform,
        '-o-transform': transform,
        'transform': transform
    });
    $(img).attr('data-rotate', degree);
    ue.fireEvent('contentchange');
};

// 显示ocr遮罩
Umeditor.showOcrMaskByIndex = function (index) {
    $(ue.body).find('.img-layout').each(function (i, image) {
        if (i == index) {
            ocrMaskImage = image;
            return false;
        }
    });
}

// 显示ocr遮罩
Umeditor.showOcrMask = function () {
}

//插入结果
Umeditor.insertOcrText = function (text) {
    if (ocrMaskImage) {
        var cursorNode = ocrMaskImage;
        // 设置光标在img标签后面
        var rng = ue.selection.getRange();
        rng.setStartAfter(ocrMaskImage).setCursor(false, true);
        var parent = ImagePlugin.handleBlockLayoutBeforeOrAfterInput();
        if (parent) {
            cursorNode = parent;
        }
        Umeditor.setHtml(text);
        // 光标设置在文字前面
        var next = $(cursorNode).nextAll('p')[0];
        if (next) {
            var rng = ue.selection.getRange();
            rng.setStartAtFirst(next);
            rng.setCursor();
            // 移动到光标位置
            Umeditor.scrollToSection();
        }
    }
}

/**
 *  删除长按的img节点
*/
Umeditor.delLongPressImgNote = function () {
    if (lastLongPressImgNote) {
        var imgParent = $(lastLongPressImgNote).parent('.block-layout')[0];
        if (imgParent) {
            $(imgParent).remove();
        } else {
            $(lastLongPressImgNote).remove();
        }

        ue.fireEvent('contentchange');
        isContentChange = true;
    }
};

// 根据index替换img节点
Umeditor.replaceImgNoteByIndex = function (index, imgHtml) {
    $(ue.body).find('.img-layout').each(function (i, img) {
        if (i === parseInt(index, 10)) {
            var parent = $(img).parent('.block-layout');
            if (parent) {
                $(parent).replaceWith(imgHtml);
            } else {
                $(img).replaceWith(imgHtml);
            }
            ue.fireEvent('contentchange');
            isContentChange = true;
            return false;
        }
    });
};

/**
 *   根据index删除img节点
*/
Umeditor.delImgNoteByIndex = function (index) {
    $(ue.body).find('.img-layout').each(function (i, img) {
        if (i == index) {
            var parent = $(img).parent('.block-layout');
            if (parent) {
                $(parent).remove();
            } else {
                $(img).remove();
            }
            ue.fireEvent('contentchange');
            isContentChange = true;
            return false;
        }
    });
};

function inEditorTransformImage (document) {
    var allNode = document.getNodesByTagName('img');
    for (var i = 0, nodei; nodei = allNode[i++];) {
        // 如果是链接卡片的图片则不处理
        var nodeClassName = nodei.getAttr('class');
        if (hasClass(nodei, 'link-card-img')) {
            continue;
        }
        // 替换图片标签
        var imgEle = replaceImg(nodei, document);
        // 转换img标签
        inTransformImg(imgEle);
        // 给img加上父标签div
        var parent = imgEle.parentNode;
        var className = parent.getAttr('class');
        if (!className || (' ' + className + ' ').indexOf(' block-layout ') === 0) {
            var div = UE.uNode.createElement('div');
            div.setAttr('class', 'block-layout');
            parent.replaceChild(div, imgEle);
            div.appendChild(imgEle);
        }
    }
}

function replaceImg (nodei, document) {
    var src = nodei.getAttr('src');
    var dataWidth = nodei.getAttr('data-imgwidth');
    var dataHeight = nodei.getAttr('data-imgheight');
    var dataRotate = nodei.getAttr('data-rotate');
    var sourcePath = nodei.getAttr('source-path');
    var newImg = UE.uNode.createElement('img');
    newImg.setAttr('class', 'img-layout');
    if (!src) {
        src = nodei.getAttr('data-url');
    }
    if (src) {
        newImg.setAttr('src', src);
    }
    if (dataWidth) {
        newImg.setAttr('data-imgwidth', dataWidth);
    }
    if (dataHeight) {
        newImg.setAttr('data-imgheight', dataHeight);
    }
    if (dataRotate) {
        newImg.setAttr('data-rotate', dataRotate);
    }
    if (sourcePath) {
        newImg.setAttr('source-path', sourcePath);
    }
    nodei.parentNode.replaceChild(newImg, nodei);
    return newImg;
}

function inTransformImg (element) {
    // 处理地址
    var downImgList = transformImgSrc(element);
    //  如果是本地图片，设置宽高
    transformWidthHeight(element);
    // element.setAttr('onerror', 'this.src=\'file:///android_asset/themes/empty.png\';this.onerror=null;');
    element.setStyle('background', '#f2f2f2');
    setTimeout(function () {
        Umeditor.pasteImgLoad(downImgList);
    }, 300);

}

function transformImgSrc (element) {
    var downImgList = '';
    var src = element.getAttr('src');
    if (src) {
        element.setAttr('data-url', src);
        element.setAttr('src');
        downImgList += src + '##';
    }
    return downImgList;
}

function transformWidthHeight (element) {
    var outWidth = 0;
    var outHeight = 0;
    var tempWidth = parseInt(element.getAttr('data-imgwidth'), 10);
    var tempHeight = parseInt(element.getAttr('data-imgheight'), 10);
    var tempRotate = element.getAttr('data-rotate');
    if (tempWidth && tempHeight) {
        outWidth = tempWidth;
        outHeight = tempHeight;
        if (tempRotate) {
            if (tempRotate == 90 || tempRotate == 270) {
                var tempValue = outWidth;
                outWidth = outHeight;
                outHeight = tempValue;
            }
        }
    }

    if (outWidth == 0 || outHeight == 0) {
        element.setAttr('width', '100%');
        element.setAttr('height', '200');

    } else {
        var webviewWidth = screen.width - 36;
        var imgWidth = outWidth;
        var imgHeight = outHeight;
        if (imgWidth <= webviewWidth) {
            element.setAttr('width', imgWidth + '');
            element.setAttr('height', imgHeight + '');
        } else {
            element.setAttr('width', '100%');
            imgHeight = parseInt(webviewWidth * imgHeight / imgWidth, 10).toString();
            element.setAttr('height', imgHeight);
        }
    }
}

// --------------------------------------------ImagePlugin ------------------------------------------------------------
// -------------------------------------------AudioPlugin--------------------------------------------------------------

var AudioPlugin = {};
// 记录长按的语音节点，用于删除
var lastLongPressAudioNote = {};

AudioPlugin.init = function () {
    ue.addListener('touchstart', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        if (e.target.tagName.toLowerCase() == 'div' && $(e.target).hasClass('audio-container') && ue.body.contentEditable != 'false') {
            timeOutEvent = setTimeout(function () {
                timeOutEvent = 0;
                lastLongPressAudioNote = $(e.target).find('audio');
                var src = $(lastLongPressAudioNote).attr('src');
                Umeditor.audioLongPress(src);
            }, 500);
        }
    });

    ue.addListener('touchmove', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        if (e.target.tagName.toLowerCase() == 'div' && $(e.target).hasClass('audio-container') && ue.body.contentEditable != 'false') {
            clearTimeout(timeOutEvent);
            timeOutEvent = 0;
        }
    });

    ue.addListener('touchend', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        if (e.target.tagName.toLowerCase() == 'div' && $(e.target).hasClass('audio-container') && ue.body.contentEditable != 'false') {
            clearTimeout(timeOutEvent);
        }
    });

    ue.addListener('afterpaste', function () {
        // 黏贴后加语音监听
        Umeditor.updateAudioStyle();
        Umeditor.reloadImgCard();
    });
}

// 监听粘贴语音时设置回contentEditable属性属性丢失，设置回contentEditable属性
AudioPlugin.doBeforepaste = function (type, root) {
    root.traversal(function (node) {
        var className = node.getAttr('class');
        if ('audio-container clearfix' === className) {
            node.setAttr('contentEditable', 'false');
        }
        if (node.tagName === 'audio' && className && className.indexOf('loaded') !== -1) {
            node.setAttr('class', '');
        }
    });
};

/**
 * 更新临时识别结果
 *
 * @param text
 */
Umeditor.updateTempContentValue = function (text) {
    if ($(ue.body).find('.temp_content').size() === 0) {
        Umeditor.setHtml('<p contenteditable=\'false\' class=\'temp_content\'>' + text + '</p>');
    } else {
        $(ue.body).find('.temp_content').html(text);
    }
};

/**
 * 清除临时识别结果样式
 */
Umeditor.clearTempContentStyle = function () {
    $(ue.body).find('.temp_content').removeAttr('contenteditable').removeClass('temp_content').addClass('audio_text');
};

/**
 *  设置最终识别结果
 *
 * @param text
 */
Umeditor.setResultContentValue = function (text) {
    $(ue.body).find('.temp_content').remove();
    Umeditor.setHtml('<p class=\'audio_text\'>' + text + '</p>');
};

/**
 * 暂停音频播放
 * @param path
 */
Umeditor.pauseAudio = function () {
    stopAllAudio();
};

/**
 * 插入音频
 * @param path
 */
Umeditor.insertAudio = function (url) {
    var html = '<div class="audio-container clearfix" contenteditable="false"><audio src="' + url + '" preload="auto" controls="controls"></audio><button class="play-state pause"></button><input type="range" min="0" max="100" class="range" value="0" step="any"/><p class="duration"><span class="current">00:00</span><span class="time_split"> / </span><span class="total">--:--</span></p><button contenteditable="false" class="remove-audio"/></div>';
    // 如果光标在div内，则需要添加<p><br></p>，不然光标会出错
    var rng = ue.selection.getRange();
    var div = UE.dom.domUtils.findParentByTagName(rng.startContainer, 'div', true);
    if (div) {
        html += '<p><br></p>';
    }
    Umeditor.setHtml(html);
    Umeditor.updateAudioStyle();
};

/**
 * 更新音频地址
 * @param path
 */
Umeditor.updateAudioPath = function (url) {
    var tempValue = url.split('/');
    var fileName = '';
    if (tempValue != null && tempValue.length > 0) {
        fileName = tempValue[tempValue.length - 1];
    }
    $(ue.body).find('audio').each(function (index, audio) {
        var oldPath = $(audio).attr('src');
        if (oldPath == fileName) {
            $(audio).attr('src', url);
            $(audio).siblings('.play-state').removeAttr('disabled');

        }
    });

};

/**
 *   根据src删除audio标签
*/
Umeditor.delAudioBySrc = function (src) {
    if (lastLongPressAudioNote) {
        var parent = $(lastLongPressAudioNote).parent();
        if ($(parent).is('div')) {
            $(parent).remove();
            ue.fireEvent('contentchange');
            isContentChange = true;
        }
    }
};

Umeditor.onListening = function () {
    ue.focus(true);
    stopAllAudio();
}

Umeditor.updateAudioStyle = function () {
    $(ue.body).find('audio').each(function (index, audioItem) {
        if (!$(audioItem).hasClass('loaded')) {
            $(audioItem).addClass('loaded');
            $(audioItem).siblings('.play-state').removeClass('play').removeClass('pause').addClass('pause');
            $(audioItem).siblings('input').removeClass('play').removeClass('pause').addClass('pause');
            var totalTime = $(audioItem).attr('total_time');
            if (totalTime != undefined) {
                $(audioItem).siblings('.total').text(totalTime);
            }
            audioItem.addEventListener('loadedmetadata',
                function () {
                    var playTime = $(this).siblings('input').val();
                    this.setAttribute('audioInterval', null);
                    this.currentTime = playTime * audioItem.duration / 100.0;
                    var durationEl = $(audioItem).siblings('.duration');
                    durationEl.find('.total').html(timeToStr(this.duration));
                    durationEl.find('.current').html(timeToStr(this.currentTime));
                    $(audioItem).attr('total_time', timeToStr(this.duration));
                });
            $(audioItem).bind('ended', function () {
                stopAudio(audioItem);
                var $playstate = $(this).siblings('.play-state');
                $playstate.removeClass('play').removeClass('pause').addClass('pause');
                var $input = $(this).siblings('input');
                $input.attr('value', 100);
                $input.val(100);
                $input.css('background', '#4a4a4a');
                $input.removeClass('play').removeClass('pause').addClass('pause');
            });
            $(audioItem).siblings('input').bind('input',
                function (e) {
                    var $input = $(this);
                    var thisAudio = $(this).siblings('audio')[0];
                    $input.attr('value', this.value);
                    $input.css('background', 'linear-gradient(to right, #4a4a4a 0%, #4a4a4a '
                        + this.value + '%,#e9e9e9 ' + this.value + '%, #e9e9e9)');

                    thisAudio.currentTime = this.value * thisAudio.duration / 100;
                    $input.siblings('.duration').find('.current').html(timeToStr(thisAudio.currentTime));

                    if (thisAudio.ended || thisAudio.paused) {
                        playAudio(thisAudio);
                        var $playstate = $(thisAudio).siblings('.play-state');
                        $playstate.removeClass('pause').addClass('play');
                        $(thisAudio).siblings('input').removeClass('pause').addClass('play');
                    }
                });

            if (ue.body.contentEditable !== 'false') {
                $(audioItem).siblings('.play-state').on('click',
                    function (e) {
                        if (e.target.className.indexOf('play-state') >= 0) {
                            var $this = $(e.target);
                            if ($this.hasClass('pause')) {
                                stopAllAudio();
                                $this.removeClass('pause').addClass('play');
                                $this.siblings('input').removeClass('pause').addClass('play');
                                var thisAudio = $this.siblings('audio')[0];
                                playAudio(thisAudio);
                                Umeditor.onAudioPlay();
                            } else {
                                var $this = $(e.target);
                                $this.removeClass('play').addClass('pause');
                                $this.siblings('input').removeClass('play').addClass('pause');
                                var thisAudio = $this.siblings('audio')[0];
                                stopAudio(thisAudio);
                                Umeditor.onAudioPause();
                            }
                        }

                    });

                $(audioItem).siblings('.remove-audio').click(function () {
                    $(audioItem).parent().remove();
                    Umeditor.onRemoveAudio();
                    ue.fireEvent('contentchange');
                    isContentChange = true;
                });
            }
        }
    });
}

function timeToStr (time) {
    var m = 0,
        s = 0,
        mm = '00',
        ss = '00';
    time = Math.floor(time % 3600);
    m = Math.floor(time / 60);
    s = Math.floor(time % 60);
    ss = s < 10 ? '0' + s : s + '';
    mm = m < 10 ? '0' + m : m + '';
    return mm + ':' + ss;
}

function playAudio (thisAudio) {
    var me = this;
    thisAudio.play();

    var audioInterval = setInterval(function () {
        thisAudio.setAttribute('audioInterval', audioInterval);
        $(thisAudio).siblings('.duration').find('.current').html(me.timeToStr(thisAudio.currentTime));

        var durationEl = $(thisAudio).siblings('.duration');
        durationEl.find('.total').html(timeToStr(thisAudio.duration));
        var $input = $(thisAudio).siblings('input');
        var playTime = 100 * thisAudio.currentTime / thisAudio.duration;
        $input.css('background', 'linear-gradient(to right, #4a4a4a 0%, #4a4a4a '
            + playTime + '%,#e9e9e9 ' + playTime + '%, #e9e9e9)');
        $input.val(playTime);
        $input.attr('value', playTime);
    },
        60);
}

function stopAudio (thisAudio) {
    thisAudio.pause();
    var audioInterval = thisAudio.getAttribute('audioInterval');
    if (audioInterval) {
        clearInterval(audioInterval);
    }
    thisAudio.setAttribute('audioInterval', null);
}

function stopAllAudio () {
    var audios = ue.document.getElementsByTagName('audio');
    $(ue.body).find('audio').each(function (index, audioItem) {
        $(audioItem).siblings('.play-state').removeClass('play').removeClass('pause').addClass('pause');
        $(audioItem).siblings('input').removeClass('play').removeClass('pause').addClass('pause');
        stopAudio(audioItem);
    });
}

// -------------------------------------------AudioPlugin--------------------------------------------------------------

// -------------------------------------------LinkCardPlugin--------------------------------------------------------------
var LinkCardPlugin = {};
// 记录长按的链接卡片节点，用于删除
var lastLongPressLinkCardNote = {};

LinkCardPlugin.init = function () {
    ue.addListener('touchstart', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        var node = findParentByClass(e.target, 'link-card-layout', true);
        if (node) {
            timeOutEvent = setTimeout(function () {
                timeOutEvent = 0;
                lastLongPressLinkCardNote = node;
                var guid = $(lastLongPressLinkCardNote).attr('guid');
                var src = $(lastLongPressLinkCardNote).attr('link');
                var x = e.touches[0].screenX;
                var y = e.touches[0].screenY;
                Umeditor.linkCardLongPress(x, y, guid, src);
            }, 500);
        }
    });

    ue.addListener('touchmove', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        var node = findParentByClass(e.target, 'link-card-layout', true);
        if (node) {
            clearTimeout(timeOutEvent);
            timeOutEvent = 0;
        }
    });

    ue.addListener('touchend', function (type, e) {
        if (ue.body.contentEditable === 'false') {
            return;
        }
        var node = findParentByClass(e.target, 'link-card-layout', true);
        if (node) {
            clearTimeout(timeOutEvent);
        }
    });

    ue.addListener('click', function (type, e) {
        var node = findParentByClass(e.target, 'link-card-layout', true);
        if (node) {
            var guid = $(node).attr('guid');
            var src = $(node).attr('link');
            Umeditor.linkCardClick(guid, src);
        }
    });
};

// 生成卡片id
function generateshenCardIDUnique () {
    var guid = '';
    var timestamp = (new Date()).valueOf().toString();
    guid = timestamp + parseInt(10 * Math.random(), 10) + parseInt(10 * Math.random(), 10) + parseInt(10 * Math.random(), 10) + parseInt(10 * Math.random(), 10);
    return guid;
}

// 创建链接卡片html
function createLinkCardHtml (src) {
    var guid = generateshenCardIDUnique();
    return '<div class=\'block-layout\'>&#8203;<div contenteditable=\'false\' class=\'link-card-layout\' guid=\'' + guid + '\' link=\'' + src + '\'><div class=\'link-card-img\' style=\'background-image: url(file:///android_asset/themes/link.png);\'></div><p class=\'link-card-title\'>' + src + '</p></div><br></div>';
}

var linkExp = /((http|https):\/\/)?[\w\-_]+(\.[\w\-_]+)+([\w\-\.,@?^=%&amp;:/~\+#!]*[\w\-\@?^=%&amp;/~\+#!])?/g;
function isLink (text) {
    return new RegExp(linkExp).test(text);
}

// 是否包含支持的class
function hasClass (nodeEle, clazzs) {
    var clazz = nodeEle.getAttr('class');
    if (!clazz) {
        return false;
    }
    var clas = clazz.split(' ');
    for (var i = 0; i < clas.length; i++) {
        if (clazzs[clas[i]]) {
            return true;
        }
    }
    return false;
}
function startWith (str, startStr) {
    if (!str || !startStr || startStr.length === 0 || startStr.length > str.length) {
        return false;
    }
    if (str.substr(0, startStr.length) === startStr) {
        return true;
    }
    return false;
}

LinkCardPlugin.doBeforepaste = function (type, root) {
    function parseHtml (root) {
        if (!root.children || root.children.length === 0) {
            // 判断文本内容
            if (root.type === 'text' && isLink(root.data)) {
                var cardHtml = '';
                var patt = new RegExp(linkExp);
                var index = 0;
                while ((result = patt.exec(root.data))) {
                    cardHtml += root.data.substring(index, result.index);
                    var url = result[0];
                    if (!startWith(url, 'http')) {
                        url = 'http://' + url;
                    }
                    cardHtml += createLinkCardHtml(url);
                    index = patt.lastIndex;
                }
                cardHtml += root.data.substring(index, root.data.length);
                var card = UE.htmlparser(cardHtml, true);
                root.parentNode.replaceChild(card, root);
            }
            return;
        }

        var node = root.children[0];
        var next;
        while (node) {
            next = node.nextSibling();
            if (!hasClass(node, { 'audio-container': 1, 'block-layout': 1, 'img-layout': 1, 'link-card-layout': 1 })) {
                parseHtml(node);
            }
            node = next;
        }
    }
    parseHtml(root);
    htmlFormat(root, true);
};

/**
 *   根据src删除链接卡片标签
*/
Umeditor.delLinkCard = function () {
    if (lastLongPressLinkCardNote) {
        var parent = findParentByClass(lastLongPressLinkCardNote, 'block-layout', true);
        if (parent) {
            $(parent).remove();
            ue.fireEvent('contentchange');
            isContentChange = true;
        }
    }
};

// 重新加载卡片图片
Umeditor.reloadImgCard = function () {
    setTimeout(function () {
        $(ue.body).find('.link-card-img').each(function (index, imgCard) {
            var imgurl = $(imgCard).attr('img-url');
            if (imgurl) {
                var img = new Image();
                img.src = imgurl;
                var fnUpdateBackground = function (imgCard) {
                    $(imgCard).css('background-image', 'url(' + imgurl + ')');
                    if (img.width <= 60 && img.height <= 60) {
                        $(imgCard).css('background-size', 'auto');
                    } else if (img.width > img.height) {
                        $(imgCard).css('background-size', 'auto 100%');
                    } else {
                        $(imgCard).css('background-size', '100% auto');
                    }
                };
                if (img.complete) {
                    fnUpdateBackground(imgCard);
                    $(imgCard).removeAttr('img-url');
                } else {
                    img.onload = function () {
                        fnUpdateBackground(imgCard);
                        $(imgCard).removeAttr('img-url');
                    };
                }
            }
        });
    }, 500);
};

// -------------------------------------------LinkCardPlugin--------------------------------------------------------------

// ---------------------------callbacks----------------------------
// html加载完成
Umeditor.doHtmlReady = function () {
    Umeditor.callback('callback-html-loaded', '');
}

// ------------------------------- 字体样式设置 -------------------------------------
// 参 数：isBold##isHead##isList##isCenter##isUndo
Umeditor.getEditorFontStatus = function () {
    var rng = ue.selection.getRange();
    var doms = rng.startContainer.localName !== 'body'
        ? rng.startContainer : rng.endContainer;
    var dom = doms;
    var h2 = UE.dom.domUtils.findParentByTagName(dom, 'h2', true);
    var p = UE.dom.domUtils.findParentByTagName(dom, 'p', true);
    var li = UE.dom.domUtils.findParentByTagName(dom, 'li', true);
    var unorderedlist = ue.queryCommandValue('insertunorderedlist');
    var $dom = $(li);
    if (h2) {
        $dom = $(h2);
    } else if (p) {
        $dom = $(p);
    }
    var isBold = $dom.find('strong')[0] ? 1 : 0;
    var isHead = $dom.is('h2') ? 1 : 0;
    var isCenter = ue.queryCommandValue('justify') === 'center' ? 1 : 0;
    var isUndo = canUndo ? 1 : 0;
    var isList = unorderedlist === 'disc' ? 1 : 0;
    var isTodo = unorderedlist === 'checkbox' ? 1 : 0;
    return {
        isBold: isBold,
        isHead: isHead,
        isList: isList,
        isCenter: isCenter,
        isUndo: isUndo,
        isTodo: isTodo
    }
}

// 设置粗体
Umeditor.setFontBold = function (notReverse) {
    var rng = ue.selection.getRange();
    var doms = rng.startContainer.localName !== 'body'
        ? rng.startContainer : rng.endContainer;
    var status = Umeditor.getEditorFontStatus();
    // 将选中的设置加粗
    if (rng.startOffset != rng.endOffset) {
        ue.execCommand('bold');
        return false;
    }
    var notOpreate = rng.startOffset === 0 && rng.endOffset === 0 && !status.isBold;
    var notEmpty = false;
    if (doms.nodeType === 3) {
        notEmpty = doms.wholeText;
    }
    // 如果是空行，则不执行加粗操作
    if (notOpreate && !notEmpty) {
        return false;
    }
    var p = UE.dom.domUtils.findParentByTagName(doms, 'p', true);
    var h2 = UE.dom.domUtils.findParentByTagName(doms, 'h2', true);
    var dom = p || h2;
    rng.selectNodeContents(dom);
    rng.select();
    rng.collapse();
    ue.execCommand('bold');
    rng.setCursor();
    isContentChange = true;
}

// 设置标题
Umeditor.setFontHead = function (notReverse) {
    var rng = ue.selection.getRange();
    var doms = rng.startContainer.localName !== 'body'
        ? rng.startContainer : rng.endContainer;
    var p = UE.dom.domUtils.findParentByTagName(doms, 'p', true);
    var h2 = UE.dom.domUtils.findParentByTagName(doms, 'h2', true);
    var status = Umeditor.getEditorFontStatus();
    // 先无序后标题：清除无序格式，转为标题
    if (status.isList && notReverse !== true) {
        Umeditor.setFontList(true);
        rng = ue.selection.getRange();
    }
    if (status.isTodo) {
        Umeditor.insertChecklist(true);
        rng = ue.selection.getRange();
    }
    if (h2) {
        ue.execCommand('Paragraph', 'p');
        isContentChange = true;
    } else if (p) {
        ue.execCommand('Paragraph', 'h2');
        isContentChange = true;
    }
}

// 设置是否有序
Umeditor.setFontList = function (notReverse) {
    // 插入todolist时，如果光标在音频div内，则往内容的最后插入
    var rng = ue.selection.getRange();
    var doms = rng.startContainer.localName !== 'body'
        ? rng.startContainer : rng.endContainer;
    var div = UE.dom.domUtils.findParentByTagName(doms, 'div', true);
    var status = Umeditor.getEditorFontStatus();
    if (status.isCenter && notReverse !== true) {
        Umeditor.setFontCenter(true);
        rng = ue.selection.getRange();
    }
    // 先标题后无序：清除标题格式，转为无序
    if (status.isHead && notReverse !== true) {
        Umeditor.setFontHead(true);
        rng = ue.selection.getRange();
    }
    if (status.isTodo && notReverse !== true) {
        Umeditor.insertChecklist(true);
        rng = ue.selection.getRange();
    }
    if (div && $(div).hasClass('play-state')) {
        ue.focus(true);
    }
    // 如果光标不再li内，并且如果该行存在img，则在其后插入
    var li = UE.dom.domUtils.findParentByTagName(rng.startContainer, 'li', true);
    if (!li) {
        var nodeType = rng.startContainer.nodeType;
        var parent = rng.startContainer;
        if (nodeType === 3) {
            parent = $(rng.startContainer).parent()[0];
        }
        var imgs = $(parent).find('img');
        var card = $(parent).find('.link-card-layout');
        if ((imgs && imgs.length > 0) || (card && card.length > 0)) {
            var p = $('<p><br></p>')[0];
            $(p).insertAfter(parent);
            rng.setStartAtFirst(p);
            rng.setCursor();
        }
    }
    ue.execCommand('insertunorderedlist', 'disc');
    // 移动到光标位置
    Umeditor.scrollToSection();
    isContentChange = true;
}

// 设置居中
Umeditor.setFontCenter = function (notReverse) {
    var rng = ue.selection.getRange();
    var doms = rng.startContainer.localName !== 'body'
        ? rng.startContainer : rng.endContainer;
    var p = UE.dom.domUtils.findParentByTagName(doms, 'p', true);
    var h2 = UE.dom.domUtils.findParentByTagName(doms, 'h2', true);
    var dom = p;
    if (h2) {
        dom = h2;
    }
    var status = Umeditor.getEditorFontStatus();
    if (status.isList && notReverse !== true) {
        Umeditor.setFontList(true);
        rng = ue.selection.getRange();
    }
    if (status.isTodo && notReverse !== true) {
        Umeditor.insertChecklist(true);
        rng = ue.selection.getRange();
    }
    if (dom) {
        var justify = ue.queryCommandValue('justify');
        if (justify === 'center') {
            ue.execCommand('justify', 'left');
        } else {
            ue.execCommand('justify', 'center');
        }
        ue.fireEvent('contentchange');
        isContentChange = true;
    }
}

// 获取格式状态
Umeditor.getFontStatus = function () {
    var status = Umeditor.getEditorFontStatus();
    var isBold = status.isBold;
    var isHead = status.isHead;
    var isList = status.isList;
    var isCenter = status.isCenter;
    var isUndo = status.isUndo;
    var str = isBold + '##' + isHead + '##' + isList + '##' + isCenter + '##' + isUndo;
    Umeditor.callback('callback-editor-font-status', str);
}

// 设置是否可撤销
Umeditor.undo = function () {
    ue.execCommand('undo');
    isContentChange = true;
}

// ------------------------------- 字体设置终止 --------------------------------------

Umeditor.editorFocusChange = function (isFocus) {
    Umeditor.callback('callback-editor-focus-change', isFocus);
}

// 连接卡片点击
Umeditor.linkCardClick = function (guid, src) {
    var params = guid + '##' + src;
    Umeditor.callback('callback-editor-linkcard-click', params);
};

// 图片点击
Umeditor.imgClick = function (src, content) {
    var params = src + '##' + content;
    Umeditor.callback('callback-editor-img-click', params);
}

// 图片长按
Umeditor.imgLongPress = function (src, x, y, width, height, rotate, content) {
    var params = src + '##' + x + '##' + y + '##' + width + '##' + height + '##' + rotate + '##' + content;
    Umeditor.callback('callback-editor-img-long-press', params);
}

// 连接卡片长按
Umeditor.linkCardLongPress = function (x, y, guid, src) {
    Umeditor.callback('callback-editor-linkcard-long-press', x + '##' + y + '##' + guid + '##' + src);
}

// 音频长按
Umeditor.audioLongPress = function (src) {
    Umeditor.callback('callback-editor-audio-long-press', src);
}

// 音频长按
Umeditor.onAudioPlay = function (src) {
    Umeditor.callback('callback-editor-audio-play', src);
}

// 音频长按
Umeditor.onAudioPause = function (src) {
    Umeditor.callback('callback-editor-audio-pause', src);
}

// 音频快捷删除
Umeditor.onRemoveAudio = function () {
    Umeditor.callback('callback-editor-audio-fast-remove', '');
}

// 复制图片的异步加载
Umeditor.pasteImgLoad = function (downImgList) {
    Umeditor.callback('callback-editor-paste-img-load', downImgList);
}

Umeditor.onNoteContentChange = function (content) {
    Umeditor.callback('callback-note-content-change', content);
}

Umeditor.callback = function (callbackScheme, callbackPath) {
    var url = callbackScheme + ':';
    if (callbackPath) {
        url = url + callbackPath;
    }
    if (nativeState.androidApiLevel < 17) {
        Umeditor.callbackThroughIFrame(url);
    } else {
        if (nativeCallbackHandler && nativeCallbackHandler.postMessage) {
            nativeCallbackHandler.postMessage(
                JSON.stringify({
                    type: callbackScheme,
                    data: callbackPath
                }));
        }
    }
};

/**
 *  @brief      Executes a callback by loading it into an IFrame.
 *  @details    The reason why we're using this instead of window.location is that window.location
 *              can sometimes fail silently when called multiple times in rapid succession.
 *              Found here:
 *              http://stackoverflow.com/questions/10010342/clicking-on-a-link-inside-a-webview-that-will-trigger-a-native-ios-screen-with/10080969#10080969
 *
 *  @param      url     The callback URL.
 */
Umeditor.callbackThroughIFrame = function (url) {
    var iframe = document.createElement('IFRAME');
    iframe.setAttribute('sandbox', '');
    iframe.setAttribute('src', url);

    // IMPORTANT: the IFrame was showing up as a black box below our text.  By setting its borders
    // to be 0px transparent we make sure it's not shown at all.
    //
    // REF BUG: https://github.com/wordpress-mobile/WordPress-iOS-Editor/issues/318
    //
    iframe.style.cssText = 'border: 0px transparent;';

    document.documentElement.appendChild(iframe);
    iframe.parentNode.removeChild(iframe);
    iframe = null;
};