package com.baidu.netdisk.account

import android.app.Activity
import android.app.Dialog
import android.net.Uri
import android.os.Build
import android.os.Bundle
import android.os.Handler
import com.baidu.netdisk.R
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.ui.WapLauncherActivity
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.xpan.SmartDeviceManager
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import java.lang.ref.WeakReference

/**
 * 通用二维码Scheme协议解析帮助类,用于区分百度系和非百度系的产品扫码跳转
 *
 * @author rendawei
 * @since 2019/09/11
 */
class OpenUrlJumpHelper(activity: Activity) {

    private val mActivity: WeakReference<Activity> = WeakReference(activity)

    private val mSmartManager = SmartDeviceManager()
    private var mLoadDialog: Dialog? = null

    companion object {

        /**
         * 通用二维码跳转特征因子
         */
        private const val OPEN_URL_UNION_SEGMENT = "union"
        private const val OPEN_URL_JUMP_SEGMENT = "jump"

        fun isOpenUrl(content: String): Boolean {
            val result = Boolean.runCatching {
                Uri.parse(content)?.let {
                    return (it.host?.endsWith(ServerURL.PAN_DOMAIN) == true &&
                            it.pathSegments.containsAll(arrayListOf(OPEN_URL_UNION_SEGMENT, OPEN_URL_JUMP_SEGMENT)))
                } ?: false
            }
            return result.isSuccess && result.getOrDefault(false)
        }
    }

    fun processOpenUrl(openUrl: String) {
        doSafeLifeCallback {
            mLoadDialog = DialogBuilder().buildLoadingDialogWithoutText(it).apply {
                show()
            }
            mSmartManager.decodeOpenQRCodeScheme(it.applicationContext, DecodeReceiver(it, Handler(), null), AccountUtils.getInstance().bduss
                    , AccountUtils.getInstance().uid, openUrl)
        }
    }

    private fun doSafeLifeCallback(block: ((activity: Activity) -> Any?)) {
        mActivity.get()?.let {
            it.takeUnless {
                it.isFinishing || (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR1 && it.isDestroyed)
            }?.apply {
                block.invoke(this)
            }
        }
    }

    fun jumpToDecodeScheme(scheme: String?) {
        doSafeLifeCallback {
            scheme?.apply {
                WapLauncherActivity.start(it, this)
            }
        }
    }

    fun showNotSupportDialog() {
        doSafeLifeCallback {
            QrLoginUtil.showSchemeNotValidDialog(it, R.string.qr_schema_not_netdisk_title, R.string.qr_schema_not_netdisk)
        }
    }

    fun dismissDialog() {
        mLoadDialog?.takeIf {
            it.isShowing
        }?.dismiss()
    }

    private inner class DecodeReceiver(activity: Activity, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<Activity>(activity, handler, resultView) {

        override fun onSuccess(reference: Activity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            dismissDialog()
            resultData?.apply {
                val scheme = getString(BaseExtras.RESULT)
                jumpToDecodeScheme(scheme)
            }
        }

        override fun onFailed(reference: Activity, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            dismissDialog()
            if (errType == ErrorType.NETWORK_ERROR) {
                ToastHelper.showToast(R.string.xpan_net_error)
                return true
            }
            showNotSupportDialog()
            return true
        }
    }
}