package com.baidu.netdisk.cloudimage.ui.timeline

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.*
import android.util.AttributeSet
import android.view.View
import com.baidu.netdisk.util.CalculationUtil


/**
 * Created by linwentao on 2019-07-31.
 *
 * Describe: 图片预览底部高亮引导
 */
class GuideView(context: Context?, attrs: AttributeSet?) : View(context, attrs) {
    // 默认背景
    private val BACKGROUND_COLOR = Color.parseColor("#80000000")
    // 矩形
    private var mRect: Rect? = null
    // 画笔
    private var mPaint: Paint = Paint(Paint.ANTI_ALIAS_FLAG)
    // 擦除效果
    private val Xfermode = PorterDuffXfermode(PorterDuff.Mode.CLEAR)
    // 矩形左坐标
    private var mLeft: Int = 0
    // 矩形上坐标
    private var mTop: Int = 0
    // 矩形右坐标
    private var mRight: Int = 0
    // 矩形下坐标
    private var mBottom: Int = 0
    // 屏幕宽
    private var mScreenWidth: Int = 0
    // 屏幕高
    private var mScreenHeight: Int = 0

    /**
     * 初始化
     *
     */
    init {
        mScreenWidth = resources.displayMetrics.widthPixels
        mScreenHeight = resources.displayMetrics.heightPixels

        mPaint.color = Color.WHITE
        mPaint.isAntiAlias = true
        mPaint.style = Paint.Style.FILL
    }

    @SuppressLint("DrawAllocation")
    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        canvas?.let {
            canvas.saveLayer(
                    0f,
                    0f,
                    width.toFloat(),
                    height.toFloat(),
                    null,
                    Canvas.ALL_SAVE_FLAG
            )
            canvas.drawColor(BACKGROUND_COLOR)
            mPaint.xfermode = Xfermode
            mRect = Rect(mLeft, mTop, mRight, mBottom)
            canvas.drawRect(mRect, mPaint)
            mPaint.xfermode = null
        }
    }

    /**
     * 设置美图秀秀高亮view
     *
     * @param view
     * @param isHideDelete 是否隐藏了删除按钮
     */
    fun setMetouGuidView(view: View, isHideDelete: Boolean) {
        val localtion = IntArray(2)
        view.getLocationOnScreen(localtion)
        mLeft = localtion[0] + 20
        mTop = mScreenHeight + CalculationUtil.getStatusBarHeight(context) - 200
        mRight = if (isHideDelete) {
            mScreenWidth / 4
        } else {
            mScreenWidth / 5
        }
        mBottom = mScreenHeight + CalculationUtil.getStatusBarHeight(context)
        invalidate()
    }

    /**
     * 设置打印高亮view
     *
     * @param view
     * @param isHideDelete 是否隐藏了删除按钮
     */
    fun setPrintGuidView(view: View, isHideDelete: Boolean) {
        val localtion = IntArray(2)
        view.getLocationOnScreen(localtion)
        if (isHideDelete) {
            mLeft = mScreenWidth * 2 / 4
            mRight = mScreenWidth * 3 / 4
        } else {
            mLeft = mScreenWidth * 2 / 5
            mRight = mScreenWidth * 3 / 5
        }
        mTop = mScreenHeight + CalculationUtil.getStatusBarHeight(context) - 200
        mBottom = mScreenHeight + CalculationUtil.getStatusBarHeight(context)
        invalidate()
    }

    /**
     * 关闭引导页
     */
    fun dismiss() {
        visibility = GONE
    }

}

