package com.baidu.netdisk.cloudp2p.service

import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.cloudp2p.network.api.P2PErrorCode
import com.baidu.netdisk.cloudp2p.network.model.AddFollowResponse
import com.baidu.netdisk.ui.verify.VerifyPopupActivity

/**
 * 加好友的helper
 *
 * @author wangyang34
 * @since 2019年9月5日
 */
object CloudP2PAddFriendHelper {

    /**
     * 封装的校验用的receiver
     */
    private class WrapResultReceiver(internal var context: Context,
                                     handler: Handler,
                                     var resultReceiver: ResultReceiver?,
                                     internal var uk: Long,
                                     internal var type: String,
                                     internal var displayName: String?,
                                     internal var remark: String?,
                                     internal var scene: String?,
                                     internal var message: String?,
                                     internal var answer: String?,
                                     internal var needInsertIfSuccess: Boolean) : ResultReceiver(handler) {

        override fun onReceiveResult(resultCode: Int, resultData: Bundle) {
            val fromVerify = resultData.getBoolean(CloudP2PService.EXTRA_VERIFY_CODE, false)
            if (fromVerify) {
                val token = resultData.getString(CloudP2PService.EXTRA_TOKEN)
                val captchaType = resultData.getString(CloudP2PService.EXTRA_CAPTCHA_TYPE)
                CloudP2PServiceHelper.addFollow(context, resultReceiver, uk, type,
                        displayName, remark, scene, message, answer, needInsertIfSuccess, token, captchaType)
                return
            }
            val errno = resultData.getInt(BaseExtras.ERROR, 0)
            val response = resultData.getParcelable<AddFollowResponse>(BaseExtras.RESULT)
            if (response != null && errno == P2PErrorCode.ERROR_CODE_VERIFICATION) {
                VerifyPopupActivity.showPopup(context, this,
                        response.mCKey, response.mCType, response.mVerifyDialogTips)
            } else {
                resultReceiver?.send(resultCode, resultData)
            }
        }
    }

    /**
     * 添加好友
     *
     * @param context 上下文
     * @param resultReceiver 结果接收
     * @param uk 要添加的用户的百度uk
     * @param type 添加数据的来源类型， normal表示正常添加，contacts表示重通讯录添加
     * @param displayName 当来源为通讯录时，为对应用户的手机或邮箱名
     * @param remark 备注，可无
     * @param scene 场景，若为豁免场景，则带值（1，2，3），其他场景为空
     * @param message  验证信息，可为空
     * @param answer 回答问题，可为空
     * @param token 校验后从server获取的token
     * @param captchaType 校验后从server获取的token的类型
     */
    fun addFollow(context: Context,
                  resultReceiver: ResultReceiver?,
                  uk: Long, type: String,
                  displayName: String?,
                  remark: String?,
                  scene: String?,
                  message: String?,
                  answer: String?,
                  needInsertIfSuccess: Boolean,
                  token: String,
                  captchaType: String) {
        val wrapReceiver = WrapResultReceiver(context, Handler(), resultReceiver, uk, type,
                displayName, remark, scene, message, answer, needInsertIfSuccess)
        CloudP2PServiceHelper.addFollow(context, wrapReceiver, uk, type,
                displayName, remark, scene, message, answer, needInsertIfSuccess, token, captchaType)
    }
}
