package com.baidu.netdisk.cloudp2p.service

import android.content.Context
import android.net.Uri
import android.os.Handler
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.BaseServiceHelper
import java.util.ArrayList

/**
 * 发消息的helper
 *
 * @author wangyang34
 * @since 2019年9月5日
 */
object CloudP2PSendMessageHelper {

    /**
     * 发送消息(单人、群)（文本、分享）
     *
     * @param context   context
     * @param resultReceiver 接收器
     * @param sendType 发送类型
     * @param receiver receiver
     * @param toName 名称
     * @param toAvatarUrl toAvatarUrl
     * @param msgType 消息类型
     * @param files 文件
     * @param msg 消息体
     * @param msgId 消息id
     * @param fileSource 文件来源
     */
    fun <T, E> send(context: Context,
                    resultReceiver: ResultReceiver,
                    sendType: Int,
                    receiver: ArrayList<String>?,
                    toName: String?,
                    toAvatarUrl: String?,
                    msgType: Int,
                    files: ArrayList<E>?,
                    msg: T?,
                    msgId: Long,
                    fileSource: Int) {
        send(context, resultReceiver, sendType, receiver, toName, toAvatarUrl, msgType, files,
                msg, msgId, fileSource, null)
    }

    /**
     * 发送消息(单人、群)（文本、分享）
     *
     * @param context   context
     * @param resultReceiver 接收器
     * @param sendType 发送类型
     * @param receiver receiver
     * @param toName 名称
     * @param toAvatarUrl toAvatarUrl
     * @param msgType 消息类型
     * @param files 文件
     * @param msg 消息体
     * @param msgId 消息id
     * @param fileSource 文件来源
     * @param mMessagesUri 消息的uri
     */
    fun <T, E> send(context: Context,
                    resultReceiver: ResultReceiver,
                    sendType: Int,
                    receiver: ArrayList<String>?,
                    toName: String?,
                    toAvatarUrl: String?,
                    msgType: Int,
                    files: ArrayList<E>?,
                    msg: T?,
                    msgId: Long,
                    fileSource: Int,
                    mMessagesUri: Uri?) {
        val wrapReceiver = VerifyResultReceiver(context, Handler(), resultReceiver, mMessagesUri)
        val intent = CloudP2PServiceHelper.send(context, wrapReceiver, sendType, receiver, toName, toAvatarUrl, msgType, files,
                msg, msgId, fileSource)
        wrapReceiver.intent = intent
        BaseServiceHelper.startTargetVersionService(context, intent)
    }
}