package com.baidu.netdisk.cloudp2p.service

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.cloudp2p.network.model.SendMsgShareResponse
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.ui.verify.VerifyPopupActivity

/**
 * 封装的校验用的receiver
 *
 * @author wangyang34
 * @since 2019年9月5日
 */
class VerifyResultReceiver(private var context: Context,
                           private var handler: Handler,
                           private var resultReceiver: ResultReceiver,
                           private var messagesUri: Uri?) : ResultReceiver(handler) {
    /** 消息id */
    var msgId: Long = 0

    /** intent */
    var intent: Intent? = null

    override fun onReceiveResult(resultCode: Int, resultData: Bundle) {
        val fromVerify = resultData.getBoolean(CloudP2PService.EXTRA_VERIFY_CODE, false)
        if (intent != null && fromVerify) {
            deleteErrorMessage()
            val token = resultData.getString(CloudP2PService.EXTRA_TOKEN)
            val captchaType = resultData.getString(CloudP2PService.EXTRA_CAPTCHA_TYPE)
            intent?.putExtra(CloudP2PService.EXTRA_TOKEN, token)
            intent?.putExtra(CloudP2PService.EXTRA_CAPTCHA_TYPE, captchaType)
            context.startService(intent)
            return
        }
        val response = resultData.getParcelable<SendMsgShareResponse>(BaseExtras.RESULT)
        if (response != null && response?.needVcode()) {
            msgId = resultData?.getLong(CloudP2PService.EXTRA_MSG_ID)
            VerifyPopupActivity.showPopup(context, this,
                    response.mCKey, response.mCType, response.mVerifyDialogTips)
        } else {
            resultReceiver.send(resultCode, resultData)
        }
    }

    /**
     * 删除出错的信息
     */
    private fun deleteErrorMessage() {
        if (messagesUri == null) {
            return
        }
        context?.contentResolver?.delete(messagesUri,
                CloudP2PContract.Messages.MSG_ID + "=?", arrayOf(msgId.toString()))
    }
}