package com.baidu.netdisk.label.view.base

import android.annotation.SuppressLint
import java.util.ArrayList

import android.content.Context
import android.graphics.Paint
import android.graphics.drawable.Drawable
import android.util.AttributeSet
import android.view.Gravity
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.android.util.deviceinfo.DeviceDisplayUtils
import com.baidu.netdisk.label.interf.ILabelInfo
import com.baidu.netdisk.sns.ui.magicindicator.buildins.UIUtil
import com.netdisk.themeskin.loader.SkinManager

/**
 * 瀑布流标签的基类，所有业务类的标签样式继承BaseLabelListView
 *
 * @author wanglihong01
 *
 * @param <T>
 * 业务数据
</T> */
abstract class BaseLabelListView<T> constructor(private val mContext: Context, attrs: AttributeSet? = null,
                                                       defStyle: Int = 0)
    : LinearLayout(mContext, attrs, defStyle), ILabelInfo<T> {

    /**
     * 接口监听
     */
    private var mIOnItemClickListener: IOnItemClickListener? = null

    /**
     * 数据源
     */
    private val mDatas = ArrayList<T>()

    /**
     * 标签间的横向间距
     */
    private var itemMargins = 10

    /**
     * 标签间的纵向间距
     */
    private var itemTopMargins = 0

    /**
     * 文字大小
     */
    private var textSize = 14

    /**
     * 文字是否粗体
     */
    private var textIsBold: Boolean = false

    /**
     * 圆角度数
     */
    private var radius = 10

    private var isFirst = true

    internal var groupWidth: Int = 0
    internal var remainWidth: Int = 0

    private var viewType: Int = 0

    private var btnidth: Float = 0f

    private var lastBtn : RelativeLayout? = null

    private var mViewMode: Boolean = true
    // 不是全部展示类型时，最后一个按钮的位置
    private var mLastIndex: Int = -1
    // 编辑模式textview到右边的padding的距离
    private var mPaddingRight: Int = 0
    private var mGravity = -1

    companion object {
        // 全部都显示
        const val TYPE_FULL: Int = 0
        // 一行显示，最后一个按钮是+
        const val TYPE_ADD_BTN: Int = 1
        // 两行显示，最后一个按钮是...
        const val TYPE_TWO_LINE: Int = 2
        // 编辑模式最后一个按钮是"+自定义标签"
        const val TYPE_EDIT_SHOW: Int = 3
        // 一行显示全部
        const val TYPE_ONE_LINE: Int = 4


        // 最后一个按钮的position
        const val INDEX_LAST_BTN: Int = -1

    }

    init {
        init()
    }

    private fun init() {
        this.orientation = VERTICAL
        // 两个标签之间的距离
        itemMargins = DeviceDisplayUtils.dip2px(mContext, 4f)
//        textSize = DeviceDisplayUtils.dip2px(mContext, 14f)
        radius = DeviceDisplayUtils.dip2px(mContext, 20f)
        // 每个标签右上角的"x"
        mPaddingRight = DeviceDisplayUtils.dip2px(mContext, 6f)
    }

    /**
     * 添加数据
     *
     * @param data
     */
    fun setData(_datas: List<T>, type : Int = 0) {

        viewType = type
        initViewTypeParams(_datas)

        mDatas.clear()

        mDatas.addAll(_datas)

        removeAllViews()

        addLabelList(mDatas)
    }

    fun setData(_datas: List<T>, type : Int = 0, gravity: Int) {
        mGravity = gravity
        setData(_datas, type)
    }

    /**
     * 添加数据
     *
     * @param data
     */
    fun setData(_datas: List<T>, type : Int = 0, selectMode: Boolean) {

        mViewMode = selectMode
        setData(_datas, type)
    }

    // 初始化在各种各种模式下
    @SuppressLint("RestrictedApi")
    fun initViewTypeParams(_datas: List<T>) {
        lastBtn = null
        when (viewType) {
            TYPE_ADD_BTN -> {
                lastBtn = createLastLabel("+", null,
                        SkinManager.getInstance().getColor(R.color.personal_lable_userinfo_recommend_txt_bg))
            }
            TYPE_TWO_LINE -> {
                lastBtn = createLastLabel("...", null,
                        SkinManager.getInstance().getColor(R.color.personal_lable_userinfo_recommend_txt_bg))
                mLastIndex = _datas.size
            }
            TYPE_EDIT_SHOW -> {
                lastBtn = createLastLabel("+ 自定义标签", SkinManager.getInstance().getDrawable(R.drawable.ic_customize),
                        SkinManager.getInstance().getColor(R.color.bg_dn_common_color_light))
                mLastIndex = _datas.size
            }
        }
    }

    // 切换到编辑状态右上角x是否显示
    fun switchEditMode(selectMode: Boolean) {
        mViewMode = selectMode
        var index: Int = 0
        for (i in 0..childCount -1) {
            if (getChildAt(i) != null) {
                val layout: LinearLayout = getChildAt(i) as LinearLayout
                for (j in 0..layout.childCount - 1) {
                    val child: View? = layout?.getChildAt(j);
                    if ((child != null) && (child is RelativeLayout)) {
                        val cancel = child.findViewById<ImageView>(R.id
                                .character_cancel_label)
                        if (index != mLastIndex) {
                            if (mViewMode) {
                                cancel?.visibility = View.GONE
                            } else {
                                cancel?.visibility = View.VISIBLE
                            }
                        } else {
                            cancel?.visibility = View.GONE
                        }
                    }

                    index++
                }
            }
        }
    }

    /**
     * 界面重新绘制的时候会调用该方法
     */
    override fun onMeasure(widthMeasureSpec: Int, heightMeasureSpec: Int) {
        // 获取容器宽度
        groupWidth = measuredWidth - paddingLeft - paddingRight
        if (groupWidth > 0 && isFirst) {
            isFirst = false
            removeAllViews()
            addLabelList(mDatas)
        }
        super.onMeasure(widthMeasureSpec, heightMeasureSpec)
    }

    /**
     * 添加标签列表
     */
    private fun addLabelList(datas: List<T>) {
        // 一行剩下的空间
        remainWidth = groupWidth
        // 设置margintop为0，只有加完第一行后才会变成10
        itemTopMargins = 0
        var line = -1;
        if (groupWidth > 0) {
            val paint = Paint()
            removeAllViews()
            var layout: LinearLayout? = null
            var labelText: RelativeLayout
            var params: LayoutParams

            var i = 0
            val length = datas.size
            while (i < length) {
                val data = datas[i]
                // 创建标签
                labelText = createLabel(data, i)
                val label: TextView = labelText.findViewById(R.id.character_label)
                paint.textSize = label.paint.textSize
                var itemPadding = label.compoundPaddingLeft + label.compoundPaddingRight + mPaddingRight
                // 获取标签宽度
                val itemWidth = paint.measureText(getLabelName(data)) + itemPadding

                if (((viewType == TYPE_ADD_BTN) || (viewType == TYPE_TWO_LINE)) && (line == viewType - 1)) {
                    if (remainWidth < (itemWidth + btnidth + itemMargins)) {
                        layout?.addView(lastBtn)
                        params = lastBtn?.layoutParams as LayoutParams

                        params.setMargins(itemMargins, 0, 0, 0)
                        remainWidth = (remainWidth - btnidth + 0.5f - itemMargins).toInt()
                        break
                    }
                }

                // 此行能放下这个标签
                if ((remainWidth > (itemWidth + itemMargins)) && (layout != null)) {
                    /**
                     * 一行剩余空间大于添加标签的宽度，说明可以继续往一行添加
                     */
                    layout.addView(labelText)
                    params = labelText.layoutParams as LayoutParams
                    params.setMargins(itemMargins, 0, 0, 0)
                    remainWidth = (remainWidth - itemWidth - 0.5f - itemMargins).toInt()
                } else if (viewType == TYPE_ONE_LINE && (line == 0)) {
                    // 满一行就退出
                    break;
                } else {
                    /**
                     * 如果一行已经添加不了，就另起一行继续添加标签
                     */
                    layout = LinearLayout(mContext)
                    layout.orientation = LinearLayout.HORIZONTAL
                    layout.addView(labelText)

                    if (mGravity == Gravity.CENTER) {
                        params = LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT)
                        params.setMargins(0, itemTopMargins, 0, 0)
                        params.gravity = mGravity
                        addView(layout, params)
                    } else {
//                        addView(layout)
//                        params = layout.layoutParams as LayoutParams
//                        params.setMargins(0, itemTopMargins, 0, 0)
                        params = LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT)
                        params.setMargins(0, itemTopMargins, 0, 0)
                        params.gravity = mGravity
                        addView(layout, params)
                    }

                    remainWidth = groupWidth
                    line++
                    remainWidth = (remainWidth - itemWidth - 0.5f).toInt()
                    // 除了第一行，其他的距离上面都有边距
                    itemTopMargins = UIUtil.dip2px(mContext, 8.0).toInt()
                }
                i++
            }

            var label: TextView? = lastBtn?.findViewById(R.id.character_label)
            if (viewType == TYPE_EDIT_SHOW && (label != null)) {
                paint.textSize = label.textSize
                val itemPadding = label.compoundPaddingLeft + label.compoundPaddingRight + mPaddingRight
                // 获取标签宽度
                val itemWidth = paint.measureText(label.text as String?) + itemPadding
                if ((remainWidth > itemWidth + itemMargins) && (layout != null)) {
                    /**
                     * 一行剩余空间大于添加标签的宽度，说明可以继续往一行添加
                     */
                    layout?.addView(lastBtn)
                    params = lastBtn?.layoutParams as LayoutParams

                    params.setMargins(itemMargins, 0, 0, 0)
                } else {
                    /**
                     * 如果一行已经添加不了，就另起一行继续添加标签
                     */
                    layout = LinearLayout(mContext)
                    layout.orientation = LinearLayout.HORIZONTAL
                    layout.addView(lastBtn)

                    if (mGravity == Gravity.CENTER) {
                        params = LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT)
                        params.setMargins(0, itemTopMargins, 0, 0)
                        params.gravity = mGravity
                        addView(layout, params)
                    } else {
                        addView(layout)
                        params = layout.layoutParams as LayoutParams
                        params.setMargins(0, itemTopMargins, 0, 0)
                    }

                    remainWidth = groupWidth
                    line++
                }
                remainWidth = (remainWidth - itemWidth + 0.5f - itemMargins).toInt()
            }
        }

    }

    /**
     * 创建标签
     *
     * @param data
     * @param position
     * @return
     */
    private fun createLabel(data: T, position: Int): RelativeLayout {
        val labelText = GradientTextView(mContext)
                .setTextColor(getTextColor(data))
                .setBackgroundColor(getBackgroundColor(data))
                .setStrokeRadius(radius)
                .setTextSize(textSize)
                .setTextBold(textIsBold)
                .setSelectMode(mViewMode)
                .setLabelText(getLabelName(data))
                .setOnClickListener(mIOnItemClickListener, getLabelName(data), position)
                .build()
        return labelText
    }

    /**
     * 在个人资料页未设置单行和已设置多行时分别添加"+"和"..."
     */
    private fun createLastLabel(text : String, bg : Drawable?, bgColor: Int): RelativeLayout {
        lastBtn = GradientTextView(mContext)
                .setTextColor(SkinManager.getInstance().getColor(R.color.text_content_color))
                .setBackgroundColor(bgColor)
                .setStrokeColor(-1)
                .setStrokeRadius(radius)
                .setTextSize(textSize)
                .setTextBold(textIsBold)
                .setSelectMode(true)
                .setLabelText(text)
                .setOnClickListener(mIOnItemClickListener, text, INDEX_LAST_BTN).build()

        takeIf {
            lastBtn != null
        }.apply {
            val paint = Paint()
            var label: TextView = lastBtn?.findViewById(R.id.character_label) as TextView

            paint?.apply {
                setTextSize(label!!.paint.textSize)
                val itemPadding = label!!.getCompoundPaddingLeft() + label!!.getCompoundPaddingRight()
                // 获取标签宽度
                val itemWidth = measureText(text) + itemPadding
                btnidth = itemPadding + itemWidth
                if (bg != null) {
                    label.setBackgroundDrawable(bg)
                }
            }


        }

        return lastBtn as RelativeLayout
    }

    /**
     * 设置字体大小
     *
     * @param size
     */
    fun setSize(size: Int) {
        this.textSize = size
    }

    /**
     * 设置字体风格
     *
     * @param size
     */
    fun setBoldStyle(isBold: Boolean) {
        this.textIsBold = isBold
    }

    /**
     * 设置标签圆角
     *
     * @param radius
     */
    fun setStrokeRadius(radius: Int) {
        this.radius = radius
    }

    /**
     * 设置监听事件
     *
     * @param listener
     */
    fun setclickListener(listener: IOnItemClickListener) {
        this.mIOnItemClickListener = listener
    }


    interface IOnItemClickListener{

        fun onClick(name: String, position: Int)

        fun onDeleteTag(name: String, position: Int)

    }

}