package com.baidu.netdisk.p2p

import android.app.Service
import android.content.*
import android.os.IBinder
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.service.NetdiskService
import com.baidu.netdisk.kotlin.extension.d

/**
 * 下载服务的守护进程
 *
 * @author wangyang34
 * @since 2019年7月16日
 */
@Tag("NetdiskDownloadGuardService")
class NetdiskDownloadGuardService : Service(), ServiceConnection {

    /** p2p服务的代理类 */
    private var mP2PServerProxy: P2PServiceProxy? = null

    /** p2p通知栏中心 */
    private var mP2PNotificationCenter: P2PNotificationCenter? = null

    override fun onCreate() {
        super.onCreate()
        "onCreate".d()
        val intent = Intent(applicationContext, NetdiskService::class.java)
        applicationContext?.bindService(intent, this, Context.BIND_AUTO_CREATE)
    }

    override fun onBind(intent: Intent?): IBinder? {
        "onBind".d()
        return null
    }

    override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
        "onServiceConnected".d()
        if (service !is NetdiskService.LocalBinder) {
            return
        }
        val netdiskBinder = service.getService<Any>("")
        if (netdiskBinder !is NetdiskService) {
            return
        }
        "onServiceConnected init".d()
        applicationContext?.unbindService(this)
        mP2PServerProxy = P2PServiceProxy(applicationContext, netdiskBinder)
        mP2PServerProxy?.startProcess()

        mP2PNotificationCenter = P2PNotificationCenter(this, mP2PServerProxy)
        mP2PNotificationCenter?.onCreate()
    }

    override fun onServiceDisconnected(name: ComponentName?) {
        "onServiceDisconnected".d()
    }

    override fun onDestroy() {
        super.onDestroy()
        "onDestroy".d()
        mP2PNotificationCenter?.onDestroy()
        mP2PServerProxy?.stopProcess()
    }
}