package com.baidu.netdisk.p2p

import android.app.Notification
import android.app.Service
import android.content.Intent
import android.os.IBinder
import android.os.Messenger
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.kotlin.extension.e
import io.reactivex.Observable
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import java.lang.ref.WeakReference
import java.util.concurrent.TimeUnit

/**
 * 下载服务 为了降低主进程的崩溃率放在了新的进程中
 *
 * @author wangyang34
 * @since 2019年7月16日
 */
@Tag("NetdiskDownloadService")
class NetdiskDownloadService : Service() {

    companion object {

        /** 绑定下载通知栏消息 */
        const val MESSAGE_BIND_DOWNLOAD_NOTIFICATION = 1

        /** 解绑下载通知栏消息 */
        const val MESSAGE_UNBIND_DOWNLOAD_NOTIFICATION = 2

        /** 下载通知栏 */
        const val DOWNLOAD_NOTIFICATION = "download_notification"

        /** 下载对应的id */
        const val DOWNLOAD_ID = "download_id"

    }

    /** p2p服务端管理类 */
    private var mP2PServerManager: P2PServerManager? = null

    /** 异步任务句柄 */
    private var mDisposable: Disposable? = null

    override fun onCreate() {
        super.onCreate()
        "onCreate".d()
        initP2P()
    }

    override fun onBind(intent: Intent?): IBinder? {
        "onBind".d()
        return Messenger(P2PServiceHandler(WeakReference(this))).binder
    }

    override fun onDestroy() {
        super.onDestroy()
        "onDestroy".d()
        cancelInitP2P()
        unbindDownloadNotification()
    }

    /**
     * 初始化p2p
     */
    private fun initP2P() {
        "initP2P".d()
        mDisposable?.dispose()
        mDisposable = Observable.just(0)
                .delay(1, TimeUnit.SECONDS)
                .subscribeOn(Schedulers.io())
                .filter { mP2PServerManager == null }
                .subscribe { mP2PServerManager = P2PServerManager() }
    }

    /**
     * 取消初始化p2p
     */
    private fun cancelInitP2P() {
        "cancelInitP2P".d()
        mDisposable?.dispose()
        mDisposable = null
    }

    /**
     * 绑定下载通知栏
     * @param notification 通知栏
     * @param id
     */
    fun bindDownloadNotification(notification: Notification, id: Int) {
        "bindDownloadNotification notification: $notification id: $id".d()
        startForeground(id, notification)
    }

    /**
     * 解绑下载通知栏
     */
    fun unbindDownloadNotification() {
        "unbindDownloadNotification".d()
        try {
            stopForeground(true)
        } catch (e: Exception) {
            "stopForeground".e()
        }
    }
}