package com.baidu.netdisk.p2p

import android.app.Notification
import android.content.ComponentName
import android.content.ServiceConnection
import android.os.Bundle
import android.os.DeadObjectException
import android.os.IBinder
import android.os.Message
import android.os.Messenger
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.p2p.NetdiskDownloadService.Companion.DOWNLOAD_ID
import com.baidu.netdisk.p2p.NetdiskDownloadService.Companion.DOWNLOAD_NOTIFICATION
import com.baidu.netdisk.p2p.NetdiskDownloadService.Companion.MESSAGE_BIND_DOWNLOAD_NOTIFICATION
import com.baidu.netdisk.p2p.NetdiskDownloadService.Companion.MESSAGE_UNBIND_DOWNLOAD_NOTIFICATION

/**
 * p2p服务连接
 *
 * @author wangyang34
 * @since 2019年7月31日
 */
@Tag("P2PServiceConnection")
class P2PServiceConnection(val reference: ServiceConnection) : ServiceConnection {

    /** 消息传递 */
    private var mMessenger: Messenger? = null

    /** 延迟消息 */
    private var mMessage: Message? = null

    override fun onServiceDisconnected(name: ComponentName?) {
        "onServiceDisconnected".d()
        mMessenger = null
        reference.onServiceDisconnected(name)
    }

    override fun onServiceConnected(name: ComponentName?, service: IBinder?) {
        "onServiceDisconnected".d()
        mMessenger = Messenger(service)
        postDelayMessage()
        reference.onServiceConnected(name, service)
    }


    /**
     * 发送延迟消息
     */
    private fun postDelayMessage() {
        if (mMessenger == null || mMessage == null) {
            return
        }
        mMessenger?.apply {
            "postDelayMessage mMessage = $mMessage".d()
            try {
                send(mMessage)
            } catch (e: DeadObjectException) {
                mMessage = null
            }
            mMessage = null
        }
    }

    /**
     * 绑定通知栏
     * @param notification 下载通知栏
     * @param id
     */
    fun bindNotification(notification: Notification, id: Int) {
        "P2PServiceConnection--bindNotification".d()
        val message = Message()
        message.what = MESSAGE_BIND_DOWNLOAD_NOTIFICATION
        val bundle = Bundle()
        bundle.putParcelable(DOWNLOAD_NOTIFICATION, notification)
        bundle.putInt(DOWNLOAD_ID, id)
        message.data = bundle
        mMessage = message
        postDelayMessage()
    }

    /**
     * 解绑通知栏
     */
    fun unbindNotification() {
        "P2PServiceConnection--unbindNotification".d()
        val message = Message()
        message.what = MESSAGE_UNBIND_DOWNLOAD_NOTIFICATION
        mMessage = message
        postDelayMessage()
    }
}