@file:Suppress("DEPRECATION")

package com.baidu.netdisk.p2p

import android.app.Notification
import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.content.Intent
import android.content.IntentFilter
import android.os.Build
import android.support.annotation.RequiresApi
import android.support.v4.app.NotificationCompat
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.kotlin.extension.e
import com.baidu.netdisk.sns.util.Utility
import com.baidu.netdisk.util.NotificationUtil.VIDEO_NOTIFICATION

/**
 * 下载服务 为了降低主进程的崩溃率放在了新的进程中
 *
 * @author wangyang34
 * @since 2019年11月19日
 */
@Tag("VideoNotification")
class VideoNotification {

    /** 通知栏id */
    private val channelID = "com.baidu.netdisk.video"

    /** 通知栏name  */
    private val channelName = "video_notification"

    /** 面板点击 */
    private val notificationPanelAction = "com.baidu.netdisk.p2p.VideoNotification.panel"

    /** 通知栏 */
    var mNotification: Notification

    /** 广播 */
    private val mBroadcast: VideoNotificationBroadcast = VideoNotificationBroadcast()

    /**
     * 构造
     */
    constructor() {
        val context = BaseApplication.getInstance()
        val intent = Intent(notificationPanelAction)
        val pendingIntent = PendingIntent.getBroadcast(context, 0, intent, PendingIntent.FLAG_CANCEL_CURRENT)
        mNotification = NotificationCompat.Builder(context, channelID)
                .setContentTitle(context.resources.getString(R.string.video_notification_title))
                .setSmallIcon(R.drawable.stat_sys_download_anim1)
                .setContentIntent(pendingIntent)
                .build()
        mNotification.defaults = mNotification.defaults and Notification.DEFAULT_SOUND.inv()
        mNotification.defaults = mNotification.defaults and Notification.DEFAULT_VIBRATE.inv()
        mNotification.flags = mNotification.flags or Notification.FLAG_ONGOING_EVENT
    }

    /**
     * 显示通知栏
     */
    fun show() {
        "show".d()
        val filter = IntentFilter()
        filter.addAction(notificationPanelAction)
        BaseApplication.getInstance().registerReceiver(mBroadcast, filter)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            createNotificationChannel()
        }
        val manager = Utility.safetyGetNotificationManager() ?: return
        try {
            manager?.notify(VIDEO_NOTIFICATION, mNotification)
        } catch (e: Exception) {
            "show $e".e()
        }
    }

    /**
     * 关闭
     */
    fun close() {
        "close".d()
        val manager = Utility.safetyGetNotificationManager() ?: return
        manager.cancel(VIDEO_NOTIFICATION)
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            deleteNotificationChannel()
        }
        BaseApplication.getInstance().unregisterReceiver(mBroadcast)
    }

    /**
     * 创建通知栏channel
     */
    @RequiresApi(api = Build.VERSION_CODES.O)
    private fun createNotificationChannel() {
        "createNotificationChannel".d()
        val manager = Utility.safetyGetNotificationManager() ?: return
        val channel = NotificationChannel(channelID, channelName, NotificationManager.IMPORTANCE_LOW)
        channel.setSound(null, null)
        channel.enableVibration(false)
        channel.vibrationPattern = longArrayOf(0)
        manager.createNotificationChannel(channel)
    }

    /**
     * 删除通知栏channel
     */
    @RequiresApi(api = Build.VERSION_CODES.O)
    private fun deleteNotificationChannel() {
        "deleteNotificationChannel".d()
        val manager = Utility.safetyGetNotificationManager() ?: return
        manager.deleteNotificationChannel(channelID)
    }
}