package com.baidu.netdisk.qrunlock.server

import android.content.Context
import android.content.Intent
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.ISchedulerService
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.job.PriorityScheduler
import com.baidu.netdisk.qrunlock.server.job.QueryLockStatusJob
import com.baidu.netdisk.qrunlock.server.job.ReportUnlockJob

/**
 *
 * @author dnegjie09
 * @description:
 * @date：2019-10-26 14:29
 *
 */
class QrUnlockService(private val mScheduler: PriorityScheduler) : ISchedulerService {

    companion object {

        /** 查询锁状态  */
        val QUERY_LOCK_STATUS = "query_lock_status"

        /** 上报解锁  */
        val REPORT_UNLOCK = "report_unlock"
    }

    override fun onHandleIntent(intent: Intent?, context: Context?) {

        if (intent == null) {
            NetDiskLog.d(IQRUnlockServiceConstant.QR_UNLOCK_TAG, "ERROR : onHandleIntent -> intent == null")
            return
        }

        val receiver = intent.getParcelableExtra<ResultReceiver>(BaseExtras.RESULT_RECEIVER)
        val uid = intent.getStringExtra(BaseExtras.UID)
        val bduss = intent.getStringExtra(BaseExtras.BDUSS)
        val uuid = intent.getStringExtra(IQRUnlockServiceConstant.UUID)
        val action = intent.action

        when (action) {

            QUERY_LOCK_STATUS -> {
                NetDiskLog.d(IQRUnlockServiceConstant.QR_UNLOCK_TAG, "命中 QUERY_LOCK_STATUS")
                mScheduler.addJobWithHigh(QueryLockStatusJob(receiver, bduss, uid, uuid))
            }

            REPORT_UNLOCK -> {
                NetDiskLog.d(IQRUnlockServiceConstant.QR_UNLOCK_TAG, "命中 REPORT_UNLOCK")
                mScheduler.addJobWithHigh(ReportUnlockJob(receiver, bduss, uid, uuid, intent.getStringExtra(IQRUnlockServiceConstant.STATUS)))
            }

        }
    }
}