package com.baidu.netdisk.qrunlock.server.api

import com.baidu.netdisk.base.network.BaseApi
import com.baidu.netdisk.base.network.NetworkTask
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.net.HttpParams
import com.baidu.netdisk.kernel.architecture.net.RequestCommonParams
import com.baidu.netdisk.kernel.architecture.net.Response
import com.baidu.netdisk.qrunlock.server.IQRUnlockServiceConstant
import com.baidu.netdisk.qrunlock.server.parser.QuaryLockStatusParser
import com.baidu.netdisk.qrunlock.server.parser.ReportUnLockParser
import com.baidu.netdisk.qrunlock.server.response.QuaryLockStatusResponse
import com.baidu.netdisk.qrunlock.server.response.ReportUnLockResponse

/**
 *
 * @author dnegjie09
 * @description:
 * @date：2019-10-26 17:24
 *
 */
class UnlockApi(val bduss: String, val uid: String): BaseApi(bduss, uid) {



    /**
     * 获取锁状态
     * @return 数据获取成功
     * @throws Exception exception
     */
    @Throws(Exception::class)
    fun quaryLockStatus(uuid: String): QuaryLockStatusResponse {

        // 1、接口地址
        var url = ServerURL.getPanDomain() + "/api/pcygj/lockstatus"

//        if (NetDiskLog.isDebug()) {
//            url = "http://10.95.105.176:8080/api/pcygj/lockstatus"
//        }

        NetDiskLog.d(IQRUnlockServiceConstant.QR_UNLOCK_TAG, "quaryLockStatus uuid: " + uuid)

        // 2、接口参数
        val params = HttpParams()
        params.add("uuid", uuid)
        params.add("clienttype", RequestCommonParams.getClientType())

        // 3、请求
        val request = buildGetRequest(url, params)

        // 4、解析器
        val parser = QuaryLockStatusParser()

        return NetworkTask<QuaryLockStatusResponse>().send(request, parser)
    }

    /**
     * 获取锁状态
     * @return 数据获取成功
     * @throws Exception exception
     */
    @Throws(Exception::class)
    fun reportUnlock(uuid: String, status: String): ReportUnLockResponse {

        // 1、接口地址
        var url = ServerURL.getPanDomain() + "/api/pcygj/changelock"

//        if (NetDiskLog.isDebug()) {
//            url = "http://10.95.105.176:8080/api/pcygj/changelock"
//        }

        // 2、接口参数
        val params = HttpParams()
        params.add("uuid", uuid)
        params.add("clienttype", RequestCommonParams.getClientType())
        params.add("status", status)

        // 3、请求
        val request = buildGetRequest(url, params)

        // 4、解析器
        val parser = ReportUnLockParser()

        return NetworkTask<ReportUnLockResponse>().send(request, parser)
    }


}