package com.baidu.netdisk.qrunlock.server.job

import android.os.Bundle
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.job.BaseJob
import com.baidu.netdisk.kernel.architecture.net.exception.RemoteException
import com.baidu.netdisk.qrunlock.server.api.UnlockApi
import com.baidu.netdisk.qrunlock.server.IQRUnlockServiceConstant
import com.baidu.netdisk.qrunlock.server.response.QuaryLockStatusResponse
import org.json.JSONException
import java.io.IOException
import java.security.KeyManagementException
import java.security.KeyStoreException
import java.security.NoSuchAlgorithmException
import java.security.UnrecoverableKeyException

/**
 *
 * @author dnegjie09
 * @description: 锁状态查询任务
 * @date：2019-10-26 17:02
 *
 */
class QueryLockStatusJob(private val receiver : ResultReceiver?,
                         private val bduss : String,
                         private val uid : String,
                         private val uuid : String): BaseJob(TAG) {

    companion object {
        const val TAG = "QueryLockStatusJob"
    }

    override fun performExecute() {

        // 查询结果
        var quaryLockStatusResponse: QuaryLockStatusResponse? = null
        try {
            quaryLockStatusResponse = UnlockApi(bduss = bduss, uid = uid).quaryLockStatus(uuid = uuid)
        }catch (e: KeyManagementException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: UnrecoverableKeyException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: NoSuchAlgorithmException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: KeyStoreException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: JSONException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: IOException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: RemoteException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        }

        val bundle = Bundle()
        bundle.putParcelable(IQRUnlockServiceConstant.QUARY_LOCK_STATUS_RESULT, quaryLockStatusResponse)

        // 结果返回
        if (quaryLockStatusResponse == null || quaryLockStatusResponse.errno != IQRUnlockServiceConstant.SUCCESS_CODE) {
            receiver?.send(BaseStatus.FAILED, bundle)
        } else {
            receiver?.send(BaseStatus.SUCCESS, bundle)
        }
    }
}