package com.baidu.netdisk.qrunlock.server.job

import android.os.Bundle
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.job.BaseJob
import com.baidu.netdisk.kernel.architecture.net.Response
import com.baidu.netdisk.kernel.architecture.net.exception.RemoteException
import com.baidu.netdisk.qrunlock.server.api.UnlockApi
import com.baidu.netdisk.qrunlock.server.IQRUnlockServiceConstant
import com.baidu.netdisk.qrunlock.server.response.ReportUnLockResponse
import org.json.JSONException
import java.io.IOException
import java.security.KeyManagementException
import java.security.KeyStoreException
import java.security.NoSuchAlgorithmException
import java.security.UnrecoverableKeyException

/**
 *
 * @author dnegjie09
 * @description: 锁状态查询任务
 * @date：2019-10-26 17:02
 *
 */
class ReportUnlockJob(private val receiver : ResultReceiver?,
                      private val bduss : String,
                      private val uid : String,
                      private val uuid : String,
                      private val status : String): BaseJob(TAG) {

    companion object {
        const val TAG = "QueryLockStatusJob"
    }

    override fun performExecute() {

        // 查询结果
        var reportUnlockResponse: ReportUnLockResponse? = null
        try {
            reportUnlockResponse = UnlockApi(bduss = bduss, uid = uid).reportUnlock(uuid = uuid, status = status)
        }catch (e: KeyManagementException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: UnrecoverableKeyException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: NoSuchAlgorithmException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: KeyStoreException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: JSONException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: IOException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        } catch (e: RemoteException) {
            NetDiskLog.e(IQRUnlockServiceConstant.QR_UNLOCK_TAG, e.message, e)
        }

        // 结果返回
        val bundle = Bundle()
        bundle.putParcelable(IQRUnlockServiceConstant.REPORT_UNLOCK_RESULT, reportUnlockResponse)

        // 结果返回
        if (reportUnlockResponse == null || reportUnlockResponse.errno != IQRUnlockServiceConstant.SUCCESS_CODE) {
            receiver?.send(BaseStatus.FAILED, bundle)
        } else {
            receiver?.send(BaseStatus.SUCCESS, bundle)
        }
    }
}