package com.baidu.netdisk.qrunlock.ui

import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.support.v4.content.LocalBroadcastManager
import android.view.View
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.RelativeLayout
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountChangeHandler
import com.baidu.netdisk.backup.pim.contact.ContactSDKManager
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.qrunlock.server.IQRUnlockServiceConstant
import com.baidu.netdisk.qrunlock.server.QrUnlockServiceHelper
import com.baidu.netdisk.qrunlock.server.model.QrLockStatusData
import com.baidu.netdisk.qrunlock.server.response.QuaryLockStatusResponse
import com.baidu.netdisk.qrunlock.server.response.ReportUnLockResponse
import com.baidu.netdisk.qrunlock.util.isErrorNetWork
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.preview.audio.AudioPlayerActivity
import com.baidu.netdisk.ui.preview.audio.helper.AudioPlayListHelper
import com.baidu.netdisk.ui.preview.audio.service.AudioPlayService
import com.baidu.netdisk.ui.preview.audio.view.AudioCircleViewHelper
import com.baidu.netdisk.ui.preview.audio.view.AudioCircleViewManager
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import java.net.URLDecoder


/**
 *
 * @author dnegjie09
 * @description: be used to pc unlock
 * 用于pc端扫码解锁 UI
 * @date：2019-10-25 20:37
 */
class QrPcUnlockActivity : BaseActivity(), View.OnClickListener {

    companion object {

        const val TAG = "QrPcUnlockActivity"

        // 二维码url
        const val QR_URL = "qrurl"

    }

    /**
     * 二维码解析出的url
     */
    private lateinit var url: String

    /**
     * pc端用户uid
     */
    private lateinit var uuid: String

    /**
     * 解锁页面
     */
    private lateinit var mUnlockPage: RelativeLayout

    /**
     * 解锁图标
     */
    private lateinit var mUnlockImage: ImageView

    /**
     * pc用户名
     */
    private lateinit var mPcUserName: TextView

    /**
     * 说明文本
     */
    private lateinit var mSubContext: TextView

    /**
     * 解锁按钮
     */
    private lateinit var mUnlockBtn: LinearLayout

    /**
     * 切换账号按钮
     */
    private lateinit var mSwitchUserBtn: LinearLayout

    /**
     * 异常重试按钮
     */
    private lateinit var mRetryBotton: LinearLayout

    /**
     * 取消
     */
    private lateinit var mCannal: LinearLayout

    /**
     * 错误页面
     */
    private lateinit var mErrorPage: LinearLayout

    /**
     * 错误提示文本
     */
    private lateinit var mErrorMsg: TextView

    /**
     * 错误提示文本
     */
    private lateinit var mRetry: TextView


    /**
     * 加载过程态
     */
    private lateinit var mLoding: LinearLayout


    /**
     * 加载过程态
     */
    private lateinit var mLodingText: TextView

    /**
     * pc用户姓名
     */
    private var mName: String? = ""

    /**
     *
     */
    private var canReport = true


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        NetdiskStatisticsLogForMutilFields.getInstance()
                .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.SWEEP_QR_CODE_UNLOCK_PC)

        url = intent.getStringExtra(QR_URL)
        url = URLDecoder.decode(url)
        uuid = url.substring(url.indexOf("uuid:") + 5, url.length-1)

        NetDiskLog.d(TAG, "rul: $url, uuid: $uuid")

    }


    override fun getLayoutId(): Int {
        return R.layout.activity_qr_unlock
    }

    override fun initView() {

        // init titlebar
        initTitleBar()

        mUnlockPage = findViewById(R.id.unlock_page)

        mUnlockImage = findViewById(R.id.unlock_image)
        mPcUserName = findViewById(R.id.pc_user_name)
        mSubContext = findViewById(R.id.sub_context)

        mUnlockBtn = findViewById(R.id.unlock_botton)
        mUnlockBtn.setOnClickListener(this)

        mSwitchUserBtn = findViewById(R.id.switch_user_botton)
        mSwitchUserBtn.setOnClickListener(this)

        mRetryBotton = findViewById(R.id.retry_botton)
        mRetryBotton.setOnClickListener(this)

        mCannal = findViewById(R.id.cancel)
        mCannal.setOnClickListener(this)

        mErrorPage = findViewById(R.id.error_page)
        mErrorMsg = findViewById(R.id.error_msg)
        mRetry = findViewById(R.id.retry_btn)
        mRetry.setOnClickListener(this)


        mLoding = findViewById(R.id.loding)
        mLodingText = findViewById(R.id.loading_text)


    }

    /**
     * 初始化titlebar
     */
    private fun initTitleBar() {

        mTitleBar = BaseTitleBar(this)

        mTitleBar.setMiddleTitle(R.string.qr_unlock)

        mTitleBar.setTopTitleBarClickListener(object: ICommonTitleBarClickListener {
            override fun onRightButtonClicked(view: View?) {
            }

            override fun onBackButtonClicked() {
                onBackPressed()
            }
        })
    }

    override fun onClick(v: View?) {

        when (v?.id) {

            R.id.unlock_botton -> {
                // 解锁点击事件
                canReport = false
                reportUnlockStatus("3", ReportUnlockResultReceiver(this, Handler(), null))
            }

            R.id.switch_user_botton -> {
                // 切换账户点击事件 退出当前登录
                logout()
            }

            R.id.cancel -> {
                // 取消点击事件
                if (canReport) {
                    reportUnlockStatus("1", null)
                }
                onBackPressed()
            }

            R.id.retry_btn -> {
                // 展示重试提示态
                mErrorMsg.setText(R.string.qr_retry_unlockling)

                // 网络异常重试
                if (!checkNetWork()) {
                    return
                }
                quaryUnlockStatus()
            }

            R.id.retry_botton -> {
                // 异常重试
                if (!checkNetWork()) {
                    return
                } else {
                    showLoadingText()
                    mLodingText.setText(R.string.qr_quary_unlock_status)
                }
                quaryUnlockStatus()
            }

        }
    }

    /**
     * 退出登录
     */
    private fun logout() {

        ContactSDKManager.getInstance().cancelContactSync(ContactSDKManager.PIM_CANCEL_SYNC_BY_LOGOUT)
        AccountChangeHandler.logout(this, true, AccountChangeHandler.QR_LOGOUT_SOURCE_SEARCHBOX)

        // 切换账号是需要要将播放列表清空
        // 防止新登录账号持有上个用户记录，一直提示播放错误
        if (AudioPlayListHelper.getInstance().playingAudios != null) {
            AudioPlayListHelper.getInstance().clear()
        }

        // 账号切换隐藏悬浮球
        PersonalConfig.getInstance().remove(AudioPlayerActivity.IS_MANUAL_CLOSE_AUDIO_CIRCLE)
        PersonalConfig.getInstance().commit()
        AudioCircleViewManager.getInstance().hideAudio()
        AudioCircleViewManager.getInstance().unBindPlayerService()
        AudioCircleViewHelper.getInstance().clearAudioLocation()

        // 退出当前账号检查当前音频播放状态
        LocalBroadcastManager.getInstance(applicationContext)
                .sendBroadcast(Intent(AudioPlayService.ACTION_STOP_AUDIO_PLAY))

    }


    override fun onResume() {
        super.onResume()

        // 向服务器强求桌面锁状态
        if (!checkNetWork()) {
            return
        } else {
            showLoadingText()
        }
        quaryUnlockStatus()
    }



    /**
     * 查询锁状态
     */
    private fun quaryUnlockStatus() {

        QrUnlockServiceHelper.queryLockStatus(uuid, QuaryLockStatusResultReceiver(this, Handler(), null))

    }

    /**
     * 解锁请求
     */
    private fun reportUnlockStatus(status: String, reportUnlockResultReceiver: ReportUnlockResultReceiver?) {

        if (!checkNetWork()) {
            return
        } else {
            //  展示解锁过程感知状态
            showLoadingText()
            mLodingText.setText(R.string.qr_unlockling)

        }
        QrUnlockServiceHelper.reportUnlock(uuid, status, reportUnlockResultReceiver)

    }

    /**
     * 解锁请求
     */
    private fun checkNetWork(): Boolean {

        return if (isErrorNetWork(this)) {
            // 展示网络异常页面
            mUnlockPage.visibility = View.GONE
            mErrorPage.visibility = View.VISIBLE
            mErrorMsg.setText(R.string.windows_unlock_net_error_error_code)
           false
        } else {
            true
        }
    }

    private fun hideloagingText() {
        mLoding.visibility = View.INVISIBLE
    }

    private fun showLoadingText() {

        mLoding.visibility = View.VISIBLE

    }

    /**
     * 展示解锁页面信息
     */
    private fun showUnlockPage(errorCode: Int, data: QrLockStatusData? = null) {

        hideloagingText()

        NetDiskLog.d(TAG, "errorCode:$errorCode")
        when (errorCode) {

            /**
             * 请求成功
             */
            IQRUnlockServiceConstant.SUCCESS_CODE -> {
                mUnlockPage.visibility = View.VISIBLE
                mErrorPage.visibility = View.GONE

                // 表示已解锁
                if (data?.status == 3) {

                    mUnlockImage.setImageResource(R.drawable.ic_pc)
                    mPcUserName.visibility = View.VISIBLE
                    mPcUserName.text = data.uname
                    mName = data.uname

                    mSubContext.visibility = View.VISIBLE
                    mSubContext.setText(R.string.unlocked)

                    mUnlockBtn.visibility = View.GONE
                    mSwitchUserBtn.visibility = View.GONE
                    mRetryBotton.visibility = View.GONE
                    return
                }

                if (data?.isCurrent == 1) {
                    // 展示正常解锁页面信息
                    mUnlockImage.setImageResource(R.drawable.ic_pc)
                    mPcUserName.visibility = View.VISIBLE
                    mPcUserName.text = data.uname
                    mName = data.uname

                    mSubContext.visibility = View.VISIBLE
                    mSubContext.setText(R.string.windows_unlock_confirm_context)

                    mUnlockBtn.visibility = View.VISIBLE
                    mSwitchUserBtn.visibility = View.GONE
                    mRetryBotton.visibility = View.GONE
                } else if (data?.isCurrent == 0) {
                    // 显示用户不匹配页面
                    mUnlockImage.setImageResource(R.drawable.ic_pc_unlock)
                    mPcUserName.visibility = View.VISIBLE
                    mPcUserName.setText(R.string.can_not_unlock)
                    val text = this.resources.getString(R.string.windows_unlock_user_atypism,  data.uname)

                    mSubContext.visibility = View.VISIBLE
                    mSubContext.text = text
                    mUnlockBtn.visibility = View.GONE
                    mRetryBotton.visibility = View.GONE
                    mSwitchUserBtn.visibility = View.VISIBLE
                }

            }

            /**
             * 参数错误
             */
            IQRUnlockServiceConstant.PARAMETER_ERROE_CODE -> {
                showAbnormalPage(R.string.windows_unlock_parameter_error_code, true)
            }

            /**
             * 未登录
             */
            IQRUnlockServiceConstant.NOT_LOGGED_IN_ERROE_CODE -> {
                showAbnormalPage(R.string.windows_unlock_not_logged_in_error_code)
            }

            /**
             * 系统异常
             */
            IQRUnlockServiceConstant.SYSTEM_ERROE_CODE -> {
                showAbnormalPage(R.string.windows_unlock_system_error_error_code, true)
            }

            /**
             * 二维码错误或未找到
             */
            IQRUnlockServiceConstant.QR_CODE_ERROR_OR_NOT_FOUND_ERROE_CODE -> {
                showAbnormalPage(R.string.windows_unlock_qr_error_code)
            }

            /**
             * 二维码已过期
             */
            IQRUnlockServiceConstant.QR_CODE_HAS_EXPIRED_ERROE_CODE -> {
                showAbnormalPage(R.string.windows_unlock_qr_has_expired_error_code)
            }

//            /**
//             * 登录账号不符合
//             */
//            IQRUnlockServiceConstant.ACCPUNT_DOES_NOT_MATCH_ERROE_CODE -> {
//                showAbnormalPage(R.string.windows_unlock_does_not_match_error_code)
//
//            }
        }
    }

    /**
     * 异常页面展示
     */
    private fun showAbnormalPage(errorMsg: Int, showRetry: Boolean = false) {
        // 展示异常解锁页面信息
        mUnlockPage.visibility = View.VISIBLE
        mErrorPage.visibility = View.GONE

        mRetryBotton.visibility = if (showRetry) View.VISIBLE else View.GONE
        mUnlockBtn.visibility = View.GONE
        mSwitchUserBtn.visibility = View.GONE

        mUnlockImage.setImageResource(R.drawable.ic_pc_unlock)

        mPcUserName.visibility = View.VISIBLE
        mPcUserName.setText(errorMsg)
        mSubContext.visibility = View.INVISIBLE

    }


    /**
     * 展示解锁后toast提示
     */
    private fun showUnlockToast(errorCode: Int) {

        hideloagingText()

        if (IQRUnlockServiceConstant.SUCCESS_CODE == errorCode) {
            NetdiskStatisticsLogForMutilFields.getInstance()
                    .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.SWEEP_QR_CODE_UNLOCK_PC_SUCCESS)
        } else {
            NetdiskStatisticsLogForMutilFields.getInstance()
                    .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.SWEEP_QR_CODE_UNLOCK_PC_FAILED)
        }

        when (errorCode) {

            /**
             * 请求成功
             */
            IQRUnlockServiceConstant.SUCCESS_CODE -> {
                onBackPressed()
                showResultToast(R.string.unlock_success)
            }
            /**
             * 参数错误
             */
            IQRUnlockServiceConstant.PARAMETER_ERROE_CODE -> {
                showResultToast(R.string.windows_unlock_parameter_error_code)
            }

            /**
             * 未登录
             */
            IQRUnlockServiceConstant.NOT_LOGGED_IN_ERROE_CODE -> {
                showResultToast(R.string.windows_unlock_not_logged_in_error_code)
            }

            /**
             * 系统异常
             */
            IQRUnlockServiceConstant.SYSTEM_ERROE_CODE -> {
                showResultToast(R.string.windows_unlock_system_error_error_code)
            }

            /**
             * 二维码错误或未找到
             */
            IQRUnlockServiceConstant.QR_CODE_ERROR_OR_NOT_FOUND_ERROE_CODE -> {
                showResultToast(R.string.windows_unlock_qr_error_code)
            }

            /**
             * 二维码已过期
             */
            IQRUnlockServiceConstant.QR_CODE_HAS_EXPIRED_ERROE_CODE -> {
                showResultToast(R.string.windows_unlock_qr_has_expired_error_code)
            }

            /**
             * 登录账号不符合
             */
            IQRUnlockServiceConstant.ACCPUNT_DOES_NOT_MATCH_ERROE_CODE -> {

                mUnlockImage.setImageResource(R.drawable.ic_pc_unlock)
                mPcUserName.visibility = View.VISIBLE
                mPcUserName.setText(R.string.can_not_unlock)
                val text = this.resources.getString(R.string.windows_unlock_user_atypism, mName)

//                val spanText = SpannableString(text)
//                spanText.setSpan(ForegroundColorSpan(R.color.light_blue),
//                        text.indexOf("(") + 1,
//                        text.lastIndexOf(")"),
//                        Spanned.SPAN_INCLUSIVE_EXCLUSIVE)

                mSubContext.visibility = View.VISIBLE
                mSubContext.text = text
                mUnlockBtn.visibility = View.GONE
                mRetryBotton.visibility = View.GONE
                mSwitchUserBtn.visibility = View.VISIBLE

            }
        }
    }

    /**
     * 异常页面展示
     */
    private fun showResultToast(toastMsg: Int, showRetry: Boolean = false) {

        if (showRetry) {
            ToastHelper.showToast(this.resources.getString(toastMsg) + ", 请重试")
        } else {
            ToastHelper.showToast(toastMsg)
        }
    }

    /**
     * 查询锁状态Receiver
     */
    private class QuaryLockStatusResultReceiver(reference: QrPcUnlockActivity, handler: Handler,
                                                resultView: ResultView?)
        : BaseResultReceiver<QrPcUnlockActivity>(reference, handler, resultView) {

        override fun onSuccess(reference: QrPcUnlockActivity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)

            if (resultData == null) {
                return
            }

            val quaryLockStatusResponse = resultData
                    .getParcelable<QuaryLockStatusResponse>(IQRUnlockServiceConstant.QUARY_LOCK_STATUS_RESULT)

            if (quaryLockStatusResponse?.data == null) {
                return
            }
            if (!reference.isFinishing) {
                // 解锁页面展示
                reference.showUnlockPage(quaryLockStatusResponse.errno, quaryLockStatusResponse.data as QrLockStatusData)
            }
        }

        override fun onFailed(reference: QrPcUnlockActivity, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {

            val quaryLockStatusResponse = resultData
                    .getParcelable<QuaryLockStatusResponse>(IQRUnlockServiceConstant.QUARY_LOCK_STATUS_RESULT)
                    ?: return false

            if (!reference.isFinishing) {
                reference.showUnlockPage(quaryLockStatusResponse.errno, null)
            }
            return true
        }
    }

    /**
     * 上报解锁Receiver
     */
    private class ReportUnlockResultReceiver(reference: QrPcUnlockActivity, handler: Handler,
                                                resultView: ResultView?)
        : BaseResultReceiver<QrPcUnlockActivity>(reference, handler, resultView) {

        override fun onSuccess(reference: QrPcUnlockActivity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)

            if (resultData == null) {
                return
            }

            val quaryLockStatusResponse = resultData
                    .getParcelable<ReportUnLockResponse>(IQRUnlockServiceConstant.REPORT_UNLOCK_RESULT)
                    ?: return

            // 解锁页面展示
            if (!reference.isFinishing) {
                reference.showUnlockToast(quaryLockStatusResponse.errno)
            }
        }

        override fun onFailed(reference: QrPcUnlockActivity, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {

            val reportUnLockResponse = resultData
                    .getParcelable<ReportUnLockResponse>(IQRUnlockServiceConstant.REPORT_UNLOCK_RESULT)
                    ?: return false
            if (reference.isFinishing) {
                reference.showUnlockToast(reportUnLockResponse.errno)
            }
            return true
        }
    }

//    override fun isActivityDark(): Boolean {
//        return true
//    }

}