package com.baidu.netdisk.ui

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.text.Spannable
import android.text.SpannableString
import android.text.TextPaint
import android.text.method.LinkMovementMethod
import android.text.style.ClickableSpan
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.Button
import android.widget.EditText
import android.widget.RelativeLayout
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.network.NetworkException
import com.baidu.netdisk.base.utils.PersonalConfigKey
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.cloudp2p.provider.CloudP2PProviderHelper
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.label.entity.CharacterLabel
import com.baidu.netdisk.label.interf.ISaveLableListener
import com.baidu.netdisk.label.view.LabelListView
import com.baidu.netdisk.label.view.base.BaseLabelListView
import com.baidu.netdisk.label.view.base.BaseLabelListView.Companion.INDEX_LAST_BTN
import com.baidu.netdisk.label.view.base.BaseLabelListView.Companion.TYPE_EDIT_SHOW
import com.baidu.netdisk.label.view.base.BaseLabelListView.Companion.TYPE_FULL
import com.baidu.netdisk.task.TaskScoreManager
import com.baidu.netdisk.task.taskscore.TaskScoreSetLable
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.presenter.PersonalLabelPresenter
import com.baidu.netdisk.ui.view.IBaseView
import com.baidu.netdisk.ui.widget.AboveInputDialog
import com.baidu.netdisk.ui.widget.LoadingDialog
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.util.ToastHelper
import com.netdisk.themeskin.loader.SkinManager
import org.json.JSONArray
import org.json.JSONException


/**
 * 编辑标签界面
 */
class CharacterLabelEditActivity : BaseActivity(), IBaseView, ICommonTitleBarClickListener, ISaveLableListener {

    override fun getActivity(): Activity {
        return this
    }

    // 已选中标签的列表
    private var mEditLabel: LabelListView? = null;
    // 推荐标签列表
    private var mRecommendLable: LabelListView? = null;

    private var mLabelPresenter: PersonalLabelPresenter? = null

    // 表示选中标签是否是编辑状态，true表示非编辑，false表示编辑
    private var mViewMode: Boolean = true
    // 在保存个性标签时候弹出的进度对话框
    private var mProgressDialog: Dialog? = null

    // 跳转来源
    private var from = 0

    /**
     * 新版紧贴输入法的dialog
     */
    private lateinit var builder: AboveInputDialog

    companion object {

        const val TAG: String = "CharacterLabelEditActivity"
        // 页面来源
        var EXTRA_FROM: String = "extra_from"
        var EXTRA_INFO: String = "extra_info"

        // 从个人信息页
        const val FROM_PERSONALINFO: Int = 0
        // 从个人资料页
        const val FROM_USERINFO: Int = 1
        // 从会话页
        const val FROM_CONVERSATION = 2
        // 从会话列表页
        const val FROM_CONVERSATION_LIST = 3
        // 从活动中心任务列表页
        const val FROM_TASK_LIST = 4
        // 从消息列表页
        const val FROM_SHARE_LIST = 5

        // 从标签编辑页返回的上一页用于更新界面
        const val USERINFO_ACTIVITY_EDIT_LABLE_REQUEST = 102

        @JvmStatic
        fun startActivity(context : BaseActivity, info : String, from : Int, requestCode : Int) {
            val intent = Intent(context, CharacterLabelEditActivity::class.java)
            intent.putExtra(EXTRA_FROM, from)
            intent.putExtra(EXTRA_INFO, info)
            context.startActivityForResult(intent, requestCode)
        }
    }


    override fun isActivityDark(): Boolean {
        return true
    }

    override fun onCreateView(name: String?, context: Context?, attrs: AttributeSet?): View? {
        return super.onCreateView(name, context, attrs)
    }

    override fun getLayoutId(): Int {
        return R.layout.layout_edit_label
    }

    override fun initView() {
        from = intent.getIntExtra(EXTRA_FROM, -1)
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setMiddleTitle(R.string.personal_label)
        mTitleBar.setTopTitleBarClickListener(this)
        mTitleBar.setRightLabel(R.string.save)
        if (!mLabelPresenter?.mRecommedTags.isNullOrEmpty()) {
            findViewById<RelativeLayout>(R.id.add_lable_recommed).visibility = View.VISIBLE
        }

        var rightBtn = mTitleBar.rightButtonView
        if (rightBtn is Button) {
            rightBtn.setTextColor(SkinManager.getInstance().getColor(R.color.text_content_color))
        }

        initPrivacyView()

        // 标签编辑列表初始化
        mEditLabel = findViewById(R.id.edit_label)
        mEditLabel?.setclickListener(object : BaseLabelListView.IOnItemClickListener {
            override fun onDeleteTag(name: String, position: Int) {
                // 编辑状态-删除按钮点击
                mLabelPresenter?.deleteTag(position);
                refreshLabelListView()
            }

            override fun onClick(name: String, position: Int) {
                if (position == INDEX_LAST_BTN) {
                    // 非编辑模式则先切换成编辑模式，然后弹出自定义编辑框
                    if (mLabelPresenter?.mSelectedTags?.size ?: 0 < 12) {
                        addCustomLable()
                    } else {
                        ToastHelper.showToast(R.string.lable_add_too_much)
                    }
                }
            }
        })

        // 推荐标签列表
        mRecommendLable = findViewById(R.id.recommed_label)
        mRecommendLable?.setclickListener(object : BaseLabelListView.IOnItemClickListener {
            override fun onClick(name: String, position: Int) {
                mLabelPresenter?.setRecommendTag(position)
                refreshLabelListView()
            }

            override fun onDeleteTag(name: String, position: Int) {

            }
        })
        refreshLabelListView()
    }

    override fun initParams() {
        super.initParams()
        val info: String = intent.getStringExtra(EXTRA_INFO)
        mLabelPresenter = PersonalLabelPresenter(this, info)

    }

    /**
     * 右下角返回键
     */
    override fun onBackPressed() {
        onBackButtonClicked()
    }

    /**
     * 隐私设置初始化
     */
    fun initPrivacyView() {
        val privacyset = findViewById<TextView>(R.id.add_lable_privacy_set)

        val colorSpan = ForegroundColorSpan(activity.resources.getColor(R.color
                .cloud_conversation_guide_personal_tag))
        val builder: SpannableString = SpannableString("其它用户可通过标签找到你，可在 隐私设置 处修改该设置")
        val start = "其它用户可通过标签找到你，可在 ".length
        val len = "隐私设置".length
        builder.setSpan(colorSpan, start, start + len, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        builder.setSpan(object : ClickableSpan() {
            override fun onClick(widget: View) {
                val intent = Intent(activity, SettingChildActivity::class.java)
                intent.putExtra(SettingChildActivity.FRAGMENT_TAG,
                        SettingChildActivity.ADD_FRIEND_VERIFICATION_SETTING_TAG)
                startActivity(intent)
            }

            override fun updateDrawState(ds: TextPaint) {
                super.updateDrawState(ds)
                ds.isUnderlineText = false
            }
        }, start, start + len, Spannable.SPAN_EXCLUSIVE_EXCLUSIVE)
        privacyset.text = builder
        privacyset.highlightColor = resources.getColor(R.color.transparent)
        privacyset.movementMethod = LinkMovementMethod.getInstance()
    }

    /**
     * 保存按钮
     */
    override fun onRightButtonClicked(view: View?) {
        // 如果是编辑状态，先走保存标签的接口
        showProgressDialog(resources.getString(R.string.saveing_image))
        mLabelPresenter?.saveLable(this)
    }

    /**
     * 返回按钮
     */
    override fun onBackButtonClicked() {
        mLabelPresenter?.takeIf {
            !it.isChanged
        }?.apply {
            finish()
        } ?: this.apply{
            val dialogBuilder = DialogBuilder()
            dialogBuilder.buildTipsDialog(activity, R.string.lable_save,
                    R.string.lable_save_nor_not, R.string.save, R.string.cancel_complete)
            dialogBuilder.setOnDialogCtrListener(object : DialogCtrListener {
                override fun onCancelBtnClick() {
                    finish()
                }

                override fun onOkBtnClick() {
                    showProgressDialog(resources.getString(R.string.saveing_image))
                    mLabelPresenter?.saveLable(this@CharacterLabelEditActivity)
                }
            })
        }

    }

    // 刷新已编辑和推荐标签列表
    fun refreshLabelListView() {
        mEditLabel?.setData(mLabelPresenter?.mSelectedTags as List<CharacterLabel>,
                TYPE_EDIT_SHOW, false)
        mRecommendLable?.setData(mLabelPresenter?.mRecommedTags as List<CharacterLabel>,
                TYPE_FULL)
    }

    /**
     * 添加自定义标签
     */
    private fun addCustomLable() {
        builder = AboveInputDialog(activity, activity)
        builder.setDialogTitle(R.string.custom_lable)
        builder.setRightBtnText(R.string.ok)
        builder.setEditMaxLength(8)
        builder.setRightBtnOnClickListener {
            val name = builder.editText.text.toString().trim { it <= ' ' }
            if (NetworkException(context).checkNetworkExceptionNoTip()) {
                builder?.apply {
//                    showProgressDialog(resources.getString(R.string.loading))
                    mLabelPresenter?.checkCustomLable(name, this@CharacterLabelEditActivity)
                }
            } else {
                ToastHelper.showToast(R.string.add_failed)
            }
        }
        builder.show()
    }

    /**
     * 隐藏软键盘
     */
    private fun hideSoftKeyboard(editText: EditText, context: Context) {
        val imm = context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(editText.windowToken, 0)
    }

    /**
     * 显示正在保存对话框
     */
    protected fun showProgressDialog(message: String) {
        if (activity !== BaseActivity.getTopActivity() || activity.isFinishing) {
            NetDiskLog.d(TAG, "context is not topactivity")
            return
        }
        mProgressDialog = LoadingDialog.show(activity, message, null)
    }

    /**
     * 关闭进度对话框
     */
    fun dismissProgressDialog() {
        NetDiskLog.d(TAG, "dismissProgressDialog()")
        if (activity !== BaseActivity.getTopActivity() || activity.isFinishing()) {
            NetDiskLog.d(TAG, "context is not topactivity")
            return
        }
        if (mProgressDialog?.isShowing == true) {
            mProgressDialog?.dismiss()
        }
    }

    // 保存个性标签成功后关闭保存中对话框消失
    override fun saveLable() {
        dismissProgressDialog()
        ToastHelper.showToast(R.string.personal_lable_save_success)
        if (from == FROM_TASK_LIST) {
            TaskScoreManager.getInstance().completeTask(this, TaskScoreSetLable.TASK_TYPE_SET_LABLE)
        } else {
            setResult(Activity.RESULT_OK)
            finish()
        }
    }

    override fun saveLableFailed() {
        dismissProgressDialog()
        ToastHelper.showToast(R.string.save_error)
    }

    /**
     * 标签反作弊检查成功
     */
    override fun checkLableSuccess() {
//        dismissProgressDialog()
        hideSoftKeyboard(builder.editText, context!!)
        builder.dismiss()
        mLabelPresenter?.addCustomLable(builder.editText.text.toString())
        refreshLabelListView()
    }

    /**
     * 标签反作弊检查失败
     */
    override fun checkLableFailed() {
//        dismissProgressDialog()
        builder.setErrorInfo(R.string.lable_error)
    }


}