package com.baidu.netdisk.ui

import android.app.Activity
import android.app.Dialog
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.text.*
import android.view.KeyEvent
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.utils.PersonalConfigKey
import com.baidu.netdisk.cloudp2p.network.api.P2PErrorCode
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.share.personalpage.service.PersonalPageServiceDelegate
import com.baidu.netdisk.ui.cloudp2p.TEXT_SIZE
import com.baidu.netdisk.ui.widget.LoadingDialog
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.netdisk.themeskin.loader.SkinManager

/**
 * 个性签名设置页
 *
 * @author linwentao
 * @since 2019/8/30
 */
class PersonalSignatureActivity : BaseActivity(), ICommonTitleBarClickListener {

    companion object {

        /**
         * 签名信息
         */
        private const val SIGNATURE_TEXT = "signature_text"

        /**
         * 调起个性签名页面
         *
         * @param activity 上一次页面
         * @param intro 签名文案
         */
        fun startPersonalSignatureActivity(activity: Activity, intro: String) {
            val intent = Intent(activity.applicationContext, PersonalSignatureActivity::class.java)
            intent.putExtra(SIGNATURE_TEXT, intro)
            activity.startActivityForResult(intent, Activity.RESULT_CANCELED)
        }
    }

    /**
     * 个性签名文案
     */
    private var mSignatureInfo: String? = null

    /**
     * 编辑框
     */
    private lateinit var mEditText: EditText

    private var mSignatureNumber: TextView? = null

    /**
     * 设置个性签名的receiver
     */
    private var mSetUserSignatureResultReceiver: SetUserSignatureResultReceiver? = null

    /**
     * loading弹窗
     */
    private var mProgressDialog: Dialog? = null

    override fun getLayoutId(): Int {
        return R.layout.activity_personal_signature_layout
    }

    override fun initView() {
        // 标题栏
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setBackLayoutVisible(true)
        mTitleBar.setTopTitleBarClickListener(this)
        mTitleBar.setMiddleTitle(R.string.personal_signature)
        val rightButton = mTitleBar.rightButtonView as Button
        rightButton.visibility = View.VISIBLE
        rightButton.text = context.resources.getString(R.string.ok)
        rightButton.setTextColor(context.resources.getColor(R.color.add_friend_verify_send_text))
        rightButton.textSize = TEXT_SIZE
        rightButton.paint.isFakeBoldText = true

        mSetUserSignatureResultReceiver = SetUserSignatureResultReceiver(this, Handler())
        mSignatureNumber = findViewById(R.id.signature_number)

        // 编辑框
        mEditText = findViewById(R.id.personal_signature_edit)
        mSignatureInfo = intent?.getStringExtra(SIGNATURE_TEXT)
        if (!TextUtils.isEmpty(mSignatureInfo)) {
            mEditText.setText(mSignatureInfo)
        }
        mEditText.setSelection(mEditText.text.length)
        mEditText.setHorizontallyScrolling(false)
        mEditText.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEND) { //判断动作标识是否匹配发送
                onRightButtonClicked(null)
            }
            false
        }

        val length: Int = mEditText.text?.length ?: 0
        mSignatureNumber?.text = "$length/40"
        mEditText.addTextChangedListener(object : TextWatcher {

            // 改变时
            override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {

            }

            // 改变前
            override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {

            }

            // 改变中
            override fun afterTextChanged(s: Editable) {
                val textSum = s.toString().length
                when {
                    textSum <= 40 -> {
                        mSignatureNumber?.text = "$textSum/40"
                        mEditText.setTextColor(SkinManager.getInstance().getColor(R.color.bg_dn_search_page_text_color))
                    }
                }
            }
        })

    }

    override fun onRightButtonClicked(view: View?) {
        // 请求/user/editinfo
        showProgressDialog(R.string.saving)
        var intro = mEditText.text.toString()
        var action = "0"
        if (intro.isBlank()) {
            action = "1"
            intro = ""
        }
        PersonalPageServiceDelegate.setUserSignarure(context, mSetUserSignatureResultReceiver, intro, action)
    }

    override fun onBackButtonClicked() {
        this.finish()
    }


    override fun onDestroy() {
        dismissProgressDialog()
        super.onDestroy()
    }

    class SetUserSignatureResultReceiver
    /**
     * 构造方法
     *
     * @param reference  PersonalSignatureActivity
     * @param handler 句柄
     */
    internal constructor(reference: PersonalSignatureActivity, handler: Handler) : BaseResultReceiver<PersonalSignatureActivity>(reference, handler, null) {

        override fun onSuccess(reference: PersonalSignatureActivity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            reference.dismissProgressDialog()
            ToastHelper.showToast(R.string.videoplayer_save_success)
            val intro = reference.mEditText.text.toString()
            PersonalConfig.getInstance().putString(
                    PersonalConfigKey.KEY_PERSONAL_SIGNATURE_INFO, intro)
            PersonalConfig.getInstance().commit()
            val intent = Intent()
            intent.putExtra("intro", intro)
            reference.setResult(Activity.RESULT_OK, intent)
            reference.finish()
        }

        override fun onFailed(reference: PersonalSignatureActivity, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            reference.dismissProgressDialog()
            val errorNo = resultData.getInt(BaseExtras.ERROR)
            if (!CloudP2PServiceHelper.isNetWorkError(resultData)) {
                when (errorNo) {
                    P2PErrorCode.MESSAGE_TEXT_ERROR -> ToastHelper
                            .showToast(R.string.personal_signature_content)
                    P2PErrorCode.MESSAGE_TEXT_ERROR_NUMBER -> ToastHelper
                            .showToast(R.string.personal_signature_content)
                    else -> ToastHelper
                            .showToast(R.string.save_file_fail)
                }
            }
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    /**
     * 显示loading弹窗
     *
     * @param message 提示信息的id
     */
    private fun showProgressDialog(message: Int) {
        mProgressDialog = LoadingDialog.show(this, message)
        mProgressDialog?.setOnKeyListener(DialogInterface.OnKeyListener { dialog, keyCode, event ->
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                dismissProgressDialog()
            }
            false
        })
    }

    /**
     * 取消loading弹窗
     *
     */
    private fun dismissProgressDialog() {
        mProgressDialog?.let {
            if (!it.isShowing && isFinishing) {
                return
            }
            it.dismiss()
        }
    }

    override fun isActivityDark(): Boolean {
        return true
    }
}
