package com.baidu.netdisk.ui.aiapps.widget

import android.content.Context
import android.util.AttributeSet
import android.view.Gravity
import android.widget.ImageView
import android.widget.LinearLayout
import com.baidu.netdisk.R

private const val DEFAULT_DOT_MARGIN = 10
/**
 * Created by huantong on 2019/5/5
 */
class HorizontalPagerIndicator : LinearLayout {
    private var mLength: Int = 0
    private val mSelectedResource: Int
    private val mNormalResource: Int
    private val mLeftMargin: Int

    constructor(context: Context) : this(context, null)

    constructor(context: Context, attributes: AttributeSet?) :
            super(context, attributes) {
        val array = context.obtainStyledAttributes(attributes, R.styleable.HorizontalPagerIndicator)
        mSelectedResource = array.getResourceId(R.styleable.HorizontalPagerIndicator_dot_selected_resource, R.drawable.aiapps_default_dot_enable)
        mNormalResource = array.getResourceId(R.styleable.HorizontalPagerIndicator_dot_normal_resource, R.drawable.aiapps_default_dot_unable)
        mLeftMargin = array.getDimensionPixelSize(R.styleable.HorizontalPagerIndicator_dotMargin, DEFAULT_DOT_MARGIN)
        array.recycle()
        orientation = HORIZONTAL
        gravity = Gravity.CENTER
    }

    private fun updateLength(length: Int): Boolean {
        if (length == mLength) {
            return false
        }
        removeAllViews()
        // 只有一个点不显示
        mLength = if (length > 1) length else 0
        for (index in 0 until mLength) {
            val dot = ImageView(context)
            dot.setImageResource(mNormalResource)
            val params = LinearLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT) // 圆点大小
            if (index > 0) {
                params.leftMargin = mLeftMargin
            }
            dot.layoutParams = params
            addView(dot)
        }
        return true
    }

    fun updatePos(currentPos: Int, size: Int) {
        if (!updateLength(size)) {
            for (index in 0 until childCount) {
                val view = getChildAt(index)
                if (view is ImageView) {
                    view.setImageResource(mNormalResource)
                }
            }
        }
        getChildAt(currentPos)?.apply {
            if (this is ImageView) {
                this.setImageResource(mSelectedResource)
            }
        }

    }
}