package com.baidu.netdisk.ui.aiapps.widget

import android.content.Context
import android.graphics.*
import android.os.Handler
import android.os.Message
import android.support.v4.view.PagerAdapter
import android.support.v4.view.ViewPager
import android.text.TextUtils
import android.util.AttributeSet
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.ImageView
import com.baidu.netdisk.R

/**
 * Created by huantong on 2019/5/5
 */
class HorizontalScrollPager : FrameLayout {
    companion object {
        private const val POLL_FLAG = 99
        private const val DEFAULT_DELAY_TIME_SECOND : Int = 3
    }

    private val mViewPager : ViewPager
    var mItemClickListener : OnItemClickedListener? = null
    var mItemSelectedListener : OnItemSelectedListener? = null
    private var mImageResource = ArrayList<String>()
    var mPrePosition : Int = 0
    private val mHandler = PollHandler()
    private val mRollingFrequency : Long
    var mImageLoader : ImageLoader? = null
    private var mCanPollPage = false

    constructor(context : Context) : this(context, null)

    constructor(context: Context, attributes: AttributeSet?) : super(context, attributes) {
        val array = context.obtainStyledAttributes(attributes, R.styleable.HorizontalScrollPager)
        mRollingFrequency = array.getInt(R.styleable.HorizontalScrollPager_rollingFrequencySecond,
                DEFAULT_DELAY_TIME_SECOND) * (1000.toLong())
        array.recycle()
        mViewPager = ViewPager(context)
        mViewPager.layoutParams = LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
        addView(mViewPager)
        mViewPager.addOnPageChangeListener(ScrollPagerChangeListener())
        initData()
    }

    fun setItemClickListener(listener: OnItemClickedListener) {
        mItemClickListener = listener
    }

    fun setItemSelectedListener(listener: OnItemSelectedListener) {
        mItemSelectedListener = listener
    }

    fun setImageLoader(loader: ImageLoader) {
        mImageLoader = loader
    }

    fun setImageResource(resource : ArrayList<String>) {
        mImageResource = resource
        initData()
        startSinglePolling()
    }

    private fun initData() {
        mViewPager.adapter = Adapter()
        mPrePosition = 0
        mItemSelectedListener?.apply {
            onItemSelect(0, mImageResource.size)
        }
    }

    private fun stopSinglePolling() {
        mHandler.removeMessages(POLL_FLAG)
    }

    private fun startSinglePolling() {
        if (mCanPollPage && mImageResource.isNotEmpty()) {
            if (!mHandler.hasMessages(POLL_FLAG)) {
                mHandler.sendMessageDelayed(mHandler.obtainMessage(POLL_FLAG, this), mRollingFrequency)
            }
        }
    }

    private fun updateImageViewResource(imageView: ImageView, pos: Int) {
        var url = mImageResource[pos % mImageResource.size]
        if (TextUtils.isEmpty(url)) {
            url = ""
        }
        mImageLoader?.updateImageView(imageView, url)
    }

    override fun dispatchTouchEvent(ev: MotionEvent?): Boolean {
        ev?.apply {
            if (action == MotionEvent.ACTION_DOWN) {
                stopSinglePolling()
            } else if (action == MotionEvent.ACTION_UP) {
                startSinglePolling()
            }
        }
        return super.dispatchTouchEvent(ev)
    }

    private fun pollUpdateView() {
        mViewPager.currentItem = mPrePosition + 1
        startSinglePolling()
    }

    fun startPolling() {
        mCanPollPage = true
        startSinglePolling()
    }

    fun stopPolling() {
        mCanPollPage = false
        stopSinglePolling()
    }

    inner class Adapter : PagerAdapter() {

        override fun isViewFromObject(view: View, `object`: Any): Boolean {
            return `object` == view
        }

        /**
         * 只有一个banner则不支持滑动
         */
        override fun getCount(): Int {
            return when {
                mImageResource.size <= 0 -> 0
                mImageResource.size == 1 -> 1
                else -> Integer.MAX_VALUE
            }
        }

        override fun instantiateItem(container: ViewGroup, position: Int): Any {
            val imageView : ImageView = createImageView(position)
            container.addView(imageView)
            updateImageViewResource(imageView, position)
            return imageView
        }

        override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
            if (`object` is View) {
                container.removeView(`object`)
            }
        }

        private fun createImageView(pos : Int) : ImageView {
            val imageView = ImageView(context)
            imageView.layoutParams = ViewGroup.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
            imageView.setOnClickListener {
                mItemClickListener?.apply {
                    onClickItem(pos % mImageResource.size)
                }
            }
            imageView.setBackgroundResource(R.drawable.aiapps_default_banner)
            imageView.scaleType = ImageView.ScaleType.CENTER_CROP
            return imageView
        }
    }

    private var roundPaint: Paint = Paint().apply {
        color = Color.WHITE
        isAntiAlias = true
        style = Paint.Style.FILL
        xfermode = PorterDuffXfermode(PorterDuff.Mode.DST_OUT)
    }

    private var imagePaint: Paint = Paint().apply {
        xfermode = null
    }

    override fun dispatchDraw(canvas: Canvas) {
        canvas.saveLayer(RectF(0f, 0f, canvas.width.toFloat(), canvas.height.toFloat()), imagePaint, Canvas.ALL_SAVE_FLAG)
        super.dispatchDraw(canvas)
        drawTopLeft(canvas)
        drawTopRight(canvas)
        drawBottomLeft(canvas)
        drawBottomRight(canvas)
        canvas.restore()
    }

    private fun drawTopLeft(canvas: Canvas, topLeftRadius: Float = 16f) {
        val path = Path()
        path.moveTo(0f, topLeftRadius)
        path.lineTo(0f, 0f)
        path.lineTo(topLeftRadius, 0f)
        path.arcTo(RectF(0f, 0f, (topLeftRadius * 2), (topLeftRadius * 2)),
                -90f, -90f)
        path.close()
        canvas.drawPath(path, roundPaint)
    }

    private fun drawTopRight(canvas: Canvas, topRightRadius: Float = 16f) {
        if (topRightRadius > 0) {
            val width = width
            val path = Path()
            path.moveTo((width - topRightRadius), 0f)
            path.lineTo(width.toFloat(), 0f)
            path.lineTo(width.toFloat(), topRightRadius)
            path.arcTo(RectF((width - 2 * topRightRadius), 0f, width.toFloat(),
                    (topRightRadius * 2)), 0f, -90f)
            path.close()
            canvas.drawPath(path, roundPaint)
        }
    }

    private fun drawBottomLeft(canvas: Canvas, bottomLeftRadius: Float = 16f) {
        if (bottomLeftRadius > 0) {
            val height = height
            val path = Path()
            path.moveTo(0f, (height - bottomLeftRadius))
            path.lineTo(0f, height.toFloat())
            path.lineTo(bottomLeftRadius, height.toFloat())
            path.arcTo(RectF(0f, (height - 2 * bottomLeftRadius),
                    (bottomLeftRadius * 2), height.toFloat()), 90f, 90f)
            path.close()
            canvas.drawPath(path, roundPaint)
        }
    }

    private fun drawBottomRight(canvas: Canvas, bottomRightRadius: Float = 16f) {
        if (bottomRightRadius > 0) {
            val height = height
            val width = width
            val path = Path()
            path.moveTo((width - bottomRightRadius), height.toFloat())
            path.lineTo(width.toFloat(), height.toFloat())
            path.lineTo(width.toFloat(), (height - bottomRightRadius))
            path.arcTo(RectF((width - 2 * bottomRightRadius), (height - 2 * bottomRightRadius), width.toFloat(), height.toFloat()), 0f, 90f)
            path.close()
            canvas.drawPath(path, roundPaint)
        }
    }

    inner class ScrollPagerChangeListener : ViewPager.OnPageChangeListener {
        override fun onPageScrollStateChanged(state: Int) {
            // do nothing
        }

        override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
            // do nothing
        }

        override fun onPageSelected(position: Int) {
            mItemSelectedListener?.apply {
                val realCurrentPos = (position % mImageResource.size)
                onItemSelect(realCurrentPos, mImageResource.size)
            }
            mPrePosition = position
        }
    }


    class PollHandler : Handler() {

        override fun dispatchMessage(msg: Message?) {
            msg?.apply {
                if (what == POLL_FLAG) {
                    val horizontalDotView : HorizontalScrollPager = msg.obj as HorizontalScrollPager
                    horizontalDotView.pollUpdateView()
                }
            }
            if (msg?.what == POLL_FLAG) {

            }
        }
    }

    interface OnItemClickedListener {
        /**
         * 点击的位置
         */
        fun onClickItem(pos : Int)
    }

    interface OnItemSelectedListener {
        /**
         * @param selected 当前选中的位置
         * @param sum page页面总数
         */
        fun onItemSelect(selected : Int, sum : Int)
    }

    /**
     * 网络图片加载库
     */
    interface ImageLoader {
        fun updateImageView(imageView: ImageView, url : String)
    }
}