package com.baidu.netdisk.ui.businessplatform

import android.app.Activity
import android.content.Context
import android.graphics.Paint
import android.os.Bundle
import android.os.Handler
import android.support.v4.app.FragmentActivity
import android.text.style.ForegroundColorSpan
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.View
import android.view.animation.Animation
import android.view.animation.AnimationUtils
import android.view.animation.TranslateAnimation
import android.widget.LinearLayout
import com.baidu.netdisk.R
import com.baidu.netdisk.businessplatform.extension.highLight
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.kernel.android.util.RealTimeUtil
import com.baidu.netdisk.kernel.android.util.network.ConnectivityState
import com.baidu.netdisk.kernel.util.TimeUtil
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.kotlin.extension.w
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields.StatisticsKeys.PROBATIONARY_GUIDE_RUNNING_CLOSE
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields.StatisticsKeys.PROBATIONARY_GUIDE_USE_CLICK
import com.baidu.netdisk.transfer.business.io.model.NextPriviledge
import com.baidu.netdisk.transfer.service.Extras
import com.baidu.netdisk.ui.businessplatform.extension.textList
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.transfer.probationary.ProbationaryManager
import com.baidu.netdisk.ui.widget.ProbationaryGuideView
import com.baidu.netdisk.util.ToastHelper
import com.netdisk.library.objectpersistence.utils.d
import kotlinx.android.synthetic.main.business_platform_probationary_privilege_layout.view.*

@Tag("BusinessProbationaryLayout")
internal class BusinessProbationaryLayout(context: Context, attr: AttributeSet) :
        LinearLayout(context, attr), Runnable {
    private val foregroundBlue = ForegroundColorSpan(context.resources.getColor(R.color.business_platform_desc_time))

    private val foregroundYellow = ForegroundColorSpan(context.resources.getColor(R.color.business_platform_yellow))

    private var state: State? = null

    private var isInitialized: Boolean = false

    private var button: CharSequence? = null

    // 加速结束end状态
    private var canBuyPrivilege: Boolean? = null
    // 特权原价删除线
    private var primitivePrivilegePrice: CharSequence? = null
    // 特权促销价
    private var promotePrivilegePrice: CharSequence? = null
    // 买特权按钮文案
    private var buyPrivilegeButton: CharSequence? = null
    // 限时优惠
    private var promotePrivilegeTip: CharSequence? = null
    // 买会员按钮文案
    private var vipPrice: CharSequence? = null
    // 立减5元
    private var promoteVipTip: CharSequence? = null

    // 是否过期
    private var expired: Boolean? = null
    // 文案
    private var descs: Array<CharSequence>? = null

    private lateinit var onProbationaryListener: ProbationaryGuideView.OnProbationaryGuideListener

    // 传入秒，转成毫秒
    private var expireTime: Long? = null
    private var buyPrivilegeUrl: String? = null
    // 毫秒
    private var endTime: Long? = null
    private var expiredButton: String? = null
    private var unExpiredButton: String? = null
    private var expiredDescs: Array<CharSequence>? = null
    private var unExpiredDescs: Array<String>? = null
    private var tip: String? = null
    private var leftCount: Int? = null
    private var endOverdueDialogContent: String? = null
    private var svipDialogContent: String? = null
    private var privilegeLeftDialogContent: String? = null
    private var preBeginOverdueDialogContent: String? = null
    private var buyExpiredTime: Long? = null
    private var showType: Int? = null
    private val timerHandler = Handler()

    init {
        val inflater = context.getSystemService(Context.LAYOUT_INFLATER_SERVICE) as LayoutInflater
        inflater.inflate(R.layout.business_platform_probationary_privilege_layout, this)
    }

    override fun run() {
        "expired".d()
        try {
            timerHandler.removeCallbacks(this)
            expired = true
            checkExpired()
        } catch (e: Exception) {
            e.w()
        }
    }

    fun preBegin(activity: FragmentActivity, bundle: Bundle, onProbationaryListener: ProbationaryGuideView.OnProbationaryGuideListener) {
        pre_begin_layout.visibility = View.VISIBLE
        running_layout.visibility = View.GONE
        end_layout_1.visibility = View.GONE
        end_layout_2.visibility = View.GONE

        attach(activity, bundle, State.PRE_BEGIN, onProbationaryListener)

        refreshPreBeginUI()
    }

    private fun refreshPreBeginUI() {
        pre_begin_ok.text = button.d(prefix = "button")
        val background = if (expired == true || continueConsume()) R.drawable.business_platform_transfer_icon_right_btn else R.drawable.business_platform_blue_btn_bg
        pre_begin_ok.setBackgroundResource(background)
        val textColor = resources.getColor(if (expired == true || continueConsume()) R.color.business_platform_text_color_dark_gray else android.R.color.white)
        pre_begin_ok.setTextColor(textColor)

        pre_begin_flipper.textList(descs.d(prefix = "descs"))
    }

    fun running(activity: FragmentActivity, bundle: Bundle, onProbationaryListener: ProbationaryGuideView.OnProbationaryGuideListener, time: Long) {
        end_layout_1.visibility = View.GONE
        end_layout_2.visibility = View.GONE

        attach(activity, bundle, State.RUNNING, onProbationaryListener, time)

        running_flipper.textList(descs)

        if (running_layout.visibility != View.VISIBLE) running_layout.visibility = View.VISIBLE
    }

    fun end(activity: FragmentActivity, bundle: Bundle, onProbationaryListener: ProbationaryGuideView.OnProbationaryGuideListener, percent: Int, good: Boolean, privilegeBuyable: Boolean) {
        pre_begin_layout.visibility = View.GONE
        running_layout.visibility = View.GONE

        leftCount = bundle.getInt(Extras.PROBATIONARY_INFO_COUNT_DOWN)
        canBuyPrivilege = privilegeBuyable.d(prefix = "买特权开启？")

        attach(activity, bundle, State.END, onProbationaryListener, percent = percent, good = good)

        if (continueConsume()) {
            // 还有剩余券，继续用
            "还有剩余券，继续用".d()
            end_layout_2.visibility = View.GONE
            end_layout_1.visibility = View.VISIBLE
            end1_button.text = button
            end1_flipper.textList(descs)
            end1_vip_tip.visibility = GONE
            end1_vipPrice.visibility = GONE
        } else {
            // 没有卷了，可以买了
            "没有卷了，可以买了".d()
            showType = if (canBuyPrivilege.d(prefix = "canBuyPrivilege") == true &&
                    !buyPrivilegeButton.isNullOrEmpty()) {
                // 买特权按钮和买会员按钮
                "买特权按钮和买会员按钮".d()
                end_layout_1.visibility = View.GONE
                end_layout_2.visibility = View.VISIBLE

                end_descs.textList(descs)
                end2_buy_privilege_button.text = buyPrivilegeButton
                end2_promote_privilege_price.text = promotePrivilegePrice
                end2_promote_privilege_price.visibility = if (promotePrivilegePrice == null) View.GONE else View.VISIBLE
                primitive_price.text = primitivePrivilegePrice
                primitive_price.visibility = if (primitivePrivilegePrice == null) View.GONE else View.VISIBLE
                end2_promote_privilege_tip.text = promotePrivilegeTip
                end2_promote_privilege_tip.visibility = if (promotePrivilegeTip == null) View.GONE else View.VISIBLE
                end2_button.text = button
                end2_vip_price.text = vipPrice
                end2_vip_price.visibility = if (vipPrice == null) View.GONE else View.VISIBLE
                end2_promote_vip_tip.text = promoteVipTip
                end2_promote_vip_tip.visibility = if (promoteVipTip == null) View.GONE else View.VISIBLE
                if (promoteVipTip.isNullOrBlank()) 1 else 2
            } else {
                // 只显示买会员按钮
                "显示布局".d()
                end_layout_2.visibility = View.GONE
                end_layout_1.visibility = View.VISIBLE

                end1_button.text = button
                end1_vipPrice.visibility = View.GONE
                end1_vip_tip.text = promoteVipTip
                end1_vip_tip.visibility = if (promoteVipTip == null) View.GONE else View.VISIBLE
                end1_flipper.textList(descs)
                if (promoteVipTip.isNullOrBlank()) 0 else 3
            }

            showType?.let {
                NetdiskStatisticsLogForMutilFields.StatisticsKeys
                        .PROBATIONARY_GUIDE_PURCHASE_END_SHOW.updateCount(it.toString())
            }
        }
    }

    private fun attach(activity: FragmentActivity, bundle: Bundle, state: State,
                       onProbationaryListener: ProbationaryGuideView.OnProbationaryGuideListener, time: Long? = null,
                       percent: Int? = null, good: Boolean? = null) {
        visibility = if (BusinessProbationKey.canShowProbationView) {
            View.VISIBLE
        } else {
            visibility = View.GONE
            return
        }
        if (isInitialized) {
            update(bundle, state, onProbationaryListener, time, percent, good)
            if (container.visibility != View.VISIBLE) container.visibility = VISIBLE
            return
        }

        isInitialized = true

        update(bundle, state, onProbationaryListener, time, percent, good)

        pre_begin_ok.setOnClickListener {
            onPrivilegeClick(activity)
            PROBATIONARY_GUIDE_USE_CLICK.updateCount(bundle.getLong(Extras.TOTAL_DURATION)
                    .toString().d(prefix = "total_duration"))
            NetdiskStatisticsLogForMutilFields.StatisticsKeys
                    .PROBATIONARY_GUIDE_CONSUME_CLICK.updateCount(NextPriviledge.TYPE_PURCHASE)
        }
        buy_vip.setOnClickListener {
            onProbationaryListener.onBuyVipClick(true)
            showType?.let {
                NetdiskStatisticsLogForMutilFields.StatisticsKeys
                        .PROBATIONARY_GUIDE_PURCHASE_END_BUY_SVIP_CLICK.updateCount(it.toString())
            }
        }
        end_ok.setOnClickListener {
            if (continueConsume()) onProbationaryListener.onContinueProbationary()
            else {
                onProbationaryListener.onBuyVipClick(true)
                showType?.let {
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys
                            .PROBATIONARY_GUIDE_PURCHASE_END_BUY_SVIP_CLICK.updateCount(it.toString())
                }
            }
        }
        buy_privilege.setOnClickListener {
            onBuyPrivilegeClick(activity)
            showType?.let {
                        NetdiskStatisticsLogForMutilFields.StatisticsKeys
                                .PROBATIONARY_GUIDE_PURCHASE_END_BUY_PRIVILEGE_CLICK
                                .updateCount(it.toString())
            }
        }
        primitive_price.paintFlags = primitive_price.paintFlags or Paint.STRIKE_THRU_TEXT_FLAG

        val gone: (View) -> Unit = {
            visibility = View.GONE
          BusinessProbationKey.canShowProbationView = false
        }

        pre_begin_close.setOnClickListener {
            stop()
            gone(it)
        }
        running_close.setOnClickListener{
            PROBATIONARY_GUIDE_RUNNING_CLOSE.updateCount(bundle.getLong(Extras.TOTAL_DURATION)
                    .toString().d(prefix ="total_duration"))
            gone(it)
        }
        end_close_1.setOnClickListener(gone)
        end_close_2.setOnClickListener {
            gone(it)
            NetdiskStatisticsLogForMutilFields.StatisticsKeys
                    .PROBATIONARY_GUIDE_PURCHASE_END_CLOSE_CLICK.updateCount()
        }

        container.visibility = VISIBLE
    }

    private fun update(bundle: Bundle, state: State, listener: ProbationaryGuideView.OnProbationaryGuideListener,
                       time: Long? = null, percent: Int? = null, good: Boolean? = null) {
        // 状态不变多次回调，实在running状态，需要更新倒计时
        if (state.d(prefix = "传入状态") == this.state.d(prefix = "当前状态") && state == State.RUNNING) {
            updateRunning(bundle, time)
            return
        }

        this.state = state

        onProbationaryListener = listener

        fun s(key: String) = bundle.getString(key)

        state.map({
            timerHandler.removeCallbacks(this)

            // server下发秒，转成毫秒
            expireTime = bundle.getLong(Extras.BEFORE_INFO_EXPIRE_TIME, -1L) * 1000
            expiredButton = s(Extras.BEFORE_BUTTON_DOWN_EXPIRED)
            unExpiredButton = s(Extras.BEFORE_BUTTON_MSG_DOWN)
            expiredDescs = arrayOf(s(Extras.BEFORE_INFO_DOWN_EXPIRED) ?: "")
            unExpiredDescs = bundle.getStringArray(Extras.BEFORE_INTRO_DOWN)
            tip = s(Extras.BEFORE_INTRO_DOWN_DES)
            preBeginOverdueDialogContent = s(Extras.BEFORE_DIALOG_DOWN_EXPIRED)

            expired = now().d(prefix = "now") > (expireTime.d(prefix = "expireTime")
                    ?: Long.MAX_VALUE)
            expired.d(prefix = "expired")

            leftCount = bundle.getInt(Extras.PROBATIONARY_INFO_COUNT_DOWN)

            if (continueConsume()) {
                // 多张券
                pre_begin_lightening.setImageResource(R.drawable.transfer_icon_speed_right)
                pre_begin_close.setImageResource(R.drawable.transfer_icon_right_close)
                initPreBeginText()
            } else {
                // 最后一张券
                pre_begin_lightening.setImageResource(R.drawable.business_platform_icon_speed)
                pre_begin_close.setImageResource(R.drawable.business_platform_icon_download_close)
                checkExpired()
            }
        }, {
            // 使用中
            "使用中".d()
            timerHandler.removeCallbacks(this)
            leftCount = bundle.getInt(Extras.PROBATIONARY_INFO_COUNT_DOWN)

            if (continueConsume()) {
                running_lightening.setImageResource(R.drawable.transfer_icon_speed_right)
                running_close.setImageResource(R.drawable.transfer_icon_right_close)
            } else {
                running_lightening.setImageResource(R.drawable.business_platform_icon_speed)
                running_close.setImageResource(R.drawable.business_platform_icon_download_close)
            }

            updateRunning(bundle, time)

            // running状态时才执行动画，避免文件列表fragmen生命周期导致动画执行延迟，文案叠加
            startTranslateAnim(pre_begin_layout, running_layout) {
                this.state == State.RUNNING
            }
        }, {
            // 使用后
            timerHandler.removeCallbacks(this)

            endTime = now()

            val text = if (good == true) bundle.getStringArray(Extras.AFTER_INTRO_DOWN)
                    ?: emptyArray() else arrayOf(s(Extras.AFTER_INTRO_DOWN_NOT_GOOD)
                    ?: "")
            descs = text.map {
                it.replace("X", "$percent%").replace("Y", "${leftCount
                        ?: 1}").highLight("$percent", "%", switchForegroundColorSpan())
            }.toTypedArray()

            // 继续用券 还是 购买
            button = if (continueConsume()) s(Extras.AFTER_BUTTON_MSG_DOWN) else s(Extras.AFTER_VIP_BTN_TITLE)

            // 最后一张券的时候出购买的UI
            if (continueConsume()) {
                end_lightening.setImageResource(R.drawable.transfer_icon_speed_right)
            } else {
                // 可以买特权时候才有值
                primitivePrivilegePrice = s(Extras.AFTER_PRIVILEDGE_BUTTON_SUB_DES)
                promotePrivilegePrice = s(Extras.AFTER_PRIVILEDGE_BUTTON_DES)
                buyPrivilegeButton = s(Extras.AFTER_PRIVILEDGE_BUTTON_TITLE)
                promotePrivilegeTip = s(Extras.AFTER_PRIVILEDGE_TIPS)
                buyPrivilegeUrl = s(Extras.AFTER_PRIVILEDGE_BUTTON_DOWN)
                endOverdueDialogContent = s(Extras.AFTER_PRIVILEGE_OVERDUE_DIALOG).d()
                svipDialogContent = s(Extras.AFTER_SVIP_DIALOG).d()
                privilegeLeftDialogContent = s(Extras.AFTER_PRIVILEGE_LEFT_DIALOG).d()
                buyExpiredTime = bundle.getLong(Extras.AFTER_BUY_EXPIRED_TIME)
                // 只有两个按钮都显示时，开通超级会员按钮才显示第二行文案
                vipPrice = s(Extras.AFTER_VIP_PRICE)
                // 买特权按钮显示结束

                promoteVipTip = s(Extras.AFTER_VIP_PRICE_TIPS)
                NetdiskStatisticsLogForMutilFields.StatisticsKeys
                        .PROBATIONARY_GUIDE_PURCHASE_END_HAS_TASK.updateCount("${onProbationaryListener.hasRunningTask()}")

                end_lightening.setImageResource(R.drawable.business_platform_icon_speed)
            }
        })

        if (continueConsume()) {
            container.setBackgroundResource(R.color.speed_up_guide_view_bg)
            end_close_1.setImageResource(R.drawable.transfer_icon_right_close)
        } else {
            container.setBackgroundResource(R.color.business_platform_bg)
            end_close_1.setImageResource(R.drawable.business_platform_icon_download_close)
        }
    }

    private fun initPreBeginText() {
        button = unExpiredButton
        descs = unExpiredDescs?.map {
            val content = it.replace("X", "${leftCount ?: 1}")
            if (continueConsume())
                content
            else
                (content + tip).highLight(tip, switchForegroundColorSpan())
        }?.toTypedArray() ?: emptyArray()
    }

    private fun checkExpired() {
        expired?.let {
            if (it) {
                // 过期了
                timerHandler.removeCallbacks(this)

                button = expiredButton
                descs = expiredDescs
            } else {
                // 没过期
                initPreBeginText()

                expireTime?.let {
                    if (it > 0L) {
                        val period = it - now()
                        timerHandler.postDelayed(this, period.d(prefix = "period"))
                    }
                }
            }

            refreshPreBeginUI()
        }
    }

    private fun continueConsume() = leftCount?.let {
        state?.map({ it.d(prefix = "prebegin leftCount") > 1 }, { it.d(prefix = "running leftCount") > 0 }, { it.d(prefix = "end leftCount") > 0 })
    } ?: false

    private fun updateRunning(bundle: Bundle, counterDown: Long?) {
        val timeLeft = getLeftTime(counterDown) ?: return
        descs = bundle.getStringArray(Extras.INCRING_INTRO_DOWN)?.map {
            it.replace("X", "${timeLeft.first}${timeLeft.second}")
                    .highLight("${timeLeft.first}${timeLeft.second}",
                            switchForegroundColorSpan())
        }?.toTypedArray() ?: emptyArray()
    }

    private fun getLeftTime(counterDown: Long?): Pair<CharSequence,CharSequence>? {
        val time = counterDown ?: 0L
        return when {
            time > TimeUtil.ONEHOUR -> {
                val hours = if (time % TimeUtil.ONEHOUR == 0L)
                    time / TimeUtil.ONEHOUR
                else
                    time / TimeUtil.ONEHOUR + 1
                "$hours" to "小时"
            }
            time > TimeUtil.ONEMINUTE * 5 -> "${time / TimeUtil.ONEMINUTE}" to "分钟"
            time > 0 -> "${time / 1000}" to "秒"
            else -> null
        }
    }


    private fun switchForegroundColorSpan() = if (continueConsume()) foregroundYellow else foregroundBlue

    private fun tryToConsume(activity: Activity) {
        // 买会员弹窗按钮事件
        val listener = object : DialogCtrListener {
            //买超级会员
            override fun onCancelBtnClick() = onProbationaryListener.onBuyVipClick(true)

            override fun onOkBtnClick() {
                ProbationaryManager(context).turnOn(false)
            }
        }

        // 买会员弹窗
        fun dialog() = with(DialogBuilder()) {
            buildTipsDialog(activity, activity.getString(R.string.overdue_title),
                    preBeginOverdueDialogContent, activity.getString(R.string.button_cancel),
                    activity.getString(R.string.button_cancel))

            setOnDialogCtrListener(listener)
        }

        when {
            !ConnectivityState.isConnected(activity) -> ToastHelper.showLengthLongToast(activity, R.string.network_exception_message)
            expired == true -> {
                if (continueConsume()) {
                    ToastHelper.showToast(R.string.download_privilege_overdue)
                    ProbationaryManager(context).turnOn(false)
                } else dialog()
            }
            !onProbationaryListener.hasRunningTask() -> {
                ToastHelper.showToast(R.string.probationary_no_download_task_tips)
                NetdiskStatisticsLogForMutilFields.StatisticsKeys
                        .PROBATIONARY_PRIVILEGE_CLICK_NO_RUNNING_TASK.updateCount()
            }
            else -> consume()
        }
    }

    private fun onPrivilegeClick(activity: Activity) = state?.let {
        it.map({ if (expired == true) onProbationaryListener.onBuyVipClick(true) else tryToConsume(activity) },
                {},
                { onProbationaryListener.onBuyVipClick(true) })
    }

    private fun onBuyPrivilegeClick(activity: Activity) = endTime?.let {
        // 判断显示界面和点击按钮间隔1小时
        onProbationaryListener.onBuyPrivilege(buyPrivilegeUrl.d(), it, buyExpiredTime
                ?: 0, endOverdueDialogContent, svipDialogContent, privilegeLeftDialogContent)
    }

    // 返回秒
    private fun now() = RealTimeUtil.getTime()

    private fun consume() {
        "consume".d()
        timerHandler.removeCallbacks(this)
        onProbationaryListener.onProbationaryClick()
    }

    private fun stop() = timerHandler.removeCallbacks(this)

    private enum class State { PRE_BEGIN, RUNNING, END }

    private inline fun <T> State.map(preBegin: () -> T,
                                     running: () -> T,
                                     end: () -> T) = when (this) {
        State.PRE_BEGIN -> preBegin()
        State.RUNNING -> running()
        State.END -> end()
    }

    private fun startTranslateAnim(viewToGone: View, viewToVisible: View, predicate: () -> Boolean) {
        val animation = AnimationUtils.loadAnimation(context,
                R.anim.probationary_translate) as TranslateAnimation
        animation.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationStart(animation: Animation) {
                "onAnimationStart".d()
                if (predicate()) {
                    viewToVisible.visibility = View.VISIBLE
                    viewToGone.visibility = View.GONE
                }
            }

            override fun onAnimationEnd(animation: Animation) {
                "onAnimationEnd".d()
            }

            override fun onAnimationRepeat(animation: Animation) {
                "onAnimationRepeat".d()
            }
        })
        viewToVisible.startAnimation(animation)
        invalidate()
    }

    fun onResume() {
        visibility = if (BusinessProbationKey.canShowProbationView && isInitialized) {
            View.VISIBLE
        } else {
            View.GONE
        }
    }
}