package com.baidu.netdisk.ui.businessplatform

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.text.TextUtils
import android.view.KeyEvent
import android.view.View
import android.view.ViewGroup
import android.view.animation.Animation
import android.view.animation.AnimationUtils
import com.baidu.netdisk.R
import com.baidu.netdisk.base.network.NetworkException
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.android.util.deviceinfo.DeviceDisplayUtils
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.payment.viporder.service.Extras
import com.baidu.netdisk.payment.wallet.WalletProxy
import com.baidu.netdisk.platform.extension.addFragment
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.ui.webview.*
import com.baidu.netdisk.ui.webview.hybrid.action.*
import kotlinx.android.synthetic.main.business_platform_activity_single_privilege.*

@Tag("BusinessSingleInterestsActivity")
class BusinessSinglePrivilegeActivity : BaseWebViewActivity(), IHybridSinglePrivilege {

    private val VIEW_HEIGHT = "view_height"
    private val FIRST_FRAGMENT = "FIRST_FRAGMENT"
    private val SECOND_FRAGMENT = "SECOND_FRAGMENT"

    private var mFirstPage: BaseWebViewFragment? = null
    private var mSecondPage: BaseWebViewFragment? = null

    private var isOpenResultPage: Boolean = false
    private var isPaySuccess = false
    private var payResultPageUrl: String? = null
    private var pageHeight = -1

    companion object {
        const val PAY_RESULT_PAGE = "https://pan.baidu.com/buy/checkout/singlePaySuc"
        const val EXTRA_IS_OPEN_RESULT_PAGE = "EXTRA_IS_OPEN_RESULT_PAGE"
        const val PAY_RESULT_PAGE_URL = "PAY_RESULT_PAGE_URL"
        const val IS_PAY_SUCCESS = "IS_PAY_SUCCESS"
        const val OPEN_RESULT_PAGE_CODE = 1
        @JvmStatic
        fun startActivityForResult(activity: Activity, url: String, resultPageUrl: String?, requestCode: Int,
                                   isOpenResultPage: Boolean = false) {
            if (!NetworkException(activity).checkNetworkException()) {
                return
            }
            val intent = Intent(activity, BusinessSinglePrivilegeActivity::class.java).apply {
                putExtra(BaseWebViewFragment.EXTRA_URL, url)
                putExtra(EXTRA_IS_OPEN_RESULT_PAGE, isOpenResultPage)
                putExtra(PAY_RESULT_PAGE_URL, resultPageUrl)
            }
            activity.startActivityForResult(intent, requestCode)
        }

        @JvmStatic
        fun getIntent(context: Context, url: String, isOpenResultPage: Boolean = false): Intent {
            return Intent(context, BusinessSinglePrivilegeActivity::class.java).apply {
                putExtra(BaseWebViewFragment.EXTRA_URL, url)
                putExtra(EXTRA_IS_OPEN_RESULT_PAGE, isOpenResultPage)
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
        val url = intent.getStringExtra(BaseWebViewFragment.EXTRA_URL)
        isOpenResultPage = intent.getBooleanExtra(EXTRA_IS_OPEN_RESULT_PAGE, false)
        isPaySuccess = intent.getBooleanExtra(IS_PAY_SUCCESS, false)
        payResultPageUrl = intent.getStringExtra(PAY_RESULT_PAGE_URL)
        pageHeight = getPageHeight(url)
        setPageHeight(content_url, pageHeight)
    }

    override fun getLayoutId(): Int {
        return R.layout.business_platform_activity_single_privilege
    }

    override fun initView() {
        image_back.setOnClickListener{ closeSecondPage() }

        image_exit.setOnClickListener {
            if (isOpenResultPage) {
                setResult(RESULT_OK, isPaySuccess())
            }
            finish()
        }
    }

    override fun onTitleChange(url: String, title: String) {
        // do nothing
    }

    private fun setPageHeight(layout: View, height: Int) {
        val layoutParams = layout.layoutParams
        layoutParams.height = height
        layout.layoutParams = layoutParams
    }

    private fun getPageHeight(url: String): Int {
        val uri = Uri.parse(url)
        val heightParam = uri.getQueryParameter(VIEW_HEIGHT)
        // 类型转换异常，取默认值300
        val height = if (heightParam.isNullOrBlank()) 300 else heightParam.toIntOrNull()

        return DeviceDisplayUtils.getPx(height ?: 300)
    }


    override fun initFragment() {
        mFirstPage = WebViewFragmentBuilder()
                .setWebViewClient(BusinessSinglePrivilegeClient(this, this, actionManager))
                .setWebChromeClient(BusinessLoadingWebChromeClient())
                .setDownloadListener(BaseOnDownloadListener())
                .setUrlLoadable(ImmediatelyUrlLoader(DefaultCookiesSyncer(
                        applicationContext)))
                .build()

        mFragment = mFirstPage?.apply {
            arguments = Bundle().apply {
                val url = intent.getStringExtra(BaseWebViewFragment.EXTRA_URL)
                putString(BaseWebViewFragment.EXTRA_URL, url)
            }
        }

        addFragment(mFragment, R.id.content_url, FIRST_FRAGMENT)
    }

    // 加载第二页fragment
    private fun initSecondPage(url: String) {
        mSecondPage = WebViewFragmentBuilder()
                .setWebViewClient(BusinessSinglePrivilegeClient(this, this, actionManager))
                .setWebChromeClient(BusinessLoadingWebChromeClient())
                .setDownloadListener(BaseOnDownloadListener())
                .setUrlLoadable(ImmediatelyUrlLoader(DefaultCookiesSyncer(
                        applicationContext)))
                .build()
        mSecondPage?.apply {
            arguments = Bundle().apply {
                putString(BaseWebViewFragment.EXTRA_URL, url)
            }
        }

        mFragment = mSecondPage

        addFragment(mFragment, R.id.content_url_second, SECOND_FRAGMENT)
    }

    private fun openSecondPage(url: String) {
        initSecondPage(url)
        setPageHeight(content_url_second, pageHeight)
        page_first.clearAnimation()
        val outAnimation = AnimationUtils.loadAnimation(activity,
                R.anim.business_platform_out_to_left)
        outAnimation.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationStart(animation: Animation) {

            }

            override fun onAnimationEnd(animation: Animation) {
                page_first.gone()
            }

            override fun onAnimationRepeat(animation: Animation) {

            }
        })
        page_first.startAnimation(outAnimation)

        page_second.clearAnimation()
        val inAnimation = AnimationUtils.loadAnimation(activity,
                R.anim.business_platform_in_from_right)
        inAnimation.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationStart(animation: Animation) {
                page_second.show()
            }

            override fun onAnimationEnd(animation: Animation) {

            }

            override fun onAnimationRepeat(animation: Animation) {

            }
        })
        page_second.startAnimation(inAnimation)
    }

    private fun closeSecondPage() {
        page_first.clearAnimation()
        page_first.show()
        val inAnimation = AnimationUtils.loadAnimation(activity,
                R.anim.business_platform_in_from_left)
        page_first.startAnimation(inAnimation)

        page_second.clearAnimation()
        val outAnimation = AnimationUtils.loadAnimation(activity,
                R.anim.business_platform_out_to_right)
        outAnimation.setAnimationListener(object : Animation.AnimationListener {
            override fun onAnimationStart(animation: Animation) {

            }

            override fun onAnimationEnd(animation: Animation) {
                page_second.gone()
                mFragment = mFirstPage
            }

            override fun onAnimationRepeat(animation: Animation) {

            }
        })
        page_second.startAnimation(outAnimation)
    }

    /**
     * hybrid action 管理
     * @return
     */
    override fun getActionManager(): IActionManager {
        return HybridActionManager.Builder()
                .setHybridActionPayProduct(HybridActionPayProduct(this, this))
                .setHybridActionForward(HybridActionForward(this))
                .setHybridActionPreData(HybridActionPreLoad(this, this))
                .setHybridActionSinglePrivilege(HybridActionSinglePrivilege(this, this))
                .build()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == OPEN_RESULT_PAGE_CODE && resultCode == Activity.RESULT_OK) {
            val isPaySuccess = data?.getBooleanExtra(IS_PAY_SUCCESS, false)
            if (isPaySuccess == true) {
                setResult(RESULT_OK)
            }
            finish()
        } else if (requestCode == HybridActionForward.REQUEST_CODE && resultCode == Activity.RESULT_OK) {
            finish()
        }
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent): Boolean {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (isOpenResultPage) {
                setResult(RESULT_OK, isPaySuccess())
                finish()
                return true
            }
            return if (page_first.isShown) {
                super.onKeyDown(keyCode, event)
            } else {
                closeSecondPage()
                true
            }
        }
        return super.onKeyDown(keyCode, event)
    }

    override fun openHalfPage(url: String) {
        if (page_first.isShown) {
            openSecondPage(url)
        }
    }

    override fun closePage() {
        setResult(RESULT_OK, isPaySuccess())
        finish()
    }

    fun hideActivity() {
        root.gone()
    }

    private fun isPaySuccess() = if(isPaySuccess) Intent().apply { putExtra(IS_PAY_SUCCESS, isPaySuccess) } else null

    private fun getPayResultPageUrl(): String = payResultPageUrl ?: PAY_RESULT_PAGE

    override fun onInterceptPayResult(): Boolean {
        return true
    }

    override fun dispatchPayResult(resultCode: Int, resultData: Bundle?) {
        super.dispatchPayResult(resultCode, resultData)
        if (resultCode == BaseStatus.OPERATING) {
            resultData?.apply {
                if (!isEmpty) {
                    isPaySuccess = getBoolean(WalletProxy.PAY_RESULT_IS_SUCCESS, false)
                    val orderNo = getString(Extras.RESULT_SUCCESS)
                    if (!TextUtils.isEmpty(orderNo)) {
                        var url = getPayResultPageUrl()
                        val parameter = "order_no=$orderNo"
                        url += if (url.contains("?")) "&$parameter" else "?$parameter"
                        hideActivity()

                        val intent = getIntent(this@BusinessSinglePrivilegeActivity, url, true).apply {
                            putExtra(IS_PAY_SUCCESS, isPaySuccess)
                        }
                        startActivityForResult(intent, OPEN_RESULT_PAGE_CODE)
                    }
                }
            }
        }
    }
}