package com.baidu.netdisk.ui.businessplatform

import android.content.DialogInterface
import android.graphics.Color
import android.graphics.Typeface
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.text.SpannableString
import android.text.Spanned
import android.text.style.AbsoluteSizeSpan
import android.text.style.StyleSpan
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.BuildConfig
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.account.PrivilegeChangedGuideHelper
import com.baidu.netdisk.account.model.ConfigCloudUnzip
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.businessplatform.extension.isVisible
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey
import com.baidu.netdisk.kotlin.extension.Bundle
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.preview.io.model.UnzipVipGuide
import com.baidu.netdisk.preview.io.model.UnzipVipGuideBean
import com.baidu.netdisk.ui.preview.unzip.CloudUnzipHelper
import com.baidu.netdisk.ui.vip.VipActivity
import kotlinx.android.synthetic.main.layout_vip_guide.*

class BusinessVipGuideDialog : DialogFragment() {

    companion object {
        private const val FILE_SIZE = "FILE_SIZE"
        private const val VIP_GUIDE_CONFIG = "VIP_GUIDE_CONFIG"
        private const val IS_UNZIPPED = "IS_UNZIPPED"
        private const val HAS_INCENTIVE = "HAS_INCENTIVE"
        const val G = 1024 * 1024 * 1024
        private const val SHOW_STATUS = "1"
        private const val HIDE_STATUS = "0"
        private const val DEFAULT_VIP_GUIDE = "0"
        private const val DEFAULT_SVIP_GUIDE = "1"
        private const val DEFAULT_UP_SVIP_GUIDE = "2"
        private const val CLICK_TYPE_SVIP = "svip"
        private const val CLICK_TYPE_UP_SVIP = "upgradesvip"
        private const val CLICK_TYPE_VIP = "vip"
        private const val CLICK_TYPE_SINGLE = "single"
        private const val CLICK_TYPE_INCENTIVE = "incentive"

        fun newInstance(config: UnzipVipGuideBean?, fileSize: Long, showIncentive: Boolean, isUnzipped: Boolean): BusinessVipGuideDialog {
            return BusinessVipGuideDialog().apply {
                arguments = Bundle {
                    VIP_GUIDE_CONFIG - config
                    FILE_SIZE - fileSize
                    IS_UNZIPPED - isUnzipped
                    HAS_INCENTIVE - showIncentive
                }
            }
        }
    }

    private var config: UnzipVipGuideBean? = null
    private var fileSize: Long? = 0L
    private var isUnzipped = false
    private val normalVipThreshold = ConfigCloudUnzip.getLimitPackageSize(AccountUtils.ILevel.VIP)
    private val sVipThreshold = ConfigCloudUnzip.getLimitPackageSize(AccountUtils.ILevel.SVIP)
    private val isUpgradeSVip = AccountUtils.ILevel.VIP == AccountUtils.getInstance().level
    private var iBusinessVipGuideView: IBusinessVipGuideView? = null
    var hasIncentive: Boolean = false
    private var guideType: String? = null
    private var incentiveShowStatus: String = HIDE_STATUS
    private var singleShowStatus: String = HIDE_STATUS

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        dialog.window?.apply {
            decorView.setPadding(0, 0, 0, 0)
            setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
            attributes.apply {
                width = ViewGroup.LayoutParams.MATCH_PARENT
                gravity = Gravity.BOTTOM or Gravity.CENTER_HORIZONTAL
            }
        }
        return inflater.inflate(R.layout.layout_vip_guide, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initClickListener()
        initParams()
        initView()
    }


    fun initParams() {
        arguments?.apply {
            config = getParcelable(VIP_GUIDE_CONFIG)
            fileSize = getLong(FILE_SIZE, -1)
            isUnzipped = getBoolean(IS_UNZIPPED, false)
            hasIncentive = getBoolean(HAS_INCENTIVE, false)
        }
    }

    fun initView() {
        when (fileSize) {
            in 0..normalVipThreshold -> {
                guideType = config?.unzipVipGuide?.guideType.toString()
                initVipUnzipLimitDialog()
            }
            in normalVipThreshold..sVipThreshold -> {
                guideType = if (isUpgradeSVip) {
                    initUpSVipDialog()
                    config?.unzipUpgradeSVipGuide?.guideType.toString()
                } else {
                    initSVipUnzipLimitDialog()
                    config?.unzipSVipGuide?.guideType.toString()
                }
            }
            else -> {
                dismiss()
                activity?.finish()
            }
        }
        statistics(StatisticsKey.BUSINESS_UNZIP_VIP_GUIDE_SHOW)
    }

    private fun setIcon(url: String?, defResId: Int) {
        GlideHelper.getInstance().displayImageFromNetwork(url, defResId, true, icon, null)
    }

    private fun setTitle(text: String?) {
        desc.show(!text.isNullOrBlank())
        desc.text = text
    }

    private fun setMessage(msg: String?) {
        top_desc.show(!msg.isNullOrBlank())
        top_desc.text = msg
    }

    private fun setSVipButton(desc: String?, price: String? = null, coupon: String? = null) {
        svip_button.show(!desc.isNullOrBlank())
        if (!desc.isNullOrBlank() && !price.isNullOrBlank()) {
            svip_button.text = processSpan(desc, price)
        } else {
            svip_button.text = desc
        }

        svip_coupon.text = coupon
        svip_coupon.show(svip_button.isVisible() && !coupon.isNullOrBlank())
    }

    private fun setUpSVipButton(desc: String?, price: String? = null, coupon: String? = null) {
        up_svip_button.show(!desc.isNullOrBlank())
        if (!desc.isNullOrBlank() && !price.isNullOrBlank()) {
            up_svip_button.text = processSpan(desc, price)
        } else {
            up_svip_button.text = desc
        }

        up_svip_coupon.text = coupon
        up_svip_coupon.show(up_svip_button.isVisible() && !coupon.isNullOrBlank())
    }

    private fun setVipButton(desc: String?, price: String? = null, coupon: String? = null) {
        vip_button.show(!desc.isNullOrBlank())
        if (!desc.isNullOrBlank() && !price.isNullOrBlank()) {
            vip_button.text = processSpan(desc, price)
        } else {
            vip_button.text = desc
        }
        vip_coupon.text = coupon
        vip_coupon.show(vip_button.isVisible() && !coupon.isNullOrBlank())
    }

    private fun setOpenVipText(text: String?) {
        open_vip.show(!text.isNullOrBlank())
        open_vip.text = text
    }

    private fun setOtherUnzip(text: String?) {
        // 激励视频没有剩余次数时不展示激励视频入口
        if (hasIncentive) {
            other_unzip.show(!text.isNullOrBlank())
            other_unzip.text = text
        }
        incentiveShowStatus = if (other_unzip.isVisible()) {
            SHOW_STATUS
        } else {
            HIDE_STATUS
        }
    }

    private fun setSinglePrivilegeButton(desc: String?, price: String? = null, coupon: String? = null) {
        single_privilege_button.show(!desc.isNullOrBlank())
        if (!desc.isNullOrBlank() && !price.isNullOrBlank()) {
            single_privilege_button.text = processSpan(desc, price)
        } else {
            single_privilege_button.text = desc
        }
        single_privilege_coupon.text = coupon
        single_privilege_coupon.show(single_privilege_button.isVisible() && !coupon.isNullOrBlank())
        singleShowStatus = if (single_privilege_button.isVisible()) {
            SHOW_STATUS
        } else {
            HIDE_STATUS
        }
    }

    private fun initClickListener() {
        svip_button.setOnClickListener {
            //开启超级会员tab
            activity?.apply {
                PrivilegeChangedGuideHelper().toSvipPay(this,
                        VipActivity.FROM_UNZIP_TO_UPGRADE_SVIP)
            }

            statistics(StatisticsKey.BUSINESS_UNZIP_VIP_GUIDE_CLICK, CLICK_TYPE_SVIP)
        }
        up_svip_button.setOnClickListener {
            //开启超级会员tab
            activity?.apply {
                PrivilegeChangedGuideHelper().toSvipPay(this,
                        VipActivity.FROM_UNZIP_TO_UPGRADE_SVIP)
            }

            statistics(StatisticsKey.BUSINESS_UNZIP_VIP_GUIDE_CLICK, CLICK_TYPE_UP_SVIP)
        }
        vip_button.setOnClickListener {
            activity?.apply {
                PrivilegeChangedGuideHelper().toVipPay(this,
                        VipActivity.FROM_UNZIP_TO_UPGRADE_SVIP)
            }
            statistics(StatisticsKey.BUSINESS_UNZIP_VIP_GUIDE_CLICK, CLICK_TYPE_VIP)
        }
        open_vip.setOnClickListener {
            activity?.apply {
                PrivilegeChangedGuideHelper().toVipPay(this,
                        VipActivity.FROM_UNZIP_TO_UPGRADE_SVIP)
            }
            statistics(StatisticsKey.BUSINESS_UNZIP_VIP_GUIDE_CLICK, CLICK_TYPE_VIP)
        }
        single_privilege_button.setOnClickListener {
            config?.singlePayUrlConfig?.apply {
                iBusinessVipGuideView?.goSinglePay(singlePayUrl, singlePayResultPageUrl)
            }
            statistics(StatisticsKey.BUSINESS_UNZIP_VIP_GUIDE_CLICK, CLICK_TYPE_SINGLE)
        }
        other_unzip.setOnClickListener {
            iBusinessVipGuideView?.showIncentive()
            statistics(StatisticsKey.BUSINESS_UNZIP_VIP_GUIDE_CLICK, CLICK_TYPE_INCENTIVE)
        }
        image_exit.setOnClickListener {
            dismiss()
        }
    }

    override fun onDismiss(dialog: DialogInterface?) {
        super.onDismiss(dialog)
        if (iBusinessVipGuideView?.canCloseGuideView() == true){
            activity?.finish()
        }
    }

    private fun initVipUnzipLimitDialog() {
        if (isServerDataUsable(config?.unzipVipGuide)) {
            initBaseDialog(config?.unzipVipGuide)
        }
        // 配置为空，或者配置的（激励视频、单权益）引导由于条件限制被屏蔽，会导致没有按钮展示，此时显示默认引导
        if (!isButtonShow()) {
            setIcon(null, R.drawable.business_platform_icon_zip)
            setTitle(null)
            setMessage(getString(R.string.business_platform_vip_guide_desc, CloudUnzipHelper.getUnzipSize(AccountUtils.ILevel.VIP)))
            setVipButton(getString(R.string.business_platform_open_vip))
            guideType = DEFAULT_VIP_GUIDE
        }
    }

    private fun initSVipUnzipLimitDialog() {
        if (isServerDataUsable(config?.unzipSVipGuide)) {
            initBaseDialog(config?.unzipSVipGuide)
        }
        // 配置为空，或者配置的（激励视频、单权益）引导由于条件限制被屏蔽，会导致没有按钮展示，此时显示默认引导
        if (!isButtonShow()) {
            setIcon(null, R.drawable.business_platform_icon_zip)
            setTitle(null)
            setMessage(getString(R.string.business_platform_svip_guide_desc,
                    CloudUnzipHelper.getUnzipSize(AccountUtils.ILevel.SVIP)))

            setSVipButton(getString(R.string.business_platform_open_svip))

            guideType = DEFAULT_SVIP_GUIDE
        }
    }

    // 升级超级会员的弹窗，pm确认普通会员只引导升级超级会员不出现激励视频和单权益
    private fun initUpSVipDialog() {
        if (isServerDataUsable(config?.unzipUpgradeSVipGuide)) {
            initBaseDialog(config?.unzipUpgradeSVipGuide)
        }
        // 配置为空，或者配置的（激励视频、单权益）引导由于条件限制被屏蔽，会导致没有按钮展示，此时显示默认引导
        if (!isButtonShow()) {
            setIcon(null, R.drawable.business_platform_icon_zip)
            setTitle(getString(R.string.business_platform_upgrade_svip_title,
                    CloudUnzipHelper.getUnzipSize(AccountUtils.ILevel.VIP)))
            setMessage(getString(R.string.business_platform_upgrade_svip_content,
                    CloudUnzipHelper.getUnzipSize(AccountUtils.ILevel.SVIP)))

            setUpSVipButton(getString(R.string.business_platform_upgrade_vip))
            guideType =  DEFAULT_UP_SVIP_GUIDE
        }
    }

    private fun initBaseDialog(guide: UnzipVipGuide?) {
        guide?.apply {
            setIcon(this.iconUrl, R.drawable.business_platform_icon_zip)

            setTitle(title)
            setMessage(content)

            setVipButton(vipButtonText, vipPrice, vipCouponPrice)
            setSVipButton(sVipButtonText, sVipPrice, sVipCouponPrice)
            setUpSVipButton(upgradeSVip, sVipPrice, sVipCouponPrice)
            setOpenVipText(openVip)
            if (isUnzipped) {
                setOtherUnzip(otherUnzip)
                if (!BuildConfig.IS_XIAOMI)
                setSinglePrivilegeButton(unzipButtonText, unzipPrice)
            }
        }
    }

    private fun isServerDataUsable(guide: UnzipVipGuide?): Boolean {
        guide?.apply {
            return !content.isNullOrBlank() && !iconUrl.isNullOrBlank() && (!vipButtonText.isNullOrBlank()
                    || !sVipButtonText.isNullOrBlank() || !otherUnzip.isNullOrBlank() || !unzipButtonText.isNullOrBlank()
                    || !openVip.isNullOrBlank() || !upgradeSVip.isNullOrBlank())
        }
        return false
    }

    /**
     * 是否有可点按钮展示
     */
    private fun isButtonShow(): Boolean {
        return single_privilege_button.isVisible() || other_unzip.isVisible()
                || open_vip.isVisible() || vip_button.isVisible() || svip_button.isVisible()
                || up_svip_button.isVisible()
    }

     fun setBusinessVipGuideView(iView: IBusinessVipGuideView) {
        iBusinessVipGuideView = iView
    }

    interface IBusinessVipGuideView {
        fun canCloseGuideView():Boolean

        fun showIncentive()

        fun goSinglePay(payPageUrl: String, payResultPageUrl: String)
    }

    private fun processSpan(str1: String, str2: String): SpannableString {
        val tempStr = "$str1 $str2"
        return SpannableString(tempStr).apply {

            setSpan(StyleSpan(Typeface.NORMAL), str1.length, tempStr.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            setSpan(AbsoluteSizeSpan(resources.getDimension(R.dimen.business_platform_vip_guide_span_text_size).toInt()),
                    str1.length, tempStr.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        }
    }

    private fun statistics(key: String, clickType: String? = null) {
        key.updateCount(guideType ?: "0", singleShowStatus, incentiveShowStatus, clickType ?: "")
    }

    // 设置dialog透明背景，使新弹窗出现后不显露dialog
    fun setDialogVisible(visible: Boolean){
        root.alpha = if (visible) 1F else 0F
    }
}