package com.baidu.netdisk.ui.cloudp2p

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.text.InputFilter
import android.text.InputType
import android.text.TextUtils
import android.view.KeyEvent
import android.view.View
import android.view.inputmethod.EditorInfo
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.NetDiskApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.utils.PersonalConfigKey
import com.baidu.netdisk.cloudp2p.network.api.P2PErrorCode
import com.baidu.netdisk.cloudp2p.service.CloudP2PAddFriendHelper
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.ui.widget.LoadingDialog
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType

/**
 * 设置的问题
 */
@JvmField val VERITY_EXTRA_QUESTION = "verify_extra_question"

/**
 * 验证的类型
 */
@JvmField val VERITY_EXTRA_TYPE = "verify_extra_type"

/**
 * 验证的其他信息
 */
@JvmField val VERITY_EXTRA_INFO = "verify_extra_info"

/**
 * 验证的其他信息-uk
 */
@JvmField val VERITY_EXTRA_INFO_UK = "verify_extra_info_uk"

/**
 * 验证的其他信息-type
 */
@JvmField val VERITY_EXTRA_INFO_TYPE = "verify_extra_info_type"

/**
 * 验证的其他信息-展示名称
 */
@JvmField val VERITY_EXTRA_INFO_DISPLAY_NAME = "verify_extra_info_display_name"

/**
 * 验证的其他信息--备注
 */
@JvmField val VERITY_EXTRA_INFO_REMARK = "verify_extra_info_remark"

/**
 * 验证的其他信息--场景
 */
@JvmField val VERITY_EXTRA_INFO_SCENE = "verify_extra_info_scene"

/**
 * 验证类型--发送验证信息
 */
@JvmField val VERIFY_TYPE_SEND_MESSAGE = 0

/**
 * 验证类型--回答问题
 */
@JvmField val VERIFY_TYPE_QUESTION = 1

/**
 * 发送按钮文字大小
 */
@JvmField val TEXT_SIZE = 13f

/**
 * 编辑框字符上线--验证消息
 */
@JvmField val MAX_LENGTH_FOR_MESSAGE = 40

/**
 * 编辑框字符上线--回答问题
 */
@JvmField val MAX_LENGTH_FOR_ANSWER = 20

/**
 * request code --回答问题
 */
@JvmField val ADD_FRIEND_VERIFY_ANSWER_REQUEST = 101

/**
 * 添加好友验证页
 *
 * @author xujing
 * @since 2019/8/05
 */
class AddFriendVerifyActivity : BaseActivity(), ICommonTitleBarClickListener {

    companion object {

        /**
         * 调起添加好友验证页
         * @param activity 上一次页面
         * @param type 验证的类型
         * @param uk 百度账号uk
         * @param addType 加好友来源类型
         * @param scene 场景
         * @param bundle 其他信息
         */
        fun startAddFollowActivity(activity: Activity?, type: Int, uk: Long, addType:String, scene:String?, bundle: Bundle?) {
            val context: Context = activity?.applicationContext ?: NetDiskApplication.getNetdiskApplication().applicationContext
            val intent = Intent(context, AddFriendVerifyActivity::class.java)
            intent.putExtra(VERITY_EXTRA_TYPE, type)
            intent.putExtra(VERITY_EXTRA_INFO_UK, uk)
            intent.putExtra(VERITY_EXTRA_INFO_TYPE, addType)
            intent.putExtra(VERITY_EXTRA_INFO_SCENE, scene)
            intent.putExtra(VERITY_EXTRA_INFO, bundle)
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            context.startActivity(intent)
        }

        /**
         * 调起添加好友验证页
         * @param activity 上一次页面
         * @param type 验证的类型
         * @param uk 百度账号uk
         * @param addType 加好友来源类型
         * @param scene 场景
         * @param bundle 其他信息
         * @param requestCode 跳转标号
         */
        fun startAddFollowActivityForResult(activity: Activity, type: Int, uk: Long, addType:String,
                                            scene:String?, bundle: Bundle?, requestCode: Int) {
            val intent = Intent(activity.applicationContext, AddFriendVerifyActivity::class.java)
            intent.putExtra(VERITY_EXTRA_TYPE, type)
            intent.putExtra(VERITY_EXTRA_INFO_UK, uk)
            intent.putExtra(VERITY_EXTRA_INFO_TYPE, addType)
            intent.putExtra(VERITY_EXTRA_INFO_SCENE, scene)
            intent.putExtra(VERITY_EXTRA_INFO, bundle)
            activity.startActivityForResult(intent, requestCode)
        }
    }

    /**
     * 验证描述Textview
     */
    private var mDescriptionText: TextView? = null

    /**
     * 问题Textview
     */
    private var mQuestionText: TextView? = null


    /**
     * 编辑框
     */
    private var mEditText: EditText? = null

    /**
     * 当前验证类型
     */
    private var mType: Int? = -1

    /**
     * 设置的问题
     */
    private var mQuestion: String? = null

    /**
     * 设置的问题
     */
    private var mVerifyReceiver: VerifyResultReceiver? = null

    /**
     * loading弹窗
     */
    private var mProgressDialog: Dialog? = null

    override fun getLayoutId(): Int {
        return R.layout.activity_add_friend_verify_layout
    }

    override fun initView() {
        // 标题栏
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setBackLayoutVisible(true)
        mTitleBar.setTopTitleBarClickListener(this)
        mTitleBar.setMiddleTitle(R.string.friend_list_title_btn)
        val rightButton = mTitleBar.rightButtonView as Button
        rightButton.visibility = View.VISIBLE
        rightButton.text = context.resources.getString(R.string.add_friend_verify_send)
        rightButton.setTextColor(context.resources.getColor(R.color.add_friend_verify_send_text))
        rightButton.setTextSize(TEXT_SIZE)
        rightButton.paint.isFakeBoldText = true

        mVerifyReceiver = VerifyResultReceiver(this, Handler())
        mType = intent?.getIntExtra(VERITY_EXTRA_TYPE, -1)

        // 验证消息提示文案
        mDescriptionText = findViewById(R.id.add_friend_verify_description)
        // 问题的文案
        mQuestionText = findViewById(R.id.add_friend_verify_question_text)
        // 都先设为不可见，后面根据显示的类型来分别设置可见
        mDescriptionText?.visibility = View.GONE
        mQuestionText?.visibility = View.GONE
        // 编辑框
        mEditText = findViewById(R.id.add_friend_verify_edit)
        mEditText?.setInputType(InputType.TYPE_TEXT_FLAG_MULTI_LINE);
        mEditText?.setSingleLine(false);
        mEditText?.setHorizontallyScrolling(false);
        mEditText?.setOnEditorActionListener(TextView.OnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEND) { //判断动作标识是否匹配发送
                onRightButtonClicked(null)
            }
            false
        });

        // 根据类型显示对应界面元素
        when (mType) {
            VERIFY_TYPE_SEND_MESSAGE -> showMessageView()
            VERIFY_TYPE_QUESTION -> showQuestionView()
            else -> wrongVerifyTypeValue()
        }
    }

    fun wrongVerifyTypeValue() {
        ToastHelper.showToast(R.string.add_friend_failed)
        finish()
    }

    override fun onRightButtonClicked(view: View?) {
        val edit: EditText? = mEditText
        if (edit == null) {
            return
        }
        if (TextUtils.isEmpty(edit.text)) {
            ToastHelper.showToast(if(mType == VERIFY_TYPE_SEND_MESSAGE) "验证信息不得为空" else "答案不得为空")
            return
        }

        showProgressDialog(R.string.add_friend_sending_verify_message)
        if (mType == VERIFY_TYPE_SEND_MESSAGE) {
            // 发送验证消息的，记录本次填写的验证消息
            PersonalConfig.getInstance().putString(PersonalConfigKey.KEY_ADD_FRIEND_VERIFY_LAST_MESSAGE,
                    edit.text.toString())
            PersonalConfig.getInstance().commit()
        }

        val uk: Long = intent.getLongExtra(VERITY_EXTRA_INFO_UK, 0)
        val sourceType: String = intent.getStringExtra(VERITY_EXTRA_INFO_TYPE)
        val scene: String? = intent.getStringExtra(VERITY_EXTRA_INFO_SCENE)
        val bundle: Bundle? = intent.getBundleExtra(VERITY_EXTRA_INFO)
        val displayName: String? = bundle?.getString(VERITY_EXTRA_INFO_DISPLAY_NAME)
        val remark: String? = bundle?.getString(VERITY_EXTRA_INFO_REMARK)
        var message: String? = null
        var answer: String? = null
        val needInsertIfSuccess: Boolean
        if (mType == VERIFY_TYPE_SEND_MESSAGE) {
            message = edit.text.toString()
            needInsertIfSuccess = false
        } else {
            answer = edit.text.toString()
            needInsertIfSuccess = true

        }

        // 请求验证
        CloudP2PAddFriendHelper.addFollow(this, mVerifyReceiver,
                uk, sourceType, displayName, remark,
                scene, message, answer, needInsertIfSuccess, "", "")
    }

    override fun onBackButtonClicked() {
        this.finish()
    }

    /**
     * 显示发送信息类型的界面元素
     */
    private fun showMessageView() {
        mDescriptionText?.visibility = View.VISIBLE
        mDescriptionText?.setText(R.string.add_friend_verify_desc_message)
        var hintText = PersonalConfig.getInstance().getString(PersonalConfigKey.KEY_ADD_FRIEND_VERIFY_LAST_MESSAGE)
        if (TextUtils.isEmpty(hintText)) {
            hintText = context.resources.getString(R.string.add_friend_verify_edit_text_default)
        }
        val filterArray: Array<InputFilter> = arrayOf(InputFilter.LengthFilter(MAX_LENGTH_FOR_MESSAGE))
        mEditText?.setFilters(filterArray);
        mEditText?.setText(hintText)
        mEditText?.setSelection(0, hintText.length)
    }

    /**
     * 显示回答问题类型的界面元素
     */
    private fun showQuestionView() {
        mDescriptionText?.visibility = View.VISIBLE
        mDescriptionText?.setText(R.string.add_friend_verify_desc_question)
        mQuestionText?.visibility = View.VISIBLE
        mQuestion = (intent?.getBundleExtra(VERITY_EXTRA_INFO))?.getString(VERITY_EXTRA_QUESTION)
        mQuestionText?.setText(context.resources.getString(R.string.add_friend_verify_text_question, mQuestion))
        val filterArray: Array<InputFilter> = arrayOf(InputFilter.LengthFilter(MAX_LENGTH_FOR_ANSWER))
        mEditText?.setFilters(filterArray);
        mEditText?.setText("")
    }

    override fun onDestroy() {
        dismissProgressDialog()
        super.onDestroy()
    }

    class VerifyResultReceiver
    /**
     * 构造方法
     *
     * @param reference  AddFriendVerifyActivity对象
     * @param handler 句柄
     */
    internal constructor(reference: AddFriendVerifyActivity, handler: Handler) : BaseResultReceiver<AddFriendVerifyActivity>(reference, handler, null) {

        override fun onSuccess(reference: AddFriendVerifyActivity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            if (reference == null || reference.isFinishing) {
                return
            }
            reference.dismissProgressDialog()
            ToastHelper.showToast(R.string.add_friend_verify_send_succeed)
            if (reference.mType == VERIFY_TYPE_QUESTION) {
                reference.setResult(Activity.RESULT_OK)
            }
            reference.finish()
        }

        override fun onFailed(reference: AddFriendVerifyActivity, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            reference.dismissProgressDialog()
            if (resultData == null) {
                ToastHelper.showToast(R.string.add_friend_verify_send_failed)
            } else if (reference != null && !reference.isFinishing) {
                val errorNo = resultData.getInt(BaseExtras.ERROR)
                val alartMessage = resultData.getString(BaseExtras.SERVER_ALERT_MESSAGE)
                var toastMessage: String? = null
                if (!CloudP2PServiceHelper.isNetWorkError(resultData)) {
                    when (errorNo) {
                        P2PErrorCode.NICKNAME_MODIFY_FREQUENT ->
                            toastMessage = reference.resources.getString(R.string.add_friend_verify_send_frequent)
                        P2PErrorCode.ERROR_CODE_ADD_FRIEND_SENSITIVE_WORDS ->
                            toastMessage = reference.resources
                                    .getString(R.string.add_friend_verify_send_content_sensitive)
                        P2PErrorCode.ERROR_CODE_ADD_FRIEND_WRONG_ANSWER ->
                            toastMessage = reference.resources
                                    .getString(R.string.add_friend_verify_send_wrong_answer)
                        P2PErrorCode.ERROR_CODE_VERIFICATION ->
                            return super.onFailed(reference, errType, errno, resultData)
                        else -> toastMessage = reference.resources
                                .getString(R.string.add_friend_verify_send_failed)
                    }
                    // 有server下发的提示文案，则优先使用server下发的文案
                    toastMessage = if (!TextUtils.isEmpty(alartMessage)) alartMessage else toastMessage
                    if (!TextUtils.isEmpty(toastMessage)) {
                        ToastHelper.showToast(toastMessage)
                    }
                }
            }
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    /**
     * 显示loading弹窗
     *
     * @param message 提示信息的id
     */
    private fun showProgressDialog(message: Int) {
        mProgressDialog = LoadingDialog.show(this, message)
        mProgressDialog?.setOnKeyListener(DialogInterface.OnKeyListener { dialog, keyCode, event ->
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                dismissProgressDialog()
            }
            false
        })
    }

    /**
     * 取消loading弹窗
     *
     */
    private fun dismissProgressDialog() {
        mProgressDialog?.let {
            if (!it.isShowing() && isFinishing) {
                return
            }
            it.dismiss()
        }
    }

}