package com.baidu.netdisk.ui.cloudp2p

import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import com.baidu.netdisk.R
import com.baidu.netdisk.base.network.NetworkException
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.android.util.TextTools
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.GroupConversationDetailActivity.EXTRA_FROM
import com.baidu.netdisk.ui.cloudp2p.GroupConversationDetailActivity.EXTRA_FROM_GUIDE
import com.baidu.netdisk.ui.cloudp2p.GroupIntroActivity.Companion.IS_GROUPOWNER
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.EXTRA_KEY_GAVATAR_CONTENT
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.EXTRA_KEY_GID_CONTENT
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.EXTRA_KEY_GNAME_CONTENT
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.EXTRA_KEY_GNUMBER_CONTENT
import com.baidu.netdisk.ui.widget.AboveInputDialog
import com.baidu.netdisk.ui.widget.BaseFragment
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import kotlinx.android.synthetic.main.fragment_group_info.*

/**
 * author : linwentao
 * date : On 2019-10-30
 * 群组详情页title信息
 */
class GroupInfoFragment : BaseFragment() {

    companion object {

        /**
         * 初始化fragment
         *
         * @param bundle  群相关属性
         */
        @JvmStatic
        fun newInstance(bundle: Bundle) : GroupInfoFragment {
            val fragment = GroupInfoFragment()
            fragment.arguments = bundle
            return fragment
        }
    }

    /** 群号 **/
    private var groupNumber: Long = 0

    /** 群名称 **/
    private lateinit var groupName: String

    /** 群头像list **/
    private var groupAvatarList: ArrayList<String>? = null

    /**
     * 群主标识
     * 默认为3 普通用户
     */
    private var isGroupOwner: Int? = 3

    /** 群id **/
    private var groupId: Long = 0

    /**
     * 新版紧贴输入法的dialog
     */
    private lateinit var builder: AboveInputDialog

    /**
     * 来源-来自群号引导
     */
    private var from: String? = null

    /**
     * 引导过来确保复制一次
     */
    private var showNum: Int = 0

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_group_info,null, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initParams()
        initView()
    }

    /**
     * 初始化参数
     */
    fun initParams() {
        val bundle = arguments
        groupNumber = bundle?.getLong(EXTRA_KEY_GNUMBER_CONTENT) ?: 0
        groupName = bundle?.getString(EXTRA_KEY_GNAME_CONTENT).toString()
        groupAvatarList = bundle?.getStringArrayList(EXTRA_KEY_GAVATAR_CONTENT)
        isGroupOwner = bundle?.getInt(IS_GROUPOWNER)
        groupId = bundle?.getLong(EXTRA_KEY_GID_CONTENT) ?: 0
        from = bundle?.getString(EXTRA_FROM).toString()
    }

    /**
     * 数据绑定初始化
     */
    fun initView() {
        group_name.text = groupName
        if (groupNumber <= 0) {
            group_number_layout.visibility = View.GONE
        } else {
            group_number_layout.visibility = View.VISIBLE
            group_number.text = getString(R.string.p2p_group_number, groupNumber)
            if (EXTRA_FROM_GUIDE == from && showNum == 0) {
                showNum++
                ToastHelper.showToast(R.string.copy_success)
                TextTools.setClipboard(groupNumber.toString(), activity)
            }
        }
        group_avatar.setIsGroup(true)
        group_avatar.setSmallImageSize(36f)
        if (groupAvatarList?.size == 1) {
            group_avatar.setImageUrls(groupAvatarList)
        } else {
            group_avatar.setImageUrls(groupAvatarList?.take(4) as java.util.ArrayList<String>)
        }

        if (isGroupOwner()) {
            edit_group_name.visibility = View.VISIBLE
        } else {
            edit_group_name.visibility = View.GONE
        }

        edit_group_name.setOnClickListener {
            changeGroupName()
        }

        group_number_layout.setOnClickListener {
            ToastHelper.showToast(R.string.copy_success)
            TextTools.setClipboard(groupNumber.toString(), activity)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.GROUP_DETAIL_COPY_NUMBER_CLICK)
        }
    }

    /**
     * 是否是群主
     * return true为群主
     */
    private fun isGroupOwner() : Boolean {
        return isGroupOwner == CloudP2PContract.GroupsPeople.ROLE_CREATOR
    }

    /**
     * 更改群组名称
     */
    private fun changeGroupName() {
        builder = AboveInputDialog(context, activity)
        builder.setDialogTitle(R.string.group_conversation_setting_name)
        builder.setRightBtnText(R.string.ok)
        builder.setEditContent(groupName)
        builder.setEditMaxLength(10)
        builder.setRightBtnOnClickListener {
            val name = builder.editText.text.toString().trim { it <= ' ' }
            hideSoftKeyboard(builder.editText, context!!)
            if (NetworkException(context).checkNetworkException()!!) {// 网络好时
                builder.switch2LoadingMode()
                // 网络请求
                CloudP2PServiceHelper.setGroupInfo(context,
                        ModifyNameDialogResultReceiver(this, Handler()), groupId,
                        name, "", "", -1, null, -1)
            }
        }
        builder.show()
    }

    /**
     * 隐藏软键盘
     */
    private fun hideSoftKeyboard(editText: EditText, context: Context) {
        val imm = context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(editText.windowToken, 0)
    }

    /**
     * 修改群名得receiver
     */
    private class ModifyNameDialogResultReceiver internal constructor(reference: GroupInfoFragment, handler: Handler) :
            BaseResultReceiver<GroupInfoFragment>(reference, handler, null) {

        override fun onSuccess(reference: GroupInfoFragment, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            reference.group_name.text = reference.builder.editText.text.toString()
            reference.builder.dismiss()
            CloudP2PServiceHelper.listGroupMsg(reference.context, null, reference.groupId)
        }

        override fun onFailed(reference: GroupInfoFragment, errType: ErrorType,
                              errno: Int, resultData: Bundle): Boolean {
            reference.builder.switch2NormalMode()
            return if (errType == ErrorType.NETWORK_ERROR) {
                !super.onFailed(reference, errType, errno, resultData)
            } else {
                super.onFailed(reference, errType, errno, resultData)
            }
        }
    }
}