package com.baidu.netdisk.ui.cloudp2p

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.text.*
import android.view.KeyEvent
import android.view.MotionEvent
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.Button
import android.widget.EditText
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract.GroupsPeople.ROLE_CREATOR
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.util.EmojiUtil
import com.baidu.netdisk.ui.widget.LoadingDialog
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.netdisk.themeskin.loader.SkinManager
import kotlinx.android.synthetic.main.activity_group_intro_layout.*
import kotlinx.android.synthetic.main.activity_personal_signature_layout.signature_number

/**
 * 个性签名设置页
 *
 * @author linwentao
 * @since 2019/8/30
 */
class GroupIntroActivity : BaseActivity(), ICommonTitleBarClickListener {

    companion object {

        /**
         * 群简介内容
         */
        private const val INTRO_TEXT = "intro_text"

        /**
         * 是否是群主
         */
        const val IS_GROUPOWNER = "is_groupowner"

        /**
         * 群id
         */
        private const val GROUP_ID_EXTRA = "group_id_extra"

        /**
         * 调起群简介页面
         *
         * @param activity 上一次页面
         * @param intro 签名文案
         */
        @JvmStatic
        fun startGroupIntroActivity(activity: Activity, groupId: Long, intro: String, isGroupOwner: Int) {
            val intent = Intent(activity.applicationContext, GroupIntroActivity::class.java)
            intent.putExtra(GROUP_ID_EXTRA, groupId)
            intent.putExtra(INTRO_TEXT, intro)
            intent.putExtra(IS_GROUPOWNER, isGroupOwner)
            activity.startActivity(intent)
        }
    }

    /**
     * 群id
     */
    private var groupId: Long = 0

    /**
     * 个性签名文案
     */
    private var mSignatureInfo: String? = null

    /**
     * 群主标识
     * 默认为3 普通用户
     */
    private var isGroupOwner: Int? = 3

    /**
     * 编辑框
     */
    private lateinit var mEditText: EditText

    private var mSignatureNumber: TextView? = null

    /**
     * 设置个性签名的receiver
     */
    private var mSetGroupInfoResultReceiver: SetGroupInfoResultReceiver? = null

    /**
     * loading弹窗
     */
    private var mProgressDialog: Dialog? = null

    override fun getLayoutId(): Int {
        return R.layout.activity_group_intro_layout
    }

    override fun initView() {
        // 标题栏
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setBackLayoutVisible(true)
        mTitleBar.setTopTitleBarClickListener(this)
        mTitleBar.setMiddleTitle(R.string.group_conversation_setting_intro)
        val rightButton = mTitleBar.rightButtonView as Button
        rightButton.text = context.resources.getString(R.string.ok)
        rightButton.setTextColor(context.resources.getColor(R.color.add_friend_verify_send_text))
        rightButton.textSize = TEXT_SIZE
        rightButton.paint.isFakeBoldText = true

        mSetGroupInfoResultReceiver = SetGroupInfoResultReceiver(this, Handler())
        mSignatureNumber = findViewById(R.id.signature_number)

        // 编辑框
        mEditText = findViewById(R.id.personal_signature_edit)

        groupId = intent.getLongExtra(GROUP_ID_EXTRA, 0)
        mSignatureInfo = intent?.getStringExtra(INTRO_TEXT)

        if (!TextUtils.isEmpty(mSignatureInfo)) {
            mEditText.setText(mSignatureInfo)
        }

        isGroupOwner = intent?.getIntExtra(IS_GROUPOWNER, 3)
        if (isGroupOwner == ROLE_CREATOR) {
            rightButton.visibility = View.VISIBLE
            val imm = context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
            imm.toggleSoftInput(InputMethodManager.SHOW_FORCED, 0)
            mEditText.setSelection(mEditText.text.length)
            mEditText.setHorizontallyScrolling(false)
        } else {
            rightButton.visibility = View.GONE
            signature_number.visibility = View.GONE
            signature_remind.visibility = View.VISIBLE
            mEditText.isEnabled = false
        }

        mEditText.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEND) { //判断动作标识是否匹配发送
                onRightButtonClicked(null)
            }
            false
        }

        val length: Int = mEditText.text?.length ?: 0
        mSignatureNumber?.text = "$length/50"
        mEditText.addTextChangedListener(object : TextWatcher {

            // 改变时
            override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {

            }

            // 改变前
            override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {

            }

            // 改变中
            override fun afterTextChanged(s: Editable) {
                val textSum = s.toString().length
                when {
                    textSum <= 50 -> {
                        mSignatureNumber?.text = "$textSum/50"
                        mEditText.setTextColor(SkinManager.getInstance().getColor(R.color.bg_dn_search_page_text_color))
                    }
                }
            }
        })

    }

    override fun onRightButtonClicked(view: View?) {
        val intro = mEditText.text.toString()
        if (!EmojiUtil.noContainsEmoji(intro)) {
            // 请求/user/editinfo
            showProgressDialog(R.string.saving)
            CloudP2PServiceHelper.setGroupInfo(context, mSetGroupInfoResultReceiver, groupId, null,
                    "", intro, -1, "gdesc", -1)
        } else {
            ToastHelper.showToast("保存失败，暂不支持插入表情符号")
        }
    }

    override fun onBackButtonClicked() {
        hideKeyBoard()
        this.finish()
    }

    override fun dispatchTouchEvent(event: MotionEvent): Boolean {
        // 触摸外部隐藏输入法
        if (isOutOfBounds(mEditText, event)) {
            hideKeyBoard()
        }
        return super.dispatchTouchEvent(event)
    }

    /**
     * 点击位置是否在对话框外部区域
     *
     * @param view 判断的view
     * @param event
     * @return true：里面    false: 外面
     */
    private fun isOutOfBounds(view: View?, event: MotionEvent): Boolean {
        if (view == null) {
            return true
        }
        val x = event.x.toInt()
        val y = event.y.toInt()
        val location = IntArray(2)
        view.getLocationOnScreen(location)
        val left = location[0]
        val top = location[1]
        val right = left + view.measuredWidth
        val bottom = top + view.measuredHeight
        return y < top || y > bottom || x < left || x > right
    }

    /**
     * 显示键键盘
     */
    fun showKeyBoard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.showSoftInput(mEditText, InputMethodManager.SHOW_IMPLICIT)

    }

    /**
     * 隐藏键盘
     */
    private fun hideKeyBoard() {
        val imm = getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(mEditText.windowToken, 0)
    }

    override fun onDestroy() {
        dismissProgressDialog()
        super.onDestroy()
    }

    class SetGroupInfoResultReceiver
    /**
     * 构造方法
     *
     * @param reference  PersonalSignatureActivity
     * @param handler 句柄
     */
    internal constructor(reference: GroupIntroActivity, handler: Handler) : BaseResultReceiver<GroupIntroActivity>(reference, handler, null) {

        override fun onSuccess(reference: GroupIntroActivity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            reference.dismissProgressDialog()
            ToastHelper.showToast(R.string.videoplayer_save_success)
            reference.finish()
        }

        override fun onFailed(reference: GroupIntroActivity, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            reference.dismissProgressDialog()
            if (resultData.getBoolean(BaseExtras.ERROR_NETWORK)) {
                ToastHelper.showToast(R.string.personalpage_network_inavailable)
            } else {
                ToastHelper.showToast(R.string.save_file_fail)
            }
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    /**
     * 显示loading弹窗
     *
     * @param message 提示信息的id
     */
    private fun showProgressDialog(message: Int) {
        mProgressDialog = LoadingDialog.show(this, message)
        mProgressDialog?.setOnKeyListener(DialogInterface.OnKeyListener { _, keyCode, _ ->
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                dismissProgressDialog()
            }
            false
        })
    }

    /**
     * 取消loading弹窗
     *
     */
    private fun dismissProgressDialog() {
        mProgressDialog?.let {
            if (!it.isShowing && isFinishing) {
                return
            }
            it.dismiss()
        }
    }

    override fun isActivityDark(): Boolean {
        return true
    }
}
