package com.baidu.netdisk.ui.cloudp2p

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.ResultReceiver
import android.view.View
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.network.NetworkException
import com.baidu.netdisk.cloudp2p.network.model.GroupInfoBean
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.EXTRA_KEY_GID_CONTENT
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.EXTRA_KEY_INVITE_STATUS
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import kotlinx.android.synthetic.main.activity_group_manager_layout.*

/**
 * Created by linwentao on 2019-11-1.
 *
 * Describe: 群管理页面
 */
class GroupManagerActivity : BaseActivity(), ICommonTitleBarClickListener {

    companion object {

        /** 标识是否允许被搜索 **/
        private const val EXTRA_KEY_SEARCH_STATUS = "extra_key_search_status"

        /**
         * 启动Activity
         *
         * @param context
         * @param groupId 群ID
         * @param inviteStatus 允许群成员邀请
         * @param searchStatus 是否允许搜索
         */
        @JvmStatic
        fun startGroupManagerActivity(context: Context, groupId: Long, inviteStatus: Int, searchStatus: Int) {
            val intent = Intent(context, GroupManagerActivity::class.java)
            intent.putExtra(EXTRA_KEY_GID_CONTENT, groupId)
            intent.putExtra(EXTRA_KEY_INVITE_STATUS, inviteStatus)
            intent.putExtra(EXTRA_KEY_SEARCH_STATUS, searchStatus)
            context.startActivity(intent)
        }
    }

    /** 群id **/
    private var gId: Long = 0

    /** 初始的群成员邀请设置 */
    private var initInviteStatus: Int = GroupInfoBean.INVITE_STATUS_ALLOW

    /** 初始的群成员邀请设置 */
    private var inviteStatus: Int = GroupInfoBean.INVITE_STATUS_ALLOW

    /** 初始的群搜索设置 */
    private var initSearchStatus: Int = GroupInfoBean.ALLOW_SEARCH

    /**
     * 是否允许被搜索
     * 默认为1 允许搜索
     */
    private var searchStatus: Int = GroupInfoBean.ALLOW_SEARCH



    override fun onCreate(savedInstanceState: Bundle?) {
        gId = intent.getLongExtra(EXTRA_KEY_GID_CONTENT, 0)
        initInviteStatus = intent.getIntExtra(EXTRA_KEY_INVITE_STATUS,
                GroupInfoBean.INVITE_STATUS_ALLOW)
        initSearchStatus = intent.getIntExtra(EXTRA_KEY_SEARCH_STATUS,
                GroupInfoBean.ALLOW_SEARCH)
        super.onCreate(savedInstanceState)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_group_manager_layout
    }

    override fun initView() {
        if (mTitleBar == null) {
            mTitleBar = BaseTitleBar(this)
        }
        mTitleBar.setMiddleTitle(R.string.group_manager)
        mTitleBar.setRightLayoutVisible(false)
        mTitleBar.setTopTitleBarClickListener(this)

        allow_group_member_invite.setCheckButtonSwitch()
        if (initInviteStatus == GroupInfoBean.INVITE_STATUS_FORBID) {
            allow_group_member_invite.isChecked = true
            allow_group_member_invite.showInfoText(R.string.group_member_invite_content, 0)
        } else {
            allow_group_member_invite.isChecked = false
        }

        allow_group_member_invite.setOnCheckBoxClickListener {
            if (NetworkException(context).checkNetworkException()) {
                allow_group_member_invite.switchCheckboxLoadingMode()
                inviteStatus = if (allow_group_member_invite.isChecked) {
                    GroupInfoBean.INVITE_STATUS_FORBID
                } else {
                    GroupInfoBean.INVITE_STATUS_ALLOW
                }
                CloudP2PServiceHelper.setGroupInfo(context, getInviteResultReceiver(), gId, null,
                        null, null, inviteStatus, null, -1)
            } else {
                refreshGroupMemberInvite(initInviteStatus)
            }
        }

        search_qroup_number_permission.setCheckButtonSwitch()
        search_qroup_number_permission.isChecked = initSearchStatus == GroupInfoBean.ALLOW_SEARCH

        search_qroup_number_permission.setOnCheckBoxClickListener {
            if (NetworkException(context).checkNetworkException()) {
                search_qroup_number_permission.switchCheckboxLoadingMode()
                searchStatus = if (search_qroup_number_permission.isChecked) {
                    GroupInfoBean.ALLOW_SEARCH
                } else {
                    GroupInfoBean.FORBID_SEARCH
                }
                CloudP2PServiceHelper.setGroupInfo(context, getSearchPermissionResultReceiver(), gId, null,
                        null, null, -1, null, searchStatus)
            } else {
                refreshGroupSearch(initSearchStatus)
            }
        }
    }

    /**
     * 刷新群成员邀请UI
     */
    private fun refreshGroupMemberInvite(status: Int) {
        allow_group_member_invite.visibility = View.VISIBLE
        allow_group_member_invite.switchCheckboxNormalMode()
        if (status == GroupInfoBean.INVITE_STATUS_FORBID) {
            allow_group_member_invite.isChecked = true
            allow_group_member_invite.showInfoText(R.string.group_member_invite_content, 0)
        } else {
            allow_group_member_invite.isChecked = false
            allow_group_member_invite.hideInfoText()
        }
    }

    /**
     * 刷新搜索设置UI
     */
    private fun refreshGroupSearch(status: Int) {
        search_qroup_number_permission.switchCheckboxNormalMode()
        search_qroup_number_permission.isChecked = status == GroupInfoBean.ALLOW_SEARCH
    }

    /**
     * 允许群成员邀请设置
     */
    private var inviteResultReceiver: InviteResultReceiver? = null

    /**
     * 获取群成员邀请回调Receiver
     */
    private fun getInviteResultReceiver(): ResultReceiver {
        if (inviteResultReceiver == null) {
            inviteResultReceiver = InviteResultReceiver(this, Handler())
        }
        return inviteResultReceiver as InviteResultReceiver
    }

    /**
     * 允许群成员邀请设置回调
     */
    private class InviteResultReceiver internal constructor(reference: GroupManagerActivity, handler: Handler) :
            BaseResultReceiver<GroupManagerActivity>(reference, handler, null) {

        override fun onSuccess(reference: GroupManagerActivity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            reference.refreshGroupMemberInvite(reference.inviteStatus)
        }

        override fun onFailed(reference: GroupManagerActivity, errType: ErrorType,
                              errno: Int, resultData: Bundle): Boolean {
            reference.refreshGroupMemberInvite(reference.initInviteStatus)
            return if (errType == ErrorType.NETWORK_ERROR) {
                !super.onFailed(reference, errType, errno, resultData)
            } else {
                super.onFailed(reference, errType, errno, resultData)
            }
        }
    }

    /**
     * 通过搜索群号设置
     */
    private var searchPermissionResultReceiver: SearchPermissionResultReceiver? = null

    /**
     * 通过搜索群号设置回调Receiver
     */
    private fun getSearchPermissionResultReceiver(): ResultReceiver {
        if (searchPermissionResultReceiver == null) {
            searchPermissionResultReceiver = SearchPermissionResultReceiver(this, Handler())
        }
        return searchPermissionResultReceiver as SearchPermissionResultReceiver
    }

    /**
     * 通过搜索群号设置回调
     */
    private class SearchPermissionResultReceiver internal constructor(reference: GroupManagerActivity, handler: Handler) :
            BaseResultReceiver<GroupManagerActivity>(reference, handler, null) {

        override fun onSuccess(reference: GroupManagerActivity, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            ToastHelper.showToast(R.string.deal_succeed)
            reference.refreshGroupSearch(reference.searchStatus)
        }

        override fun onFailed(reference: GroupManagerActivity, errType: ErrorType,
                              errno: Int, resultData: Bundle): Boolean {
            ToastHelper.showToast(R.string.deal_failed)
            reference.refreshGroupSearch(reference.initSearchStatus)
            return if (errType == ErrorType.NETWORK_ERROR) {
                !super.onFailed(reference, errType, errno, resultData)
            } else {
                super.onFailed(reference, errType, errno, resultData)
            }
        }
    }

    override fun onRightButtonClicked(view: View?) {
    }

    override fun onBackButtonClicked() {
        finish()
    }


    override fun isActivityDark(): Boolean {
        return true
    }

}
