package com.baidu.netdisk.ui.cloudp2p

import android.content.Context
import android.content.Intent
import android.graphics.Bitmap
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.text.TextUtils
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.GridView
import android.widget.LinearLayout
import com.baidu.netdisk.BuildConfig
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.cloudp2p.network.model.GroupInfoBean
import com.baidu.netdisk.cloudp2p.network.model.MyQrcodeResponse
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.android.util.image.ImageUtil
import com.baidu.netdisk.permission.IPermission
import com.baidu.netdisk.sns.utils.SafeHandler
import com.baidu.netdisk.ui.TypeItem
import com.baidu.netdisk.ui.cloudp2p.presenter.GroupQrcodePresenter
import com.baidu.netdisk.ui.permission.PermissionContentHelper
import com.baidu.netdisk.ui.permission.presenter.PermissionPresenter
import com.baidu.netdisk.ui.share.ShareOption
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import io.reactivex.annotations.NonNull
import io.reactivex.annotations.Nullable
import kotlinx.android.synthetic.main.activity_group_qrcode.*

/**
 * Created by linwentao on 2019-09-10.
 *
 * Describe: 群二维码页面
 */
@Suppress("DEPRECATION")
class GroupQrcodeActivity : QrcodeShareBaseActivity(), SafeHandler.IHandlerHost{

    /** handler声明，用户保存成功切换到主线程 **/
    private var mSafeHandler: SafeHandler = SafeHandler(this)

    /** 群二维码图片 **/
    lateinit var groupQrcodeImage : Bitmap

    /** 初始化GroupQrcodePresenter **/
    private lateinit var mGroupQrcodePresenter: GroupQrcodePresenter

    companion object {

        /** 标识保存 **/
        private const val ACTION_SAVE = "action_save"

        /** 标识分享 **/
        private const val ACTION_SHARE = "action_share"

        /** 生成二维码成功的消息id **/
        const val MESSAGE_GENARATE_SUCCESS = 1

        /** 生成二维码失败的消息id **/
        const val MESSAGE_GENARATE_FAILED = 2

        /** 标识群id **/
        const val EXTRA_KEY_GID_CONTENT = "extra_key_gid_content"

        /** 标识群号 **/
        private const val EXTRA_KEY_GNUM_CONTENT = "extra_key_gnum_content"

        /** 标识群头像 **/
        const val EXTRA_KEY_GAVATAR_CONTENT = "extra_key_gavatar_content"

        /** 标识群名称 **/
        const val EXTRA_KEY_GNAME_CONTENT = "extra_key_gname_content"

        /** 标识群号 **/
        const val EXTRA_KEY_GNUMBER_CONTENT = "extra_key_gnumber_content"

        /** 标识群简介 **/
        private const val EXTRA_KEY_GINTRO_CONTENT = "extra_key_gintro_content"

        /** 标识群成员数量 **/
        private const val EXTRA_KEY_GPEOPLENUM_CONTENT = "extra_key_gpeoplenum_content"

        /** 标识群成员数量限制 **/
        private const val EXTRA_KEY_GSUM_CONTENT = "extra_key_gsum_content"

        /** 群成员邀请设置状态 */
        const val EXTRA_KEY_INVITE_STATUS = "extra_key_invite_status"

        /**
         * 启动Activity
         *
         * @param context
         * @param groupId 群Id
         * @param groupName 群名称
         * @param peopleNum 群人数
         * @param sum 群人数总数
         * @param inviteStatus 群成员邀请设置
         * @param groupAvatarList 群头像urlList
         */
        @JvmStatic
        fun startGroupQrcodeActivity(
                context: Context, groupId: Long, groupNumber: Long, groupName: String?, groupIntro: String?,
                peopleNum: Int, sum: Int, inviteStatus: Int, groupAvatarList: ArrayList<String>) {
            val intent = Intent(context, GroupQrcodeActivity::class.java)
            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
            intent.putExtra(EXTRA_KEY_GID_CONTENT, groupId)
            intent.putExtra(EXTRA_KEY_GNUM_CONTENT, groupNumber)
            intent.putExtra(EXTRA_KEY_GAVATAR_CONTENT, groupAvatarList)
            intent.putExtra(EXTRA_KEY_GNAME_CONTENT, groupName ?: "")
            intent.putExtra(EXTRA_KEY_GINTRO_CONTENT, groupIntro ?: "")
            intent.putExtra(EXTRA_KEY_GPEOPLENUM_CONTENT, peopleNum)
            intent.putExtra(EXTRA_KEY_GSUM_CONTENT, sum)
            intent.putExtra(EXTRA_KEY_INVITE_STATUS, inviteStatus)
            context.startActivity(intent)
        }
    }

    /** 群id **/
    private var mGroupId: Long? = 0

    /** 群号 **/
    private var groupNumber: Long = 0

    /** 群头像list **/
    private var mGroupAvatarList: ArrayList<String>? = null

    /** 群名称 **/
    private lateinit var mGroupName: String

    /** 群简介 **/
    private lateinit var mGroupIntro: String

    /** 群成员数量 **/
    private var mGroupPeopleNum: Int = 0

    /** 群成员数量限制 **/
    private var mGroupSum: Int = 0

    /** 群成员邀请设置 */
    private var inviteStatus: Int = GroupInfoBean.INVITE_STATUS_ALLOW

    /** 操作类型：保存、分享 **/
    private lateinit var mType: String

    /** 分享position **/
    private var mPosition: Int = -1

    override fun onCreate(savedInstanceState: Bundle?) {
        mGroupId = intent.getLongExtra(EXTRA_KEY_GID_CONTENT, -1)
        groupNumber = intent.getLongExtra(EXTRA_KEY_GNUM_CONTENT, -1)
        mGroupAvatarList = intent.getStringArrayListExtra(EXTRA_KEY_GAVATAR_CONTENT)
        mGroupName = intent.getStringExtra(EXTRA_KEY_GNAME_CONTENT)
        mGroupIntro = intent.getStringExtra(EXTRA_KEY_GINTRO_CONTENT)
        mGroupPeopleNum = intent.getIntExtra(EXTRA_KEY_GPEOPLENUM_CONTENT, 1)
        mGroupSum = intent.getIntExtra(EXTRA_KEY_GSUM_CONTENT, 50)
        inviteStatus = intent.getIntExtra(EXTRA_KEY_INVITE_STATUS,
                GroupInfoBean.INVITE_STATUS_ALLOW)

        CloudP2PServiceHelper.getGroupQrcode(context.applicationContext,
                GroupQrcodeResultReceiver(this, Handler()), mGroupId ?: -1)
        super.onCreate(savedInstanceState)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_group_qrcode
    }

    override fun initView() {
        super.initView()
        mTitleBar.setMiddleTitle(R.string.follow_list_tab_group_qrcode_title)
        mTitleBar.setBackgroundColor(R.color.my_qr_code_background)
        group_name.text = mGroupName
        group_num.text = resources.getString(R.string.p2p_group_qrcode_num, mGroupPeopleNum, mGroupSum)
        group_number.text = resources.getString(R.string.p2p_group_number, groupNumber)
        group_avatar.setIsGroup(true)
        group_avatar.setSmallImageSize(25f)

        if(!TextUtils.isEmpty(mGroupIntro)) {
            group_introduction.visibility = View.VISIBLE
            group_introduction.text = mGroupIntro
        }

        if (mGroupAvatarList?.size!! <= 1) {
            group_avatar.setImageUrls(mGroupAvatarList)
        } else {
            group_avatar.setImageUrls(mGroupAvatarList?.take(4) as java.util.ArrayList<String>)
        }

        mGroupQrcodePresenter = GroupQrcodePresenter(this)

        save_layout.setOnTouchListener { v, event -> onTouchAlbum(v, event) }
        save_layout.setOnClickListener { saveGroupQrcodeToLocal(ACTION_SAVE) }
        save_layout.isEnabled = false

        save_text.setOnTouchListener { v, event -> onTouchAlbum(v, event) }
        save_text.setOnClickListener { saveGroupQrcodeToLocal(ACTION_SAVE) }
        save_text.isEnabled = false

        save_icon.setOnTouchListener { v, event -> onTouchAlbum(v, event) }
        save_icon.setOnClickListener { saveGroupQrcodeToLocal(ACTION_SAVE) }
        save_icon.isEnabled = false

        updateQrcodeStatus(false)
    }

    override fun initShare() {
        mShareGridView.layoutParams = LinearLayout.LayoutParams(resources.displayMetrics.widthPixels,
                ViewGroup.LayoutParams.WRAP_CONTENT)
        mShareGridView.stretchMode = GridView.NO_STRETCH
        mShareGridView.columnWidth = resources.displayMetrics.widthPixels / mShareGridItems.size
    }

    /**
     * 保存我的二维码到本地点击处理
     * 点击分享，先保存到本地
     */
    private fun saveGroupQrcodeToLocal(type: String) {
        if (PermissionPresenter(this)
                        .requestPermissions(IPermission.STORAGE_GROUP_PERMISSION,
                                PermissionContentHelper.STORGE_PERMISSION)) {
            return
        }
        mType = type
        mGroupQrcodePresenter.showLoading()
        // 生成分享渠道的二维码
        mGroupQrcodePresenter.generateShareGroupQrcode(ImageUtil.getBitmapFromView(view_info), mSafeHandler)
    }

    /**
     * 保存到相册后更新UI
     *
     * @param msg Message 保存完通知的消息
     */
    override fun handleMessage(msg: Message?) {
        when (msg!!.what) {
            MESSAGE_GENARATE_SUCCESS -> {
                val isSuccess = mGroupQrcodePresenter.saveGroupQrcodeToLocal()
                if (isSuccess) {
                    mGroupQrcodePresenter.dismisssLoading()
                    if (mType == ACTION_SAVE) {
                        ToastHelper.showToast(R.string.save_to_album_success)
                    } else {
                        // 处理分享逻辑
                        mGroupQrcodePresenter.handleShare(mPosition)
                    }
                }
            }
            MESSAGE_GENARATE_FAILED -> {
                mGroupQrcodePresenter.dismisssLoading()
                ToastHelper.showToast(R.string.get_group_qrcode_error)
            }
        }
    }

    override fun initShareList() {
        /**
         * 微信
         */
        val wexin = TypeItem(getString(R.string.share_weixin),
                R.drawable.ic_share_weixin_selector, ShareOption.SHARE_TO_WEIXIN)

        /**
         * qq好友
         */
        val qq = TypeItem(getString(R.string.share_qq),
                R.drawable.ic_share_qq_selector, ShareOption.SHARE_QQ)

        /**
         * 新浪
         */
        val sina = TypeItem(getString(R.string.share_sina),
                R.drawable.share_icon_sina_pressed, ShareOption.SHARE_SINA)

        /**
         * 微信朋友圈
         */
        val wexinQuan = TypeItem(getString(R.string.share_weixin_quan),
                R.drawable.ic_share_quan_selector, ShareOption.SHARE_TO_WEIXIN_QUAN)

        /**
         * qq空间
         */
        val qqZone = TypeItem(getString(R.string.share_qq_zone),
                R.drawable.ic_share_zone_selector, ShareOption.SHARE_QQ_ZONE)

        if (!BuildConfig.IS_XIAOMI)
            mShareGridItems.add(wexin)

        mShareGridItems.add(qq)
        mShareGridItems.add(sina)
        if (!BuildConfig.IS_XIAOMI)
            mShareGridItems.add(wexinQuan)

        mShareGridItems.add(qqZone)
    }

    /**
     * 保存群二维码到本地按压态
     */
    private fun onTouchAlbum(v: View, event: MotionEvent): Boolean {
        when (event.action) {
            MotionEvent.ACTION_DOWN -> {
                if (save_icon != null) {
                    save_icon.isPressed = true
                    save_icon.alpha = 0.4f
                }

                if (save_text != null) {
                    save_text.isPressed = true
                    save_text.alpha = 0.4f
                }
            }
            MotionEvent.ACTION_UP -> {
                if (save_icon != null) {
                    save_icon.isPressed = false
                    save_icon.alpha = 1f
                }

                if (save_text != null) {
                    save_text.isPressed = false
                    save_text.alpha = 1f
                }
                v.performClick()
            }
            else -> {
            }
        }
        return true
    }

    /**
     * 更新二维码状态
     *
     * @param loadSuccess 二维获取成功
     */
    private fun updateQrcodeStatus(loadSuccess: Boolean) {
        val forbidInvite = inviteStatus == GroupInfoBean.INVITE_STATUS_FORBID
        val isFull = mGroupPeopleNum == mGroupSum

        share_scrollview.visibility = if (loadSuccess) View.VISIBLE else View.GONE
        val saveEnable = loadSuccess && !forbidInvite && !isFull
        save_layout.isEnabled = saveEnable
        save_text.isEnabled = saveEnable
        save_icon.isEnabled = saveEnable
        if (saveEnable) {
            save_text.setTextColor(resources.getColor(R.color.black))
            save_icon.setImageDrawable(resources.getDrawable(R.drawable.icon_save_to_album))
        } else {
            save_text.setTextColor(resources.getColor(R.color.gray))
            save_icon.setImageDrawable(resources.getDrawable(R.drawable.icon_save_to_album_disable))
        }

        when {
            forbidInvite -> {
                group_full_view.setText(R.string.group_qrcode_invite_forbid)
                group_full_view.visibility = View.VISIBLE
                share_scrollview.visibility = View.GONE
            }
            isFull -> {
                group_full_view.setText(R.string.group_qrcode_people_full_text)
                group_full_view.visibility = View.VISIBLE
                share_scrollview.visibility = View.GONE
            }
            else -> {
                group_full_view.visibility = View.GONE
            }
        }
    }

    /**
     * 分享点击
     */
    override fun onItemClick(parent: AdapterView<*>?, view: View?, position: Int, id: Long) {
        // 点击分享，先走保存到本地逻辑,然后handle处理分享逻辑
        mPosition = position
        saveGroupQrcodeToLocal(ACTION_SHARE)
    }

    override fun onRightButtonClicked(view: View?) {
    }

    override fun onBackButtonClicked() {
        finish()
    }

    /**
     * 获取群二维码的ResultReceiver
     */
    class GroupQrcodeResultReceiver internal constructor(refrence: GroupQrcodeActivity, handler: Handler)
        : BaseResultReceiver<GroupQrcodeActivity>(refrence, handler, null) {

        override fun onSuccess(@NonNull refrence: GroupQrcodeActivity, @Nullable resultData: Bundle?) {
            super.onSuccess(refrence, resultData)
            val groupQrcodeResponse: MyQrcodeResponse = resultData?.getParcelable(BaseExtras.RESULT)!!
            if (groupQrcodeResponse.myQrcode != null) {
                refrence.groupQrcodeImage = groupQrcodeResponse.myQrcode
                refrence.group_qrcode.visibility = View.VISIBLE
                refrence.group_qrcode.setImageBitmap(refrence.groupQrcodeImage)
                refrence.updateQrcodeStatus(true)
            }
        }

        override fun onFailed(@NonNull refrence: GroupQrcodeActivity, @NonNull errType: ErrorType,
                              errno: Int, @NonNull resultData: Bundle): Boolean {
            if (resultData.getBoolean(BaseExtras.ERROR_NETWORK)) {
                ToastHelper.showToast(R.string.network_error)
            } else {
                ToastHelper.showToast(R.string.get_group_qrcode_error)
            }
            refrence.updateQrcodeStatus(false)
            return super.onFailed(refrence, errType, errno, resultData)
        }
    }

    override fun isActivityDark(): Boolean {
        return true
    }
}
