package com.baidu.netdisk.ui.cloudp2p

import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.PersistableBundle
import android.view.View
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.NetDiskApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.cloudp2p.provider.CloudP2PProviderHelper
import com.baidu.netdisk.cloudp2p.service.CloudP2PService
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.android.util.network.ConnectivityState
import com.baidu.netdisk.ui.cloudp2p.ConversationActivity.EXTRA_TITLE
import com.baidu.netdisk.ui.cloudp2p.ConversationActivity.EXTRA_TO_AVATAR_URL
import com.baidu.netdisk.ui.widget.CircleImageView
import com.baidu.netdisk.ui.widget.SettingsItemView
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.imageloader.AvatarLoaderHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView

/**
 * 官方账号个人资料详情页
 */
class OfficialConversationDetailActivity: BaseActivity(), INotifyView, ICommonTitleBarClickListener {

    companion object {

        @JvmStatic
        fun startActivity(context: Context, uk: Long, title: String, avatar: String) {
            var intent:Intent = Intent(context, OfficialConversationDetailActivity::class.java)
            intent.putExtra(CloudP2PService.EXTRA_UK, uk)
            intent.putExtra(EXTRA_TITLE, title)
            intent.putExtra(EXTRA_TO_AVATAR_URL, avatar)
            context.startActivity(intent)
        }
    }

    // 消息免打扰
    private var mNotify: SettingsItemView? = null
    //账号uk
    private var mUk: Long = 0
    //账号名称
    private var mOfficalName: String? = null
    //账号头像地址
    private var mAvatartUrl: String? = null
    //更新免打扰结果的uri
    private var mDetailUri: Uri? = null;

    override fun onCreate(savedInstanceState: Bundle?, persistentState: PersistableBundle?) {
        super.onCreate(savedInstanceState, persistentState)
    }

    override fun isActivityDark(): Boolean {
        return true
    }

    override fun getLayoutId(): Int {
        return R.layout.official_detail_layout
    }

    override fun initParams() {
        super.initParams()
        mUk = intent.getLongExtra(CloudP2PService.EXTRA_UK, 0)
        mDetailUri = CloudP2PContract.PeopleMessages.buildMessagesDetailUri(mUk, AccountUtils.getInstance().getBduss())

        mOfficalName = intent.getStringExtra(EXTRA_TITLE)
        mAvatartUrl = intent.getStringExtra(EXTRA_TO_AVATAR_URL)
    }

    override fun initView() {
        if (mTitleBar == null) {
            mTitleBar = BaseTitleBar(this@OfficialConversationDetailActivity)
            mTitleBar.setTopTitleBarClickListener(this@OfficialConversationDetailActivity)
            mTitleBar.setBackLayoutVisible(true)
        }

        findViewById<TextView>(R.id.remark_name)?.apply{
            setText(mOfficalName)
        }
        findViewById<CircleImageView>(R.id.avatar_image)?.apply {
            if (mAvatartUrl.isNullOrEmpty()) {
                setImageResource(R.drawable.default_user_head_icon)
            } else {
                AvatarLoaderHelper.getInstance().displayAvatar(mAvatartUrl, R.drawable
                        .default_user_head_icon, this)
            }
        }

        mNotify = findViewById(R.id.notifition_checkbox)
        mNotify?.apply {
            setBackgroundDark()
            setCheckButtonSwitch()
            val listener = View.OnClickListener ret@{
                // 无网时
                if (!ConnectivityState.isConnected(NetDiskApplication.getInstance())) {
                    setNotifyFailed()
                    return@ret
                }

                switchCheckboxLoadingMode()
                CloudP2PServiceHelper.setOfficialIsNotify(context,
                        SetOfficialNotifyResultReceiver(this@OfficialConversationDetailActivity, Handler(), null), mUk,
                        if (isChecked) CloudP2PContract.YES else CloudP2PContract.NO)
            }

            // 进到页面先设置成loading状态
            switchCheckboxLoadingMode()
            setOnCheckBoxClickListener(listener)
            isChecked = (CloudP2PProviderHelper(AccountUtils.getInstance().bduss)
                    .getIsNotify(this@OfficialConversationDetailActivity, mUk) == CloudP2PContract.YES)

        }
    }

    override fun onResume() {
        super.onResume()
        CloudP2PServiceHelper.getAllOfficialIsNotify(this@OfficialConversationDetailActivity,
                GetOfficialNotifyResultReceiver(this@OfficialConversationDetailActivity, Handler(), null))
    }

    override fun onRightButtonClicked(view: View?) {

    }

    /**
     * 返回按钮
     */
    override fun onBackButtonClicked() {
        finish()
    }

    /**
     * 设置成功回调
     */
    override fun setNotifySuccess() {
        mNotify?.apply {
            switchCheckboxNormalMode()
            if (isChecked) {
                ToastHelper.showToast(R.string.open_notify_success)
            } else {
                ToastHelper.showToast(R.string.close_notify_success)
            }
        }
    }

    /**
     * 设置失败回调
     */
    override fun setNotifyFailed() {
        mNotify?.apply {
            isChecked = !isChecked
            if (isChecked) {
                ToastHelper.showToast(R.string.close_notify_failed)
            } else {
                ToastHelper.showToast(R.string.open_notify_failed)
            }
            switchCheckboxNormalMode()
        }
    }

    /**
     * 成功获取账号免打扰消息
     */
    override fun getNotifyFinished() {
        mNotify?.apply {
            isChecked = (CloudP2PProviderHelper(AccountUtils.getInstance().bduss)
                    .getIsNotify(this@OfficialConversationDetailActivity, mUk) == CloudP2PContract.YES)
            switchCheckboxNormalMode()
        }
    }

    /**
     * 设置官方账号免打扰
     */
    private class SetOfficialNotifyResultReceiver(reference: INotifyView,
                                                  handler: Handler,
                                                  resultView: ResultView?)
        : BaseResultReceiver<INotifyView>(reference, handler, resultView) {

        override fun onSuccess(reference: INotifyView, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            if (reference != null) {
                reference.setNotifySuccess()
            }
        }

        override fun onFailed(reference: INotifyView, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            if (reference != null) {
                reference.setNotifyFailed()
            }
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    /**
     * 设置官方账号免打扰
     */
    private class GetOfficialNotifyResultReceiver(reference: INotifyView,
                                                  handler: Handler,
                                                  resultView: ResultView?)
        : BaseResultReceiver<INotifyView>(reference, handler, resultView) {

        override fun onSuccess(reference: INotifyView, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            if (reference != null) {
                reference.getNotifyFinished()
            }
        }

        override fun onFailed(reference: INotifyView, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            if (reference != null) {
                reference.getNotifyFinished()
            }
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

}