package com.baidu.netdisk.ui.cloudp2p.presenter

import android.app.Activity
import android.app.Dialog
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.net.Uri
import android.os.Environment
import android.text.TextUtils
import android.view.LayoutInflater
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.NetdiskCommonConfig
import com.baidu.netdisk.base.storage.config.Setting
import com.baidu.netdisk.kernel.android.util.file.FileUtils
import com.baidu.netdisk.kernel.android.util.image.ImageUtil
import com.baidu.netdisk.kernel.util.encode.MD5Util
import com.baidu.netdisk.sns.utils.SafeHandler
import com.baidu.netdisk.ui.cloudp2p.MyQrcodeActivity.*
import com.baidu.netdisk.ui.cloudp2p.qrcodeimage.DrawGroupQrcodePictureUtil
import com.baidu.netdisk.ui.share.ShareOption
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.expansion.qq.QqApiUtils
import com.baidu.netdisk.util.expansion.sina.SinaApiUtils
import com.baidu.netdisk.util.expansion.wx.WXApiUtils
import com.baidu.netdisk.utils.ExternalStorageUtils
import com.sina.weibo.sdk.WbSdk
import com.sina.weibo.sdk.auth.AuthInfo
import com.sina.weibo.sdk.share.WbShareHandler
import com.tencent.tauth.IUiListener
import kotlinx.android.synthetic.main.dialog_novel_add.view.*
import java.io.File
import java.util.*

/**
 * Created by linwentao on 2019-09-11.
 *
 * Describe: 群二维码的persenter
 */
class GroupQrcodePresenter(val mActivity: Activity) {

    /** loading  */
    var mLoadingDialog: Dialog? = null

    companion object {

        /**
         * 群二维码分享文件名
         */
        private const val FILE_GROUP_SHARE_QRCODE_NAME = "group_share_qrcode.jpg"

        /**
         * 用户的唯一标识
         *
         * */
        @JvmStatic
        fun getUidMd5(): String? {
            var uid = AccountUtils.getInstance().uid
            if (!TextUtils.isEmpty(uid)) {
                uid = MD5Util.createMD5WithHex(uid, false)
                if (!TextUtils.isEmpty(uid)) {
                    uid = uid.trim { it <= ' ' }
                }
            }
            return uid
        }

        /**
         * 群二维码分享图片路径
         *
         * @return 路径
         */
        @JvmStatic
        fun getShareGroupQrCodePath(): String {
            return (Setting.getDefaultSaveDir(BaseApplication.mContext) + File.separator
                    + getUidMd5() + FILE_GROUP_SHARE_QRCODE_NAME)
        }
    }

    /**
     * 生成分享渠道的群二维码
     *
     * @param view 群组信息
     * @param handler 生成成功回调
     */
    fun generateShareGroupQrcode(view: Bitmap, handler: SafeHandler) {
        val drawQrcodePictureUtil = DrawGroupQrcodePictureUtil(mActivity)
        drawQrcodePictureUtil.setData(view, handler)
        drawQrcodePictureUtil.startDraw()
    }

    /**
     * 保存生成的二维码到本地相册
     */
    fun saveGroupQrcodeToLocal(): Boolean {
        if (Environment.MEDIA_MOUNTED != ExternalStorageUtils.getExternalStorageState()) {
            ToastHelper.showToast(mActivity, R.string.sd_inval)
            return false
        }

        val qrcodeFile = File(getShareGroupQrCodePath())
        if (qrcodeFile.exists()) {
            val destFile = File(
                    ExternalStorageUtils.getExternalStorageDirectory().toString() + "/Pictures/" + System.currentTimeMillis() + ".jpg")
            FileUtils.copyFile(qrcodeFile, destFile)

            // 发送广播通知更新图库，这样系统图库可以找到这张图片
            val uri = Uri.fromFile(destFile)
            mActivity.applicationContext
                    .sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, uri))

            return destFile.exists()
        }
        return false
    }


    /**
     * 二维码分享
     * @param position
     */
    fun handleShare( position: Int) {
        var shareOption = -1
        when (position) {
            SHARE_TO_WX -> shareOption = ShareOption.SHARE_TO_WEIXIN
            SHARE_TO_QQ -> shareOption = ShareOption.SHARE_QQ
            SHARE_TO_SINA -> shareOption = ShareOption.SHARE_SINA
            SHARE_TO_WX_QUAN -> shareOption = ShareOption.SHARE_TO_WEIXIN_QUAN
            SHARE_QQ_ZONE -> shareOption = ShareOption.SHARE_QQ_ZONE
            else -> {
            }
        }
        if (shareToSInstalled(shareOption)) {
            val opts = BitmapFactory.Options()
            opts.inPreferredConfig = Bitmap.Config.RGB_565
            val bitmap = BitmapFactory.decodeFile(getShareGroupQrCodePath(), opts)
            when (shareOption) {
                ShareOption.SHARE_TO_WEIXIN -> shareQRToWeiXin(bitmap, false)
                ShareOption.SHARE_TO_WEIXIN_QUAN -> shareQRToWeiXin(bitmap, true)
                ShareOption.SHARE_QQ -> shareQRToQQ(getShareGroupQrCodePath(), null)
                ShareOption.SHARE_QQ_ZONE -> {
                    val list = ArrayList<String>()
                    list.add(getShareGroupQrCodePath())
                    shareQRToQQZone(list, null)
                }
                ShareOption.SHARE_SINA -> shareQRToSina(bitmap)
            }
        }
    }

    /**
     * 是否安装了分享客户端
     * @param shareTo
     * @return 是否安装应用
     */
    private fun shareToSInstalled(shareTo: Int): Boolean {
        if ((shareTo == ShareOption.SHARE_TO_WEIXIN || shareTo == ShareOption.SHARE_TO_WEIXIN_QUAN) && !WXApiUtils.getInstance().isWXInstalled) {
            ToastHelper.showToast(R.string.story_share_wechat_notinstall)
            return false
        }

        if ((shareTo == ShareOption.SHARE_QQ || shareTo == ShareOption.SHARE_QQ_ZONE) && !QqApiUtils.isQQClientAvailable(mActivity)) {
            ToastHelper.showToast(R.string.story_share_qq_notinstall)
            return false
        }

        if (shareTo == ShareOption.SHARE_SINA && !SinaApiUtils.isWeiboInstalled(mActivity)) {
            ToastHelper.showToast(R.string.story_share_sina_notinstall)
            return false
        }

        return true
    }

    /**
     * 微信分享或者微信朋友圈
     *
     * @param bitmap
     * @param isShareToWXQuan
     */
    private fun shareQRToWeiXin(bitmap: Bitmap, isShareToWXQuan: Boolean) {
        WXApiUtils.getInstance().sendBitmapMsgToWX(bitmap, isShareToWXQuan)
    }

    /**
     * QQ好友分享
     *
     * @param url
     * @param listener
     */
    private fun shareQRToQQ(url: String, listener: IUiListener?) {
        QqApiUtils().shareQQ(mActivity, url, listener)
    }

    /**
     * QQ空间分享
     *
     * @param urlList
     * @param listener
     */
    private fun shareQRToQQZone(urlList: ArrayList<String>, listener: IUiListener?) {
        QqApiUtils().shareToQzone(mActivity, urlList, listener)
    }

    /**
     * 新浪微博分享
     *
     * @param bitmap
     */
    private fun shareQRToSina(bitmap: Bitmap) {
        val mAuthInfo = AuthInfo(mActivity, NetdiskCommonConfig.NETDISK_THIRD_SDK_ID.sinaAppid,
                SinaApiUtils.REDIRECT_URL, SinaApiUtils.SCOPE)
        WbSdk.install(mActivity, mAuthInfo)
        val mShareHandler = WbShareHandler(mActivity)
        mShareHandler.registerApp()
        SinaApiUtils().shareWeiBoImage(mShareHandler, mActivity,
                ImageUtil.bmpToByteArray(bitmap, true))
    }

    /**
     * 展示loading
     */
    fun showLoading() {
        if (mActivity.isFinishing) {
            return
        }
        if (mLoadingDialog == null) {
            mLoadingDialog = Dialog(mActivity, R.style.BaiduNetDiskDialogTheme)
            val view = LayoutInflater.from(mActivity).inflate(R.layout.dialog_novel_add, null)
            mLoadingDialog!!.setContentView(view)
            view.dialog_tips.setText(R.string.saving)
            mLoadingDialog!!.setOnShowListener {
                if (view.loading_lottie != null) {
                    view.loading_lottie!!.playAnimation()
                }
            }
            mLoadingDialog!!.setOnDismissListener {
                if (view.loading_lottie != null) {
                    view.loading_lottie!!.cancelAnimation()
                }
            }
            mLoadingDialog!!.setOnCancelListener {
                if (view.loading_lottie != null) {
                    view.loading_lottie!!.cancelAnimation()
                }
            }
        }
        mLoadingDialog!!.show()
    }

    /**
     * 隐藏loading
     */
    fun dismisssLoading() {
        if (mActivity.isFinishing) {
            return
        }
        if (mLoadingDialog != null) {
            mLoadingDialog!!.dismiss()
            mLoadingDialog = null
        }
    }
}