package com.baidu.netdisk.ui.cloudp2p.qrcodeimage

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Matrix
import android.text.TextPaint
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.ImageView
import android.widget.LinearLayout
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.android.util.PhoneUtil
import com.baidu.netdisk.kernel.android.util.image.ImageUtil
import com.baidu.netdisk.sns.utils.SafeHandler
import com.baidu.netdisk.statistics.ThreadJob
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.MESSAGE_GENARATE_FAILED
import com.baidu.netdisk.ui.cloudp2p.GroupQrcodeActivity.Companion.MESSAGE_GENARATE_SUCCESS
import com.baidu.netdisk.ui.cloudp2p.presenter.GroupQrcodePresenter
import java.io.File
import java.io.FileOutputStream
import java.io.IOException

/**
 * Created by linwentao on 2019-09-15.
 *
 * Describe: 描画，用于保存群二维码图片
 */
class DrawGroupQrcodePictureUtil : LinearLayout {

    /**
     * 群二维码bitmap
     */
    private lateinit var mGroupInfoView: Bitmap

    /**
     * 保存的二维码bg
     */
    private lateinit var qrcodeBg : ImageView

    /**
     * 二维码背景图的宽、高
     */
    private var qrcodeBgWidth = 0
    private var qrcodeBgHeight = 0

    /**
     * 二维码描画的handler
     */
    private lateinit var mSafeHandler: SafeHandler

    /**
     * 构造方法
     * @param context Context
     * @constructor
     */
    constructor (context: Context): super(context) {
        init(context)
    }

    constructor (context: Context, attrs: AttributeSet?): super(context, attrs) {
        init(context)
    }

    constructor (context: Context, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr) {
        init(context)
    }

    /**
     * 初始化
     *
     * @param context Context
     */
    private fun init(context: Context) {
        val rootView = LayoutInflater.from(context).inflate(R.layout.layout_draw_group_qrcode_canvas, this, false)
        qrcodeBg = rootView.findViewById(R.id.qrcode_bg)
        PhoneUtil.layoutView(qrcodeBg, context)
        qrcodeBgWidth = qrcodeBg.measuredWidth
        qrcodeBgHeight = qrcodeBg.measuredHeight
    }

    /**
     * 设置需要绘制的图片数据
     *
     * @param info 需要画的参数
     * @param handler
     */
    fun setData(view: Bitmap, handler: SafeHandler) {
        this.mGroupInfoView = view
        this.mSafeHandler = handler
    }

    /**
     * 启动描画群二维码任务
     */
    fun startDraw() {
        object : ThreadJob("doContactSyncRunnable") {
            override fun performExecute() {
                draw()
            }
        }.start()
    }

    /**
     * 开始描画分享图片
     */
    fun draw() {
        // 创建空白画布
        var config: Bitmap.Config = Bitmap.Config.ARGB_8888
        var bitmapAll: Bitmap
        try {
            bitmapAll = Bitmap.createBitmap(mGroupInfoView.width, mGroupInfoView.height + qrcodeBgHeight, config)
        } catch (e: Exception) {
            e.printStackTrace()
            config = Bitmap.Config.RGB_565
            bitmapAll = Bitmap.createBitmap(mGroupInfoView.width, mGroupInfoView.height + qrcodeBgHeight, config)
        }

        val canvas = Canvas(bitmapAll)
        canvas.drawColor(Color.WHITE)
        val paint = TextPaint()
        paint.isAntiAlias = true
        paint.isDither = true
        paint.isFilterBitmap = true

        // 绘制群组信息图片
        canvas.drawBitmap(mGroupInfoView, 0f, 0f, paint)
        canvas.save()

        // 绘制网盘logo
        val bitmap = ImageUtil.getBitmapFromView(qrcodeBg)
        // 充值logo宽高
        val newBitmap = resizeImage(bitmap, mGroupInfoView.width, qrcodeBgHeight)
        if (newBitmap != null) {
            canvas.drawBitmap(newBitmap, 0f, mGroupInfoView.height.toFloat(), paint)
            canvas.save()
        }
        // 保存二维码图片
        try {
            saveBitmapBackPath(bitmapAll)
        } catch (e: IOException) {
            e.printStackTrace()
        }
    }

    /**
     * 重置图像宽高
     *
     * @param origin Bitmap?
     * @param newWidth Int
     * @param newHeight Int
     * @return Bitmap?
     */
    private fun resizeImage(origin: Bitmap?, newWidth: Int, newHeight: Int): Bitmap? {
        if (origin == null) {
            return null
        }
        val height = origin.height
        val width = origin.width
        val scaleWidth = newWidth.toFloat() / width
        val scaleHeight = newHeight.toFloat() / height
        val matrix = Matrix()
        matrix.postScale(scaleWidth, scaleHeight)
        val newBM = Bitmap.createBitmap(origin, 0, 0, width, height, matrix, false)
        if (!origin.isRecycled) {
            origin.recycle()
        }
        return newBM
    }

    /**
     * 保存图片到本地
     * @param bm
     * @throws IOException
     */
    @SuppressLint("WrongThread")
    @Throws(IOException::class)
    private fun saveBitmapBackPath(bm: Bitmap) {
        val qrcodeFile = File(GroupQrcodePresenter.getShareGroupQrCodePath())
        if (qrcodeFile.exists()) {
            qrcodeFile.delete()
        }
        try {
            val os = FileOutputStream(qrcodeFile.absolutePath)
            bm.compress(Bitmap.CompressFormat.JPEG, 100, os)
            mSafeHandler.sendEmptyMessage(MESSAGE_GENARATE_SUCCESS)
            os.close()
        } catch (e: Exception) {
            mSafeHandler.sendEmptyMessage(MESSAGE_GENARATE_FAILED)
        }

    }
}