package com.baidu.netdisk.ui.cloudp2p.search

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.ViewGroup
import android.view.WindowManager
import android.widget.RelativeLayout
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.base.storage.config.ConfigSearchLabelRecommend
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.label.entity.CharacterLabel
import com.baidu.netdisk.ui.cloudp2p.search.GroupNumberSearchContainer.Companion.PARAM_SEARCH_TYPE_FRIEND_GROUP_LABEL
import com.baidu.netdisk.uiframe.container.*
import com.baidu.netdisk.uiframe.containerimpl.list.*
import com.baidu.netdisk.uiframe.containerimpl.video.ListEmptyContainerInfo
import com.netdisk.themeskin.loader.SkinManager
import java.util.*
import kotlin.collections.ArrayList

/**
 * 人、群、标签搜索页util
 *
 * @author xujing
 * @since 2019/11/29
 */
class FriendGroupLabelSearchUtil {

    companion object {

        /** 页面tag */
        const val TAG = "friend_group_label_search"
        // 从消息列表页添加好友卡片跳转
        const val FROM_SHARE_LIST_NEW_FRINED_CARD = "from_share_list_new_friend_card"
        // 从任务系统跳转
        const val FROM_TASK_SOCRE_CARD = "from_task_socre_card"
    }

    /**
     * 打开标签搜索页
     * RelativeLayout                                   根view
     *      GroupNumberSearchContainer                  群号搜索框
     *      CommonListContainer                         列表
     *          ListRequestContainer                    列表请求
     *          SearchSingleContainer                   判断请求结果是否单条数据
     *          VerifyCodeContainer                     处理请求返回-19逻辑
     *      ListStateContainer                          列表加载页
     *          ListLoadingContainer                    列表loading动画
     *          ListEmptyContainer                      列表空视图
     *      LabelRecommendContainer                     推荐标签
     * @param activity Activity
     * @param from 页面跳转来源
     */
    fun openPage(activity: Activity, from: String) {
        var softMode = WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN or WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE
        var runningData = Bundle()
        runningData.putString(CommonPageBuilder.KEY_PAGE_NAME, TAG)
        CommonPageBuilder(activity)
                .addGroupNodeBegin(ContainerInfo(ContainerFactory.CONTAINER_LAYOUT,
                        LayoutContainerInfo(LayoutContainerInfo.LAYOUT_CONTAINER_TYPE_RELATIVE_LAYOUT), null))
                    .addNode(ContainerInfo(ContainerFactory.CONTAINER_LABEL_SEARCH_EMPTY_CONTAINER))
                    .addNode(getSearchBoxInfo(activity))
                    .addGroupNodeBegin(getListInfo())
                        .addNode(ContainerInfo(ContainerFactory.CONTAINER_LIST_REQUEST_CONTAINER, ListRequestInfo(), null))
                        .addNode(ContainerInfo(ContainerFactory.CONTAINER_SEARCH_SINGLE_RESULT))
                        .addNode(ContainerInfo(ContainerFactory.CONTAINER_LABEL_SEARCH_HANDLE_RESPONSE))
                        .addNode(ContainerInfo(ContainerFactory.CONTAINER_LABEL_NEW_FRIEND_HANDLE_RESPONSE, from, null))
                        .addNode(ContainerInfo(ContainerFactory.CONTAINER_SEARCH_FRIEND_RELATION_HANDLE))
                    .addGroupNodeEnd()
                    .addGroupNodeBegin(getListStateInfo())
                        .addNode(getLoadingInfo())
                        .addNode(getEmptyInfo())
                    .addGroupNodeEnd()
                    .addNode(getLabelRecommendInfo(activity))
                .addGroupNodeEnd()
                .setRunningData(runningData)
                .setSoftInputMode(softMode)
                .setActivityDark(true)
                .show()
    }

    /**
     * 获取搜索框信息
     *
     * @return 搜索框信息
     */
    private fun getSearchBoxInfo(context: Context): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mWidth = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mHeight = ViewGroup.LayoutParams.WRAP_CONTENT
        val hintText = context.resources.getString(R.string.search_friend_group_label_page_hint)
        val searchBoxInfo: SearchBoxInfo = SearchBoxInfo(hintText, PARAM_SEARCH_TYPE_FRIEND_GROUP_LABEL)
        return ContainerInfo(ContainerFactory.CONTAINER_GROUP_FRIEND_SEARCH_FRAME, searchBoxInfo, layoutDecoratorInfo)
    }

    /**
     * 获取列表信息
     * @return 列表信息
     */
    private fun getListInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mColorResource = R.color.my_qr_code_background
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        // 列表数据信息
        val listDataInfo = ListDataInfo()
        listDataInfo.dataFromType = ListDataTypeIDs.LIST_DATA_TYPE_MEMORY
        listDataInfo.mLayoutInfo = ListDataInfo.LayoutInfo()
        listDataInfo.mLayoutInfo.mTypeLayoutManager = ListDataInfo.LayoutInfo.TYPE_LINERLAYOUT_MANAGER
        // 列表信息
        val listInfo = CommonListInfo()
        listInfo.mDataInfo = listDataInfo
        return ContainerInfo(ContainerID.GROUP_FRIEND_SEARCH_RESULT_LIST_ID, ContainerFactory.CONTAINER_LIST, listInfo, layoutDecoratorInfo)
    }

    /**
     * 获取列表状态信息
     *
     * @return 信息
     */
    private fun getListStateInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        return ContainerInfo(ContainerFactory.CONTAINER_LIST_STATE_VIEW, ListStateInfo(), layoutDecoratorInfo)
    }

    /**
     * 获取empty信息
     *
     * @return 信息
     */
    private fun getEmptyInfo(): ContainerInfo {
        val info = ListEmptyContainerInfo()
        info.mTipsResource = R.string.no_result
        info.mImageResource = R.drawable.empty_search
        val decoratorInfo = LayoutDecoratorInfo()
        decoratorInfo.mWidth = ViewGroup.LayoutParams.WRAP_CONTENT
        decoratorInfo.mHeight = ViewGroup.LayoutParams.WRAP_CONTENT
        decoratorInfo.mColorResource = R.color.transparent
        return ContainerInfo(ContainerFactory.CONTAINER_LIST_EMPTY_LOADING, info, decoratorInfo)
    }

    /**
     * 获取loading信息
     *
     * @return 信息
     */
    private fun getLoadingInfo(): ContainerInfo {
        val decoratorInfo = LayoutDecoratorInfo()
        decoratorInfo.mWidth = ViewGroup.LayoutParams.WRAP_CONTENT
        decoratorInfo.mHeight = ViewGroup.LayoutParams.WRAP_CONTENT
        return ContainerInfo(ContainerFactory.CONTAINER_LIST_LOADING, null, decoratorInfo)
    }

    /**
     * 获取搜索推荐标签信息
     *
     * @return 搜索推荐标签信息
     */
    private fun getLabelRecommendInfo(context : Context): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mWidth = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mHeight = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        return ContainerInfo(ContainerFactory.CONTAINER_LABEL_RECOMMEND_CONTAINER, getRecommendLabelData(context), layoutDecoratorInfo)
    }

    /**
     * 获取搜索提示文案信息
     *
     * @return 搜索推荐标签信息
     */
    private fun getPageGuideHintInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mWidth = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mHeight = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        return ContainerInfo(ContainerFactory.CONTAINER_SEARCH_PAGE_GUIDE_HINT_CONTAINER, null, layoutDecoratorInfo)
    }

    /**
     * 获取搜索推荐标签数据
     *
     * @return 搜索推荐标签数据
     */
    private fun getRecommendLabelData(context : Context): ArrayList<CharacterLabel>? {
        var result: ArrayList<CharacterLabel> = ArrayList<CharacterLabel>();
        val configRecommendLabel = ConfigSearchLabelRecommend(
                ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.SEARCH_LABEL_RECOMMED_CONFIG))
        configRecommendLabel.labelList?.apply {
            if (this.isEmpty()) {
                return null
            }
            var colors: IntArray = BaseApplication.getInstance().resources.getIntArray(R.array.label_colors)
            forEach {
                if (!TextUtils.isEmpty(it)) {
                    val label = CharacterLabel()
                    label.name = it
                    val backColor = SkinManager.getInstance()
                            .getColor(R.color.bg_dn_common_color_light)
                    label.backgroudColor = backColor
                    val color = Random().nextInt(colors.size)
                    label.textColor = colors[color]
                    result.add(label)
                }
            }
        }
        return result
    }

}