package com.baidu.netdisk.ui.cloudp2p.search

import android.content.Context
import android.graphics.Typeface
import android.net.Uri
import android.text.*
import android.text.style.StyleSpan
import android.view.LayoutInflater
import android.view.View
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.base.network.FallbackManager.HTTPS
import com.baidu.netdisk.base.network.ServerURL.PAN_DOMAIN
import com.baidu.netdisk.kernel.util.encode.Base64Util
import com.baidu.netdisk.kernel.util.encode.RC4Util
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.AddFollowActivity.SEARCH_NEED_ENCRYPT
import com.baidu.netdisk.ui.cloudp2p.AddFollowActivity.SEARCH_NOT_NEED_ENCRYPT
import com.baidu.netdisk.ui.cloudp2p.VerifyCodeDialog
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.container.ContainerID
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListContainer
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListResponse
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListResponse.Companion.NEED_VERIFY_CODE
import com.baidu.netdisk.uiframe.containerimpl.list.CommonRequestInfo
import com.baidu.netdisk.uiframe.containerimpl.list.ListStateContainer
import com.baidu.netdisk.uiframe.containerimpl.list.ParserFactory.Companion.ENCODE_PARSER
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.CommonListAdapter
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.MemoryListAdapterProxy
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs
import com.baidu.netdisk.util.ToastHelper
import com.netdisk.themeskin.loader.SkinManager
import kotlinx.android.synthetic.main.label_search_frame.view.*
import org.json.JSONArray
import java.util.regex.Pattern


/**
 * 群号/好友搜索框
 *
 * @author linwentao
 * @since 2019/11/7
 */
class GroupNumberSearchContainer : BaseContainer() {
    companion object {

        /** 搜索类型--人和群 **/
        const val SEARCH_TYPE_FRIEND_DROUP = 1

        /** 搜索类型--人、群、标签 **/
        const val SEARCH_TYPE_FRIEND_GROUP_LABEL = 2

        /** 搜索接口类型参数--人、群、标签 **/
        const val PARAM_SEARCH_TYPE_FRIEND_GROUP_LABEL = 7

        /** 搜索接口类型参数--人、群**/
        const val PARAM_SEARCH_TYPE_FRIEND_GROUP = 3

        /** 默认每次列表请求搜索标签的条目 */
        const val REQUEST_SEARCH_LABEL_COUNT = 5

    }

    /** 正则表达式判断手机号 **/
    private val mIsPhonePattern = Pattern.compile("^1\\d{10}$")

    /** 正则表达式判断是否是数字 **/
    private val mNumPattern = Pattern.compile("[0-9]*")

    /** 搜索的key **/
    private var mSearchKey: String? = null

    /** 是否加密 **/
    private var mIsNeedEncrypt = SEARCH_NOT_NEED_ENCRYPT

    private lateinit var  searchText : EditText

    /** 清除按钮 **/
    private lateinit var  cleanButton : ImageView

    private var mSearchBoxInfo: SearchBoxInfo? = null;

    /** 起始值  */
    private var mStartIndex = 0

    override fun onCreateView(): View {
        val inflater = LayoutInflater.from(activity)
        val view = inflater.inflate(R.layout.label_search_frame, null)
        searchText = view.findViewById(R.id.search_text)
        cleanButton = view.findViewById(R.id.clean_button)
        val quitButton: TextView = view.findViewById(R.id.search_cancle)

        quitButton.setOnClickListener {
            activity.finish()
        }

        cleanButton.setOnClickListener {
            searchText.text.clear()
            clearSearchResult()
            postShowRecommend()
        }

        searchText.addTextChangedListener(object : TextWatcher {
            // 改变时
            override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
                if (TextUtils.isEmpty(s)) {
                    view.search_button.isEnabled = false
                    view.clean_button.visibility = View.GONE
                } else {
                    view.search_button.isEnabled = true
                    view.clean_button.visibility = View.VISIBLE
                }
            }

            // 改变前
            override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {

            }

            // 改变中
            override fun afterTextChanged(s: Editable) {

            }
        })

        searchText.setOnEditorActionListener { _, actionId, _ ->
            if (actionId == EditorInfo.IME_ACTION_SEARCH) { //判断动作标识是否搜索
                mStartIndex = 0
                onSearch()
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                        NetdiskStatisticsLogForMutilFields.StatisticsKeys.FRIEND_TAB_SEARCH_BOX_PAGE_INPUT_CLICK_EARCH)
            }
            false
        }
        return view
    }

    override fun onInitData() {
        mInfo?.apply {
            this.data?.apply {
                mSearchBoxInfo = this as SearchBoxInfo
            }
        }
    }

    override fun onAfterInitData() {
        setHintText()
    }

    override fun onCommonEvent(ev: CommonEvent): Boolean {
        when (ev.what) {
            MessageIDs.MESSAGE_COMMON_LIST_REQUEST_RESPONSE -> {
                if (ev.obj !is CommonListResponse) {
                    return false
                }
                val result = ev.obj as CommonListResponse
                if (result.errno == NEED_VERIFY_CODE) {
                    showVCodeDialog(result.mErrNoCaptcha)
                }
            }
            MessageIDs.MESSAGE_RECOMMEND_LABEL_ITEM_CLICK -> {
                val query = ev.obj as String
                setSearchText(query)
                mStartIndex = 0
                onSearch()
            }

            MessageIDs.MESSAGE_COMMON_LIST_LOAD_MORE -> {
                onSearch()
            }
            MessageIDs.MESSAGE_COMMON_LIST_LOAD_RETRY -> {
                mStartIndex -= REQUEST_SEARCH_LABEL_COUNT
                onSearch()
            }
        }
        return false
    }

    /**
     * 设置搜索框文字内容
     *
     * @param text 检索词
     */
    private fun setSearchText(text: String) {
        searchText.setText(text)
        searchText.setSelection(text.length);
        if (TextUtils.isEmpty(text)) {
            cleanButton.visibility = View.GONE
            postShowRecommend()
        } else {
            cleanButton.visibility = View.VISIBLE
            postHideRecommend()
        }
    }

    /**
     * 发起搜索
     *
     */
    private fun onSearch() {
        hideKeyBoard()
        postHideRecommend()
        networkRequest(null, null)
    }

    /**
     * 隐藏推荐标签布局
     *
     */
    private fun postHideRecommend() {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_HIDE_RECOMMEND_LABEL
        postEventToPage(event)
    }

    /**
     * 显示推荐标签布局
     *
     */
    private fun postShowRecommend() {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_SHOW_RECOMMEND_LABEL
        postEventToPage(event)
    }

    /**
     * 隐藏键盘
     */
    private fun hideKeyBoard() {
        val imm = activity.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        imm.hideSoftInputFromWindow(searchText.windowToken, 0)
    }

    /**
     * 设置搜索框默认文案
     */
    private fun setHintText() {
        // 设置hint文本
        var hintText = context.resources.getString(R.string.search_hint)
        mSearchBoxInfo?.getHintText()?.apply {
            hintText = this
        }
        val spannableString = SpannableString(hintText)
        val span = StyleSpan(Typeface.BOLD)
        spannableString.setSpan(span, 0, hintText.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        searchText.hint = spannableString

    }

    /**
     * 搜索接口请求
     */
    private fun networkRequest(vcode: String?, input: String?) {
        getEncrypt(searchText.text.toString())
        val userJson = JSONArray()
        userJson.put(mSearchKey)
        mSearchKey = if (SEARCH_NEED_ENCRYPT == mIsNeedEncrypt) {
            Base64Util.encode(RC4Util.makeRc4WithCharset(userJson.toString(), "UTF-8"))
        } else {
            userJson.toString()
        }
        val params = ArrayList<Pair<String, String>>()
        val url = Uri.Builder()
                .scheme(HTTPS)
                .encodedAuthority(PAN_DOMAIN)
                .path("mbox/usergroup/search")
                .build().toString()
        params.add(Pair("user_list", mSearchKey.toString()))
        params.add(Pair("encrypt", mIsNeedEncrypt.toString()))
        val searchtypeParam = mSearchBoxInfo?.getSearchTypeParam() ?: PARAM_SEARCH_TYPE_FRIEND_GROUP
        params.add(Pair("searchtype", searchtypeParam.toString()))
        params.add(Pair("start", "$mStartIndex"))
        params.add(Pair("limit", "${REQUEST_SEARCH_LABEL_COUNT}"))

        if(!vcode.isNullOrEmpty()) {
            params.add(Pair("vcode", vcode))
        }
        if(!input.isNullOrEmpty()) {
            params.add(Pair("input", input))
        }
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_COMMON_LIST_REQUEST
        if (SEARCH_NEED_ENCRYPT == mIsNeedEncrypt) {
            event.obj = CommonRequestInfo((mStartIndex == 0), url, true, params, ENCODE_PARSER)
        } else {
            event.obj = CommonRequestInfo((mStartIndex == 0), url, true, params)
        }
        postEventToPage(event)
        mStartIndex += REQUEST_SEARCH_LABEL_COUNT
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_PAGE_SEARCH_CLICK)
    }

    /**
     * 清除搜索结果
     */
    private fun clearSearchResult() {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_STATE_CHANGE
        event.arg1 = ListStateContainer.LIST_STATE_IDLE
        postEventToPage(event)
        val listContainer = fragment.findContainerById(ContainerID.GROUP_FRIEND_SEARCH_RESULT_LIST_ID) as CommonListContainer
        val adapter = listContainer.recyclerView?.adapter as CommonListAdapter
        val adapterProxy = adapter.adapterProxy as MemoryListAdapterProxy
        adapterProxy.clear()
    }

    /**
     * 判断是否需要加密
     *
     * @param key
     */
    private fun getEncrypt(key: String) {
        val phone = getPhoneNumber(key)
        if (TextUtils.isEmpty(phone)) {
            if (isNumeric(key)) {
                // 通过uk查找好友的用户数/次数
                NetdiskStatisticsLogForMutilFields.getInstance()
                        .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_USER_BY_UK)
            }
            mSearchKey = key
            mIsNeedEncrypt = SEARCH_NOT_NEED_ENCRYPT
        } else {
            // 通过手机号查找好友的用户数/次数
            NetdiskStatisticsLogForMutilFields.getInstance()
                    .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_USER_BY_PHONE_NUMBER)
            mSearchKey = phone
            mIsNeedEncrypt = SEARCH_NEED_ENCRYPT
        }
    }

    /**
     * 判断是否为手机号
     *
     * @param phone
     * @return
     */
    private fun getPhoneNumber(phone: String?): String? {
        var phone = phone
        if (phone == null || phone.length < 11) {
            return null
        }

        phone = phone.replace("-".toRegex(), "")
        phone = phone.replace(" ".toRegex(), "")

        if (phone.length < 11) {
            return null
        }

        val onlyPhone = phone.substring(phone.length - 11)

        val matcher = mIsPhonePattern.matcher(onlyPhone)
        return if (matcher.matches()) {
            onlyPhone
        } else null
    }

    /**
     * 字符串是否为纯数字
     *
     * @param str
     * @return
     */
    private fun isNumeric(str: CharSequence): Boolean {
        return mNumPattern.matcher(str).matches()
    }

    /**
     * 命中-19频控，需要验证
     *
     * @param errNoCaptcha
     * @return
     */
    private fun showVCodeDialog(errNoCaptcha: Int) {
        VerifyCodeDialog.show(activity, R.string.input_verify_dialog_title, R.string.search_user_vcode_desc, errNoCaptcha,
                "mbox") { vcode, text ->
                    if (!TextUtils.isEmpty(text) && !TextUtils.isEmpty(vcode)) {
                        mStartIndex -= REQUEST_SEARCH_LABEL_COUNT
                        networkRequest(vcode, text)
                    } else {
                        ToastHelper.showToast(R.string.vcode_invalide)
                    }
                }
    }
}