package com.baidu.netdisk.ui.cloudp2p.search

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.WRAP_CONTENT
import android.widget.RelativeLayout
import com.baidu.netdisk.R
import com.baidu.netdisk.uiframe.container.*
import com.baidu.netdisk.uiframe.container.ContainerFactory.*
import com.baidu.netdisk.uiframe.container.ContainerID.GROUP_FRIEND_SEARCH_RESULT_LIST_ID
import com.baidu.netdisk.uiframe.container.LayoutContainerInfo.LAYOUT_CONTAINER_TYPE_RELATIVE_LAYOUT
import com.baidu.netdisk.uiframe.containerimpl.list.*
import com.baidu.netdisk.uiframe.containerimpl.video.ListEmptyContainerInfo

/**
 * 标签搜索页util
 *
 * @author linwentao
 * @since 2019/11/7
 */
class GroupNumberSearchUtil {

    companion object {

        /** 页面tag */
        const val TAG = "group_number"
    }

    /**
     * 打开标签搜索页
     * RelativeLayout                                   根view
     *      GroupNumberSearchContainer                  群号搜索框
     *      CommonListContainer                         列表
     *          ListRequestContainer                    列表请求
     *          SearchSingleContainer                   判断请求结果是否单条数据
     *          VerifyCodeContainer                     处理请求返回-19逻辑
     *      ListStateContainer                          列表加载页
     *          ListLoadingContainer                    列表loading动画
     *          ListEmptyContainer                      列表空视图
     * @param activity Activity
     */
    fun openPage(activity: Activity) {
        var runningData = Bundle()
        runningData.putString(CommonPageBuilder.KEY_PAGE_NAME, TAG)
        CommonPageBuilder(activity)
                .addGroupNodeBegin(ContainerInfo(CONTAINER_LAYOUT,
                        LayoutContainerInfo(LAYOUT_CONTAINER_TYPE_RELATIVE_LAYOUT), null))
                    .addNode(getLabelSearchInfo(activity))
                    .addGroupNodeBegin(getListInfo())
                        .addNode(ContainerInfo(CONTAINER_LIST_REQUEST_CONTAINER, ListRequestInfo(false), null))
                        .addNode(ContainerInfo(CONTAINER_SEARCH_SINGLE_RESULT))
                        .addNode(ContainerInfo(CONTAINER_LABEL_SEARCH_HANDLE_RESPONSE))
                        .addNode(ContainerInfo(CONTAINER_SEARCH_FRIEND_RELATION_HANDLE))
                    .addGroupNodeEnd()
                    .addGroupNodeBegin(getListStateInfo())
                       .addNode(getLoadingInfo())
                       .addNode(getEmptyInfo())
                    .addGroupNodeEnd()
                .addGroupNodeEnd()
                .setRunningData(runningData)
                .setActivityDark(true)
                .show()
    }

    /**
     * 获取搜索框信息
     *
     * @return 标签信息
     */
    private fun getLabelSearchInfo(context : Context): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mWidth = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mHeight = WRAP_CONTENT
        val hintText = context.resources.getString(R.string.search_hint)
        val searchBoxInfo: SearchBoxInfo = SearchBoxInfo(hintText, GroupNumberSearchContainer.PARAM_SEARCH_TYPE_FRIEND_GROUP)
        return ContainerInfo(CONTAINER_GROUP_FRIEND_SEARCH_FRAME, searchBoxInfo, layoutDecoratorInfo)
    }

    /**
     * 获取列表信息
     * @return 列表信息
     */
    private fun getListInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mColorResource = R.color.my_qr_code_background
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        // 列表数据信息
        val listDataInfo = ListDataInfo()
        listDataInfo.dataFromType = ListDataTypeIDs.LIST_DATA_TYPE_MEMORY
        listDataInfo.mLayoutInfo = ListDataInfo.LayoutInfo()
        listDataInfo.mLayoutInfo.mTypeLayoutManager = ListDataInfo.LayoutInfo.TYPE_LINERLAYOUT_MANAGER
        // 列表信息
        val listInfo = CommonListInfo()
        listInfo.mDataInfo = listDataInfo
        return ContainerInfo(GROUP_FRIEND_SEARCH_RESULT_LIST_ID, CONTAINER_LIST, listInfo, layoutDecoratorInfo)
    }

    /**
     * 获取列表状态信息
     *
     * @return 信息
     */
    private fun getListStateInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        return ContainerInfo(CONTAINER_LIST_STATE_VIEW, ListStateInfo(), layoutDecoratorInfo)
    }

    /**
     * 获取empty信息
     *
     * @return 信息
     */
    private fun getEmptyInfo(): ContainerInfo {
        val info = ListEmptyContainerInfo()
        info.mTipsResource = R.string.no_result
        info.mImageResource = R.drawable.empty_search
        val decoratorInfo = LayoutDecoratorInfo()
        decoratorInfo.mWidth = WRAP_CONTENT
        decoratorInfo.mHeight = WRAP_CONTENT
        decoratorInfo.mColorResource = R.color.transparent
        return ContainerInfo(CONTAINER_LIST_EMPTY_LOADING, info, decoratorInfo)
    }

    /**
     * 获取loading信息
     *
     * @return 信息
     */
    private fun getLoadingInfo(): ContainerInfo {
        val decoratorInfo = LayoutDecoratorInfo()
        decoratorInfo.mWidth = WRAP_CONTENT
        decoratorInfo.mHeight = WRAP_CONTENT
        return ContainerInfo(CONTAINER_LIST_LOADING, null, decoratorInfo)
    }
}