package com.baidu.netdisk.ui.cloudp2p.search

import android.view.LayoutInflater
import android.view.View
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.util.CollectionUtils
import com.baidu.netdisk.label.entity.CharacterLabel
import com.baidu.netdisk.label.view.LabelListView
import com.baidu.netdisk.label.view.base.BaseLabelListView
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs
import com.baidu.netdisk.uiframe.message.MessageIDs.MESSAGE_HIDE_RECOMMEND_LABEL
import com.baidu.netdisk.uiframe.message.MessageIDs.MESSAGE_SHOW_RECOMMEND_LABEL

/**
 * 标签搜索推荐容器
 *
 * @author xuijng31
 * @since 2019/11/8
 */
class LabelSearchRecommendContainer : BaseContainer() {
    private val TAG = "LabelSugContainer"

    /** 根布局  */
    private var mRootView: View? = null

    /** 推荐标签列表布局  */
    private var mRecommendLabel: LabelListView? = null

    /** 推荐标签列表数据  */
    private var mRecommendLabelList: List<CharacterLabel>? = null

    override fun onCreateView(): View? {
        val inflater = LayoutInflater.from(activity)
        mRootView = inflater.inflate(R.layout.search_recommend_label_layout, null)
        mRecommendLabel = mRootView?.findViewById(R.id.search_recommed_label_list)
        mRecommendLabel?.setclickListener(object : BaseLabelListView.IOnItemClickListener {
            override fun onClick(name: String, position: Int) {
                postClickLabel(name)
            }

            override fun onDeleteTag(name: String, position: Int) {

            }
        })
        return mRootView
    }

    override fun onInitData() {
        super.onInitData()
        mRecommendLabelList = mInfo.data as ArrayList<CharacterLabel>
        if (CollectionUtils.isEmpty(mRecommendLabelList)) {
            // 推荐标签配置为空时，不展示布局
            mRootView?.visibility = View.GONE
            return
        }
        mRecommendLabelList?.apply {
            mRecommendLabel?.setSize(12)
            mRecommendLabel?.setBoldStyle(true)
            mRecommendLabel?.setData(this, BaseLabelListView.TYPE_FULL)
        }
    }

    override fun onCommonEvent(ev: CommonEvent): Boolean {
        when (ev.what) {
            MESSAGE_HIDE_RECOMMEND_LABEL -> {
                mRootView?.visibility = View.GONE
            }

            MESSAGE_SHOW_RECOMMEND_LABEL -> {
                if (!CollectionUtils.isEmpty(mRecommendLabelList)) {
                    mRootView?.visibility = View.VISIBLE
                }
            }
        }
        return false
    }

    /**
     * 发送点击某个推荐标签的消息
     * @param label 标签名
     */
    fun postClickLabel(label: String) {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_RECOMMEND_LABEL_ITEM_CLICK
        event.obj = label
        postEventToPage(event)
    }
}