package com.baidu.netdisk.ui.cloudp2p.search

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.text.TextUtils
import android.view.ViewGroup
import android.view.ViewGroup.LayoutParams.WRAP_CONTENT
import android.view.WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN
import android.view.WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE
import android.widget.RelativeLayout
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.base.storage.config.ConfigSearchLabelRecommend
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.label.entity.CharacterLabel
import com.baidu.netdisk.uiframe.container.*
import com.baidu.netdisk.uiframe.container.CommonPageBuilder.KEY_PAGE_NAME
import com.baidu.netdisk.uiframe.container.ContainerFactory.*
import com.baidu.netdisk.uiframe.container.ContainerID.LABEL_SEARCH_RESULT_LIST_ID
import com.baidu.netdisk.uiframe.container.LayoutContainerInfo.LAYOUT_CONTAINER_TYPE_RELATIVE_LAYOUT
import com.baidu.netdisk.uiframe.containerimpl.list.*
import com.baidu.netdisk.uiframe.containerimpl.video.ListEmptyContainerInfo
import com.netdisk.themeskin.loader.SkinManager
import java.util.*
import kotlin.collections.ArrayList

/**
 * 标签搜索页util
 *
 * @author wangyang34
 * @since 2019/11/6
 */
class LabelSearchUtil {

    companion object {

        /** 页面tag */
        const val TAG = "label_search"
    }

    /**
     * 打开标签搜索页
     * RelativeLayout                                   根view
     *      LabelSearchEmptyContainer                   空view 用于处理页面的特殊逻辑
     *      LabelSearchContainer                        标签搜索框
     *      CommonListContainer                         列表
     *          ListRequestContainer                    列表请求
     *          LabelSearchHandleResponse               去重
     *      ListStateContainer                          列表加载页
     *          ListLoadingContainer                    列表loading动画
     *          ListEmptyContainer                      列表空视图
     *      LabelRecommendContainer                     推荐标签
     *      LabelSugContainer                           sug
     *
     * @param activity Activity
     * @param query 默认检索词
     */
    fun openPage(activity: Activity, query: String = "") {
        var softMode = SOFT_INPUT_ADJUST_PAN
        if (TextUtils.isEmpty(query)) {
            softMode = softMode or SOFT_INPUT_STATE_VISIBLE
        }
        var runningData = Bundle()
        runningData.putString(KEY_PAGE_NAME, TAG)
        CommonPageBuilder(activity)
                .addGroupNodeBegin(ContainerInfo(CONTAINER_LAYOUT,
                        LayoutContainerInfo(LAYOUT_CONTAINER_TYPE_RELATIVE_LAYOUT), null))
                    .addNode(ContainerInfo(CONTAINER_LABEL_SEARCH_EMPTY_CONTAINER))
                    .addNode(getLabelSearchInfo(query))
                    .addGroupNodeBegin(getListInfo())
                        .addNode(ContainerInfo(CONTAINER_LIST_REQUEST_CONTAINER, ListRequestInfo(), null))
                        .addNode(ContainerInfo(CONTAINER_LABEL_SEARCH_HANDLE_RESPONSE, ListRequestInfo(), null))
                        .addNode(ContainerInfo(CONTAINER_SEARCH_FRIEND_RELATION_HANDLE))
                    .addGroupNodeEnd()
                    .addGroupNodeBegin(getListStateInfo())
                        .addNode(getLoadingInfo())
                        .addNode(getEmptyInfo())
                    .addGroupNodeEnd()
                    .addNode(getLabelRecommendInfo(activity))
                    .addNode(getLabelSugInfo())
                .addGroupNodeEnd()
                .setRunningData(runningData)
                .setSoftInputMode(softMode)
                .setActivityDark(true)
                .show()
    }

    /**
     * 获取列表状态信息
     *
     * @return 信息
     */
    private fun getListStateInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        return ContainerInfo(CONTAINER_LIST_STATE_VIEW, ListStateInfo(), layoutDecoratorInfo)
    }

    /**
     * 获取loading信息
     *
     * @return 信息
     */
    private fun getLoadingInfo(): ContainerInfo {
        val decoratorInfo = LayoutDecoratorInfo()
        decoratorInfo.mWidth = WRAP_CONTENT
        decoratorInfo.mHeight = WRAP_CONTENT
        return ContainerInfo(CONTAINER_LIST_LOADING, null, decoratorInfo)
    }

    /**
     * 获取empty信息
     *
     * @return 信息
     */
    private fun getEmptyInfo(): ContainerInfo {
        val info = ListEmptyContainerInfo()
        info.mTipsResource = R.string.list_label_empty_tips
        info.mImageResource = R.drawable.list_label_empty_tips
        val decoratorInfo = LayoutDecoratorInfo()
        decoratorInfo.mWidth = WRAP_CONTENT
        decoratorInfo.mHeight = WRAP_CONTENT
        return ContainerInfo(CONTAINER_LIST_EMPTY_LOADING, info, decoratorInfo)
    }

    /**
     * 获取标签搜索框信息
     *
     * @param query 检索词
     * @return 标签信息
     */
    private fun getLabelSearchInfo(query: String): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mWidth = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mHeight = ViewGroup.LayoutParams.WRAP_CONTENT
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.title_bar_general_root_view))
        return ContainerInfo(CONTAINER_LABEL_SEARCH_FRAME, query, layoutDecoratorInfo)
    }

    /**
     * 获取标签sug信息
     *
     * @return 标签sug信息
     */
    private fun getLabelSugInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mWidth = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mHeight = ViewGroup.LayoutParams.WRAP_CONTENT
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        return ContainerInfo(CONTAINER_LABEL_SUG_CONTAINER, null, layoutDecoratorInfo)
    }

    /**
     * 获取搜索推荐标签信息
     *
     * @return 搜索推荐标签信息
     */
    private fun getLabelRecommendInfo(context : Context): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mWidth = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mHeight = ViewGroup.LayoutParams.MATCH_PARENT
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        return ContainerInfo(CONTAINER_LABEL_RECOMMEND_CONTAINER, getRecommendLabelData(context), layoutDecoratorInfo)
    }

    /**
     * 获取搜索推荐标签数据
     *
     * @return 搜索推荐标签数据
     */
    private fun getRecommendLabelData(context : Context): ArrayList<CharacterLabel>? {
        var result: ArrayList<CharacterLabel> = ArrayList<CharacterLabel>();
        val configRecommendLabel = ConfigSearchLabelRecommend(
                ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.SEARCH_LABEL_RECOMMED_CONFIG))
        configRecommendLabel.labelList?.apply {
            if (this.isEmpty()) {
                return null
            }
            var colors: IntArray = BaseApplication.getInstance().resources.getIntArray(R.array.label_colors)
            forEach {
                if (!TextUtils.isEmpty(it)) {
                    val label = CharacterLabel()
                    label.name = it
                    val backColor = SkinManager.getInstance()
                            .getColor(R.color.bg_dn_common_color_light)
                    label.backgroudColor = backColor
                    val color = Random().nextInt(colors.size)
                    label.textColor = colors[color]
                    result.add(label)
                }
            }
        }
        return result
    }

    /**
     * 获取列表信息
     * @return 列表信息
     */
    private fun getListInfo(): ContainerInfo {
        val layoutDecoratorInfo = LayoutDecoratorInfo()
        layoutDecoratorInfo.mColorResource = R.color.my_qr_code_background
        layoutDecoratorInfo.mRule.add(LayoutRule(RelativeLayout.BELOW, R.id.search_frame))
        // 列表数据信息
        val listDataInfo = ListDataInfo()
        listDataInfo.dataFromType = ListDataTypeIDs.LIST_DATA_TYPE_MEMORY
        listDataInfo.mLayoutInfo = ListDataInfo.LayoutInfo()
        listDataInfo.mLayoutInfo.mTypeLayoutManager = ListDataInfo.LayoutInfo.TYPE_LINERLAYOUT_MANAGER
        // 列表信息
        val listInfo = CommonListInfo()
        listInfo.mDataInfo = listDataInfo
        return ContainerInfo(LABEL_SEARCH_RESULT_LIST_ID, CONTAINER_LIST, listInfo, layoutDecoratorInfo)
    }
}