package com.baidu.netdisk.ui.cloudp2p.search

import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import com.baidu.netdisk.NetDiskApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.android.util.deviceinfo.DeviceDisplayUtils
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.containerimpl.sug.SugItem
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs

/**
 * 标签sug容器
 *
 * @author xuijng31
 * @since 2019/11/8
 */
class LabelSugContainer : BaseContainer(), SugItem.SugItemClickListener  {

    companion object {

        /** tag */
        const val TAG = "LabelSugContainer"

        /** 最多展示的sug条目 */
        const val MAX_SHOW_SUG_COUNT = 10
    }

    /** 根布局  */
    private lateinit var mRootView: View

    /** Linear布局  */
    private lateinit var mLinearView: LinearLayout

    override fun onCreateView(): View {
        val inflater = LayoutInflater.from(activity)
        mRootView = inflater.inflate(R.layout.label_search_sug_layout, null)
        mLinearView = mRootView.findViewById(R.id.label_search_sug)
        return mRootView
    }

    override fun onClickSug(sug: String?) {
        // 点击某个sug的回调
        NetDiskLog.d(TAG, "点击某个sug: " + sug)
        sug?.apply {
            postClickSug(this)
        }
    }

    override fun onCommonEvent(ev: CommonEvent): Boolean {
        when (ev.what) {
            MessageIDs.MESSAGE_SHOW_SUG -> {
                // 每次显示sug都先清除一下之前的view
                mLinearView.removeAllViews()
                var sugList = ev.obj as ArrayList<String>
                mRootView.visibility = View.VISIBLE
                val marginLeft = DeviceDisplayUtils.dip2px(NetDiskApplication.getInstance(), 35f);
                sugList.apply {
                    forEach {
                        if (mLinearView.childCount >= MAX_SHOW_SUG_COUNT) {
                            return false;
                        }
                        if (!TextUtils.isEmpty(it)) {
                            var sugItem = SugItem(context, it)
                            sugItem.setTextMarginLeft(marginLeft)
                            sugItem.setOnClickListener(this@LabelSugContainer)
                            mLinearView.addView(sugItem.getView())
                        }
                    }
                }
            }
            MessageIDs.MESSAGE_HIDE_SUG -> {
                mLinearView.removeAllViews()
                mRootView.visibility = View.GONE
            }
        }
        return false
    }

    /**
     * 发送点击某个sug的消息
     * @param sug 点击的sug
     */
    fun postClickSug(sug: String) {
        val event = CommonEvent()
        event.what = MessageIDs.MESSAGE_SUG_ITEM_CLICK
        event.obj = sug
        postEventToPage(event)
    }
}