package com.baidu.netdisk.ui.cloudp2p.search

import android.view.View
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.utils.PersonalConfigKey
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.cloudp2p.provider.CloudP2PProviderHelper
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.util.CollectionUtils
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.task.TaskScoreManager
import com.baidu.netdisk.task.taskscore.TaskScoreSearchLable
import com.baidu.netdisk.uiframe.card.CardStoreCardID
import com.baidu.netdisk.uiframe.card.CommonCardInfo
import com.baidu.netdisk.uiframe.cardimpl.FriendSearchInfo
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListResponse
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs

/**
 * 从"添加好友"卡片跳转搜索，如果搜索到好友则删除该会话卡片
 * 从"任务系统"跳抓安搜索，如果搜索结果用户代表标签则完成任务
 *
 * @author wanglihong01
 * @since 2019/12/3
 */
@Tag("SearchNewFriendHandleResponse")
class SearchNewFriendHandleResponse : BaseContainer() {

    public var find: Boolean = false

    override fun onCreateView(): View? {
        return null
    }

    override fun onCommonEvent(ev: CommonEvent): Boolean {
        // 如果data为空则表示不进行检查
        if (mInfo.data == null) {
            return false;
        } else if (mInfo.data !is String){
            return false;
        }

        when (ev.what) {
            MessageIDs.MESSAGE_COMMON_LIST_REQUEST_RESPONSE -> {
                if (ev.obj is CommonListResponse) {
                    // 处理来自添加好友卡片结果
                    if (FriendGroupLabelSearchUtil.FROM_SHARE_LIST_NEW_FRINED_CARD.equals(mInfo.data)) {
                        // 如果没有找到则继续查找
                        if (!find) {
                            updateFriendCard((ev.obj as CommonListResponse).mCardList)
                        }
                    } else if (FriendGroupLabelSearchUtil.FROM_TASK_SOCRE_CARD.equals(mInfo.data)) {
                        // 处理来自任务系统结果
                        updateTaskScoreCard((ev.obj as CommonListResponse).mCardList)
                    }
                }
            }
        }
        return false
    }

    /**
     * 从"添加好友"卡片跳转的搜索
     * 根据搜索结果存在好友信息，则删除会话列表页中的"添加好友"卡片
     */
    fun updateFriendCard(cardList: ArrayList<CommonCardInfo>?) {
        if (CollectionUtils.isEmpty(cardList)) {
            return
        }

        if (PersonalConfig.getInstance().getBoolean(PersonalConfigKey.KEY_GUIDE_NEW_FRIEND_NOT_SEARCHED,
                        true)) {
            cardList?.forEach result@{
                if (it.type == CardStoreCardID.CARD_ITEM_TYPE_FRIEND_INFO_CARD) {
//                            NetDiskLog.d(TAG, "updaateFriendCard find friend info card");
                    (CloudP2PProviderHelper(AccountUtils.getInstance().bduss))
                            .insertOrDeleteGuideConversation(activity, false,
                                    CloudP2PContract.Messages.TYPE_GUIDE_NEWFRIEND)
                    PersonalConfig.getInstance().putBoolean(PersonalConfigKey
                            .KEY_GUIDE_NEW_FRIEND_NOT_SEARCHED, false);
                    PersonalConfig.getInstance().commit()
                    find = true
                    return@result
                }
            }
        } else {
            find = true;
        }
    }

    /**
     * 从任务系统跳转搜索查找结果中存在标签用户
     */
    fun updateTaskScoreCard(cardList: ArrayList<CommonCardInfo>?) {
        if (CollectionUtils.isEmpty(cardList)) {
            return
        }

        cardList?.forEach result@{
            it?.data?.apply {
                if (this is FriendSearchInfo) {
                    tags?.takeIf { it.size != 0 }?.apply {
                        TaskScoreManager.getInstance().completeTask(context,
                                TaskScoreSearchLable.TASK_SCORE_SEARCH_LABEL)
                        return@result
                    }
                }
            }
        }
    }

}