package com.baidu.netdisk.ui.cloudp2p.search

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.net.Uri
import android.support.v4.content.LocalBroadcastManager
import android.support.v7.widget.SimpleItemAnimator
import android.text.TextUtils
import android.view.View
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.servicechannel.ServiceChannelConstants
import com.baidu.netdisk.servicechannel.entity.Channel
import com.baidu.netdisk.uiframe.UIFrameConstans
import com.baidu.netdisk.uiframe.card.CommonCardInfo
import com.baidu.netdisk.uiframe.cardimpl.FriendSearchInfo
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.containerimpl.homepage.HomePageServiceRefreshContainer
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListContainer
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.CommonListAdapter

/**
 * 好友关系变化处理容器类
 *
 * @author xujing31
 * @since 2019/12/5
 */
class SearchRelationChangeHandleContainer : BaseContainer() {
    companion object{

        /** 好友关系 **/
        private const val IS_FRIEND = 1

        /** 非好友关系 **/
        private const val IS_NOT_FRIEND = 0

        /** 双向好友关系 是双向好友 **/
        private const val TWO_WAY_FRIEND = 10
    }


    /** 关系变更接收  */
    private var mRelationChangeReceiver: BroadcastReceiver? = null

    /** 列表容器  */
    private lateinit var mListContainer: CommonListContainer

    override fun onCreateView(): View? {
        mListContainer = parent as CommonListContainer
        registerReceiver()
        return null
    }

    override fun onDestroyView() {
        unregisterReceiver()
    }


    /**
     * 注册广播监听
     */
    private fun registerReceiver() {
        if (mRelationChangeReceiver != null) {
            return
        }
        mRelationChangeReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                if (intent == null) {
                    return
                }
                val uk = intent.getLongExtra(UIFrameConstans.DATA_FRIEND_UK, 0)
                when (intent.action) {
                    UIFrameConstans.ACTION_USER_INFO_PAGE_ADD_SUCCEED -> {
                        uk?.apply {
                            updateRelation(uk, true)
                        }
                    }
                    UIFrameConstans.ACTION_USER_INFO_PAGE_DELETE_SUCCEED -> {
                        uk?.apply {
                            updateRelation(uk, false)
                        }
                    }
                }
            }
        }
        mRelationChangeReceiver?.apply {
            val filter = IntentFilter()
            filter.addAction(UIFrameConstans.ACTION_USER_INFO_PAGE_ADD_SUCCEED)
            filter.addAction(UIFrameConstans.ACTION_USER_INFO_PAGE_DELETE_SUCCEED)
            LocalBroadcastManager.getInstance(context).registerReceiver(this, filter)
        }
    }

    /**
     * 注销广播监听
     */
    private fun unregisterReceiver() {
        mRelationChangeReceiver?.apply {
            LocalBroadcastManager.getInstance(context).unregisterReceiver(this)
            mRelationChangeReceiver = null
        }
    }

    /**
     * 更新好友关系
     * @param uk 变更的好友的uk
     * @param isFriend 变更后是否是好友关系
     */
    private fun updateRelation(uk: Long, isFriend: Boolean) {
        // 修改内存数据
        val adapter = mListContainer.getRecyclerView().getAdapter() as CommonListAdapter
                ?: return
        val memoryData = adapter.listData as List<CommonCardInfo>
        memoryData?.apply {
            if (this.size == 0) {
                return
            }
            var iterator = this.iterator()
            while (iterator.hasNext()) {
                val cardInfo: CommonCardInfo? = iterator.next()
                if (cardInfo == null || cardInfo.data !is FriendSearchInfo) {
                    continue
                }
                var friendSearchInfo = cardInfo.data as FriendSearchInfo
                if (friendSearchInfo.uk != uk) {
                    continue
                }
                if (isFriend) {
                    // 变更为好友关系
                    if (friendSearchInfo.rel > 0) {
                        friendSearchInfo.rel = TWO_WAY_FRIEND
                    } else {
                        friendSearchInfo.followFlag = IS_FRIEND
                    }
                } else {
                    // 变更为非好友关系
                    friendSearchInfo.rel = IS_NOT_FRIEND
                    friendSearchInfo.followFlag = IS_NOT_FRIEND
                }
            }
            adapter.notifyDataSetChanged()
        }

    }
}