package com.baidu.netdisk.ui.cloudp2p.search
import android.view.View
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.cloudp2p.provider.CloudP2PProviderHelper
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.ConversationActivity
import com.baidu.netdisk.ui.cloudp2p.GroupInfoActivity
import com.baidu.netdisk.ui.cloudp2p.UserInfoActivity
import com.baidu.netdisk.uiframe.card.CardStoreCardID.CARD_ITEM_TYPE_FRIEND_INFO_CARD
import com.baidu.netdisk.uiframe.card.CardStoreCardID.CARD_ITEM_TYPE_GROUP_INFO_CARD
import com.baidu.netdisk.uiframe.card.CommonCardInfo
import com.baidu.netdisk.uiframe.cardimpl.FriendSearchInfo
import com.baidu.netdisk.uiframe.cardimpl.GroupSearchInfo
import com.baidu.netdisk.uiframe.cardimpl.SearchCardHelper
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.container.ContainerID
import com.baidu.netdisk.uiframe.containerimpl.FriendGroupRequestContainer.Companion.FROM_SEARCH
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListContainer
import com.baidu.netdisk.uiframe.containerimpl.list.ListStateContainer.LIST_STATE_FINISH
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.CommonListAdapter
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.MemoryListAdapterProxy
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs.MESSAGE_STATE_CHANGE
import java.util.*

/**
 * 搜索结果为单个人或者群时，自动跳转到相应界面的逻辑处理
 *
 * author linwentao
 * created 2019/11/12
 */
class SearchSingleContainer : BaseContainer() {

    companion object{

        /** 用户在群里 **/
        private const val INSIDE_GROUP = 1
    }

    override fun onCreateView(): View? {
        return null
    }

    override fun onCommonEvent(event: CommonEvent?): Boolean {
        // 收到完成消息,判断搜索结果是否是单条结果
        when (event?.what) {
            MESSAGE_STATE_CHANGE -> {
                if (event.arg1 != LIST_STATE_FINISH) {
                    return false
                }

                val listContainer = fragment.findContainerById(ContainerID.GROUP_FRIEND_SEARCH_RESULT_LIST_ID) as CommonListContainer
                val adapter = listContainer.recyclerView?.adapter as CommonListAdapter
                val adapterProxy = adapter.adapterProxy as MemoryListAdapterProxy
                val listData = adapterProxy.listData as ArrayList<CommonCardInfo>

                if (listData.size == 0) {
                    NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                            NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_RESULT_PAGE_NULL_SHOW)
                } else {
                    NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                            NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_RESULT_PAGE_SHOW)
                }

                val data = ArrayList<CommonCardInfo>()
                for (info : CommonCardInfo? in listData) {
                    if (info != null && (info.type == CARD_ITEM_TYPE_FRIEND_INFO_CARD ||
                            info.type == CARD_ITEM_TYPE_GROUP_INFO_CARD)) {
                        data.add(info)
                        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                                NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_RESULT_CONTAINS_GROUP_USER, info.type)
                    }
                }

                if (data.size != 1) {
                    return false
                }

                if(data[0].data == null) {
                    return false
                }

                if (data[0].type == CARD_ITEM_TYPE_FRIEND_INFO_CARD) {
                    val friendSearchInfo = data[0].data as FriendSearchInfo
                    UserInfoActivity.startUserInfoActivity(activity, friendSearchInfo.uk, null)

                } else if (data[0].type == CARD_ITEM_TYPE_GROUP_INFO_CARD) {
                    val groupSearchInfo = data[0].data as GroupSearchInfo

                    if (groupSearchInfo.inside == INSIDE_GROUP) {
                        // 在对话框页面内插入一条系统提示消息
                            val helper = CloudP2PProviderHelper(AccountUtils.getInstance().bduss)
                            val resolver = BaseApplication.getInstance().contentResolver
                            helper.addGroupTextMessage(resolver, groupSearchInfo.gid,
                                    context.getString(R.string.scan_join_group_already_in),
                                    null, 0, "", "",
                                    CloudP2PContract.Messages.TYPE_GROUP_SYSTEM, false)
                        // 跳转进入群组对话框页面
                            val intent = ConversationActivity.getStartIntent(activity,
                                    CloudP2PContract.Groups.buildGroupUri(groupSearchInfo.gid, AccountUtils.getInstance().bduss),
                                    groupSearchInfo.name, null, false, false, null)
                            activity.startActivity(intent)
                    } else {
                        val uri = SearchCardHelper.BASE_JOIN_URI.buildUpon()
                                .appendQueryParameter("gid", groupSearchInfo.gid.toString())
                                .appendQueryParameter("sign", groupSearchInfo.sign)
                                .appendQueryParameter("invite_uk", groupSearchInfo.inviteUk.toString())
                                .appendQueryParameter("invite_time", groupSearchInfo.inviteTime.toString())
                                .appendQueryParameter("from", FROM_SEARCH)
                        GroupInfoActivity().openPage(activity, uri.toString())
                    }
                }
                adapterProxy.clear()
            }
        }
        return false
    }
}
