package com.baidu.netdisk.ui.gc

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.util.TypedValue
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.ui.widget.BaseFragment
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.storage.config.ConfigGarbageFile
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.gc.io.parser.model.GarbageFileItem
import com.baidu.netdisk.gc.service.Extras
import com.baidu.netdisk.gc.statistics.StatisticsKey
import com.baidu.netdisk.kernel.android.util.network.ConnectivityState
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.ui.businessplatform.extension.toast
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.gc.adapter.EmptyFolderAdapter
import com.baidu.netdisk.ui.gc.presenter.GarbageFilesPresenter
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.vip.VipActivity
import kotlinx.android.synthetic.main.fragment_empty_folder_list.*

@Tag("EmptyFolderListFragment")
class EmptyFolderListFragment : BaseFragment(), IGarbageFilesView<GarbageFileItem> {

    companion object {
        const val PAGE_LIMIT = 30
        const val SVIP_PAGE_LIMIT = 1000
        const val REQUEST_CODE = 10
        const val SVIP_LIMIT = 3000
        const val AUTO_LOAD_COUNT = 50
    }

    private val mGarbageFilesPresenter = GarbageFilesPresenter(this)
    private lateinit var mAdapter: EmptyFolderAdapter
    private var mTaskId = ""
    private var mHasMore: Boolean = false
    private var mStart: Int = 0
    private var mIsLoadingMore: Boolean = false
    private lateinit var mGuideDialog: VipGuideDialog
    private var mTotalNum = 0
    private var mSelectedNum = 0

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        super.onCreateView(inflater, container, savedInstanceState)
        return inflater.inflate(R.layout.fragment_empty_folder_list, null, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        mAdapter = EmptyFolderAdapter(::updateBottomBar)
        initView()
        // 发送网络请求
        startLoading()
    }

    fun initView() {
        activity?.apply {
            (this as? BaseActivity)?.titleBar?.setMiddleTitle(R.string.garbage_empty_folder_clean_title)
        }
        rv_filelist.apply {
            layoutManager = LinearLayoutManager(context)
            adapter = mAdapter
            setOnLoadMoreListener {
                getFilesList()
            }
        }
        btn_delete.isEnabled = false
        btn_delete.setOnClickListener {
            StatisticsKey.GC_EMPTY_FOLDER_CLEAN_NOW.updateCount()
            if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP) {
                showDeleteDialog()
            } else {
                showGuideSVipDialog()
            }
        }

        tv_select_all.setOnClickListener {
            mAdapter.onSelectAllStateChange()
        }
        arguments?.apply {
            mTaskId = getString(GarbageFileListActivity.TASK_ID) ?: ""
            mTotalNum = getInt(GarbageFileListActivity.GARBAGE_FILE_COUNT)
        }
        head_text.text = resources.getString(R.string.garbage_empty_list_head_desc, mTotalNum)
        empty_view.setRefreshListener {
            startLoading()
        }
    }

    private fun startLoading() {
        empty_view.visibility = View.VISIBLE
        empty_view.setLoading(R.string.loading)
        rv_filelist.visibility = View.GONE
        getFilesList()
    }

    private fun stopLoading(count: Int) {
        if (count > 0) {
            hidEmptyView()
        } else {
            displayEmptyView(false)
        }
    }


    private fun needAutoLoad(count: Int) =
            count <= AUTO_LOAD_COUNT && hasMore()

    private fun updateFooterView() {
        with(rv_filelist) {
            val svip = AccountUtils.getInstance().isSVip
            // 条目超出列表上限，并且hasMore是true，才会加footer
            if (limitCount() && mHasMore) {
                if (footerContainer.childCount == 0) {
                    val view = LayoutInflater.from(context).inflate(if (svip) R.layout
                            .footer_garbage_count_limit else
                        R.layout.footer_garbage_footer_svip_common, null)
                    addFooterView(view)
                    if (svip) {
                        view.findViewById<TextView>(R.id.tv_limit_desc).text = getString(R.string
                                .garbage_empty_folder_footer_text_svip)
                    } else {
                        setNotSvipFooterGuide(view)
                    }
                }
            } else {
                if (footerContainer.childCount > 0) {
                    footerContainer.removeAllViews()
                }
            }
        }
    }

    // 设置Footer文案（区分 普通用户/vip）
    private fun setNotSvipFooterGuide(view: View) {
        val tvGuide = view.findViewById<TextView>(R.id.guide_desc)
        val btnSvip = view.findViewById<Button>(R.id.svip_button)
        val cfg = ConfigGarbageFile(ServerConfigKey.getCfgConfigBodyByType(
                ServerConfigKey.ConfigType.GARBAGE_FILE_CONFIG))
        if (AccountUtils.getInstance().isVip) {
            tvGuide.text = cfg.vipShowAllEmptyFolderGuideDesc
            btnSvip.text = cfg.vipShowAllEmptyFolderButton
            btnSvip.setOnClickListener {
                footerToSVipGuideIntent(cfg.vipShowAllEmptyFolderGuideUrl,
                        StatisticsKey.GC_EMPTY_FOLDER_SHOW_ALL_UPGRADE_SVIP)
            }
        } else {
            tvGuide.text = cfg.showAllEmptyFolderGuideDesc
            btnSvip.text = cfg.showAllEmptyFolderButton
            btnSvip.setOnClickListener {
                footerToSVipGuideIntent(cfg.showAllEmptyFolderGuideUrl,
                        StatisticsKey.GC_EMPTY_FOLDER_SHOW_ALL_OPEN_SVIP)
            }
        }
    }

    private fun footerToSVipGuideIntent(url: String?, countStr: String?) {
        activity?.apply {
            if (url.isNullOrEmpty()) {
                val intent = VipActivity.getStartIntent(
                        this,
                        VipActivity.GARBAGE_SHOW_EMPTY_TO_SVIP,
                        VipActivity.TO_SVIP)
                startActivityForResult(intent, REQUEST_CODE)
            } else {
                startActivity(RichMediaActivity.getStartIntent(
                        this, url))
            }
            countStr?.updateCount()
        }
    }

    override fun onGetDataSuccess(hasMore: Boolean, start: Int, data: List<GarbageFileItem>) {
        mStart = start
        mHasMore = hasMore
        mAdapter.setData(data)
        stopLoading(mAdapter.itemCount)
        updateFooterView()
        mIsLoadingMore = false
        rv_filelist.setLoadMoreEnabled(hasMore())
    }

    override fun onGetDataFail() {
        activity?.let {
            if (mAdapter.itemCount == 0) {
                displayEmptyView(true)
            } else {
                toast(R.string.dup_files_list_get_fail)
            }
        }
        mIsLoadingMore = false
    }

    override fun onDeleteSuccess() {
        "onDeleteSuccess".d()
        updateHeadView()
        mAdapter.onDeleteSuccess()
        if (needAutoLoad(mAdapter.itemCount)) {
            getFilesList()
            toast(R.string.garbage_delete_success)
        } else {
            if (mAdapter.itemCount == 0) {
                activity?.apply {
                    GarbageCleanResultActivity.startActivity(this, Extras.TYPE_EMPTY,
                            if (mSelectedNum < mTotalNum) mSelectedNum else mTotalNum)
                    finish()
                }
            } else {
                toast(R.string.garbage_delete_success)
            }

        }
        updateFooterView()
        rv_filelist.setLoadMoreEnabled(hasMore())
    }

    fun hasMore() = mHasMore && !limitCount()

    fun limitCount(): Boolean {
        val svip = AccountUtils.getInstance().isSVip
        val count = mAdapter.itemCount
        return ((svip && count >= SVIP_LIMIT) || (!svip && count >= PAGE_LIMIT))
    }

    override fun onDeleteFail() {
        toast(R.string.garbage_delete_fail)
    }

    private fun displayEmptyView(isError: Boolean) {
        head_text.visibility = View.GONE
        rl_bottom.visibility = View.GONE
        rv_filelist.visibility = View.GONE
        empty_view.visibility = View.VISIBLE
        if (isError) {
            empty_view.setLoadError(R.string.garbage_clean_load_error)
            empty_view.setRefreshVisibility(View.VISIBLE)
        } else {
            empty_view.setLoadNoData(R.string.garbage_empty_folder_empty_hint)
        }
    }

    private fun hidEmptyView() {
        head_text.visibility = View.VISIBLE
        rl_bottom.visibility = View.VISIBLE
        rv_filelist.visibility = View.VISIBLE
        empty_view.visibility = View.GONE
    }

    private fun updateBottomBar(number: Int,selectAll:Boolean) {
        // 如果没有选择任何item， 则清理按钮置灰
        btn_delete.isEnabled = number != 0

        // 更新选择的文件数量
        bottom_file_info.text = getString(
                R.string.garbage_empty_bottom_info, number)
        tv_select_all.text = getString(if (selectAll)  R.string
                .garbage_cancel_select_all else R.string.garbage_select_all)
    }


    private fun getFilesList() {
        if (mIsLoadingMore) return
        mIsLoadingMore = true
        mGarbageFilesPresenter.getGarbageFilesList(Extras.TYPE_EMPTY, mTaskId, mStart,
                if (AccountUtils.getInstance().isSVip) SVIP_PAGE_LIMIT else PAGE_LIMIT)
    }

    private fun deleteSelectedFiles() {
        mGarbageFilesPresenter.deleteGarbageFiles(Extras.TYPE_EMPTY, mTaskId, mAdapter
                .getSelectItems())
    }


    private fun showDeleteDialog() {
        val content = getString(R.string.garbage_empty_delete_content, mAdapter.getSelectCount())
        val builder = DialogBuilder()
        val dialog = builder.buildTipsDialog(activity, content,
                getString(R.string.garbage_delete_small_hint),
                getString(R.string.garbage_clean_confirm),
                getString(R.string.garbage_clean_cancel))

        val titleView = dialog.findViewById<TextView>(R.id.txt_confirmdialog_title)
        val contentView = dialog.findViewById<TextView>(R.id.text_content)
        titleView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 16F)
        titleView.setPadding(100, 0, 100, 0)
        titleView.setLineSpacing(5F, 1f)
        contentView.setTextSize(TypedValue.COMPLEX_UNIT_DIP, 14F)
        contentView.setTextColor(resources.getColor(R.color.baby_born_days_color))

        builder.setOnDialogCtrListener(object : DialogCtrListener {
            override fun onOkBtnClick() {
                if (!ConnectivityState.isConnected(context)) {
                    toast(R.string.network_exception_message)
                } else {
                    deleteSelectedFiles()
                    dialog.dismiss()
                }
            }

            override fun onCancelBtnClick() {
                dialog.dismiss()
            }
        })
        dialog.show()
    }

    private fun showGuideSVipDialog() {
        activity?.apply {
            if (!::mGuideDialog.isInitialized) {
                mGuideDialog = VipGuideDialog(this)
            }

            mGuideDialog.init {
                val cfg = ConfigGarbageFile(
                        ServerConfigKey.getCfgConfigBodyByType(
                                ServerConfigKey.ConfigType.GARBAGE_FILE_CONFIG))
                setTopImage("", R.drawable.business_garbage_vip_guide_icon)
                val showSizeThreshold = cfg.showDeleteEmptySizeDescThreshold
                if (AccountUtils.getInstance().isVip) {
                    setDesc(getSvipGuideDialogDesc(
                            cfg.vipDeleteEmptyFoldersGuideDesc,
                            cfg.vipDeleteEmptyFoldersGuideDescWithSize,
                            showSizeThreshold))
                    setSVipButton(cfg.vipDeleteEmptyFoldersButton) {
                        vipGuideDialogIntent(this@apply, cfg.vipDeleteEmptyFoldersGuideUrl,
                                StatisticsKey.GC_EMPTY_FOLDER_DELETE_UPGRADE_SVIP)
                    }
                } else {
                    setDesc(getSvipGuideDialogDesc(
                            cfg.deleteEmptyFoldersGuideDesc,
                            cfg.deleteEmptyFoldersGuideDescWithSize,
                            showSizeThreshold))
                    setSVipButton(cfg.deleteEmptyFoldersButton) {
                        vipGuideDialogIntent(this@apply, cfg.deleteEmptyFoldersGuideUrl,
                                StatisticsKey.GC_EMPTY_FOLDER_DELETE_OPEN_SVIP)
                    }
                }
            }
        }
    }

    private fun getSvipGuideDialogDesc(desc: String, descX: String, threshold: Int): String {
        return if (mTotalNum < threshold) {
            desc
        } else {
            if (descX.contains("X")) {
                descX.replace("X", mTotalNum.toString(), true)
            } else {
                descX
            }
        }
    }

    private fun vipGuideDialogIntent(activity: Activity, url: String?, countStr: String?) {
        if (url.isNullOrEmpty()) {
            val intent = VipActivity.getStartIntent(activity, VipActivity
                    .GARBAGE_DELETE_EMPTY_TO_SVIP,
                    VipActivity.TO_SVIP)
            startActivityForResult(intent, REQUEST_CODE)
        } else {
            startActivity(RichMediaActivity.getStartIntent(activity, url))
        }
        countStr?.updateCount()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (resultCode) {
            Activity.RESULT_OK -> {
                if (AccountUtils.getInstance().isSVip) {
                    if (rv_filelist.footerContainer.childCount > 0) {
                        rv_filelist.footerContainer.removeAllViews()
                    }
                    rv_filelist.setLoadMoreEnabled(hasMore())
                    if (needAutoLoad(mAdapter.itemCount)) {
                        getFilesList()
                    }
                }
                if (::mGuideDialog.isInitialized && mGuideDialog.isShowing) mGuideDialog.dismiss()
            }
        }
    }

    private fun updateHeadView() {
        mSelectedNum += mAdapter.getSelectCount()
        val num = (mTotalNum - mSelectedNum).takeIf {
            it > 0
        } ?: 0
        head_text.text = resources.getString(R.string.garbage_empty_list_head_desc,
                num)
    }

    override fun onDestroy() {
        mGarbageFilesPresenter.onDestroy()
        super.onDestroy()
    }
}