package com.baidu.netdisk.ui.gc

import android.content.Context
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.BaseServiceHelper
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.ServiceTypes
import com.baidu.netdisk.gc.IGarbageClean
import com.baidu.netdisk.gc.io.parser.model.GarbageFileItem
import com.baidu.netdisk.gc.service.Actions
import com.baidu.netdisk.gc.service.Extras
import com.baidu.netdisk.kotlin.extension.invoke

class GarbageCleanManager : IGarbageClean {
    private val builder = BaseServiceHelper::buildIntent

    override fun deleteGarbageFiles(context: Context, bduss: String, uid: String,
                                    type: String, taskId:String, deleteItems: List<GarbageFileItem>, receiver:
                                    ResultReceiver?) {
        if (!BaseServiceHelper.isNetWorkAvailable(context, receiver)) {
            return
        }
        DeleteGarbageFilesTask(context, bduss, uid, type, taskId, deleteItems, receiver)
    }

    override fun getGarbageFilesList(context: Context, bduss: String, uid: String, type: String,
                                     taskId: String, start: Int, limit: Int, receiver:
                                     ResultReceiver?) {
        if (!BaseServiceHelper.isNetWorkAvailable(context, receiver)) {
            return
        }
        val intent = builder(context, bduss, uid, receiver).invoke {
            BaseExtras.EXTRA_SERVICE_TYPE - ServiceTypes.GARBAGE_CLEAN_SERVICE
            Extras.GARBAGE_TYPE - type
            Extras.TASK_ID - taskId
            Extras.START - start
            Extras.LIMIT - limit
        }.setAction(Actions.USER_GC_FILES_LIST)
        BaseServiceHelper.startTargetVersionService(context, intent)
    }

    override fun scanGarbageFile(context: Context, receiver: ResultReceiver?, bduss: String, uid: String, type:
    String, path: String?, isCancel: Boolean, taskId: String?) {
        if (!BaseServiceHelper.isNetWorkAvailable(context, receiver)) {
            return
        }

        val intent = builder(context, bduss, uid, receiver).invoke {
            BaseExtras.EXTRA_SERVICE_TYPE - ServiceTypes.GARBAGE_CLEAN_SERVICE
            Extras.GARBAGE_TYPE - type
            Extras.SCAN_PATH - path
            Extras.IS_CANCEL_SCAN - isCancel
            Extras.TASK_ID - taskId
        }.setAction(Actions.SCAN_GARBAGE_FILE)
        BaseServiceHelper.startTargetVersionService(context, intent)
    }

    override fun queryProgress(context: Context, receiver: ResultReceiver?, bduss: String, uid: String, type: String,
                               option: String, taskId: String) {
        if (!BaseServiceHelper.isNetWorkAvailable(context, receiver)) {
            return
        }

        val intent = builder(context, bduss, uid, receiver).invoke {
            BaseExtras.EXTRA_SERVICE_TYPE - ServiceTypes.GARBAGE_CLEAN_SERVICE
            Extras.GARBAGE_TYPE - type
            Extras.QUERY_OPTION - option
            Extras.TASK_ID - taskId
        }.setAction(Actions.QUERY_SCAN_STATUS)
        BaseServiceHelper.startTargetVersionService(context, intent)
    }

    override fun asyncQueryProgress(context: Context, receiver: ResultReceiver?, bduss: String, uid: String, type: String, option: String, taskId: String) {
        if (!BaseServiceHelper.isNetWorkAvailable(context, receiver)) {
            return
        }

        val intent = builder(context, bduss, uid, receiver).invoke {
            BaseExtras.EXTRA_SERVICE_TYPE - ServiceTypes.GARBAGE_CLEAN_SERVICE
            Extras.GARBAGE_TYPE - type
            Extras.QUERY_OPTION - option
            Extras.TASK_ID - taskId
        }.setAction(Actions.SYNC_QUERY_SCAN_STATUS)
        BaseServiceHelper.startTargetVersionService(context, intent)
    }
}