package com.baidu.netdisk.ui.gc

import android.animation.Animator
import android.app.Activity
import android.arch.lifecycle.MutableLiveData
import android.content.Intent
import android.os.Bundle
import android.support.v4.content.ContextCompat
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.businessplatform.extension.ObserverOnlyOnce
import com.baidu.netdisk.businessplatform.extension.formatStorageSize
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.gc.io.parser.model.QueryInfo
import com.baidu.netdisk.gc.service.Extras
import com.baidu.netdisk.gc.statistics.StatisticsKey
import com.baidu.netdisk.gc.statistics.StatisticsKey.GC_CLEAN_FINISH_PAGE_SHOW
import com.baidu.netdisk.kotlin.extension.invoke
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.invisible
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.ui.businessplatform.extension.immerseGarbageStatusBar
import com.baidu.netdisk.ui.gc.presenter.QueryGarbageProgressPresenter
import com.baidu.netdisk.ui.lottie.NetdiskLottieView
import kotlinx.android.synthetic.main.activity_garbage_clean_result.*
import kotlinx.android.synthetic.main.view_garbage_common_title.*

/**
 * @author chencaixing
 * @time 2019-09-23 11:39
 */
class GarbageCleanResultActivity : BaseActivity() {

    private var mTotalFree = MutableLiveData<Long?>()
    private var mType:String? = null
    companion object {
        private const val RELEASE_MEMORY_SIZE = "release_memory_size"
        private const val CLEAN_FILE_SIZE = "clean_files_size"

        @JvmStatic
        fun startActivity(activity: Activity, type: String, fileSizes: Int, memorySize:
        Long = 0) {
            activity.startActivity(Intent(activity, GarbageCleanResultActivity::class.java).invoke {
                RELEASE_MEMORY_SIZE - memorySize
                CLEAN_FILE_SIZE - fileSizes
                Extras.GARBAGE_TYPE - type
            })
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val presenter = QueryGarbageProgressPresenter(object : IQueryGarbageProgressView {
            override fun updateProgress(progress: Int, queryInfo: QueryInfo?) {
                mTotalFree.value = queryInfo?.totalSize
            }

            override fun queryProgressFailed() {
            }

            override fun getActivity(): Activity {
                return this@GarbageCleanResultActivity
            }

        }
        )
        presenter.queryScanProgress(Extras.TYPE_DUP, "", Extras.OPTION_HISTORY)
        mTotalFree.ObserverOnlyOnce(this) {
            if (it != null && it > 0) {
                tv_total_free.show()
                tv_total_free.text = resources.getString(R.string.garbage_total_free_memory, it.formatStorageSize())
            }

        }

        GC_CLEAN_FINISH_PAGE_SHOW.updateCount(mType ?: Extras.TYPE_DUP)

    }

    override fun getLayoutId() = R.layout.activity_garbage_clean_result

    override fun initView() {
        val fileSizes = intent.getIntExtra(CLEAN_FILE_SIZE, 0)
        iv_back.setOnClickListener {
            finish()
        }
        mType = intent.getStringExtra(Extras.GARBAGE_TYPE)
        when (mType) {
            Extras.TYPE_DUP -> {
                title_text.text = getString(R.string.garbage_dupfile_clean_title)
                val memorySize = intent.getLongExtra(RELEASE_MEMORY_SIZE, 0L)
                tv_free_memory.text = resources.getString(R.string.garbage_clean_memory_size,
                        memorySize.formatStorageSize())
                tv_delete_files.text = resources.getString(R.string.garbage_delete_file_size, fileSizes)
            }
            Extras.TYPE_EMPTY ->{
                title_text.text = getString(R.string.garbage_empty_folder_clean_title)
                tv_free_memory.gone()
                tv_delete_files.text = resources.getString(R.string.garbage_clean_empty_folder_result, fileSizes)
            }
        }
        fl_cai.setOnClickListener {
            playAnimation(lottie_cai, lottie_kua)
            tv_end.apply {
                show()
                setCompoundDrawablesWithIntrinsicBounds(ContextCompat.getDrawable(context, R
                        .drawable
                        .ic_garbage_cai),
                        null, null, null)
                text = getString(R.string.garbage_result_cai_end)

            }
            StatisticsKey.GC_RESULT_PAGE_CAI_CLICK.updateCount(mType?:"")
        }
        fl_kua.setOnClickListener {
            playAnimation(lottie_kua, lottie_cai)
            tv_end.apply {
                show()
                setCompoundDrawablesWithIntrinsicBounds(ContextCompat.getDrawable(context, R
                        .drawable
                        .ic_garbage_kua),
                        null, null, null)
                text = getString(R.string.garbage_result_kua_end)
            }
            StatisticsKey.GC_RESULT_PAGE_KUA_CLICK.updateCount(mType?:"")
        }
    }

    fun playAnimation(playView: NetdiskLottieView, stopView: NetdiskLottieView) {
        stopView.cancelAnimation()
        stopView.gone()
        ll_bottom.invisible(true)
        with(playView) {
            if (!isAnimating) {
                show()
                playAnimation()
                addAnimatorListener(object : Animator.AnimatorListener {
                    override fun onAnimationRepeat(animation: Animator?) {
                    }

                    override fun onAnimationEnd(animation: Animator?) {
                        gone()
                    }

                    override fun onAnimationCancel(animation: Animator?) {
                    }

                    override fun onAnimationStart(animation: Animator?) {

                    }

                })
            }
        }


    }

    override fun onResume() {
        super.onResume()
        window.immerseGarbageStatusBar()
    }

    override fun onThemeUpdate() {
        super.onThemeUpdate()
        window.immerseGarbageStatusBar()
    }
}

