package com.baidu.netdisk.ui.gc

import android.arch.lifecycle.Observer
import android.content.Context
import android.content.Intent
import android.os.Bundle
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.businessplatform.extension.formatStorageSize
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.gc.GarbageFileHelper
import com.baidu.netdisk.gc.GarbageFileScanBean
import com.baidu.netdisk.gc.service.Extras
import com.baidu.netdisk.gc.statistics.StatisticsKey
import com.baidu.netdisk.kotlin.extension.invoke
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.platform.extension.toast
import com.baidu.netdisk.ui.businessplatform.extension.immerseGarbageStatusBar
import com.baidu.netdisk.util.NotificationUtil
import kotlinx.android.synthetic.main.activity_garbage_main.*

/**
 * @author chencaixing
 * @time 2019-10-21 14:39
 */
class GarbageMainActivity : BaseActivity() {

    override fun getLayoutId() = R.layout.activity_garbage_main
    private var mFrom  = FROM_OTHER
    companion object{
        // 扫描过程展现来源
        const val FROM_PARAS = "gc_from"

        // 引导飘条
        const val FROM_GUIDE_LAYOUT = 1
        // 九宫格
        const val FROM_NINE_SQUARE = 2
        // 容量构成页
        const val FROM_CAPACITY = 3
        // 通知栏
        const val FROM_NOTIFICATION = 4
        // 其它
        const val FROM_OTHER = 99

        @JvmStatic
        fun getStartIntent(context: Context, from: Int): Intent {
            return Intent(context, GarbageMainActivity::class.java)(){
                FROM_PARAS - from
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        mFrom = intent?.getIntExtra(FROM_PARAS,FROM_OTHER)?:FROM_OTHER
        StatisticsKey.GC_GARBAGE_MAIN_PAGE_SHOW.updateCount(mFrom.toString())
    }

    override fun initView() {
        iv_back.setOnClickListener {
            finish()
        }
        rl_clean_dup_files.setOnClickListener {
            onItemClick(GarbageFileHelper.getTask(Extras.TYPE_DUP), Extras.TYPE_DUP)
        }

        rl_clean_empty_folder.setOnClickListener {
            onItemClick(GarbageFileHelper.getTask(Extras.TYPE_EMPTY), Extras.TYPE_EMPTY)
        }

        GarbageFileHelper.observeTask(Extras.TYPE_DUP,this, Observer {
            if (it == null) {
                dup_file_size.gone()
                dup_arrow.gone()
                return@Observer
            }
            dup_arrow.show()
            dup_file_size.show()

            if (it.progress == 100) {
                dup_file_size.text = it.totalSize.formatStorageSize()
            } else {
                dup_file_size.text = getString(R.string.garbage_file_main_scanning, it.progress)
            }
        })

        GarbageFileHelper.observeTask(Extras.TYPE_EMPTY, this, Observer {
            if (it == null) {
                empty_folder_count.gone()
                empty_arrow.gone()
                return@Observer
            }
            empty_arrow.show()
            empty_folder_count.show()

            if (it.progress == 100) {
                empty_folder_count.text = "${it.totalNum}个"
            } else {
                empty_folder_count.text = getString(R.string.garbage_file_main_scanning, it.progress)
            }
        })
    }

    private fun canStartNewScanTask(curScanType: String): Boolean {
        return when(curScanType) {
            Extras.TYPE_DUP -> {
                GarbageFileHelper.getTask(Extras.TYPE_EMPTY)?.progress?.let {
                    if (it < 100 && GarbageFileHelper.getTask(Extras.TYPE_DUP)?.progress != 100) return false
                }
                return true
            }

            Extras.TYPE_EMPTY -> {
                GarbageFileHelper.getTask(Extras.TYPE_DUP)?.progress?.let {
                    if (it < 100 && GarbageFileHelper.getTask(Extras.TYPE_EMPTY)?.progress != 100) return false
                }
                return true
            }
            else -> return true
        }
    }

    private fun onItemClick(task: GarbageFileScanBean?, scanType: String) {
        if (canStartNewScanTask(scanType)) {
            cleanNotification(scanType)
            if (task?.progress == 100) {
                ScanGarbageResultActivity.startActivity(this, task.taskId, task.totalNum, task.totalSize, scanType)
            } else {
                startActivity(ScanGarbageFileActivity.getStartIntent(this, mFrom, scanType))
            }
        } else {
            toast(R.string.garbage_other_scan_task_not_finish)
        }
    }

    private fun cleanNotification(scanType: String) {
        when(scanType) {
            Extras.TYPE_DUP -> {
                NotificationUtil.clearGarbageDupFileNotification(this)
            }

            Extras.TYPE_EMPTY -> {
                NotificationUtil.clearGarbageEmptyFolderNotification(this)
            }
        }
    }

    override fun onResume() {
        super.onResume()
        window.immerseGarbageStatusBar()
    }

    override fun onThemeUpdate() {
        super.onThemeUpdate()
        window.immerseGarbageStatusBar()
    }
}