package com.baidu.netdisk.ui.gc

import android.animation.ValueAnimator
import android.annotation.SuppressLint
import android.app.Activity
import android.app.Dialog
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.gc.GarbageFileHelper
import com.baidu.netdisk.gc.io.ErrorCode
import com.baidu.netdisk.gc.io.parser.model.QueryInfo
import com.baidu.netdisk.gc.service.Extras.GARBAGE_TYPE
import com.baidu.netdisk.gc.service.Extras.OPTION_SCAN
import com.baidu.netdisk.gc.service.Extras.TYPE_DUP
import com.baidu.netdisk.gc.statistics.StatisticsKey
import com.baidu.netdisk.gc.statistics.StatisticsKey.GC_BACKGROUND_SCANNING_CLICK
import com.baidu.netdisk.gc.statistics.StatisticsKey.GC_STOP_DIALOG_CANCEL_CLICK
import com.baidu.netdisk.gc.statistics.StatisticsKey.GC_STOP_DIALOG_CONTINUE_CLICK
import com.baidu.netdisk.gc.statistics.StatisticsKey.GC_STOP_DIALOG_SHOW
import com.baidu.netdisk.gc.statistics.StatisticsKey.GC_STOP_SCANNING_CLICK
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.gc.presenter.QueryGarbageProgressPresenter
import com.baidu.netdisk.ui.gc.presenter.ScanGarbageFilePresenter
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.vip.VipActivity
import com.baidu.netdisk.ui.widget.BaseFragment
import kotlinx.android.synthetic.main.fragment_scan_garbage_file.*
import kotlinx.android.synthetic.main.view_garbage_common_title.*

@Tag("ScanGarbageFileFragment")
class ScanGarbageFileFragment : BaseFragment(), IScanGarbageFileView, IQueryGarbageProgressView {

    companion object {
        const val PROGRESS_ANIMATION_INTERVAL = 100
        const val REQUEST_CODE = 100
    }

    private var scanType: String = ""
    private var presenter: ScanGarbageFilePresenter? = null
    private var progressPresenter: QueryGarbageProgressPresenter? = null
    private var stopScanDialog: Dialog? = null
    private var isScanning = true
    private var handler: Handler? = null
    private var mProgressStart = 0
    private var mProgressAnimator: ValueAnimator? = null
    private lateinit var scanGarbageAdapter: ScanGarbageAdapter
    private var isBackGroundScanning = false
    private var isPause = false

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        super.onCreateView(inflater, container, savedInstanceState)
        return inflater.inflate(R.layout.fragment_scan_garbage_file, null, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initParams()
        scanGarbageAdapter = ScanGarbageAdapter(scanType)
        GarbageFileHelper.setIsBackGround(scanType, false)

        handler = Handler()
        initPresenter()
        val taskId = GarbageFileHelper.getTaskId(scanType)
        if (taskId.isBlank()) {
            presenter?.onScanGarbageFile(scanType, null, false, null)
        } else {
            mProgressStart = GarbageFileHelper.getTask(scanType)?.progress.d(prefix ="progress") ?: 0
            updateProgressView(mProgressStart.d(prefix = "mProgressStart"))
            startQueryProgress(taskId)
            isBackGroundScanning = true
        }

        initView()
    }

    private fun initView() {
        scan_desc.setText(scanGarbageAdapter.scanningDesc)
        title_text.setText(scanGarbageAdapter.titleText)
        iv_back.setOnClickListener {
            onBackKeyPressed()
        }

        showStopButton()

        background_scan.text = getString(if (isBackGroundScanning) R.string.garbage_file_background_scan_continue
        else R.string.garbage_file_background_scan)
        background_scan.setOnClickListener {
            GarbageFileHelper.setIsBackGround(scanType, true)
            progressPresenter?.asyncScanProgress(scanType, GarbageFileHelper.getTaskId(scanType), OPTION_SCAN)
            GC_BACKGROUND_SCANNING_CLICK.updateCount(scanType)
            cancelProgressAnimation()
            activity?.finish()
        }
    }

    private fun initParams() {
        scanType = arguments?.getString(GARBAGE_TYPE) ?: TYPE_DUP
    }

    private fun initPresenter() {
        presenter = ScanGarbageFilePresenter(this)
        progressPresenter = QueryGarbageProgressPresenter(this)
    }

    override fun scanGarbageSuccess(taskId: String) {
        GarbageFileHelper.setTaskId(scanType, taskId)
        "scanResult.taskId :: $taskId".d()
        startQueryProgress(taskId)
    }

    override fun scanGarbageFailed(errorCode: Int) {
        isScanning = false
        showScanErrorView(errorCode)
    }

    override fun scanGarbageLimit() {
        isScanning = false
        showScanCountLimit()
    }

    @SuppressLint("SetTextI18n")
    override fun updateProgress(progress: Int, queryInfo: QueryInfo?) {
        if (progress == 100) {
            updateProgressView(progress.d(prefix = "progress")) {
                isScanning = false
                stopScanDialog?.dismiss()

                if (isPause) return@updateProgressView

                handler?.postDelayed({
                    showScanFinish()
                }, 100)
                handler?.postDelayed({
                    enterScanResult(GarbageFileHelper.getTaskId(scanType), queryInfo?.totalFileNum ?: 0,
                            queryInfo?.totalSize ?: 0)
                }, 2000)
            }
        } else {
            progressPresenter?.queryScanProgress(scanType, GarbageFileHelper.getTaskId(scanType), OPTION_SCAN, true)
            updateProgressView(progress)
        }
    }

    override fun queryProgressFailed() {
        cancelProgressAnimation()
        isScanning = false
        GarbageFileHelper.clearTask(scanType)
        stopScanDialog?.dismiss()
        showScanErrorView()
    }

    private fun startScan() {
        presenter?.onScanGarbageFile(scanType, null, false, null)
        isScanning = true
        mProgressStart = 0
        showScanning()
    }

    private fun startQueryProgress(taskId: String) {
        progressPresenter?.queryScanProgress(scanType, taskId, OPTION_SCAN)
    }

    private fun showScanErrorView(errorCode: Int = 0) {
        scan_lyout.show()
        scan_success_empty.gone()
        animation_scan.cancelAnimation()
        animation_scan.gone()
        background_scan.visibility = View.INVISIBLE
        center_icon.setImageResource(R.drawable.business_icon_clean_fail)
        center_icon.show()
        when(errorCode) {
            ErrorCode.HAS_SCAN_TASK_NOT_FINISH -> {
                scan_desc.setText(R.string.garbage_other_scan_task_not_finish)
                stop_scan.gone()
            }
            else -> {
                scan_desc.setText(R.string.garbage_dupfile_scan_network_error)
                stop_scan.apply {
                    setText(R.string.garbage_dupfile_restart_scan)
                    show()
                    setOnClickListener {
                        startScan()
                        showStopButton()
                    }
                }
            }
        }
        scan_progress_layout.gone()
        scan_finish_tip.gone()
    }

    private fun showScanCountLimit() {
        scan_lyout.show()
        scan_success_empty.gone()
        animation_scan.cancelAnimation()
        animation_scan.gone()
        background_scan.visibility = View.INVISIBLE
        center_icon.setImageResource(R.drawable.business_icon_clean_fail)
        center_icon.show()
        scan_desc.setText(if (AccountUtils.getInstance().isSVip) R.string.garbage_file_scan_count_limit_svip
        else R.string.garbage_file_scan_count_limit)
        scan_progress_layout.gone()
        stop_scan.gone()
        scan_finish_tip.gone()
        vip_guide_layout.show(!AccountUtils.getInstance().isSVip)

        if (AccountUtils.getInstance().isVip) {
            open_svip_desc_up.text = scanGarbageAdapter.vipScanGarbageTimesGuideDesc
            open_svip_desc_mid.text = scanGarbageAdapter.vipScanGarbageTimesGuideDescMid
            open_svip.text = scanGarbageAdapter.vipScanGarbageTimesButton
            open_svip.setOnClickListener {
                vipGuideDialogIntent(scanGarbageAdapter.vipScanGarbageTimesGuideUrl,
                        StatisticsKey.GC_SCAN_TIMES_OUT_UPGRADE_SVIP)

            }
        } else {
            open_svip_desc_up.text = scanGarbageAdapter.scanGarbageTimesGuideDesc
            open_svip_desc_mid.text = scanGarbageAdapter.scanGarbageTimesGuideDescMid
            open_svip.text = scanGarbageAdapter.scanGarbageTimesButton
            open_svip.setOnClickListener {
                vipGuideDialogIntent(scanGarbageAdapter.scanGarbageTimesGuideUrl,
                        StatisticsKey.GC_SCAN_TIMES_OUT_OPEN_SVIP)
            }
        }
    }

    private fun vipGuideDialogIntent(url: String?, countStr: String?) {
        if (url.isNullOrEmpty()) {
            val intent = VipActivity.getStartIntent(activity,
                    scanGarbageAdapter.scanLimitBuySVipFrom,
                    VipActivity.TO_SVIP)
            startActivityForResult(intent, REQUEST_CODE)
        } else {
            startActivity(RichMediaActivity.getStartIntent(activity, url))
        }
        countStr?.updateCount(scanType)
    }

    private fun showScanning() {
        scan_lyout.show()
        scan_progress_layout.show()
        scan_success_empty.gone()
        background_scan.show()
        animation_scan.show()
        if (!animation_scan.isAnimating) {
            animation_scan.playAnimation()
        }
        center_icon.gone()
        scan_finish_tip.gone()
        scan_desc.setText(scanGarbageAdapter.scanningDesc)
        vip_guide_layout.gone()
    }

    private fun updateProgressView(progress: Int, onScanSuccess: (() -> Unit)? = null) {
        if (progress == mProgressStart || isPause) {
            scan_progress.text = progress.toString()
            mProgressStart = progress
            return
        }
        if (mProgressAnimator != null) {
            cancelProgressAnimation()
        }
        mProgressAnimator = ValueAnimator.ofInt(mProgressStart, progress).apply {
            duration = (PROGRESS_ANIMATION_INTERVAL * (progress - mProgressStart)).toLong()
            addUpdateListener {
                mProgressStart = it.animatedValue.toString().toInt()
                scan_progress?.text = mProgressStart.toString()

                if (mProgressStart == 100) {
                    onScanSuccess?.invoke()
                }
            }
            start()
        }
    }

    private fun showStopButton() {
        stop_scan.setText(R.string.garbage_dupfile_stop_scan)
        stop_scan.show()
        stop_scan.setOnClickListener {
            // 停止扫描
            stopScan()
            GC_STOP_SCANNING_CLICK.updateCount(scanType)
        }
    }

    private fun showScanFinish() {
        scan_lyout.show()
        scan_success_empty.gone()
        animation_scan.cancelAnimation()
        animation_scan.gone()
        animation_finish.show()
        animation_finish.playAnimation()
        center_icon.gone()
        scan_desc.setText(R.string.garbage_file_scan_finish_tip)
        scan_progress_layout.gone()
        stop_scan.gone()
        background_scan.visibility = View.INVISIBLE
        scan_finish_tip.show()
        vip_guide_layout.gone()
    }

    private fun stopScan() {
        DialogBuilder().apply {
            stopScanDialog = buildTipsDialog(activity, null, resources.getString(R.string.garbage_file_scan_stop_scan),
                    resources.getString(R.string.garbage_file_scan_continue),
                    resources.getString(R.string.garbage_file_scan_cancel))
            setOnDialogCtrListener(object : DialogCtrListener {
                override fun onOkBtnClick() {
                    GC_STOP_DIALOG_CONTINUE_CLICK.updateCount(scanType)
                }

                override fun onCancelBtnClick() {
                    GarbageFileHelper.getTaskId(scanType).let {
                        if (!it.isBlank()) {
                            presenter?.onScanGarbageFile(scanType, null, true, it)
                        }
                    }

                    GarbageFileHelper.clearTask(scanType)
                    GC_STOP_DIALOG_CANCEL_CLICK.updateCount(scanType)
                    cancelProgressAnimation()
                    activity?.finish()
                }

            })
        }
        stopScanDialog?.show()
        GC_STOP_DIALOG_SHOW.updateCount(scanType)
    }

    override fun onBackKeyPressed(): Boolean {
        if (isScanning) {
            if (isBackGroundScanning) { // 已经进入过后台扫描时，back键和返回按钮不再显示挽留弹窗,而是继续后台扫描
                GarbageFileHelper.setIsBackGround(scanType, true)
                progressPresenter?.asyncScanProgress(scanType, GarbageFileHelper.getTaskId(scanType), OPTION_SCAN)
                cancelProgressAnimation()
                activity?.finish()
            } else {
                stopScan()
            }
        } else {
            cancelProgressAnimation()
            activity?.finish()
        }
        return true
    }

    override fun onDestroy() {
        handler?.removeCallbacksAndMessages(null)
        handler = null
        progressPresenter?.onDestroy()
        super.onDestroy()
    }

    private fun cancelProgressAnimation() {
        mProgressAnimator?.apply {
            removeAllUpdateListeners()
            cancel()
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (resultCode) {
            Activity.RESULT_OK -> {
                if (AccountUtils.getInstance().isSVip) {
                    startScan()
                }
            }
        }
    }

    override fun onResume() {
        super.onResume()
        isPause = false
        val task = GarbageFileHelper.getTask(scanType)
        task?.apply {
            scan_progress.text = progress.toString()
            if (progress == 100) {
                enterScanResult(taskId, totalNum, totalSize)
            } else {
                if (!animation_scan.isAnimating) {
                    animation_scan.playAnimation()
                }
            }
        }

    }

    override fun onPause() {
        super.onPause()
        isPause = true
        cancelProgressAnimation()
        animation_scan.cancelAnimation()
    }

    private fun enterScanResult(taskId: String?, fileCount: Int, fileSpace: Long) {
        animation_finish.cancelAnimation()
        ScanGarbageResultActivity.startActivity(activity, taskId, fileCount, fileSpace, scanType)
        activity?.finish()
    }
}
