package com.baidu.netdisk.ui.gc

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.gc.GarbageFileHelper
import com.baidu.netdisk.gc.service.Extras
import com.baidu.netdisk.gc.service.Extras.TYPE_DUP
import com.baidu.netdisk.gc.service.Extras.TYPE_EMPTY
import com.baidu.netdisk.gc.statistics.StatisticsKey
import com.baidu.netdisk.gc.statistics.StatisticsKey.GC_SCANNING_FINISH_PAGE_SHOW
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.invoke
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.ui.businessplatform.extension.immerseGarbageStatusBar
import com.baidu.netdisk.util.NotificationUtil
import kotlinx.android.synthetic.main.activity_scan_garbage_result.*
import kotlinx.android.synthetic.main.view_garbage_common_title.*

@Tag("ScanGarbageResultActivity")
class ScanGarbageResultActivity: BaseActivity() {
    companion object{
        const val TASK_ID = "task_id"
        const val GARBAGE_FILE_COUNT = "garbage_file_count"
        const val GARBAGE_FILE_SPACE = "garbage_file_space"
        fun startActivity(activity: Activity?, taskId: String?, fileCount: Int, fileSpace: Long, type: String =
                Extras.TYPE_EMPTY) {
            if (taskId.isNullOrEmpty()) {
                return
            }
            activity?.startActivity(Intent(activity, ScanGarbageResultActivity::class.java).invoke {
                TASK_ID - taskId
                GARBAGE_FILE_COUNT - fileCount
                GARBAGE_FILE_SPACE - fileSpace
                Extras.GARBAGE_TYPE - type
            })
        }
    }

    private var mTotalSize = 0L
    private var mTotalNum = 0
    private var mTaskId = ""
    private var scanType: String = ""
    private lateinit var scanGarbageAdapter: ScanGarbageAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        cleanNotification()
        cleanScanInfo()
        GC_SCANNING_FINISH_PAGE_SHOW.updateCount(scanType)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_scan_garbage_result
    }

    override fun initView() {
        iv_back.setOnClickListener {
            finish()
        }
        title_text.setText(scanGarbageAdapter.titleText)
        if (mTotalNum > 0)
            showSuccessView()
        else
            showSuccessViewEmpty()
    }

    override fun initParams() {
        scanType = intent.getStringExtra(Extras.GARBAGE_TYPE)
        mTaskId = intent.getStringExtra(GarbageFileListActivity.TASK_ID)
        mTotalNum = intent.getIntExtra(GarbageFileListActivity.GARBAGE_FILE_COUNT, 0)
        mTotalSize = intent.getLongExtra(GarbageFileListActivity.GARBAGE_FILE_SPACE, 0L)
        scanGarbageAdapter = ScanGarbageAdapter(scanType)
    }

    private fun showSuccessView() {
        scan_success_empty.gone()
        scan_success.show()

        scan_path.text = resources.getString(R.string.garbage_dupfile_scan_path, resources.getString(R.string.garbage_file_scan_range))
        scanGarbageAdapter.showSuccessView(this, scan_success, mTotalNum, mTotalSize)
        right_clean_bottom_desc.setText(scanGarbageAdapter.cleanButtonBottomDesc)
        right_clean.setOnClickListener {
            StatisticsKey.GC_RESULT_CLEAN_NOW.updateCount(scanType)
            GarbageFileListActivity.startActivity(this, mTaskId,
                    mTotalNum, mTotalSize, scanType)
            finish()
        }
    }

    private fun showSuccessViewEmpty() {
        scan_success.gone()
        scan_success_empty.show()
        scan_empty_result.setText(scanGarbageAdapter.noGarbageFileDesc)
    }

    override fun onResume() {
        super.onResume()
        window.immerseGarbageStatusBar()
    }

    override fun onThemeUpdate() {
        super.onThemeUpdate()
        window.immerseGarbageStatusBar()
    }

    private fun cleanScanInfo() {
        GarbageFileHelper.clearTask(scanType)
    }

    private fun cleanNotification() {
        when(scanType) {
            TYPE_DUP -> {
                NotificationUtil.clearGarbageDupFileNotification(this)
            }

            TYPE_EMPTY -> {
                NotificationUtil.clearGarbageEmptyFolderNotification(this)
            }
        }
    }

}