package com.baidu.netdisk.ui.gc

import android.app.Activity
import android.app.Dialog
import android.graphics.Typeface
import android.os.Bundle
import android.text.SpannableString
import android.text.Spanned
import android.text.style.AbsoluteSizeSpan
import android.text.style.StyleSpan
import android.view.Gravity
import android.view.ViewGroup
import com.baidu.netdisk.R
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.platform.extension.show
import kotlinx.android.synthetic.main.dialog_common_vip_guide.*
import kotlinx.android.synthetic.main.dialog_common_vip_guide.desc
import kotlinx.android.synthetic.main.dialog_common_vip_guide.icon
import kotlinx.android.synthetic.main.dialog_common_vip_guide.open_vip
import kotlinx.android.synthetic.main.dialog_common_vip_guide.single_privilege_button
import kotlinx.android.synthetic.main.dialog_common_vip_guide.svip_button
import kotlinx.android.synthetic.main.dialog_common_vip_guide.svip_coupon
import kotlinx.android.synthetic.main.dialog_common_vip_guide.vip_button

class VipGuideDialog(private val activity: Activity, private val cancelAble: Boolean = true)
    : Dialog(activity, R.style.BottomDialog) {

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window?.apply {
            decorView.setPadding(0, 0, 0, 0)
            attributes.apply {
                width = ViewGroup.LayoutParams.MATCH_PARENT
                gravity = Gravity.BOTTOM or Gravity.CENTER_HORIZONTAL
            }
        }
        setCanceledOnTouchOutside(cancelAble)
        // setCancelable(cancelAble)
        setContentView(R.layout.dialog_common_vip_guide)
        image_exit.setOnClickListener {
            dismiss()
        }

    }

    fun setTopImage(url: String, resId: Int) {
        GlideHelper.getInstance().displayImageFromNetwork(url, resId, true, icon, null)
    }

    fun setTopDesc(guideDesc: String) {
        top_desc.show()
        top_desc.text = guideDesc
    }

    fun setDesc(guideDesc: String) {
        desc.show()
        desc.text = guideDesc
    }

    fun setSVipButton(text: String, price: String? = null, coupon: String? = null, svipClick: (()-> Unit)) {
        svip_button.show()
        if (!price.isNullOrBlank()) {
            svip_button.text = processSpan(text, price)
        } else {
            svip_button.text = text
        }

        svip_button.setOnClickListener {
            svipClick.invoke()
        }

        svip_coupon.text = coupon
        svip_coupon.show(!coupon.isNullOrBlank())
    }

    fun setVipButton(text: String, price: String? = null, coupon: String? = null, vipClick: (()-> Unit)) {
        vip_button.show()
        if (!price.isNullOrBlank()) {
            vip_button.text = processSpan(text, price)
        } else {
            vip_button.text = text
        }

        vip_button.setOnClickListener {
            vipClick.invoke()
        }

        vip_coupon.text = coupon
        vip_coupon.show(!coupon.isNullOrBlank())
    }

    fun setBlueButton(text: String, price: String? = null, coupon: String? = null, click: (()-> Unit)) {
        single_privilege_button.show()
        if (!price.isNullOrBlank()) {
            single_privilege_button.text = processSpan(text, price)
        } else {
            single_privilege_button.text = text
        }

        single_privilege_button.setOnClickListener {
            click.invoke()
        }

        single_privilege_coupon.text = coupon
        single_privilege_coupon.show(!coupon.isNullOrBlank())
    }

    fun setGodText(text: String, click: () -> Unit) {
        open_vip.show()
        open_vip.text = text
        open_vip.setOnClickListener {
            click.invoke()
        }
    }

    fun setBlackText(text: String, click: () -> Unit) {
        other.show()
        other.text = text
        other.setOnClickListener {
            click.invoke()
        }
    }

    private fun processSpan(str1: String, str2: String): SpannableString {
        val tempStr = "$str1 $str2"
        return SpannableString(tempStr).apply {

            setSpan(StyleSpan(Typeface.NORMAL), str1.length, tempStr.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
            setSpan(AbsoluteSizeSpan(
                    activity.resources.getDimension(R.dimen.business_platform_vip_guide_span_text_size).toInt()),
                    str1.length, tempStr.length, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE)
        }
    }

    fun init(block: VipGuideDialog.() -> Unit) {
        show()
        block.invoke(this)
    }
}