package com.baidu.netdisk.ui.gc.adapter

import android.support.v4.content.ContextCompat
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.base.utils.FileType
import com.baidu.netdisk.businessplatform.widget.BaseViewHolder
import com.baidu.netdisk.gc.io.parser.model.GarbageGroupListItem
import com.baidu.netdisk.gc.io.parser.model.GarbageFileItem
import com.baidu.netdisk.kernel.util.FormatUtils
import com.baidu.netdisk.platform.extension.show

class DupFileAdapter(private var onSelectChange: (Int, Long) -> Unit) :
        RecyclerView.Adapter<BaseViewHolder>() {
    private val mGroupData = mutableListOf<GarbageGroupListItem>()
    private var mData = listOf<GarbageFileItem>()

    private var mTotalSize = 0L
    var mSmartChoose: Boolean = true
        set(value) {
            if (value != field) {
                field = value
                smartChoose(value)
            }
        }

    fun setData(newData:List<GarbageGroupListItem>){
        val data = newData.toMutableList()
        // 1.处理新数据的选中状态以及group的选择数
        data.onSmartChoose(mSmartChoose)
        // 2.处理可能存在的组被截断的情况，进行数据merge
        val newFirst = data.firstOrNull()
        mGroupData.lastOrNull().mergeGroup(newFirst) {
            data.remove(newFirst)
        }
        // 3.添加新数据
        mGroupData.addAll(data)
        // 4.清空可能存在的脏数据
        cleanUnUseData()
        mData = getData()
        onSelectChange(getSelectedNum(), mTotalSize)
        notifyDataSetChanged()
    }

   private inline fun GarbageGroupListItem?.mergeGroup(other: GarbageGroupListItem?, onFinish:()
   ->Unit){
       if (this != null && group_index == other?.group_index){
           if(groupBroken){
               initSelectCount(mSmartChoose)
               data.firstOrNull()?.initSelect(mSmartChoose)
           }
           if (other.groupBroken){
               other.initSelectCount(mSmartChoose)
               other.data.firstOrNull()?.initSelect(mSmartChoose)
           }
           selectCount += other.selectCount
           if (data.size > 0) {
               groupBroken = false
           }
           data.addAll(other.data)
           onFinish.invoke()
       }
   }

    override fun getItemCount()  = mData.size

    override fun onBindViewHolder(holder: BaseViewHolder, position: Int) {
        mData[position].apply {
                holder<TextView>(R.id.name_dup_file_list).text = server_filename
                holder<TextView>(R.id.size_dup_file_list).text = FormatUtils.formatFileSize(size)
                holder<TextView>(R.id.path_dup_file_list).text = path
                holder<ImageView>(R.id.cb_dup_files_list).isSelected = selected
                GlideHelper.getInstance().displayImageFromDrawable(FileType.getType(server_filename,
                        false).mResourceIdSmall, holder.getView(R.id.img_dup_file_list))
            val group = getGroup(group_index)
                holder<View>(R.id.rl_item).setBackgroundColor(ContextCompat.getColor
                (holder.itemView.context,
                        (if (group?.selectCount == group?.data?.size)
                            R.color.dup_files_selected_item_bg else
                            R.color.white)))
                holder<View>(R.id.dup_files_divider).show(position !=0 &&
                        this == group?.data?.firstOrNull())
                holder.itemView.setOnClickListener {
                    val preSelected = selected
                    selected = !preSelected
                    if (selected) {
                        mTotalSize += size
                    } else {
                        mTotalSize -= size
                    }
                    group?.apply {
                        if (selected) selectCount ++ else selectCount --
                    }
                    notifyDataSetChanged()
                    onSelectChange(getSelectedNum(), mTotalSize)
                }
        }
    }

    private fun smartChoose(select:Boolean){
        mTotalSize = 0
        mGroupData.onSmartChoose(select)
        onSelectChange(getSelectedNum(), mTotalSize)
        notifyDataSetChanged()
    }

    private fun MutableList<GarbageGroupListItem>.onSmartChoose(select:Boolean){
        asSequence().filter {
            (it.data.size >1).apply {
                if (!this) it.groupBroken = true
            }
        }.flatMap {
            it.initSelectCount(select)
            it.data.asSequence()
        }.forEach {
            it.initSelect(select)
        }
    }


    private fun GarbageFileItem.initSelect(smartChoose:Boolean){
        selected = if (smartChoose) smart_choose != 1 else false
        if (selected) mTotalSize += size
    }

    private fun getGroup(groupIndex:Int): GarbageGroupListItem?{
      return   mGroupData.find {
            it.group_index == groupIndex
        }
    }
    private fun GarbageGroupListItem.initSelectCount(smartChoose:Boolean){
        selectCount = if (smartChoose) {
            if (smart_choose == -1)
                data.size else data.size - 1
        } else 0
    }

    fun getItem(position: Int):Pair<GarbageGroupListItem,GarbageFileItem>?{
        var count = 0
        var lastCount = 0
        mGroupData.forEach {
            count += it.data.size
            if (position < count) {
                return it to it.data[position - lastCount]
            }
            lastCount = count
        }
        return null
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseViewHolder {
        return BaseViewHolder(LayoutInflater.from(parent.context).inflate(R
                .layout.item_dup_files_list, parent, false))
    }


    fun onDeleteSuccess(){
        mTotalSize = 0
        mGroupData.forEach {
            group -> group.data.removeAll {
                it.selected.apply {
                    if (this) group.selectCount --
                }
            }

        }
        cleanUnUseData()
        mData = getData()
        onSelectChange(0, 0)
        notifyDataSetChanged()
    }


    private fun cleanUnUseData(){
        val last = mGroupData.lastOrNull()
        mGroupData.removeAll {
            it.data.size <= 1 &&
                    last?.group_index != it.group_index
        }
    }

    fun getSelectedNum() =  getSelectItems().size

    fun getSelectedSize() = mTotalSize

    fun getSelectItems() = mData.filter {
        it.selected
    }

    private fun getData() = mGroupData.filter {
        it.data.size > 1
    }.flatMap {
        it.data
    }
}
