package com.baidu.netdisk.ui.gc.broadcast

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.businessplatform.extension.formatStorageSize
import com.baidu.netdisk.gc.constant.GarbageFileBroadcastParams
import com.baidu.netdisk.gc.service.Extras
import com.baidu.netdisk.gc.service.Extras.TASK_STATUS_SUCCESS
import com.baidu.netdisk.gc.service.Extras.TYPE_DUP
import com.baidu.netdisk.gc.service.Extras.TYPE_EMPTY
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.invoke
import com.baidu.netdisk.ui.gc.GarbageMainActivity
import com.baidu.netdisk.ui.gc.ScanGarbageFileActivity
import com.baidu.netdisk.ui.gc.ScanGarbageResultActivity
import com.baidu.netdisk.util.NotificationUtil

@Tag("GarbageFileQueryReceiver")
class GarbageFileQueryReceiver : BroadcastReceiver() {
    override fun onReceive(context: Context?, intent: Intent?) {
        if (BaseActivity.getTopAvailableActivity() is ScanGarbageFileActivity ||
                BaseActivity.getTopAvailableActivity() is ScanGarbageResultActivity ) {
            return
        }

        val taskId = intent?.getStringExtra(GarbageFileBroadcastParams.EXTRA_GARBAGE_FILE_TASK_ID) ?: ""
        val taskStatus = intent?.getStringExtra(GarbageFileBroadcastParams.EXTRA_GARBAGE_FILE_TASK_STATUS) ?: ""
        val taskType = intent?.getStringExtra(GarbageFileBroadcastParams.EXTRA_GARBAGE_FILE_TASK_TYPE) ?: ""
        val taskTotalSize = intent?.getLongExtra(GarbageFileBroadcastParams.EXTRA_GARBAGE_FILE_TASK_FILE_SIZE, 0) ?: 0
        val taskFileCount = intent?.getIntExtra(GarbageFileBroadcastParams.EXTRA_GARBAGE_FILE_TASK_FILE_COUNT,
                0) ?: 0

        context?.let { noNullContext ->
            when (taskType) {
                TYPE_DUP -> {
                    NotificationUtil.showGarbageDupFileScanFinish(noNullContext,
                            getDupFileTitle(noNullContext, taskStatus, taskTotalSize, taskFileCount),
                            getIntent(noNullContext, taskId, taskStatus, taskType, taskTotalSize, taskFileCount))
                }
                TYPE_EMPTY -> {
                    NotificationUtil.showGarbageEmptyFolderScanFinish(noNullContext,
                            getEmptyFolderTitle(noNullContext, taskStatus, taskTotalSize, taskFileCount),
                            getIntent(noNullContext, taskId, taskStatus, taskType, taskTotalSize, taskFileCount))
                }
            }
        }
    }

    private fun getDupFileTitle(context: Context, taskStatus: String,  totalSize: Long, totalFileNum: Int): String {
        return if (TASK_STATUS_SUCCESS == taskStatus) {
            if (totalSize > (1024 * 1024 * 1024))
                context.getString(R.string.garbage_dup_notification_title_more, totalSize.formatStorageSize())
            else
                context.getString(R.string.garbage_dup_notification_title_less)
        } else {
            context.getString(R.string.garbage_dup_file_notification_failed)
        }
    }

    private fun getEmptyFolderTitle(context: Context, taskStatus: String,  totalSize: Long, totalFileNum: Int): String {
        return if (TASK_STATUS_SUCCESS == taskStatus) {
            if (totalSize > 10)
                context.getString(R.string.garbage_empty_notification_title_more, totalFileNum)
            else
                context.getString(R.string.garbage_empty_notification_title_less)
        } else {
            context.getString(R.string.garbage_empty_notification_failed)
        }
    }

    private fun getIntent(context: Context, taskId: String, taskStatus: String, taskType: String,  totalSize: Long,
                          taskFileCount: Int): Intent {
        return if (TASK_STATUS_SUCCESS == taskStatus) {
            Intent(context, ScanGarbageResultActivity::class.java).invoke {
                ScanGarbageResultActivity.TASK_ID - taskId
                ScanGarbageResultActivity.GARBAGE_FILE_COUNT - taskFileCount
                ScanGarbageResultActivity.GARBAGE_FILE_SPACE - totalSize
                Extras.GARBAGE_TYPE - taskType
            }
        } else {
            GarbageMainActivity.getStartIntent(context, GarbageMainActivity.FROM_NOTIFICATION)
        }
    }
}