package com.baidu.netdisk.ui.gc.presenter

import android.app.Dialog
import android.os.*
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.businessplatform.extension.safeDismiss
import com.baidu.netdisk.gc.io.Status
import com.baidu.netdisk.gc.io.parser.model.GarbageFileItem
import com.baidu.netdisk.gc.service.Extras
import com.baidu.netdisk.gc.service.Extras.HAS_MORE
import com.baidu.netdisk.gc.service.Extras.LIST_DATA
import com.baidu.netdisk.ui.businessplatform.extension.showLoading
import com.baidu.netdisk.ui.gc.GarbageCleanManager
import com.baidu.netdisk.ui.gc.IGarbageFilesView
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType

/**
 * @author chencaixing
 * @time 2019-09-23 11:40
 */
class  GarbageFilesPresenter<T:Parcelable>(private val iGarbageFilesView: IGarbageFilesView<T>) {

    private val mManager = GarbageCleanManager()
    private var mFilesResultReceiver: ResultReceiver? = null
    private var mDeleteResultReceiver: ResultReceiver? = null
    private var mDeleteProResultReceiver: ResultReceiver? = null
    private var mProgressDialog: Dialog? = null
    private var mHandler: Handler? = null

    // 获取垃圾文件列表
    fun getGarbageFilesList(type: String, taskId: String, start: Int, limit: Int) {
        if (mFilesResultReceiver == null) {
            mFilesResultReceiver = GarbageFilesResultReceiver(Handler(), this)
        }
        mManager.getGarbageFilesList(iGarbageFilesView.activity, AccountUtils.getInstance()
                .bduss,
                AccountUtils.getInstance().uid, type, taskId, start, limit, mFilesResultReceiver)
    }


    // 删除选中的垃圾文件
    fun deleteGarbageFiles( type: String, taskId:String, deleteItems: List<GarbageFileItem>) {
        showLoadingDialog(R.string.loading_dialog_deleting)
        if (mDeleteResultReceiver == null) {
            mDeleteResultReceiver =  DeleteResultReceiver( type, Handler(), this)
        }
        mManager.deleteGarbageFiles(iGarbageFilesView.activity, AccountUtils.getInstance().bduss,
                AccountUtils.getInstance().uid, type, taskId, deleteItems, mDeleteResultReceiver)
    }

    fun getDeleteProgress(type: String, deleteTaskId: String) {
        if (null == mHandler) mHandler = Handler()
        mHandler?.apply {
            postDelayed({
                if (mDeleteProResultReceiver == null) {
                    mDeleteProResultReceiver = DeleteStatusResultReceiver(type,
                            deleteTaskId,
                            this, this@GarbageFilesPresenter)
                }
                mManager.queryProgress(iGarbageFilesView.activity, mDeleteProResultReceiver, AccountUtils
                        .getInstance().bduss,
                        AccountUtils.getInstance().uid, type, Extras.OPTION_DELETE, deleteTaskId)
            }, 2000)
        }

    }

    private fun showLoadingDialog(textRes: Int) {
        if (mProgressDialog != null && mProgressDialog?.isShowing == true) {
            return
        }
        mProgressDialog = iGarbageFilesView.activity?.showLoading(textRes)
    }

    fun dismissDialog() {
        mProgressDialog?.safeDismiss()
    }

    fun onDestroy() {
        if (null != mHandler) {
            mHandler?.removeCallbacksAndMessages(null)
            mHandler = null
        }
    }

  fun pageFinish()  = iGarbageFilesView.activity == null || iGarbageFilesView.activity.isFinishing

    inner class GarbageFilesResultReceiver(handler: Handler, private val garbageFilesPresenter:
    GarbageFilesPresenter<T>)
        : BaseResultReceiver<GarbageFilesPresenter<T>>(garbageFilesPresenter, handler, null) {

        override fun onInterceptResult(reference: GarbageFilesPresenter<T>, resultCode: Int,
                                       resultData: Bundle?): Boolean {
            return garbageFilesPresenter.pageFinish()
        }
        override fun onFailed(reference: GarbageFilesPresenter<T>, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            garbageFilesPresenter.iGarbageFilesView.onGetDataFail()
            return super.onFailed(reference, errType, errno, resultData)
        }

        override fun onSuccess(reference: GarbageFilesPresenter<T>, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            resultData?.apply {
                val nextIndex = getInt(BaseExtras.RESULT)
                val hasMore = getBoolean(HAS_MORE)
                val listData = getParcelableArrayList<T>(LIST_DATA)
                garbageFilesPresenter.iGarbageFilesView.onGetDataSuccess(hasMore, nextIndex,listData)
            }
        }
    }

    inner class DeleteResultReceiver(private val type: String,
                                     handler: Handler, private val garbageFilesPresenter:
                                     GarbageFilesPresenter<T>)
        : BaseResultReceiver<GarbageFilesPresenter<T>>(garbageFilesPresenter, handler, null) {

        override fun onInterceptResult(reference: GarbageFilesPresenter<T>, resultCode: Int,
                                       resultData: Bundle?): Boolean {
            return garbageFilesPresenter.pageFinish()
        }

        override fun onFailed(reference: GarbageFilesPresenter<T>, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            dismissDialog()
            garbageFilesPresenter.iGarbageFilesView.onDeleteFail()
            return super.onFailed(reference, errType, errno, resultData)
        }

        override fun onSuccess(reference: GarbageFilesPresenter<T>, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            when (resultData?.getString(Extras.DELETE_STATUS)) {
                Status.SUCCESS -> {
                    dismissDialog()
                    garbageFilesPresenter.iGarbageFilesView.onDeleteSuccess()
                }
                Status.FAILED -> {
                    dismissDialog()
                    garbageFilesPresenter.iGarbageFilesView.onDeleteFail()
                }
                else -> {
                    val deleteTaskId = resultData?.getString(Extras.TASK_ID)
                    if (deleteTaskId != null) {
                        garbageFilesPresenter.getDeleteProgress( type,
                                deleteTaskId)
                    } else {
                        dismissDialog()
                        garbageFilesPresenter.iGarbageFilesView.onDeleteFail()
                    }

                }
            }
        }
    }

    inner class DeleteStatusResultReceiver(private val type: String,
                                           private val deleteTaskId:String,
                                           handler: Handler, private val garbageFilesPresenter:
                                           GarbageFilesPresenter<T>)
        : BaseResultReceiver<GarbageFilesPresenter<T>>(garbageFilesPresenter, handler, null) {

        override fun onInterceptResult(reference: GarbageFilesPresenter<T>, resultCode: Int,
                                       resultData: Bundle?): Boolean {
            return garbageFilesPresenter.pageFinish()
        }

        override fun onFailed(reference: GarbageFilesPresenter<T>, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            dismissDialog()
            garbageFilesPresenter.iGarbageFilesView.onDeleteFail()
            return super.onFailed(reference, errType, errno, resultData)
        }

        override fun onSuccess(reference: GarbageFilesPresenter<T>, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            if (resultData?.getInt( BaseExtras.RESULT,0) != 100){
                garbageFilesPresenter.getDeleteProgress( type, deleteTaskId)
            } else {
                dismissDialog()
                garbageFilesPresenter.iGarbageFilesView.onDeleteSuccess()
            }

        }
    }

}