package com.baidu.netdisk.ui.gc.presenter

import android.os.Bundle
import android.os.Handler
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.gc.io.parser.model.QueryInfo
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.ui.gc.GarbageCleanManager
import com.baidu.netdisk.ui.gc.IQueryGarbageProgressView
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView

@Tag("QueryGarbageProgressPresenter")
class QueryGarbageProgressPresenter(private val view: IQueryGarbageProgressView) {

    private var handler: Handler? = null
    private  val  progressReceiver = QueryScanProgressResultReceiver(this, Handler(), null)

    fun queryScanProgress(type: String, taskId: String, option: String, isDelay: Boolean = false) {
        if (isDelay) {
            if (null == handler) handler = Handler()
            handler?.postDelayed({
                if (view.activity != null && !view.activity.isFinishing) {
                    GarbageCleanManager().queryProgress(view.activity, progressReceiver, AccountUtils.getInstance().bduss,
                            AccountUtils.getInstance().uid, type, option, taskId)
                }
            }, 2000)
        } else {
            GarbageCleanManager().queryProgress(view.activity, progressReceiver, AccountUtils.getInstance().bduss,
                    AccountUtils.getInstance().uid, type, option, taskId)
        }

    }

    fun asyncScanProgress(type: String, taskId: String, option: String) {
        GarbageCleanManager().asyncQueryProgress(view.activity, null, AccountUtils.getInstance().bduss,
                AccountUtils.getInstance().uid, type, option, taskId)
    }

    fun onDestroy() {
        if (handler != null) {
            handler?.removeCallbacksAndMessages(null)
            handler = null
        }
    }

    private inner class QueryScanProgressResultReceiver(reference: QueryGarbageProgressPresenter,
                                                   handler: Handler, view:
    ResultView?) : BaseResultReceiver<QueryGarbageProgressPresenter>(reference, handler, view) {

        override fun onInterceptResult(reference: QueryGarbageProgressPresenter, resultCode: Int, resultData: Bundle?): Boolean {
            val activity = reference.view.activity
            return if (activity == null || activity.isFinishing) {
                !super.onInterceptResult(reference, resultCode, resultData)
            } else super.onInterceptResult(reference, resultCode, resultData)
        }

        override fun onSuccess(reference: QueryGarbageProgressPresenter, resultData: Bundle?) {
            // 刷新页面。继续下一次请求
            resultData?.apply {
                if (!reference.view.activity.isFinishing) {
                    val pro = getInt(BaseExtras.RESULT)

                    val info = getParcelable<QueryInfo>(BaseExtras.RESULT_SUCCESS)

                    reference.view.updateProgress(pro.d(prefix = "progress onSuccess"), info)
                }
            }
            super.onSuccess(reference, resultData)
        }

        override fun onFailed(reference: QueryGarbageProgressPresenter, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            "QueryGarbageProgress Failed errno:: $errno".d()
            reference.view.queryProgressFailed()
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}