package com.baidu.netdisk.ui.gc.presenter

import android.os.Bundle
import android.os.Handler
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.gc.io.ErrorCode.HAS_SCAN_TASK_NOT_FINISH
import com.baidu.netdisk.gc.io.ErrorCode.SCAN_COUNT_LIMIT
import com.baidu.netdisk.gc.statistics.StatisticsKey
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.ui.gc.GarbageCleanManager
import com.baidu.netdisk.ui.gc.IScanGarbageFileView
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView

@Tag("ScanGarbageFilePresenter")
class ScanGarbageFilePresenter(private val view: IScanGarbageFileView) {
    private val receiver = ScanGarbageFileResultReceiver(this, Handler(), null)

    fun onScanGarbageFile(type: String, path: String?, isCancel: Boolean, taskId: String?) {
        GarbageCleanManager().scanGarbageFile(view.activity, receiver, AccountUtils.getInstance().bduss,
                AccountUtils.getInstance().uid, type, path, isCancel, taskId)
        if (!isCancel) {
            StatisticsKey.GC_DUP_FILES_SCAN.updateCount(AccountUtils.getInstance().level.toString())
        }
    }

    inner class ScanGarbageFileResultReceiver(reference: ScanGarbageFilePresenter, handler: Handler, view: ResultView?) :
            BaseResultReceiver<ScanGarbageFilePresenter>(reference, handler, view) {

        override fun onInterceptResult(reference: ScanGarbageFilePresenter, resultCode: Int, resultData: Bundle?): Boolean {
            val activity = reference.view.activity
            return if (activity == null || activity.isFinishing) {
                !super.onInterceptResult(reference, resultCode, resultData)
            } else super.onInterceptResult(reference, resultCode, resultData)
        }

        override fun onFailed(reference: ScanGarbageFilePresenter, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            if (errType == ErrorType.NETWORK_ERROR) {
                reference.view.scanGarbageFailed()
            } else {
                when (errno) {
                    SCAN_COUNT_LIMIT -> {
                        reference.view.scanGarbageLimit()
                    }
                    else -> reference.view.scanGarbageFailed(errno)
                }
            }
            return super.onFailed(reference, errType, errno, resultData)
        }

        override fun onSuccess(reference: ScanGarbageFilePresenter, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            resultData?.apply {
                val result = getString(BaseExtras.RESULT)
                reference.view.scanGarbageSuccess(result.d())
            }
        }
    }
}