package com.baidu.netdisk.ui.manager

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import android.widget.ImageView
import com.baidu.android.common.util.CommonParam
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.base.storage.config.IncentiveVideoPrivateConfig
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.android.util.network.ConnectivityState
import com.baidu.netdisk.kernel.architecture.AppCommon
import com.baidu.netdisk.kernel.architecture.net.RequestCommonParams
import com.baidu.netdisk.kernel.util.encode.RC4Util
import com.baidu.netdisk.platform.business.IncentiveSDK
import com.baidu.netdisk.platform.business.callback.IncentiveCallbackImpl
import com.baidu.netdisk.platform.business.callback.IncentivePropertyCallback
import com.baidu.netdisk.transfer.probationary.DownloadSceneIncentiveReporter
import com.baidu.netdisk.ui.view.SceneLayout
import com.baidu.sofire.ac.FH

object IncentiveVideoHelper {
    /**
     * 积分场景id
     */
    const val POINT_SIGNED = 10011

    /**
     * 云解压场景id
     */
    const val CLOUD_UNCOMP = 10031

    /**
     * 视频倍速场景id
     */
    const val VIDEO_SPEEDUP = 10032

    /**
     * 音频倍速场景id
     */
    const val AUDIO_SPEEDUP = 10033

    /**
     * 下载文件场景id
     */
    const val DOWNLOAD_LIST = 10035

    /**
     * 我的奖品地址
     */
    @JvmStatic
    public val MY_PRIZE_URL = "https://pan.baidu" +
            ".com/act/pullnew/mainpage?id=136591542165&action=myprize&pt=combine"

    private var isLogin = false

    /**
     * 下载场景激励视频展示前提条件是否满足 文件大小和速度
     */
    private var mDownloadIsSatify = false

    private var sDialog: Dialog? = null

    /**
     * 初始化激励视频SDK
     *
     * @param context
     */
    fun init(context: Context) {
        if (AccountUtils.getInstance().isLogin && !isLogin && ConnectivityState.isConnected(context)) {
            if (AccountUtils.getInstance().cloudUK == 0L) {
                CloudP2PServiceHelper.getSelfInfo(context, null, true)
                return
            }
            isLogin = true
            val bundle = getBundle(context)
            IncentiveSDK.init(context, bundle)
        }
    }

    /**
     * 初始化需要的参数
     *
     * @return
     */
    private fun getBundle(context: Context): Bundle {
        val bundle = Bundle()
        bundle.putLong("UK", AccountUtils.getInstance().cloudUK)
        // 110去掉端上所有调用imei代码的位置，激励视频imei设置成空
        bundle.putString("imei", null)
        bundle.putString("cuid", CommonParam.getCUID(context))
        bundle.putString("User-Agent", RequestCommonParams.getUserAgent())
        bundle.putString("rc4key", RC4Util.getRC4Key())
        bundle.putString("version", AppCommon.VERSION_DEFINED)
        bundle.putString("BDUSS", AccountUtils.getInstance().bduss)
        bundle.putString("STOKEN", AccountUtils.getInstance().stoken)
        bundle.putString("zid", FH.gz(context))
        bundle.putInt("VIP", AccountUtils.getInstance().level)
        bundle.putString("devuid", AppCommon.DEVUID)
        bundle.putString("channel", RequestCommonParams.getChannel())
        bundle.putString("UID", AccountUtils.getInstance().uid)
        return bundle
    }

    /**
     * 退出登录时下次进来重新拉
     *
     * @param logout
     */
    fun setLogoutState(logout: Boolean) {
        isLogin = logout
        resetDownloadScene()
    }

    /**
     * 下载场景激励视频展示前提条件是否满足
     */
    fun setDownloadSceneCondition(isSatify: Boolean) {
        mDownloadIsSatify = isSatify
    }

    fun setDownloadSceneSucceed() {
        DownloadSceneIncentiveReporter.getInstance().setDownloadSceneSucceed()
    }

    fun checkDownloadSceneCondition(): Boolean {
        // Svip不展示
        if (AccountUtils.getInstance().level >= AccountUtils.ILevel.SVIP) {
            return false
        }

        // 文件大小50M和限速值
        return mDownloadIsSatify

    }

    fun checkDownloadSceneConfigEnable(): Boolean {
        // mispan配置激励视频下载位是否启用
        val config = IncentiveVideoPrivateConfig(
                ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.INCENTIVE_VIDEO_PRIVATE_CONFIG))
        return config.downloadTryEnable
    }

    /**
     * 重置激励视频极速下载场景
     */
    private fun resetDownloadScene() {
        mDownloadIsSatify = false
        DownloadSceneIncentiveReporter.getInstance().reset()
    }

    /**
     * 展示激励视频结果弹窗
     *
     * @param activity
     * @param popupText 弹窗文案
     * @param hasMore
     */
    fun showIncentiveVideoDialog(activity: Activity, popupText: String?, imageUrl: String?,
                                 hasMore: Boolean, onlineUrl: String, defaultIconID: Int,
                                 confirmTextId: Int,
                                 listener: IncentiveSuccessDialogClickListener) {

        val confirmtxt = activity.getString(confirmTextId)
        val content = activity.getString(R.string.popup_extra_text)
        val extraText = if (!popupText.isNullOrEmpty()) popupText else activity
                .getString(R.string.popup_content_default_text)

        val builder = ImageNormalAlertDialogBuilder()
                .setContentText(content).setContentTextColor(R.color.dark_gray_99)
                .setContentTextSize(14)
                .setConfirmButtonBgRes(R.drawable.chain_dialog_btn_selector)
                .setConfirmButtonTextColorRes(R.color.white)
                .setCancelBgRes(R.drawable.popup_inventivevideo_close)
                .setSubContentText(extraText).setSubContentTextColor(R.color.lighter_black)
                .setSubContentTextSize(17)
                .setConfirmButtonText(confirmtxt)

        val rightText = if (hasMore) {
            activity.getString(R.string.popup_SecondConfirmbutton_text)
        } else if (!TextUtils.isEmpty(onlineUrl)) {
            activity.getString(R.string.popup_SecondConfirm_text)
        } else ""

        builder.setSecondConfirmText(rightText).setSecondConfirmTextColorRes(R.color.blue)

        val dialog = builder.build(activity)
        dialog.window?.apply {
            val imageView = decorView.findViewById<ImageView>(R.id.dialog_content_image)
            GlideHelper.getInstance()
                    .displayImageFromNetwork(imageUrl, defaultIconID, defaultIconID,
                            defaultIconID, true, imageView, null)
        }

        builder.setDialogCtrListener(object : NewExpandDialogCtrListener {
            override fun onSecondConfirmClick() {
                listener.learnMoreClick()

            }

            override fun onOkBtnClick() {
                listener.onUseNowBtnClick()
            }

            override fun onCancelBtnClick() {
                listener.onCloseBtnClick()
            }
        })

        if (!activity.isFinishing) {
            dialog.show()
        }
    }

    public fun check(activity: FragmentActivity, sceneId: Int, callback: IncentiveCallbackImpl) {
        IncentiveSDK.check(activity, sceneId, callback)
    }

    public fun check(activity: FragmentActivity, sceneId: Int, sceneLayout: SceneLayout, callback:
    IncentiveCallbackImpl) {
        IncentiveSDK.check(activity, sceneId, sceneLayout, callback)
    }

    public fun checkWithFragment(fragment: Fragment, sceneId: Int, callback:
    IncentiveCallbackImpl) {
        IncentiveSDK.check(fragment, sceneId, callback)
    }

    public fun getRemains(activity: FragmentActivity, sceneId: Int, callback:
    IncentivePropertyCallback) {
        IncentiveSDK.getRemains(activity, sceneId, callback)
    }
}