package com.baidu.netdisk.ui.preview.image

import android.graphics.drawable.Drawable
import android.support.v4.content.ContextCompat
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.base.storage.config.ConfigAIApps
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.cloudfile.io.model.CloudFile
import com.baidu.netdisk.cloudimage.ImageInfo
import com.baidu.netdisk.cloudimage.storage.config.CloudImageConfigKey
import com.baidu.netdisk.cloudimage.ui.location.LocationTimelineActivity
import com.baidu.netdisk.cloudimage.ui.person.PersonTimelineActivity
import com.baidu.netdisk.cloudimage.ui.things.ThingsTimelineActivity
import com.baidu.netdisk.kernel.android.util.file.FileUtils
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.util.CollectionUtils
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.kernel.util.FormatUtils
import com.baidu.netdisk.kernel.util.TimeUtil
import com.baidu.netdisk.preview.image.PreviewFileBean
import com.baidu.netdisk.preview.image.PreviewImageCategory
import com.baidu.netdisk.sharedirectory.provider.ShareDirectoryContract
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.stats.NetdiskStatsEngine
import com.baidu.netdisk.stats.StatisticsType
import com.baidu.netdisk.ui.aiapps.AiAppsInitUtil
import com.baidu.netdisk.ui.cloudfile.AlbumTimelineActivity
import com.baidu.netdisk.ui.preview.OpenFileHelper
import com.baidu.netdisk.ui.widget.BottomDrawerLayout
import com.google.gson.Gson
import java.util.*

/**
 * 大图预览底部详细信息
 *
 * @author wangyang34
 * @since 2019年12月3日
 */
class ImageBottomDetailView(val activity: ImagePagerActivity, val bean: PreviewFileBean, private val drawerLayout: BottomDrawerLayout, val from: Int) {

    /** 获取共享目录的根目录名字回调 */
    private var mGetShareDirectoryRootNameCallback: GetShareDirectoryRootNameCallback? = null

    /** 共享目录的根路径 */
    private var mSharedRootPath: String? = null

    /** 共享目录的根路径 */
    private var mShareRootDirectory: CloudFile? = null

    /** 图片信息 */
    private val info: ImageInfo? = bean.imageInfo

    /** 是否显示打印 */
    private var mNeedShowPrintBtn = true

    init {
        info?.let {
            if (ShareDirectoryContract.Directories.isSharedToMeRootAndSubDirectory(it.mParentPath)) {
                mSharedRootPath = ShareDirectoryContract.Directories.getShareDirectoryRootPath(it.mParentPath)
                mShareRootDirectory = if (TextUtils.isEmpty(mSharedRootPath)) null else mSharedRootPath?.let { CloudFile(it) }
            }
        }
        mNeedShowPrintBtn = if (bean.isType(PreviewImageCategory.VIDEO_IMAGE)) {
            false
        } else {
            showPrintEntrance(from)
        }
    }

    /**
     * 绑定数据
     */
    fun onBindInfo() {
        onBindMoreOperation()
        onBindImageInfo()
        onBindDateInfo()
        onBindLocationInfo()
        onBindPathInfo()

        var showPerson = true
        if (info == null
                || !PersonalConfig.getInstance().getBoolean(CloudImageConfigKey.SUPPORT_PERSONAL_FACE, true)
                || CollectionUtils.isEmpty(info.persons)
                || info.mIsLocal) {
            showPerson = false
        }
        var showTags = true
        if (info == null || CollectionUtils.isEmpty(info.tags) || info.mIsLocal) {
            showTags = false
        }
        if (!showPerson && !showTags) {
            val title = drawerLayout.findViewById<View>(R.id.relevant_title)
            title.visibility = View.GONE
        }
        onBindPersonInfo(showPerson)
        onBindTagsInfo(showTags)
    }

    /**
     * 绑定更多操作数据
     */
    private fun onBindMoreOperation() {
        val moreOperation = drawerLayout.findViewById<RecyclerView>(R.id.more_operation)

        val mItems = MoreOperationConfig(activity).getConfig(mNeedShowPrintBtn)
        val adapter = SimpleMoreOperationAdapter(activity, mItems)
        val layoutManager = GridLayoutManager(activity, 5)
        moreOperation.layoutManager = layoutManager
        moreOperation.adapter = adapter
    }

    /**
     * 绑定图片信息数据
     */
    private fun onBindImageInfo() {
        val title = drawerLayout.findViewById<View>(R.id.image_title)
        val panel = drawerLayout.findViewById<View>(R.id.image_info_panel)
        val name = drawerLayout.findViewById<TextView>(R.id.image_name)
        val size = drawerLayout.findViewById<TextView>(R.id.image_size)
        val icon = drawerLayout.findViewById<ImageView>(R.id.item_operation_icon)
        if (info == null) {
            title.visibility = View.GONE
            panel.visibility = View.GONE
            return
        }
        name.text = bean.fileName
        var sizeText = FormatUtils.formatFileSize(info.mSize)
        if (info.mHeight > 0 && info.mWidth > 0) {
            sizeText = activity.getString(R.string.image_size, sizeText, info.mHeight, info.mWidth)
        }
        size.text = sizeText
        val drawableLoading: Drawable? = ContextCompat.getDrawable(activity, R.color.white_f5)
        GlideHelper.getInstance().displayImageFromNetwork(bean.resourceUrl,
                drawableLoading, drawableLoading, drawableLoading, true, icon, null)
    }

    /**
     * 绑定路径信息数据
     */
    private fun onBindPathInfo() {
        val pathPanel = drawerLayout.findViewById<View>(R.id.image_path_panel)
        if (info == null || info.mIsLocal || info.mParentPath == null) {
            pathPanel.visibility = View.GONE
            return
        }
        val imagePath = pathPanel.findViewById<TextView>(R.id.image_path)
        val imagePathDescription = pathPanel.findViewById<TextView>(R.id.image_path_description)
        if (TextUtils.isEmpty(info.fsId)) {
            imagePath.text = "存储位置"
            imagePathDescription.text = "手机本地"
            return
        }
        // 设置目录信息
        val pathText: String?
        if (!TextUtils.isEmpty(mSharedRootPath)) {
            if (mGetShareDirectoryRootNameCallback == null) {
                mGetShareDirectoryRootNameCallback = GetShareDirectoryRootNameCallback(
                        activity.applicationContext, pathPanel, info.mParentPath)
            }
            val rootName = mGetShareDirectoryRootNameCallback!!.rootName
            if (TextUtils.isEmpty(rootName)) {
                pathText = null
                activity.supportLoaderManager.initLoader(Int.MAX_VALUE, null, mGetShareDirectoryRootNameCallback)
            } else { // 用根目录名字替换掉根目录路径
                pathText = info.mParentPath.replace(mSharedRootPath!!, FileUtils.ROOT + rootName)
            }
        } else {
            pathText = if (info.mParentPath == FileUtils.ROOT) activity.getString(R.string.image_path_root) else info.mParentPath
        }
        imagePath.text = pathText
        pathPanel.setOnClickListener(View.OnClickListener {
            if (TextUtils.isEmpty(info.mParentPath)) {
                return@OnClickListener
            }
            val dest = CloudFile(info.mParentPath)
            mGetShareDirectoryRootNameCallback?.let {
                val fileName = FileUtils.getFileName(info.mParentPath)
                if (TextUtils.isEmpty(fileName)) {
                    dest.filename = it.rootName
                } else {
                    dest.filename = fileName
                }
            }
            OpenFileHelper().openDirActivity(activity, dest)
            NetdiskStatisticsLogForMutilFields.getInstance()
                    .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.IMAGE_DETAIL_MORE_FOLDER)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.NEW_IMAGE_DETAIL_MORE_FOLDER)
        })
    }

    /**
     * 绑定日期信息数据
     */
    private fun onBindDateInfo() {
        val datePanel = drawerLayout.findViewById<View>(R.id.image_date_panel)
        if (info == null || info.mIsLocal) {
            datePanel.visibility = View.GONE
            return
        }
        val imageDate = drawerLayout.findViewById<TextView>(R.id.image_date)
        imageDate.text = if (info.mDateTaken > 0) {
            formatTimeInfo(info.mDateTaken * 1000, info.mYear, info.mMonth, info.mDay)
        } else {
            activity.getString(R.string.cloudimage_timeline_cluster_no_date)
        }
        datePanel.setOnClickListener {
            mShareRootDirectory?.let { rootDir ->
                mGetShareDirectoryRootNameCallback?.let {
                    rootDir.filename = it.rootName
                }
            }
            if (from == ImagePagerActivity.ALBUM_LOCAL_IMAGE) {
                AlbumTimelineActivity.startTimeline(activity, info.mYear, info.mMonth,
                        info.mDay, 0, mShareRootDirectory)
            } else {
                AlbumTimelineActivity.startTimeline(activity, info.mYear, info.mMonth,
                        info.mDay, 1, mShareRootDirectory)
            }
            NetdiskStatisticsLogForMutilFields.getInstance()
                    .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.IMAGE_DETAIL_MORE_DATE)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.NEW_IMAGE_DETAIL_MORE_DATE)
        }
    }

    /**
     * 绑定位置信息数据
     */
    private fun onBindLocationInfo() {
        val locationPanel = drawerLayout.findViewById<View>(R.id.image_location_panel)
        // 设置位置信息
        if (info == null || TextUtils.isEmpty(info.mCountry) || info.mIsLocal) {
            locationPanel.visibility = View.GONE
            return
        }
        val imageLocation = drawerLayout.findViewById<TextView>(R.id.image_location)
        val isForeign = activity.resources.getString(R.string.location_default_country) != info.mCountry
        imageLocation.text = if (isForeign) {
            formatLocationInfo(info.mCountry, info.mProvince, info.mCity)
        } else {
            formatLocationInfo(info.mCity, info.mDistrict, info.mStreet)
        }
        locationPanel.setOnClickListener {
            LocationTimelineActivity.startActivity(activity, info.mCountry, info.mProvince,
                    info.mCity, mSharedRootPath, mShareRootDirectory, -1)
            NetdiskStatisticsLogForMutilFields.getInstance()
                    .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.IMAGE_DETAIL_MORE_LOCATION)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.NEW_IMAGE_DETAIL_MORE_LOCATION)
        }
    }

    /**
     * 绑定人物信息
     * @param show 是否显示元素
     */
    private fun onBindPersonInfo(show: Boolean) {
        val persons = drawerLayout.findViewById<RecyclerView>(R.id.image_relevant_people)
        if (info == null || !show) {
            persons.visibility = View.GONE
            return
        }
        val simplePersonAdapter = SimplePersonAdapter(info.persons)
        simplePersonAdapter.setItemClickListener { person ->
            PersonTimelineActivity.startActivity(activity, person, mShareRootDirectory, -1)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.IMAGE_DETAIL_PERSON_CLICK)
        }
        val layoutManager = LinearLayoutManager(activity, LinearLayoutManager.HORIZONTAL, false)
        persons.layoutManager = layoutManager
        persons.adapter = simplePersonAdapter
    }

    /**
     * 绑定事物信息
     * @param show 是否显示元素
     */
    private fun onBindTagsInfo(show: Boolean) {
        val tags = drawerLayout.findViewById<RecyclerView>(R.id.image_relevant_tag)
        if (info == null || !show) {
            tags.visibility = View.GONE
            return
        }
        val simpleTagAdapter = SimpleTagAdapter(info.tags)
        simpleTagAdapter.setmItemClickListener { tag ->
            ThingsTimelineActivity.startActivity(activity, tag.tagId, tag.tagName,
                    mSharedRootPath, mShareRootDirectory, -1)
            val map = HashMap<String, String>()
            map["tag"] = tag.tagName
            NetdiskStatsEngine.getInstance().getNetdiskStats(StatisticsType.JSON)
                    .statCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.IMAGE_DETAIL_THING_CLICK, Gson().toJson(map))
        }
        val layoutManager = LinearLayoutManager(activity, LinearLayoutManager.HORIZONTAL, false)
        tags.layoutManager = layoutManager
        tags.adapter = simpleTagAdapter
    }

    /**
     * 格式化时间信息
     *
     * @param dateTaken 拍照时间
     * @param year 年
     * @param month 月
     * @param day 日
     * @return 格式化后时间
     */
    private fun formatTimeInfo(dateTaken: Long, year: Int, month: Int, day: Int): String? {
        val time = DateUtil.formatTimeEndWithMin(dateTaken)
        if (year == 0 || month == 0 || day == 0) {
            return time
        }
        val calendar = Calendar.getInstance()
        calendar[year, month - 1] = day
        val week = TimeUtil.getDayOfWeek(calendar[Calendar.DAY_OF_WEEK])
        return "$time $week"
    }

    /**
     * 格式化位置信息
     *
     * @param country 国家
     * @param province 省
     * @param city 城市
     * @return 位置信息
     */
    private fun formatLocationInfo(country: String?, province: String?, city: String?): String? {
        val formatProvince = if (TextUtils.isEmpty(province)) {
            ""
        } else {
            "-$province"
        }
        val formatCity = if (TextUtils.isEmpty(city)) {
            ""
        } else {
            "-$city"
        }
        return "$country$formatProvince$formatCity"
    }

    /**
     * 是否显示打印入口
     *
     * @param from 来源
     * @return 布尔
     */
    private fun showPrintEntrance(from: Int): Boolean {
        val config = ConfigAIApps(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.AIAPPS))
        return (!AiAppsInitUtil.DISABLED && !config.hideImagePrintEntrance
                && (from == ImagePagerActivity.FILE_IMAGE
                || from == ImagePagerActivity.STORY_IMAGE
                || from == ImagePagerActivity.CLOUD_IMAGE
                || from == ImagePagerActivity.LATEST_IMAGE
                || from == ImagePagerActivity.ALBUM_CLOUD_IMAGE))
    }
}