package com.baidu.netdisk.ui.preview.image

import com.baidu.netdisk.R
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.router.YOUA_GUIDE_URL
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields.StatisticsKeys.IAMGE_PAGER_YOUA_GUIDE_CLICK
import com.baidu.netdisk.ui.aiapps.AiAppsInitUtil
import com.baidu.netdisk.xpan.provider.aiapps.AiAppsContract

/**
 * 大图预览底部详细信息的配置
 *
 * @author wangyang34
 * @since 2019年12月3日
 */
class MoreOperationConfig(val activity: ImagePagerActivity) {

    companion object {

        /** 删除 */
        const val MORE_OPERATION_DELETE = 0

        /** 小程序 */
        const val MORE_OPERATION_AIAPP = 1

        /** router */
        const val MORE_OPERATION_ROUTER = 2
    }

    /**
     * 获取配置
     * @param needShowPrintBtn 是否需要打印
     * @return 配置列表
     */
    fun getConfig(needShowPrintBtn: Boolean): List<MoreOperationInfo> {
        // 首先读取mispan配置
        val config = ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.IMAGE_BOTTOM_MORE_OPERATION)
        val configImageBottomDetail = ConfigImageBottomDetail(config)
        configImageBottomDetail.mItems?.let {
            if (it.size > 10) {
                it.subList(0, 10)
            } else {
                it
            }
        }
        return defaultConfig(needShowPrintBtn)
    }

    /**
     * 默认配置
     * @param needShowPrintBtn 是否需要打印
     * @return 配置列表
     */
    private fun defaultConfig(needShowPrintBtn: Boolean): List<MoreOperationInfo> {
        val items = ArrayList<MoreOperationInfo>()
        val title = activity.resources.getString(R.string.imagepager_bt_delete)
        activity.bottomToolBarFragment?.let {
            items.add(MoreOperationInfo(title, MORE_OPERATION_DELETE, "", R.drawable.edit_more_delete,"", ""))
        }
        if (needShowPrintBtn) {
            val print = activity.resources.getString(R.string.imagepager_print)
            items.add(MoreOperationInfo(print, MORE_OPERATION_AIAPP, "", R.drawable.timeline_edit_tools_print,
                    "", AiAppsContract.AiApps.ENTRANCE_IMAGE_PRINT))
        }
        if (!AiAppsInitUtil.DISABLED) {
            val youa = activity.resources.getString(R.string.imagepager_youa)
            items.add(MoreOperationInfo(youa, MORE_OPERATION_ROUTER, "", R.drawable.edit_more_music,
                    IAMGE_PAGER_YOUA_GUIDE_CLICK, YOUA_GUIDE_URL))
        }

        return items
    }
}