package com.baidu.netdisk.ui.preview.image

import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.router.RouterDispatcher.navigate
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.preview.image.MoreOperationConfig.Companion.MORE_OPERATION_AIAPP
import com.baidu.netdisk.ui.preview.image.MoreOperationConfig.Companion.MORE_OPERATION_DELETE
import com.baidu.netdisk.ui.preview.image.MoreOperationConfig.Companion.MORE_OPERATION_ROUTER

/**
 * 更多操作的适配器
 *
 * @author wangyang34
 * @since 2019年12月3日
 *
 * @param mItems 列表数据
 */
class SimpleMoreOperationAdapter(private val mActivity: ImagePagerActivity,
                                 private val mItems: List<MoreOperationInfo>)
    : RecyclerView.Adapter<SimpleMoreOperationAdapter.ViewHolder>() {

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = LayoutInflater.from(parent.context).inflate(R.layout.item_more_operation, parent, false)
        return ViewHolder(view)
    }

    override fun getItemViewType(position: Int): Int {
        return mItems[position].type
    }

    override fun getItemCount(): Int {
        return mItems.size
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val item: MoreOperationInfo = mItems[position]
        holder.title?.let {
            it.text = item.title
        }
        holder.icon?.let {
            if (item.res != 0) {
                GlideHelper.getInstance().displayImageFromDrawableWithDefaut(item.res,
                        R.color.timeline_image_default_bg_color, it)
            } else {
                GlideHelper.getInstance().displayImage(item.icon, R.color.timeline_image_default_bg_color,
                        it, false)
            }
        }
        holder.root.setOnClickListener {
            when(item.type) {
                MORE_OPERATION_DELETE -> mActivity.bottomToolBarFragment?.deletePic()
                MORE_OPERATION_AIAPP -> mActivity.showAiAppsDialog(item.extra)
                MORE_OPERATION_ROUTER -> commonJump(item)
            }
        }
    }

    /**
     * 打开webview
     */
    private fun commonJump(item: MoreOperationInfo) {
        if (TextUtils.isEmpty(item.extra)) {
            return
        }
        navigate(item.extra)
        if (!TextUtils.isEmpty(item.statistics)) {
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(item.statistics)
        }
    }

    /**
     * holder
     * @param root 根view
     */
    inner class ViewHolder(val root: View) : RecyclerView.ViewHolder(root) {

        /** 图标 */
        var icon: ImageView? = null

        /** 标题 */
        var title: TextView? = null

        init {
            icon = root.findViewById(R.id.item_operation_icon)
            title = root.findViewById(R.id.item_operation_title)
        }
    }
}
