package com.baidu.netdisk.ui.preview.unzip

import android.annotation.SuppressLint
import android.app.Activity
import android.app.Dialog
import android.arch.lifecycle.MutableLiveData
import android.arch.lifecycle.Observer
import android.content.*
import android.database.Cursor
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import android.view.Gravity
import android.view.KeyEvent
import android.view.View
import android.view.ViewGroup
import android.view.animation.AnimationSet
import android.view.animation.AnimationUtils
import android.widget.EditText
import android.widget.TextView
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.NetDiskApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.account.PrivilegeChangedGuideHelper
import com.baidu.netdisk.account.constant.PrivilegeConstant
import com.baidu.netdisk.account.model.ConfigCloudUnzip
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.base.service.BaseServiceHelper
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.base.storage.config.IncentiveVideoConfig
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.base.storage.config.Setting
import com.baidu.netdisk.businessplatform.extension.safeDismiss
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey
import com.baidu.netdisk.cloudfile.io.model.CloudFile
import com.baidu.netdisk.cloudfile.storage.config.CloudFileConfigKey
import com.baidu.netdisk.cloudfile.storage.db.CloudFileContract
import com.baidu.netdisk.io.ErrorCode
import com.baidu.netdisk.kernel.android.ext.WeakRefResultReceiver
import com.baidu.netdisk.kernel.android.util.file.FileUtils
import com.baidu.netdisk.kernel.android.util.network.ConnectivityState
import com.baidu.netdisk.kernel.android.util.network.NetWorkVerifier
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.db.cursor.ObjectCursor
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.net.ParallelAsyncTask
import com.baidu.netdisk.kernel.util.CollectionUtils
import com.baidu.netdisk.main.caller.FSCreateObjectApiCompManager
import com.baidu.netdisk.onlineactivity.OnlineActivityContract
import com.baidu.netdisk.permission.IPermission
import com.baidu.netdisk.permission.Permissions
import com.baidu.netdisk.platform.business.callback.DialogCallback
import com.baidu.netdisk.platform.business.callback.IncentivePropertyCallback
import com.baidu.netdisk.preview.cloudunzip.CloudUnzipBroadcastParams
import com.baidu.netdisk.preview.cloudunzip.CloudUnzipConstant
import com.baidu.netdisk.preview.io.model.UnzipVipGuideBean
import com.baidu.netdisk.preview.service.PreviewServiceDelegate
import com.baidu.netdisk.task.OnlineTaskManager
import com.baidu.netdisk.transfer.task.DownloadTaskManager
import com.baidu.netdisk.transfer.task.process.download.CloudFileDownloadProcessorFactory
import com.baidu.netdisk.transferlist.DownloadTaskManagerProxy
import com.baidu.netdisk.ui.businessplatform.BusinessSinglePrivilegeActivity
import com.baidu.netdisk.ui.businessplatform.BusinessVipGuideDialog
import com.baidu.netdisk.ui.businessplatform.extension.toast
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogCategory
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogCtrListener
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogFactory
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogStyle
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.manager.*
import com.baidu.netdisk.ui.permission.PermissionContentHelper
import com.baidu.netdisk.ui.permission.presenter.PermissionPresenter
import com.baidu.netdisk.ui.transfer.OnAddDownloadTaskListener
import com.baidu.netdisk.ui.transfer.SmoothVideoDownloadHelper
import com.baidu.netdisk.ui.vip.VipActivity
import com.baidu.netdisk.ui.webview.CommonWebViewActivity
import com.baidu.netdisk.ui.widget.EditLoadingDialog
import com.baidu.netdisk.ui.widget.LoadingDialog
import com.baidu.netdisk.util.ToastHelper
import kotlinx.android.synthetic.main.unzip_progress_dialog_layout.*
import org.json.JSONException
import org.json.JSONObject
import java.io.File
import java.util.*

class BusinessUnzipGuideDialogActivity : BaseActivity(), CloudUnzipPresenter.UnzipListFinishListener,
        CloudUnzipPresenter.UnzipListQueryTaskListener, QueryUnzipGuideListener,
        CloudUnzipPresenter.UnzipCopyFinishListener, CloudUnzipPresenter.UnzipCopyQueryTaskListener,
        BusinessVipGuideDialog.IBusinessVipGuideView {

    private val TAG = "BusinessUnzipGuideDialogActivity"
    /**
     * 购买云解压的 RequestCode
     */
    private val REQUEST_CODE_TO_BUY_CLOUD_ZIP: Int = 101

    private var mType: Int = 0
    private var mPath: String? = null
    private var mSubPath: String? = null
    private var mSubPaths: ArrayList<String>? = null
    private var mToPath: String? = null
    private var mSize = 0L
    private var mProduct: String? = null
    private var mFsid: String? = null
    private var mPrimaryId: Long = 0
    private var mUk: Long = 0
    private var mExtra: String? = null
    private var mFileMd5: String? = null
    private var mCloudFile: CloudFile? = null

    private var mCloudUnzipPresenter: CloudUnzipPresenter? = null
    private var mCurrentTaskId: String? = null
    private var mCopySuccessFileSize = -1
    private var mDownloadResultReceiver: DownloadResultReceiver? = null

    /**
     * 免费特权使用云解压，目前主要做统计行为区分
     */
    private var mUseFreePrivilege: Boolean = false

    // 注册广播
    private var mUnzipListReceiver: BroadcastReceiver? = null
    private var mUnzipCopyReceiver: BroadcastReceiver? = null

    private var mErrorDialog: Dialog? = null
    private var mUnzipListDialog: Dialog? = null
    private var mUnzipListProgressText: TextView? = null
    private var mUnzipListProgress = 0
    private var mUnzipCopyDialog: Dialog? = null
    private var mUnzipCopyProgressText: TextView? = null
    private var mUnzipCopyProgress = 0

    private var mProgressDialog: Dialog? = null
    private var mPasswordDialog: EditLoadingDialog? = null
    private var mPwdEditText: EditText? = null
    private var mPassword: String? = null
    private var mPwdDialogShowCount = 0

    private var vipGuideDialog: BusinessVipGuideDialog? = null
    private var guideBean :UnzipVipGuideBean? = null
    // first:是否有剩余激励视频次数  second:文件是否历史解压过 third 是否成功获取配置
    private val vipGuideLiveData: MutableLiveData<Triple<Boolean?, Boolean?,Boolean?>> =
            MutableLiveData()

    private var mCanCloseGuideView = true

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        window.setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT)
        mCloudUnzipPresenter = CloudUnzipPresenter(this)
        // 解析Intent
        parseIntent()

        mDownloadResultReceiver = DownloadResultReceiver(this, Handler())
        beginProcess()

        mCloudUnzipPresenter?.getUnZipFreePrivilege()
        // 压缩文件点击，在此收拢
        StatisticsKey.BUSINESS_ZIP_FILE_CLICK.updateCount(CloudUnzipHelper.getZipFileSizeRange(mSize))
    }

    override fun initView() {

    }

    override fun getLayoutId(): Int {
        return R.layout.business_platform_activity_unzip_dialog
    }

    /**
     * 解析Intent，对解析过程做安全校验，此页面为Exported=true，防止被攻击
     */
    private fun parseIntent() {
        try {
            mType = intent.getIntExtra(CloudUnzipConstant.KEY_TYPE, -1)
            mPath = intent.getStringExtra(CloudUnzipConstant.EXTRA_KEY_PATH)
            mSubPath = intent.getStringExtra(CloudUnzipConstant.EXTRA_KEY_SUBPATH)
            mSubPaths = intent.getStringArrayListExtra(CloudUnzipConstant.EXTRA_KEY_SUBPATHS)
            mSize = intent.getLongExtra(CloudUnzipConstant.EXTRA_KEY_SIZE, 0L)
            mToPath = intent.getStringExtra(CloudUnzipConstant.EXTRA_KEY_TOPATH)
            mProduct = intent.getStringExtra(CloudUnzipConstant.EXTRA_KEY_PRODUCT)
            mFsid = intent.getStringExtra(CloudUnzipConstant.EXTRA_KEY_FSID)
            mPrimaryId = intent.getLongExtra(CloudUnzipConstant.EXTRA_KEY_PRIMARYID, 0L)
            mUk = intent.getLongExtra(CloudUnzipConstant.EXTRA_KEY_UK, 0L)
            mExtra = intent.getStringExtra(CloudUnzipConstant.EXTRA_KEY_EXTRA)
            mFileMd5 = intent.getStringExtra(CloudUnzipConstant.EXTRA_KEY_FILE_MD5)
            mCloudFile = intent.getParcelableExtra(CloudUnzipConstant.EXTRA_KEY_CLOUD_FILE)
        } catch (e: Exception) {
            // 第三方调起过程中，启动失败
            // 1、调起方参数问题 2、被恶意攻击
            NetDiskLog.d(TAG, "UnzipShowDialogActivity 启动失败，失败原因 : ${e.toString()}")
            finish()
        }
    }

    private fun beginProcess() {
        when (mType) {
            CloudUnzipConstant.TYPE_UNZIP_LIST, CloudUnzipConstant.TYPE_UNZIP_DIR_LIST -> {
                showUnzipListProcess()
            }
            CloudUnzipConstant.TYPE_UNZIP_COPY, CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD,
            CloudUnzipConstant.TYPE_UNZIP_PREVIEW -> {
                showUnzipCopyProcess()
            }
            else -> {
                finish()
            }
        }
    }

    private fun showUnzipListProcess() {
        // 压缩包使用过免费解压服务，并且在有效期内
        if (mCloudUnzipPresenter?.isFreeZip(mFileMd5) == true) {
            getUnzipList(true)
            return
        }
        if (CloudUnzipHelper.enabled()) {
            if (CloudUnzipHelper.canUnzip(mSize)) {
                getUnzipList(false)
            } else {
                showNoPermissionDialog(mSize)
            }
        } else {
            showNoPermissionDialog(mSize)
        }
    }

    /**
     * 加载解压文件列表
     */
    private fun getUnzipList(useFreePrivilege: Boolean) {
        if (useFreePrivilege) {
            showLoadingDialog(getString(R.string.unzip_free_file_list_loading))
        } else {
            showLoadingDialog(getString(R.string.unzip_file_list_loading))
        }
        mUseFreePrivilege = useFreePrivilege
        mCloudUnzipPresenter?.unzipList(mPath, mSubPath, this, 0, 50, mProduct, mFsid, mPrimaryId, mUk, mExtra,
                mPassword)
    }

    private fun showNoPermissionDialog(fileSize: Long) {
        // 获取引导要引导购买的级别
        val toBuyLevel = mCloudUnzipPresenter?.getVipLevelToBuy(fileSize)
        val freeCount = mCloudUnzipPresenter?.unzipFreeCount ?: 0
        when (toBuyLevel) {
            CloudUnzipPresenter.PACKAGE_TOO_BIG -> showErrorDialog(getString(R.string.unzip_file_size_large_dialog_title),
                    getString(R.string.unzip_file_size_large_dialog_content))
            else -> {
                if (freeCount > 0) { // 有券未用引导使用云解压券
                    showUseCloudZipPrivilege()
                } else {
                    vipGuideLiveData.observe(this, Observer {
                        if (it?.toList()?.all {
                                    it != null
                                } == true) {
                            dismissLoadingDialog()
                            showGuideVipDialog(it.first ?: false, it.second ?: false)
                        }
                    })
                    queryIncentiveRemains()
                    mCloudUnzipPresenter?.queryUnzipVipGuide(this, mFsid, mPath)
                    showLoadingDialog("")
                }
            }
        }
    }

    private fun showUnzipCopyProcess() {
        var loading = ""
        when (mType) {
            CloudUnzipConstant.TYPE_UNZIP_COPY -> loading = if (AccountUtils.getInstance().isVip) {
                getString(R.string.unzip_file_copy_loading,
                        getString(if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP)
                            R.string.unzip_svip_call
                        else
                            R.string.unzip_vip_call))
            } else {
                getString(R.string.unzip_free_file_copy_loading)
            }
            CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD -> loading = if (AccountUtils.getInstance().isVip) {
                getString(R.string.unzip_file_download_loading,
                        getString(if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP)
                            R.string.unzip_svip_call
                        else
                            R.string.unzip_vip_call))
            } else {
                getString(R.string.unzip_free_file_download_loading)
            }
            CloudUnzipConstant.TYPE_UNZIP_PREVIEW -> loading = if (AccountUtils.getInstance().isVip) {
                getString(R.string.unzip_file_preview_loading,
                        getString(if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP)
                            R.string.unzip_svip_call
                        else
                            R.string.unzip_vip_call))
            } else {
                getString(R.string.unzip_free_file_preview_loading)
            }
            else -> finish()
        }
        showLoadingDialog(loading)
        mCloudUnzipPresenter?.unzipCopy(this, mPath, mSubPaths, mToPath, mProduct, mFsid, mPrimaryId, mUk, mExtra,
                mPassword)
    }

    private fun showErrorDialog(title: String, content: String) {
        val dialogBuilder = DialogBuilder()
        mErrorDialog = dialogBuilder.buildOneButtonDialog(this, title, content, getString(R.string.know_it))
        dialogBuilder.setOnDialogCtrListener(object : DialogCtrListener {
            override fun onOkBtnClick() {
                finish()
            }

            override fun onCancelBtnClick() {
                finish()
            }
        })
        dialogBuilder.setCancelable(false)
    }


    private fun showPasswordDialog(count: Int) {
        if (mPasswordDialog == null) {
            mPasswordDialog = EditLoadingDialog.build(this, EditLoadingDialog.Type.PASSWORD)
        }

        mPasswordDialog?.apply {
            val title: Int
            if (count > 0) {
                title = R.string.unzip_password_error
                showSubtitleTitle()
            } else {
                title = R.string.unzip_password_dialog_title
            }
            setTitle(title)

            // mPwdEditText = editText as EditText
            setRightBtnText(R.string.ok)
            setRightBtnOnClickListener {
                mPassword = editText.text.toString()
                dismiss()
                beginProcess()
            }
            setLeftBtnOnClickListener { finish() }
            setCanceledOnTouchOutside(false)
            show()
        }
    }

    private fun showUnzipListDialog() {
        val builder = DialogBuilder()
        mUnzipListDialog = builder.buildCustomViewDialog(this, R.string.unzip_file_list_dialog_title,
                R.string.unzip_dialog_hide_btn, R.string.cancel, R.layout.unzip_progress_dialog_layout).apply {
            startTransportAnim(this)
            mUnzipListProgressText = findViewById<TextView>(R.id.progress_text)
            progress_text.text = "$mUnzipListProgress%"
            if (mUseFreePrivilege) {
                desc_text.text = getString(R.string.unzip_free_file_list_dialog_content,
                        CloudUnzipHelper.getUnzipSize(AccountUtils.ILevel.SVIP))
            } else {
                desc_text.text = getString(R.string.unzip_file_list_dialog_content, CloudUnzipHelper.getUnzipSize())
            }

            builder.setOnDialogCtrListener(object : DialogCtrListener {
                override fun onOkBtnClick() {
                    CloudUnzipHelper.cancelUnzipFileListTask()
                    saveUnzipListTaskInfo()
                    endTransportAnim(this@apply)
                    finish()
                }

                override fun onCancelBtnClick() {
                    CloudUnzipHelper.cancelUnzipFileListTask()
                    endTransportAnim(this@apply)
                    finish()
                }
            })
            builder.setCancelable(false)
            show()
        }
    }

    @SuppressLint("SetTextI18n")
    private fun showUnzipCopyDialog() {
        val builder = DialogBuilder()
        val title: Int
        val contentText: Int
        val cancel: Int
        val confirm: Int
        when (mType) {
            CloudUnzipConstant.TYPE_UNZIP_COPY -> {
                title = R.string.unzip_file_copy_dialog_title
                contentText = if (AccountUtils.getInstance().isVip) {
                    R.string.unzip_file_copy_dialog_content
                } else {
                    R.string.unzip_free_file_copy_dialog_content
                }
                confirm = R.string.unzip_dialog_hide_btn
                cancel = -1
            }
            CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD -> {
                title = R.string.unzip_file_download_dialog_title
                contentText = if (AccountUtils.getInstance().isVip) {
                    R.string.unzip_file_download_dialog_content
                } else {
                    R.string.unzip_free_file_download_dialog_content
                }
                confirm = R.string.unzip_dialog_hide_btn
                cancel = R.string.cancel
            }
            CloudUnzipConstant.TYPE_UNZIP_PREVIEW -> {
                title = R.string.unzip_file_preview_dialog_title
                contentText = if (AccountUtils.getInstance().isVip) {
                    R.string.unzip_file_preview_dialog_content
                } else {
                    R.string.unzip_free_file_preview_dialog_content
                }
                confirm = R.string.cancel
                cancel = -1
            }
            else -> {
                finish()
                return
            }
        }

        mUnzipCopyDialog = builder.buildCustomViewDialog(this, title, confirm, cancel, R.layout.unzip_progress_dialog_layout).apply {
            startTransportAnim(this)
            mUnzipCopyProgressText = findViewById<TextView>(R.id.progress_text)
            progress_text.text = "$mUnzipCopyProgress%"

            desc_text.text = if (AccountUtils.getInstance().isVip) {
                getString(contentText, getString(if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP)
                    R.string.unzip_svip_call
                else
                    R.string.unzip_vip_call))
            } else {
                getString(contentText)
            }

            builder.setOnDialogCtrListener(object : DialogCtrListener {
                override fun onOkBtnClick() {
                    endTransportAnim(this@apply)
                    when (mType) {
                        CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD -> CloudUnzipHelper.hideUnzipTask()
                        CloudUnzipConstant.TYPE_UNZIP_PREVIEW -> CloudUnzipHelper.cancelUnzipTask(mCurrentTaskId)
                        CloudUnzipConstant.TYPE_UNZIP_COPY -> {
                            CloudUnzipHelper.hideUnzipTask()
                            CloudUnzipHelper.sendUnzipNotificationBroadcast(FileUtils.getFileName(mPath))
                        }
                    }
                    finish()
                }

                override fun onCancelBtnClick() {
                    endTransportAnim(this@apply)
                    CloudUnzipHelper.cancelUnzipTask(mCurrentTaskId)
                    finish()
                }
            })
            builder.setCancelable(false)
            show()
        }
    }

    private fun registerUnzipListBroadcast() {
        mUnzipListReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                if (isDestroying) {
                    return
                }
                mUnzipListProgress = intent.getIntExtra(CloudUnzipBroadcastParams.EXTRA_UNZIP_LIST_PROGRESS, 0)
                NetDiskLog.d(TAG,
                        "mUnzipListReceiver mUnzipListProgress: $mUnzipListProgress taskid: $mCurrentTaskId")
                mUnzipListProgressText?.text = ("$mUnzipListProgress%")
                abortBroadcast()
            }
        }
        val filter = IntentFilter(CloudUnzipBroadcastParams.ACTION_UNZIP_LIST_PROGRESS)
        registerReceiver(mUnzipListReceiver, filter, Permissions.BROADCAST, null)
    }

    private fun registerUnzipDialogBroadcast() {
        mUnzipCopyReceiver = object : BroadcastReceiver() {
            override fun onReceive(context: Context, intent: Intent) {
                if (isDestroying) {
                    return
                }
                val type = intent.getIntExtra(CloudUnzipBroadcastParams.EXTRA_UNZIP_TASK_TYPE, 0)
                val status = intent.getStringExtra(CloudUnzipBroadcastParams.EXTRA_UNZIP_TASK_STATUS)
                val job = intent.getIntExtra(CloudUnzipBroadcastParams.EXTRA_UNZIP_JOB_TYPE, 0)
                when (type) {
                    CloudUnzipConstant.TYPE_UNZIP_COPY -> if (CloudUnzipConstant.TASK_STATUS_RUNNING == status) {
                        mUnzipCopyProgress = intent.getIntExtra(CloudUnzipBroadcastParams.EXTRA_UNZIP_TASK_PROGRESS, 0)
                        mUnzipCopyProgressText?.text = ("$mUnzipCopyProgress%")
                    }
                    CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD, CloudUnzipConstant.TYPE_UNZIP_PREVIEW ->
                        if (CloudUnzipConstant.TASK_STATUS_RUNNING == status) {
                            if (job == CloudUnzipConstant.TYPE_COPY_TASK_JOB) {
                                mUnzipCopyProgress = intent.getIntExtra(CloudUnzipBroadcastParams.EXTRA_UNZIP_TASK_PROGRESS, 0)
                                mUnzipCopyProgressText?.text = ("$mUnzipCopyProgress%")
                            } else if (job == CloudUnzipConstant.TYPE_DIFF_QUERY_TASK_JOB) {
                                mUnzipCopyProgress += ((100 - mUnzipCopyProgress) * 0.5).toInt()
                                mUnzipCopyProgressText?.text = ("$mUnzipCopyProgress%")
                            }
                        }
                    else -> finish()
                }
                abortBroadcast()
            }
        }
        val filter = IntentFilter(CloudUnzipBroadcastParams.ACTION_UNZIP_DIALOG_PROGRESS)
        registerReceiver(mUnzipCopyReceiver, filter, Permissions.BROADCAST, null)
    }

    /**
     * 开始解压动画
     */
    private fun startTransportAnim(dialog: Dialog) {
        // 开始动画
        val unzippingAnimView = dialog.findViewById<View>(R.id.unzipping_anim)
        val unzippingAnimation = AnimationUtils.loadAnimation(this, R.anim.cloud_unzipping) as AnimationSet
        unzippingAnimView.clearAnimation()
        unzippingAnimView.startAnimation(unzippingAnimation)
    }

    /**
     * 结束解压动画
     */
    private fun endTransportAnim(dialog: Dialog) {
        val unzippingAnimView = dialog.findViewById<View>(R.id.unzipping_anim)
        unzippingAnimView.clearAnimation()
    }

    private fun dismissLoadingDialog() {
        if (mProgressDialog?.isShowing == true) {
            mProgressDialog?.dismiss()
        }
    }

    private fun showLoadingDialog(text: String) {
        mProgressDialog?.dismiss()
        mProgressDialog = LoadingDialog.show(this, text).apply {
            setOnKeyListener(DialogInterface.OnKeyListener { dialog, keyCode, event ->
                if (keyCode == KeyEvent.KEYCODE_BACK) {
                    finish()
                    return@OnKeyListener true
                }
                false
            })
        }
    }

    private fun showGuideVipDialog(hasIncentiveVideo: Boolean, isUnzipped: Boolean) {
        if(guideBean == null){
            toast(R.string.unzip_error_file_system)
            finish()
        }
        vipGuideDialog = BusinessVipGuideDialog.newInstance(guideBean, mSize, hasIncentiveVideo,
                isUnzipped)
        vipGuideDialog?.setBusinessVipGuideView(this)
        vipGuideDialog?.show(supportFragmentManager, "guideDialog")
    }

    /**
     * 下载文件夹
     */
    fun addFolder2DownloadList(name: String, toPath: String, downloadType: Int) {
        if (TextUtils.isEmpty(name) || TextUtils.isEmpty(toPath)) {
            return
        }

        if (PermissionPresenter(this@BusinessUnzipGuideDialogActivity)
                        .requestPermissions(IPermission.STORAGE_GROUP_PERMISSION,
                                PermissionContentHelper.STORGE_PERMISSION)) {
            return
        }

        object : ParallelAsyncTask<Void, Void, Int>() {
            override fun doInBackground(vararg params: Void): Int? {
                val sb = StringBuilder()
                sb.append(CloudFileContract.Files.FILE_IS_DIRECTORY + "=" + CloudFileContract.Files.IS_FILE + " AND ")
                sb.append(CloudFileContract.Files.FILE_SERVER_PATH + " LIKE ?")
                val selection = sb.toString()
                val arg = "$toPath/%"
                val args = arrayOf(arg)
                NetDiskLog.d(TAG, "addFolder2DownloadList::selection = $selection arg: $arg")
                var objectCursor: ObjectCursor<CloudFile>? = null
                var cursor: Cursor? = null
                val fileList = ArrayList<CloudFile>()
                try {
                    cursor = NetDiskApplication.getInstance().contentResolver.query(
                            CloudFileContract.Files.buildFilesUri(AccountUtils.getInstance().bduss),
                            CloudFileContract.Query.PROJECTION, selection, args, null)
                    if (cursor == null || cursor.count == 0) {
                        return 0
                    }
                    objectCursor = ObjectCursor(cursor, CloudFile.FACTORY)
                    while (objectCursor.moveToNext()) {
                        val fw = objectCursor.model ?: continue
                        fileList.add(fw)
                    }
                    NetDiskLog.d(TAG, "fileList size = " + fileList.size)
                    NetWorkVerifier.reset()
                    if (!CollectionUtils.isEmpty(fileList)) {
                        val factory = CloudFileDownloadProcessorFactory(
                                FSCreateObjectApiCompManager.createNetdiskTaskGenerator(fileList, downloadType), null, OnAddDownloadTaskListener())

                        val downloadTaskManager = DownloadTaskManagerProxy(this@BusinessUnzipGuideDialogActivity)
                        downloadTaskManager.addDownloadListTask(fileList, factory, null,
                                DownloadTaskManager.DOWNLOAD_PRIORITY_NETDISK)
                    }
                } catch (e: Exception) {
                    NetDiskLog.e(TAG, "", e)
                } finally {
                    cursor?.close()
                    objectCursor?.close()
                }
                return fileList.size
            }

            override fun onPostExecute(result: Int?) {
                if (isDestroying) {
                    return
                }
                when {
                    result == 0 -> ToastHelper.showToast(R.string.download_empty_dir)
                    TextUtils.isEmpty(mCurrentTaskId) -> dismissLoadingDialog()
                    else -> {
                        ToastHelper.showToast(R.string.download_file_all_added)
                        mUnzipCopyProgress = 100
                        mUnzipCopyProgressText?.text = "$mUnzipCopyProgress%"
                        val intent = intent
                        setResult(Activity.RESULT_OK, intent)
                    }
                }
                finish()
            }
        }.execute()
    }

    private fun saveUnzipListTaskInfo() {
        val taskIds = PersonalConfig.getInstance().getString(CloudFileConfigKey.UNZIP_FILE_TASK_INFO)
        if (!TextUtils.isEmpty(mCurrentTaskId)) {
            val jsonObject = JSONObject()
            try {
                jsonObject.put("taskid", mCurrentTaskId)
                jsonObject.put("type", mType)
                jsonObject.put("path", mPath)
                jsonObject.put("subPath", mSubPath)
                jsonObject.put("size", mSize)
                jsonObject.put("product", mProduct)
                jsonObject.put("fsid", mFsid)
                jsonObject.put("primaryid", mPrimaryId)
                jsonObject.put("uk", mUk)
                jsonObject.put("extra", mExtra)
            } catch (e: JSONException) {
                NetDiskLog.e(TAG, e.message, e)
            }

            if (!TextUtils.isEmpty(taskIds)) {
                PersonalConfig.getInstance().putString(CloudFileConfigKey.UNZIP_FILE_TASK_INFO,
                        "$taskIds;$jsonObject")
            } else {
                PersonalConfig.getInstance().putString(CloudFileConfigKey.UNZIP_FILE_TASK_INFO, jsonObject.toString())
            }
            PersonalConfig.getInstance().commit()

            val save = PersonalConfig.getInstance().getString(CloudFileConfigKey.UNZIP_FILE_TASK_INFO)
            NetDiskLog.d(TAG, "save task info = $save")
        }
    }

    /**
     * 处理接口receiver返回的错误码
     *
     * @param bundle
     * @param errorMsg
     */
    private fun handleReceiverFailed(bundle: Bundle, errorMsg: Int) {
        when {
            BaseServiceHelper.isNetWorkError(bundle) -> // 网络不通
                ToastHelper.showToast(getString(R.string.unzip_network_exception_message,
                        getString(if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP)
                            R.string.unzip_svip_call
                        else
                            R.string.unzip_vip_call)))
            bundle.containsKey(BaseExtras.ERROR) -> {
                val errno = bundle.getInt(BaseExtras.ERROR)
                NetDiskLog.d(TAG, "handleReceiverFailed errno: $errno")
                when {
                    errno == UnzipErrorCodeHelper.ERROR_FILE_ENCRYPTION -> {
                        NetDiskLog.d(TAG, "showPasswordDialog mPwdDialogShowCount:$mPwdDialogShowCount")
                        if (mPwdDialogShowCount > CloudUnzipConstant.MAXIMUM_PASSWORD_ERROR) {
                            ToastHelper.showToast(R.string.unzip_password_error_over)
                            finish()
                            return
                        }

                        showPasswordDialog(mPwdDialogShowCount)
                        mPwdDialogShowCount++
                        return
                    }
                    UnzipErrorCodeHelper.isShowErrorDialog(errno) -> {
                        showErrorDialog(getString(UnzipErrorCodeHelper.getErrorTitleRes(errno, errorMsg)),
                                getString(UnzipErrorCodeHelper.getErrorContentRes(errno, errorMsg)))
                        return
                    }
                    UnzipErrorCodeHelper.isErrorByNoSpace(errno) -> {
                        showNoSpaceDialog()
                        return
                    }
                    errno == UnzipErrorCodeHelper.ERROR_NO_LONGER_VIP -> {
                        ToastHelper.showToast(R.string.unzip_file_copy_failed_notification_title)
                    }
                    else -> ToastHelper.showToast(UnzipErrorCodeHelper.getErrorCodeRes(errno, errorMsg))
                }
            }
            else -> ToastHelper.showToast(errorMsg)
        }
        finish()
    }

    private fun showNoSpaceDialog() {
        if (AccountUtils.getInstance().isSVip) {
            showErrorDialog(getString(R.string.dialog_tip_fail_title),
                    getString(R.string.dialog_tip_fail_unzip_description))
        } else {
            val noSpaceDialog = ConfigDialogFactory.buildDialog(getTopAvailableActivity(),
                    ConfigDialogStyle.DIALOG_STYLE_ONE_BUTTON,
                    ConfigDialogCategory.DIALOG_CATEGORY_SPACE,
                    ConfigDialogCategory.DIALOG_SPACE_ACTION_UNZIP, object : ConfigDialogCtrListener {

                override fun onOkBtnClick(link: String) {
                    if (TextUtils.isEmpty(link)) {
                        CommonWebViewActivity.startActivity(
                                getTopAvailableActivity(), ServerURL.getVipWap())
                    } else {
                        if (link.contains(ServerURL.getVipWap())) {
                            VipActivity.startActivity(getTopAvailableActivity(),
                                    VipActivity.FROM_UNZIP_VIP,
                                    VipActivity.TO_SVIP)
                        } else {
                            CommonWebViewActivity.startActivity(
                                    getTopAvailableActivity(), link)
                        }
                    }
                }

                override fun onCancelBtnClick() {
                    OnlineTaskManager.getInstance().handleActivity(
                            BaseApplication.getInstance().applicationContext,
                            OnlineActivityContract.ActivityType.NO_SPACE_ACTIVITY_TYPE,
                            OnlineTaskManager.FORM_POP)
                }
            })
            noSpaceDialog.setOnDismissListener { finish() }
            noSpaceDialog.show()
        }
    }

    private fun downloadZipDir() {
        val path = Setting.getDefaultSaveDir(this)
        if (!ensureDefaultFolderExist(path)) {
            finish()
            return
        } else {
            diffCloudFile()
        }
    }

    private fun ensureDefaultFolderExist(folderPath: String): Boolean {
        val fPath = File(folderPath)
        if (!fPath.exists() || !fPath.isDirectory) {
            if (!fPath.mkdir()) {
                ToastHelper.showToast(R.string.download_folder_not_exist)
                return false
            }
        }
        return true
    }

    private fun diffCloudFile() {
        val name = FileUtils.getFileName(mPath)
        PreviewServiceDelegate.queryDiffResult(applicationContext, mDownloadResultReceiver, name, mToPath, mType,
                mSubPaths, CloudUnzipBroadcastParams.SEND_DIALOG_BROADCAST, mCopySuccessFileSize)
    }

    private fun getUnzipListActivityType(): Int {
        return CloudUnzipConstant.TYPE_CAN_DOWNLOAD
    }

    // 有券未用，则显示使用云解压券的对话框
    private fun showUseCloudZipPrivilege() {
        val builder = DialogBuilder()
        val dialog = builder.buildCustomViewDialog(this, -1, R.string.popup_unzip_confirmbutton_text,
                R.string.cloud_zip_privilege_not_to_use, R.layout.dialog_cloud_zip_hint)
        val contentText = dialog.findViewById<TextView>(R.id.cloud_zip_text_content)
        contentText.text = getString(R.string.cloud_zip_pivilege_use_hint)
        // 单行文案居中显示；多行左对齐
        contentText.gravity = Gravity.CENTER
        builder.setOnDialogCtrListener(object : DialogCtrListener {
            override fun onOkBtnClick() {
                useCloudZipPrivilege()
            }

            override fun onCancelBtnClick() {
                finish()
            }
        })
        if (!isFinishing && null != dialog) {
            dialog.show()
//            dialog.setOnDismissListener {}
        }
    }

    //使用云解压券
    private fun useCloudZipPrivilege() {
        // 检查网络状态
        if (!ConnectivityState.isConnected(this)) {
            ToastHelper.showLengthLongToast(this, R.string.network_exception_message)
            finish()
            return
        }
        // 检查券失效
        val freeCount = mCloudUnzipPresenter?.unzipFreeCount ?: 0
        if (freeCount <= 0) {
            ToastHelper.showLengthLongToast(this, R.string.cloud_zip_privilege_out_of_use)
            finish()
        } else {
            getUnzipList(true)
        }
    }

    // 购买解压券
    private fun buyCloudZipPrivilege(fileSize: Long, payPageUrl: String, payResultPageUrl: String) {
        // 检查网络状态
        if (!ConnectivityState.isConnected(this)) {
            ToastHelper.showLengthLongToast(this, R.string.network_exception_message)
            finish()
            return
        }
        val level = AccountUtils.getInstance().level
        val freeCount = mCloudUnzipPresenter?.unzipFreeCount ?: 0
        val limitSize = AccountUtils.getInstance().getPrivilegeValue<ConfigCloudUnzip>(PrivilegeConstant.CLOUD_UNZIP).mLimitPackageSize
        val content: String
        val confirmTextId: Int
        val cancelTextId: Int
        // 检查是否为超级会员/会员
        if (level == AccountUtils.ILevel.SVIP) {
            content = getString(R.string.cloud_zip_privilege_use_hint_svip, CloudUnzipHelper.getUnzipSize(level))
            confirmTextId = R.string.cloud_zip_privilege_use_right_now
            showBuyCloudPrivilegeDialog(confirmTextId, -1, content, false)
        } else if (level == AccountUtils.ILevel.VIP && fileSize <= limitSize) {
            content = getString(R.string.cloud_zip_privilege_use_hint_vip, CloudUnzipHelper.getUnzipSize(level))
            confirmTextId = R.string.cloud_zip_privilege_use_right_now
            showBuyCloudPrivilegeDialog(confirmTextId, -1, content, false)
        } else if (freeCount > 0) {
            content = getString(R.string.cloud_zip_privilege_has_hint)
            confirmTextId = R.string.popup_unzip_confirmbutton_text
            cancelTextId = R.string.cloud_zip_privilege_use_then
            showBuyCloudPrivilegeDialog(confirmTextId, cancelTextId, content, true)
        } else {
            vipGuideDialog?.setDialogVisible(visible = false)
            BusinessSinglePrivilegeActivity.startActivityForResult(this, payPageUrl, payResultPageUrl, REQUEST_CODE_TO_BUY_CLOUD_ZIP)
        }
    }

    private fun showBuyCloudPrivilegeDialog(confirmTextId: Int, cancelTextId: Int, content: String,
                                            useFreePrivilege: Boolean) {
        val builder = DialogBuilder()
        val dialog = builder.buildCustomViewDialog(this, -1, confirmTextId, cancelTextId,
                R.layout.dialog_cloud_zip_hint)
        val contentText = dialog.findViewById<TextView>(R.id.cloud_zip_text_content)
        contentText.text = content
        builder.setOnDialogCtrListener(object : DialogCtrListener {
            override fun onOkBtnClick() {
                getUnzipList(useFreePrivilege)
            }

            override fun onCancelBtnClick() {
                finish()
            }
        })
        if (!isFinishing && null != dialog) {
            dialog.show()
//            dialog.setOnDismissListener {}
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            REQUEST_CODE_TO_BUY_CLOUD_ZIP -> when (resultCode) {
                Activity.RESULT_OK -> {
                    // 购买成功后查券，及时更新状态
                    FreePrivilegeHelper.getInstance().getUnZipFreePrivilege()
                    vipGuideDialog?.dismiss()
                    StatisticsKey.BUSINESS_CLOUD_ZIP_PRIVILEGE_BUY_SUCCESS.updateCount()
                    finish()
                }
                else -> vipGuideDialog?.setDialogVisible(visible = true)
            }
            PrivilegeChangedGuideHelper.REQUEST_CODE_TO_BE_VIP.toInt() -> {
                if (resultCode == Activity.RESULT_OK) finish()
            }
        }
    }

    override fun onUnzipListSuccess(dir: String?, size: Int) {
        // 跳转云解压文件列表页面
        NetDiskLog.d(TAG, "onUnzipListSuccess path: $dir size: $size")
        dismissLoadingDialog()
        mCloudUnzipPresenter?.getUnZipFreePrivilege()

        if (mType == CloudUnzipConstant.TYPE_UNZIP_LIST) {
            UnzipFileListActivity.startUnzipFileListActivity(this, mPath, mSubPath, getUnzipListActivityType(),
                    mProduct, mFsid, mPrimaryId, mUk, mExtra, mFileMd5, mPassword, mCloudFile)
        } else if (mType == CloudUnzipConstant.TYPE_UNZIP_DIR_LIST) {
            val data = intent
            setResult(Activity.RESULT_OK, data)
        }
        finish()
    }

    override fun onUnzipListFailed(bundle: Bundle?, errorMsg: Int) {
        NetDiskLog.d(TAG, "onUnzipListFailed")
        dismissLoadingDialog()
        bundle?.let {
            handleReceiverFailed(it, errorMsg)
        }
    }

    override fun onUnzipListOperating(taskid: String?) {
        NetDiskLog.d(TAG, "onUnzipListOperating taskid： $taskid")
        dismissLoadingDialog()
        mCloudUnzipPresenter?.getUnZipFreePrivilege()

        registerUnzipListBroadcast()
        showUnzipListDialog()
        mCurrentTaskId = taskid
        // 开始轮询网络请求
        mCloudUnzipPresenter?.queryUnzipListTask(taskid, this, mPath, mSubPath)
    }

    override fun onUnzipListQueryTaskSuccess(status: String?, errno: Int) {
        if (CloudUnzipConstant.TASK_STATUS_SUCCESS == status) {
            mUnzipListProgress = 100
            mUnzipListProgressText?.text = "$mUnzipListProgress%"
            if (mType == CloudUnzipConstant.TYPE_UNZIP_LIST) {
                UnzipFileListActivity.startUnzipFileListActivity(this, mPath, mSubPath, getUnzipListActivityType(),
                        mProduct, mFsid, mPrimaryId, mUk, mExtra, mFileMd5, mPassword, mCloudFile)
            } else if (mType == CloudUnzipConstant.TYPE_UNZIP_DIR_LIST) {
                val intent = intent
                setResult(Activity.RESULT_OK, intent)
            }
            finish()
        } else if (CloudUnzipConstant.TASK_STATUS_FAILED == status) {
            // 错误码提示
            val bundle = Bundle()
            bundle.putInt(BaseExtras.ERROR, errno)
            handleReceiverFailed(bundle, R.string.unzip_file_list_error)
        }
    }

    override fun onUnzipListQueryTaskFailed(bundle: Bundle?, errorMsg: Int) {
        bundle?.let {
            handleReceiverFailed(it, errorMsg)
        }
    }

    override fun onUnzipCopySuccess() {
        NetDiskLog.d(TAG, "onUnzipCopySuccess")
        // copy接口全部走异步操作，目前不会走到这里，此处为兼容
        when (mType) {
            CloudUnzipConstant.TYPE_UNZIP_COPY -> {
                dismissLoadingDialog()
                if (AccountUtils.getInstance().isVip) {
                    ToastHelper.showToast(getString(R.string.unzip_file_copy_done_toast,
                            getString(if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP)
                                R.string.unzip_svip_call
                            else
                                R.string.unzip_vip_call)))
                } else {
                    ToastHelper.showToast(getString(R.string
                            .unzip_free_file_copy_done_notification_title))
                }
                val intent = Intent()
                intent.putExtra(CloudUnzipConstant.EXTRA_KEY_TOPATH, mToPath)
                setResult(Activity.RESULT_OK, intent)
                finish()
            }
            CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD -> // 解压完成，开始diff和下载
                downloadZipDir()
            CloudUnzipConstant.TYPE_UNZIP_PREVIEW -> // 解压完成，开始diff和预览下载
                diffCloudFile()
        }
    }

    override fun onUnzipCopyFailed(bundle: Bundle?, errorMsg: Int) {
        NetDiskLog.d(TAG, "onUnzipCopyFailed")
        dismissLoadingDialog()
        bundle?.let {
            handleReceiverFailed(it, errorMsg)
        }
    }

    override fun onUnzipCopyOperating(taskid: String?) {
        NetDiskLog.d(TAG, "onUnzipCopyOperating taskid： $taskid")
        dismissLoadingDialog()
        registerUnzipDialogBroadcast()
        showUnzipCopyDialog()
        mCurrentTaskId = taskid
        // 开始轮询网络请求
        mCloudUnzipPresenter?.queryUnzipCopyTask(taskid, FileUtils.getFileName(mPath), mToPath, this, mType)
    }

    override fun onUnzipCopyQueryTaskSuccess(status: String?, errno: Int, successFileSize: Int) {
        when (status) {
            CloudUnzipConstant.TASK_STATUS_SUCCESS -> when (mType) {
                CloudUnzipConstant.TYPE_UNZIP_COPY -> {
                    mUnzipCopyProgress = 100
                    mUnzipCopyProgressText?.text = "$mUnzipCopyProgress%"
                    val intent = intent
                    intent.putExtra(CloudUnzipConstant.EXTRA_KEY_TOPATH, mToPath)
                    setResult(Activity.RESULT_OK, intent)
                    if (AccountUtils.getInstance().isVip) {
                        ToastHelper.showToast(getString(R.string.unzip_file_copy_done_toast,
                                getString(if (AccountUtils.getInstance().level == AccountUtils.ILevel.SVIP)
                                    R.string.unzip_svip_call
                                else
                                    R.string.unzip_vip_call)))
                    } else {
                        ToastHelper.showToast(getString(R.string
                                .unzip_free_file_copy_done_notification_title))
                    }
                    finish()
                }
                CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD -> {
                    mCopySuccessFileSize = successFileSize
                    NetDiskLog.d(TAG, "receive CopySuccessFileSize: $mCopySuccessFileSize")
                    if (mCopySuccessFileSize > 0) {
                        // 解压完成，开始diff和下载
                        downloadZipDir()
                    } else {
                        ToastHelper.showToast(R.string.download_empty_dir)
                        finish()
                    }
                }
                CloudUnzipConstant.TYPE_UNZIP_PREVIEW -> diffCloudFile()
                else -> finish()
            }
            CloudUnzipConstant.TASK_STATUS_FAILED -> {
                // 对预览文件中存在同名文件的错误码，单独处理,同时云解压copy任务的-30同普通转存任务一样，表示文件已经存在
                if (mType == CloudUnzipConstant.TYPE_UNZIP_PREVIEW && (errno == UnzipErrorCodeHelper.ERROR_FILE_EXIST || errno == ErrorCode.ERROR_TRANSFER_FILE_AREADY_EXIST)) {
                    diffCloudFile()
                    return
                }
                val bundle = Bundle()
                bundle.putInt(BaseExtras.ERROR, errno)
                handleReceiverFailed(bundle, R.string.unzip_file_copy_error)
            }
        }
    }

    override fun onUnzipCopyQueryTaskFailed(bundle: Bundle?, errorMsg: Int) {
        bundle?.let {
            handleReceiverFailed(it, errorMsg)
        }
    }

    override fun queryUnzipStatusFinish(isUnzipped: Boolean) {
        vipGuideLiveData.value = Triple(vipGuideLiveData.value?.first, isUnzipped,vipGuideLiveData
                .value?.third)
    }


     override fun canCloseGuideView()  = mCanCloseGuideView
    /**
     * 是否激励
     */
    private fun isShowRewardVideo() {
        IncentiveVideoHelper.init(this)
        IncentiveVideoHelper.check(this, IncentiveVideoHelper.CLOUD_UNCOMP, object :
                DialogCallback() {
             override fun onRightClose() {
                vipGuideDialog?.dismiss()
                StatisticsKey.BUSINESS_UNZIP_CLOSE_INCENTIVE.updateCount()
                finish()
            }

            override fun beforeDoStep() {
                super.beforeDoStep()
                vipGuideDialog?.setDialogVisible(false)
                StatisticsKey.BUSINESS_UNZIP_SHOW_INCENTIVE.updateCount()
            }

            override fun onCancel() {
                mCanCloseGuideView = true
                vipGuideDialog?.setDialogVisible(true)
                StatisticsKey.BUSINESS_UNZIP_CLOSE_INCENTIVE.updateCount()
            }

            override fun onSuccess(integer: Int?, s: String?, s1: String?, b: Boolean) {
                mCanCloseGuideView = false
                vipGuideDialog?.dismiss()
                val config = IncentiveVideoConfig(
                        ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.INCENTIVE_VIDEO_CONFIG))

                IncentiveVideoHelper.showIncentiveVideoDialog(this@BusinessUnzipGuideDialogActivity,
                        s, s1, b, config.onlineCloudUnzipUrl,
                        R.drawable.popup_inventivevideo_image_unzip,
                        R.string.popup_unzip_confirmbutton_text,
                        object :IncentiveSuccessDialogClickListener {
                            override fun learnMoreClick() {
                                if (b) {
                                    isShowRewardVideo()
                                } else {
                                    val intent = RichMediaActivity
                                            .getStartIntent(this@BusinessUnzipGuideDialogActivity, config.onlineCloudUnzipUrl)
                                    if (intent != null) {
                                        startActivity(intent)
                                    }
                                    finish()
                                }
                            }

                            override fun onUseNowBtnClick() {
                                getUnzipList(true)
                            }

                            override fun onCloseBtnClick() {
                                finish()
                            }

                        })

                StatisticsKey.BUSINESS_UNZIP_INCENTIVE_SUCCESS.updateCount()
            }

        }.dialogStyle {
            rightEnter().rightClose()
        })
    }

    private fun queryIncentiveRemains() {
        IncentiveVideoHelper.init(this)
        IncentiveVideoHelper.getRemains(this, IncentiveVideoHelper.CLOUD_UNCOMP,
                object : IncentivePropertyCallback {
            override fun onGetRemindCount(size: Int, currentStep: Int?, totalStep: Int?) {
                vipGuideLiveData.value = Triple(size > 0, vipGuideLiveData.value?.second,
                        vipGuideLiveData.value?.third)
            }
        })
    }

    override fun showIncentive() {
        isShowRewardVideo()
    }

    override fun goSinglePay(payPageUrl: String, payResultPageUrl: String) {
        buyCloudZipPrivilege(mSize, payPageUrl, payResultPageUrl)
    }


    /**
     * 获取增量数据的结果接收器
     */
    private class DownloadResultReceiver
    /**
     * Create a new ResultReceive to receive results. Your [.onResult] method will be called from the
     * thread running <var>handler</var> if given, or from an arbitrary thread if null.
     *
     * @param reference
     * @param handler
     */
    internal constructor(reference: BusinessUnzipGuideDialogActivity, handler: Handler) : WeakRefResultReceiver<BusinessUnzipGuideDialogActivity>(reference, handler) {

        /**
         * @param resultCode
         * @param resultData
         *
         * @see android.os.ResultReceiver.onReceiveResult
         */
        override fun onResult(unzipShowDialogActivity: BusinessUnzipGuideDialogActivity,
                              resultCode: Int, resultData: Bundle) {
            when (resultCode) {
                BaseStatus.SUCCESS -> when {
                    unzipShowDialogActivity.mType == CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD -> {
                        unzipShowDialogActivity.mToPath?.let { toPath ->
                            val name = FileUtils.getFileName(toPath)
                            unzipShowDialogActivity.addFolder2DownloadList(name,
                                    toPath, SmoothVideoDownloadHelper.ORIGN_TYPE)
                        }
                    }
                    unzipShowDialogActivity.mType == CloudUnzipConstant.TYPE_UNZIP_PREVIEW -> {
                        if (TextUtils.isEmpty(unzipShowDialogActivity.mCurrentTaskId)) {
                            unzipShowDialogActivity.dismissLoadingDialog()
                        } else {
                            unzipShowDialogActivity.mUnzipCopyProgress = 100
                            if (unzipShowDialogActivity.mUnzipCopyProgressText != null) {
                                unzipShowDialogActivity.mUnzipCopyProgressText?.text = "${unzipShowDialogActivity.mUnzipCopyProgress}%"
                            }
                        }
                        val intent = unzipShowDialogActivity.intent
                        unzipShowDialogActivity.setResult(Activity.RESULT_OK, intent)
                        unzipShowDialogActivity.finish()
                    }
                    else -> unzipShowDialogActivity.finish()
                }
                BaseStatus.FAILED -> if (unzipShowDialogActivity.mType == CloudUnzipConstant.TYPE_UNZIP_DOWNLOAD) {
                    unzipShowDialogActivity.handleReceiverFailed(resultData, R.string.unzip_file_download_error)
                } else if (unzipShowDialogActivity.mType == CloudUnzipConstant.TYPE_UNZIP_PREVIEW) {
                    unzipShowDialogActivity.handleReceiverFailed(resultData, R.string.unzip_file_preview_error)
                }
            }
        }
    }

    override fun onGetVipGuideResult(success: Boolean, bean: UnzipVipGuideBean?) {
        guideBean = bean
        vipGuideLiveData.value = Triple(vipGuideLiveData.value?.first, vipGuideLiveData.value?.second,
                success)
    }

    public override fun onDestroy() {
        super.onDestroy()
        mErrorDialog?.safeDismiss()
        mUnzipListDialog?.safeDismiss()
        mUnzipCopyDialog?.safeDismiss()
        mProgressDialog?.safeDismiss()
        if (mUnzipListReceiver != null) {
            unregisterReceiver(mUnzipListReceiver)
        }
        if (mUnzipCopyReceiver != null) {
            unregisterReceiver(mUnzipCopyReceiver)
        }
    }
}