package com.baidu.netdisk.ui.quota

import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.support.v4.content.ContextCompat
import android.support.v4.content.LocalBroadcastManager
import android.text.style.ForegroundColorSpan
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.R
import com.baidu.netdisk.base.storage.config.ConfigCapacityBar
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.businessplatform.extension.addUrlParams
import com.baidu.netdisk.businessplatform.extension.highLight
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.EXPAND_STORAGE_FROM_QUOTA_LIMIT_DIALOG
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_DIALOG_CLEAN_CLICK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_DIALOG_CLOSE
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_EXPAND_STORAGE_CLICK
import com.baidu.netdisk.kotlin.extension.Bundle
import com.baidu.netdisk.ui.ITabSwitchable
import com.baidu.netdisk.ui.MainActivity
import com.baidu.netdisk.ui.businessplatform.extension.toast
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import kotlinx.android.synthetic.main.dialog_quota_capacity_limit_notice.*

/**
 * @author chencaixing
 * @time 2019-11-25 14:45
 */
class QuotaCapacityLimitDialog : DialogFragment() {

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        dialog.window?.apply {
            setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))
        }
        return inflater.inflate(R.layout.dialog_quota_capacity_limit_notice, container, false)
    }

    companion object {
        private const val LIMIT_DAY = "limit_days"
        fun getInstance(days: String?) = QuotaCapacityLimitDialog().apply {
            arguments = Bundle {
                LIMIT_DAY - days
            }
        }
    }


    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        dialog?.setCanceledOnTouchOutside(false)
        val daysLeft = arguments?.getString(LIMIT_DAY)?.toString() ?: ""
        activity?.apply {
            this@QuotaCapacityLimitDialog.tv_content.text =
                    getString(R.string.quota_limit_dialog_content, daysLeft).highLight("${daysLeft}天",
                            ForegroundColorSpan(ContextCompat.getColor(this, R.color.bg_homepage_title_notice)))
        }

        iv_close.setOnClickListener {
            QUOTA_LIMIT_DIALOG_CLOSE.updateCount()
            dismissAllowingStateLoss()
        }
        btn_go_delete_file.setOnClickListener {
            activity?.apply {
                QUOTA_LIMIT_DIALOG_CLEAN_CLICK.updateCount()
                if (this is MainActivity || parent is MainActivity) {
                    val intentResourceTab = Intent(ITabSwitchable.Action.SWITCH_TAB)
                            .putExtra(ITabSwitchable.Extra.TAB_ID, ITabSwitchable.Id.FILELIST_ACTIVITY)
                    LocalBroadcastManager.getInstance(applicationContext).sendBroadcast(intentResourceTab)
                } else {
                    val intent = Intent(this, MainActivity::class.java)
                    intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP or Intent.FLAG_ACTIVITY_SINGLE_TOP)
                    intent.putExtra(ITabSwitchable.Extra.TAB_ID, ITabSwitchable.Id.FILELIST_ACTIVITY)
                    startActivity(intent)
                }
                toast(R.string.clean_netdisk_file)
            }
            dismissAllowingStateLoss()
        }
        btn_expand_storage_capacity.setOnClickListener {
            val capacityBarInfo = ConfigCapacityBar(
                    ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.CAPACITY_BAR_CONFIG)
            )
            if (!capacityBarInfo.mManageSpaceUrl.isNullOrEmpty()) {
                QUOTA_LIMIT_EXPAND_STORAGE_CLICK.updateCount(EXPAND_STORAGE_FROM_QUOTA_LIMIT_DIALOG)
                val url = capacityBarInfo
                        .mManageSpaceUrl.addUrlParams(QuotaOverrunHelper.QUOTA_FROM,
                        QuotaOverrunHelper.FROM_DIALOG.toString())
                startActivity(RichMediaActivity.getStartIntent(context, url))
            }
            dismissAllowingStateLoss()
        }
    }


     fun showDialog(manager: FragmentManager?, tag: String?) {
        val ft = manager?.beginTransaction()
        ft?.add(this, tag)
        ft?.commitAllowingStateLoss()
    }

}