package com.baidu.netdisk.ui.quota

import android.arch.lifecycle.Observer
import android.content.Intent
import android.os.Bundle
import android.support.v4.app.FragmentActivity
import android.view.KeyEvent
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.businessplatform.extension.getViewModel
import com.baidu.netdisk.cloudfile.io.model.QuotaOverrunResponse
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.kotlin.extension.invoke
import com.baidu.netdisk.platform.extension.addFragment
import com.baidu.netdisk.platform.extension.getFragmentByTag
import com.baidu.netdisk.platform.extension.replaceFragment
import com.baidu.netdisk.ui.businessplatform.extension.toast
import com.baidu.netdisk.ui.quota.viewmodel.PointViewModel
import com.baidu.netdisk.ui.quota.viewmodel.QuotaOverrunViewModel
import com.baidu.netdisk.ui.quota.viewmodel.onNoOverLimit

/**
 * 空间容量超限被封禁的页面
 */
@Tag("QuotaLockedActivity")
class QuotaLockedActivity: BaseActivity() {
    private var pointViewModel: PointViewModel? = null
    private var quotaInfoViewModel: QuotaOverrunViewModel? = null

    private var firstEnter: Boolean = true

    companion object {
        const val EXTRA_QUOTA_INFO = "EXTRA_QUOTA_INFO"

        fun startActivity(activity: FragmentActivity, quotaInfo: QuotaOverrunResponse) {

            activity.startActivity(Intent(activity, QuotaLockedActivity::class.java)(){
                EXTRA_QUOTA_INFO - quotaInfo
            })
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        initFragment()
        pointViewModel = getViewModel()
        quotaInfoViewModel = getViewModel()
        initObserve()
    }
    override fun getLayoutId(): Int {
        return R.layout.activity_quota_locked
    }

    override fun initView() {
    }

    override fun onResume() {
        super.onResume()
        if (firstEnter) {
            firstEnter = false
        } else {
            quotaInfoViewModel?.fetchQuotaLimitData()
        }
    }

    private fun initFragment() {
        val fragment = QuotaLockedFragment().apply {
            arguments = intent.extras
        }
        addFragment(fragment, R.id.content)
    }

    private fun initObserve() {
        pointViewModel?.observeConsumePointResult(this, Observer {
            if (it.d() == true) {
                val tag = QuotaPointUnlockFragment::class.java.name
                if(null == getFragmentByTag(tag)) {
                    val fragment = QuotaPointUnlockFragment()
                    replaceFragment(fragment, R.id.content, tag)
                }
            } else {
                // 积分扣除失败
                toast(R.string.quota_unlock_use_point_failed)
            }
        })

        quotaInfoViewModel?.observeQuotaLimitData(this) {
            it?.onNoOverLimit {
                val tag = QuotaRealUnlockFragment::class.java.name
                if (null == getFragmentByTag(tag)) {
                    val fragment =
                            QuotaRealUnlockFragment.getInstance(it.user?.nick, it.total)
                    replaceFragment(fragment, R.id.content, tag)
                }
            }
        }
    }

    override fun onKeyUp(keyCode: Int, event: KeyEvent): Boolean {
        return keyCode == KeyEvent.KEYCODE_BACK
    }
}