package com.baidu.netdisk.ui.quota

import android.arch.lifecycle.Observer
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountChangeHandler
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.backup.pim.contact.ContactSDKManager
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.base.storage.config.ConfigCapacityBar
import com.baidu.netdisk.base.storage.config.ConfigQuotaLimit
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.businessplatform.extension.addUrlParams
import com.baidu.netdisk.businessplatform.extension.formatStorageSize
import com.baidu.netdisk.businessplatform.extension.getActivityViewModel
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.EXPAND_STORAGE_FROM_QUOTA_LOCK_PAGE
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_CHANGE_ACCOUNT_CLICK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_EXPAND_STORAGE_CLICK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_FEEDBACK_CLICK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_MANAGE_NOTICE_CLICK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_PAGE_SHOW
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_POINT_CLICK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_POINT_NOT_ENOUGH
import com.baidu.netdisk.cloudfile.io.model.QuotaOverrunResponse
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.statistics.NetdiskStatisticsLog
import com.baidu.netdisk.stats.NetdiskStatsEngine
import com.baidu.netdisk.ui.businessplatform.extension.toast
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.quota.QuotaLockedActivity.Companion.EXTRA_QUOTA_INFO
import com.baidu.netdisk.ui.quota.viewmodel.PointViewModel
import com.baidu.netdisk.ui.quota.viewmodel.QuotaOverrunViewModel
import com.baidu.netdisk.ui.widget.BaseFragment
import com.baidu.ufosdk.UfoSDK
import kotlinx.android.synthetic.main.fragment_quota_locked.*

class QuotaLockedFragment: BaseFragment() {

    private var viewModel: PointViewModel? = null
    private var quotaInfoViewModel: QuotaOverrunViewModel? = null
    private var curPoint: Int? = 0

    private var quotaInfo: QuotaOverrunResponse? = null

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        super.onCreateView(inflater, container, savedInstanceState)
        return inflater.inflate(R.layout.fragment_quota_locked, null, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        viewModel = getActivityViewModel()
        quotaInfoViewModel = getActivityViewModel()
        viewModel?.getMyPoint(context!!, this, Observer {
            curPoint = it
        })

        quotaInfo = arguments?.getParcelable(EXTRA_QUOTA_INFO)

        initView()
        QUOTA_LIMIT_PAGE_SHOW.updateCount()
    }

    fun initView() {
        val config = ConfigQuotaLimit(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey
                .ConfigType.QUOTA_LIMIT_CONFIG))

        quotaInfo?.apply {
            GlideHelper.getInstance().displayImageFromNetwork(user?.avatar ?: AccountUtils
                    .getInstance().cloudAvatarURL, 0, true, avatar, null)
            user_name.text = user?.nick ?: AccountUtils.getInstance().displayName
            initQuotaView(used, total)
        }

        if (config.proclamationUrl.isNullOrBlank()) {
            enter_about.gone()
            bottom_desc.gone()
            top_desc.gravity = Gravity.CENTER_HORIZONTAL
        }

        enter_about.setOnClickListener {
            if (!config.proclamationUrl.isNullOrBlank()) {
                val url = config.proclamationUrl.addUrlParams(QuotaOverrunHelper.QUOTA_FROM,
                        QuotaOverrunHelper.FROM_QUOTA_LOCK_NOTICE.toString())
                startActivity(RichMediaActivity.getStartIntent(context, url))
            }

            QUOTA_LIMIT_MANAGE_NOTICE_CLICK.updateCount()
        }

        feed_back.setOnClickListener {
            startActivity(UfoSDK.getStartFaqIntent(context))
            QUOTA_LIMIT_FEEDBACK_CLICK.updateCount()
        }
        change_account.setOnClickListener {
            NetdiskStatisticsLog.updateCount(NetdiskStatisticsLog.StatisticsKeys.LOGOUT_COUNT)
            NetdiskStatsEngine.getInstance().uploadAll()
            ContactSDKManager.getInstance().cancelContactSync(ContactSDKManager.PIM_CANCEL_SYNC_BY_LOGOUT)
            AccountChangeHandler.logout(activity, true,
                    AccountChangeHandler.LOGOUT_SOURCE_NORMAL)
            QUOTA_LIMIT_CHANGE_ACCOUNT_CLICK.updateCount()
        }

        buy_quota.setOnClickListener {
            val capacityBarInfo = ConfigCapacityBar(
                    ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.CAPACITY_BAR_CONFIG))
            val url = capacityBarInfo
                    .mManageSpaceUrl.addUrlParams(QuotaOverrunHelper.QUOTA_FROM,
                    QuotaOverrunHelper.FROM_QUOTA_LOCK_EXPAND_STORAGE.toString())
            startActivity(RichMediaActivity.getStartIntent(activity, url))
            QUOTA_LIMIT_EXPAND_STORAGE_CLICK.updateCount(EXPAND_STORAGE_FROM_QUOTA_LOCK_PAGE)
        }

        temporary_unlock.setOnClickListener {
            if (isPointEnough()) {
                val dialog = PointUnfreezeDialog.newInstance(curPoint)
                dialog?.show(fragmentManager, PointUnfreezeDialog::class.java.name)
            } else {
                toast(R.string.quota_unlock_no_enough_point)
                QUOTA_LIMIT_POINT_NOT_ENOUGH.updateCount()
            }
            QUOTA_LIMIT_POINT_CLICK.updateCount()
        }
        quotaInfoViewModel?.observeQuotaLimitData(this) {
            it?.apply {
                initQuotaView(used, total)
            }
        }
    }

    private fun initQuotaView(used: Long, total: Long) {
        quota_used.text = getString(R.string.quota_used, used.formatStorageSize(0),
                total.formatStorageSize(0))
        if (total != 0L) {
            quota_over_limit.show()
            quota_over_limit.text = getString(R.string.quota_used_percentage,
                    (used * 1.0 / total * 100).toInt())
        } else {
            quota_over_limit.gone()
        }
    }

    private fun isPointEnough(): Boolean {
        return curPoint?.let {
            it >= 5
        } ?: false
    }
}