package com.baidu.netdisk.ui.quota

import android.animation.Animator
import android.animation.Animator.AnimatorListener
import android.animation.ValueAnimator
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.R
import com.baidu.netdisk.base.storage.config.ConfigQuotaLimit
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.businessplatform.extension.formatStorageSize
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.EXPAND_STORAGE
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_UNFREEZE_SUCCESS_ENTER_NETDISK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_UNFREEZE_SUCCESS_SHOW
import com.baidu.netdisk.kotlin.extension.Bundle
import com.baidu.netdisk.ui.ITabSwitchable
import com.baidu.netdisk.ui.businessplatform.extension.startMainActivity
import com.baidu.netdisk.ui.widget.BaseFragment
import kotlinx.android.synthetic.main.fragment_quota_unlock.*

/**
 * 付费解冻成功的页面
 */
class QuotaRealUnlockFragment: BaseFragment() {

    private var animator: ValueAnimator? = null
    private var userName: String = ""
    private var quota: Long = 0L


    companion object{
        private const val USER_NAME = "USER_NAME"
        private const val USE_QUOTA = "USE_QUOTA"

        fun getInstance(userName: String?, quota: Long): QuotaRealUnlockFragment{
            return QuotaRealUnlockFragment().apply {
                arguments = Bundle {
                    USER_NAME - userName
                    USE_QUOTA - quota
                }
            }
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        super.onCreateView(inflater, container, savedInstanceState)
        return inflater.inflate(R.layout.fragment_quota_unlock, null, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        userName = arguments?.getString(USER_NAME) ?: ""
        quota = arguments?.getLong(USE_QUOTA) ?: 0
        initView()
        QUOTA_LIMIT_UNFREEZE_SUCCESS_SHOW.updateCount(EXPAND_STORAGE)
    }

    override fun onPause() {
        super.onPause()
        activity?.finish()
    }

    fun initView() {
        val config = ConfigQuotaLimit(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey
                .ConfigType.QUOTA_LIMIT_CONFIG))

        user_name.text = getString(R.string.quota_unlock_top_desc, userName)

        top_desc.text = getString(R.string.quota_real_unlock_desc, quota.formatStorageSize(0))

        bottom_desc.text = config.payEducationalCopyBottomDesc

        QUOTA_LIMIT_UNFREEZE_SUCCESS_ENTER_NETDISK.updateCount(EXPAND_STORAGE)
        lottie_anim.playAnimation()

        enter_netdisk.setOnClickListener {
            initEnterNetdiskAnim()
        }
    }

    private fun initEnterNetdiskAnim() {
        animator = ValueAnimator.ofFloat(0f, 1f).apply {
            duration = 400L
            addUpdateListener {
                root_layout.alpha = 1 - it.animatedValue.toString().toFloat()
                root_layout.scaleX = 1 + it.animatedValue.toString().toFloat()
                root_layout.scaleY = 1 + it.animatedValue.toString().toFloat()
            }
            start()
        }

        animator?.addListener(object : AnimatorListener{
            override fun onAnimationRepeat(animation: Animator?) {
            }

            override fun onAnimationEnd(animation: Animator?) {
                activity?.apply {
                    startMainActivity(ITabSwitchable.Id.HOME_ACTIVITY)
                    finish()
                }

            }

            override fun onAnimationCancel(animation: Animator?) {
            }

            override fun onAnimationStart(animation: Animator?) {
            }
        })
    }

    override fun onDestroy() {
        super.onDestroy()
        animator?.apply {
            removeAllUpdateListeners()
            cancel()
        }
    }
}