package com.baidu.netdisk.ui.quota.viewmodel

import android.arch.lifecycle.MutableLiveData
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.support.v4.app.FragmentActivity
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.mypoint.IMyPoint
import com.baidu.netdisk.mypoint.model.MyPointResponse
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView

@Tag("PointRepository")
class PointRepository(private val pointManager: IMyPoint) {
    private val _pointLiveData = MutableLiveData<Int>()
    val pointLiveData
        get() = _pointLiveData

    private val _isUnlockSuccess = MutableLiveData<Boolean>()
    val isUnlockSuccess
        get() = _isUnlockSuccess

    fun queryMyPoint(context: Context) {
        pointManager.myPointQuery(context, QueryMyPointResultReceiver(this, Handler(), null),
                AccountUtils.getInstance().bduss, AccountUtils.getInstance().uid)
    }

    fun consumePoint(activity: FragmentActivity, points: Int, eventId: Int) {
        pointManager.consumePoint(activity, ConsumePointResultReceiver(this,
                Handler(), null), AccountUtils.getInstance().bduss, AccountUtils.getInstance().uid,
                points, eventId)
    }

    inner class QueryMyPointResultReceiver(reference: PointRepository, handler: Handler, view: ResultView?) :
            BaseResultReceiver<PointRepository>(reference, handler, view) {
        override fun onSuccess(reference: PointRepository, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            resultData?.apply {
                val myPoint = getParcelable<MyPointResponse>(BaseExtras.RESULT)
                _pointLiveData.value = myPoint?.balance?.toIntOrNull()
            }
        }
    }

    inner class ConsumePointResultReceiver(reference: PointRepository, handler: Handler, view: ResultView?) :
            BaseResultReceiver<PointRepository>(reference, handler, view) {
        override fun onSuccess(reference: PointRepository, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            _isUnlockSuccess.value = true
            "积分解冻成功".d()
        }

        override fun onFailed(reference: PointRepository, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            _isUnlockSuccess.value = false
            "积分解冻失败".d()
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}