package com.baidu.netdisk.ui.quota.viewmodel

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.LifecycleOwner
import android.arch.lifecycle.Observer
import android.support.v4.app.FragmentActivity
import android.view.View
import android.view.ViewStub
import com.baidu.netdisk.BuildConfig
import com.baidu.netdisk.NetDiskApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.base.storage.config.ConfigCapacityBar
import com.baidu.netdisk.base.storage.config.ConfigQuotaLimit
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.businessplatform.extension.addUrlParams
import com.baidu.netdisk.businessplatform.extension.secondsToDays
import com.baidu.netdisk.businessplatform.extension.updateCount
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_DIALOG_SHOW
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_HOME_PAGE_BANNER_CLICK
import com.baidu.netdisk.businessplatform.statistics.StatisticsKey.QUOTA_LIMIT_HOME_PAGE_BANNER_SHOW
import com.baidu.netdisk.cloudfile.io.model.QuotaOverrunResponse
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.home.HomePageFragment
import com.baidu.netdisk.ui.quota.QuotaCapacityLimitDialog
import com.baidu.netdisk.ui.quota.QuotaLockedActivity
import com.baidu.netdisk.ui.quota.QuotaOverrunHelper
import com.baidu.netdisk.ui.quota.QuotaOverrunHelper.FROM_HOME_PAGE_BANNER
import com.baidu.netdisk.ui.quota.QuotaOverrunHelper.QUOTA_FROM
import com.baidu.netdisk.ui.widget.CheckableItemLayout
import kotlinx.android.synthetic.main.view_homepage_title_notice.view.*

@Tag("QuotaOverrunViewModel")
class QuotaOverrunViewModel(application: Application) : AndroidViewModel(application) {

    private val repository = QuotaOverrunRepository()

    fun checkHomePageQuotaLimit(fragment: HomePageFragment, viewStub: ViewStub?) {
        if(!quotaLimitKey()) return
        val titleView = fragment.view?.findViewById<View>(R.id.rl_quota)
        fun showTitleView(view: View?, title: String? = null) {
            view?.apply {
                tv_title.text = title
                tv_des.text = context.getString(R.string.quota_limit_know_more)
                tv_des.setOnClickListener {
                    val capacityBarInfo = ConfigCapacityBar(
                            ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.CAPACITY_BAR_CONFIG)
                    )
                   val url =  ConfigQuotaLimit(ServerConfigKey.getCfgConfigBodyByType
                    (ServerConfigKey.ConfigType.QUOTA_LIMIT_CONFIG)).quotaCompositionUrl.takeIf {
                        !it.isNullOrEmpty()
                    }?.addUrlParams(QUOTA_FROM, FROM_HOME_PAGE_BANNER.toString())
                   ?:capacityBarInfo.mManageSpaceUrl
                    if (!url.isNullOrEmpty()) {
                        fragment.activity?.startActivity(RichMediaActivity.getStartIntent(context,
                                url))
                    }
                    QUOTA_LIMIT_HOME_PAGE_BANNER_CLICK.updateCount()
                }
            }
            view?.show()
        }
        observeQuotaLimitData(fragment) {
            it.onBan {
                titleView?.gone()
            }.onNoOverLimit {
                titleView?.gone()
            }.onOverLimit {
                if (fragment.editMode != CheckableItemLayout.CHOICE_MODE_MULTIPLE) {
                    viewStub?.show()
                    QUOTA_LIMIT_HOME_PAGE_BANNER_SHOW.updateCount()
                    showTitleView(fragment.view?.findViewById(R.id.rl_quota), fragment
                            .activity?.getString(R.string
                            .quota_limit_over_notice, it))
                }
            }
        }
    }


    fun fetchQuotaLimitData() {
        if(!quotaLimitKey()) return
        repository.queryQuotaOverrunInfo(NetDiskApplication.getInstance())
    }

    fun observeQuotaLimitData(lifecycleOwner: LifecycleOwner, onResult: (QuotaOverrunResponse?) -> Unit) {
        repository.quotaOverrunLiveData.observe(lifecycleOwner, Observer {
            onResult(it)
        })
    }

    fun observeQuotaLimitData(activity: FragmentActivity) {
        if(!quotaLimitKey()) return
        observeQuotaLimitData(activity) { response ->
            response.onBan {
                // 启动封禁页面
                QuotaLockedActivity.startActivity(activity, it)
            }.onOverLimit { leftDays ->
                if (!QuotaOverrunHelper.hasShowLimitDialog && leftDays in 1..30) {
                    QuotaOverrunHelper.hasShowLimitDialog = true
                    QUOTA_LIMIT_DIALOG_SHOW.updateCount()
                    QuotaCapacityLimitDialog.getInstance(leftDays.toString()).showDialog(activity
                            .supportFragmentManager,
                            QuotaCapacityLimitDialog::class.java.name)
                }
            }
        }
    }

    private inline fun QuotaOverrunResponse?.onBan(onBan: (QuotaOverrunResponse) -> Unit): QuotaOverrunResponse? {
        if (this?.cross?.forbidStatus == true) onBan(this)
        return this
    }


    private inline fun QuotaOverrunResponse?.onOverLimit(onOverLimit: (Long) -> Unit): QuotaOverrunResponse? {
        this?.cross?.run {
            if (quota > 0 && !forbidStatus && surplusTime
                    > 0) {
                surplusTime.secondsToDays().apply(onOverLimit)
            }
        }
        return this
    }

    private fun quotaLimitKey() = ConfigQuotaLimit(ServerConfigKey.getCfgConfigBodyByType
    (ServerConfigKey.ConfigType.QUOTA_LIMIT_CONFIG)).openQuotaLimit && !BuildConfig.IS_XIAOMI
}

inline fun QuotaOverrunResponse?.onNoOverLimit(onNoOverLimit: () -> Unit): QuotaOverrunResponse? {
    if (this?.cross?.quota?:0 <= 0L && this?.cross?.forbidStatus != true)
        onNoOverLimit()
    return this
}


