package com.baidu.netdisk.ui.search

import android.support.constraint.ConstraintLayout
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.BaiduSearchActivity
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs
import kotlinx.android.synthetic.main.serach_enter_baidu.view.*

/**
 * Created by zhanglei84 on 2019/10/26.
 * 全网搜索入口
 */
class SearchEnterBaiduContainer : BaseContainer(), View.OnClickListener {
    companion object {
        const val TAG = "SearchEnterBaiduContainer"
        /** 大搜url */
        const val SEARCH_IN_BAIDU = "https://www.baidu.com/s?wd="
    }

    /** 大搜入口view */
    private lateinit var mSearchInBaidu: ConstraintLayout

    /** 大搜关键词view */
    private lateinit var mSearchBaiduQueryText: TextView

    /** 蒙层 */
    private lateinit var mMask: View


    /** 搜索词 */
    private var mQueryText: String = ""

    override fun onCreateView(): View {
        NetDiskLog.d(TAG, "大搜入口 onCreateView")
        val inflater = LayoutInflater.from(activity)
        val parentView = if (parent == null) null else parent.rootView as ViewGroup
        val root = inflater.inflate(R.layout.serach_enter_baidu, parentView, false)

        mSearchBaiduQueryText = root.search_baidu_query

        mSearchInBaidu = root.search_in_baidu
        mSearchInBaidu.setOnClickListener(this)
        mSearchInBaidu.visibility = View.GONE

        mMask = root.mask
        mMask.setOnClickListener(this)
        mMask.visibility = View.GONE
        return root
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.search_in_baidu ->
                queryInBaidu()
            R.id.mask ->
                return
        }
    }

    override fun onCommonEvent(event: CommonEvent?): Boolean {
        super.onCommonEvent(event)
        when (event?.what) {
            MessageIDs.MESSAGE_START_QUERY -> {
                mQueryText = event.obj as String
                mSearchBaiduQueryText.text = mQueryText
                if (!TextUtils.isEmpty(mQueryText)) {
                    mSearchInBaidu.visibility = View.VISIBLE
                }
                NetDiskLog.d(TAG, "大搜入口收到信息" + mQueryText)
            }
            MessageIDs.MESSAGE_SELECTED_MODE -> {
                // 进入编辑态
                mMask.visibility = View.VISIBLE
            }
            MessageIDs.MESSAGE_VIEW_MODE -> {
                // 退出编辑态
                mMask.visibility = View.GONE
            }
        }
        return false
    }

    /**
     * 大搜关键词查询
     * @param query 搜索关键词
     */
    private fun queryInBaidu() {
        // 去全网搜索点击pv/uv（携带搜索词）
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                NetdiskStatisticsLogForMutilFields.StatisticsKeys.BAI_DU_SEARCH_CLICK, mQueryText)

        val queryWord = mSearchBaiduQueryText.getText().toString().trim()
        val url = SEARCH_IN_BAIDU + queryWord + " " + context.getString(R.string.netdisk)
        NetDiskLog.d(TAG, "跳转页面 url= $url")
        val intent = BaiduSearchActivity.getStartIntent(context, url)
        if (intent != null) {
            activity.startActivity(intent)
        }
    }
}