package com.baidu.netdisk.ui.trade

import android.content.Context
import android.util.SparseIntArray
import android.view.View
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.kernel.android.util.file.FileUtils
import com.baidu.netdisk.preview.image.BaseImagePreviewBeanLoader
import com.baidu.netdisk.preview.image.PreviewBeanLoaderParams
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.preview.image.ImagePagerActivity
import kotlinx.android.synthetic.main.trade_check_origin_layout.*

/**
 * 支持交易平台图片异步加载、购买引导、查看原图
 *
 * author guochao02
 * email guochao02@baidu.com
 * created 2019/12/20 18:21
 */
class TradeImagePagerActivityEx : ImagePagerActivity(), ITradImageView {

    companion object {
        /**
         * 请求图片URL使用的pid
         */
        const val EXTRA_IMAGE_PID = "com.baidu.netdisk.IMAGE_PID"
    }

    /**
     * 图片预览的url列表
     */
    private var mUrls = ArrayList<String?>()

    /**
     * 图片预览的起始位置
     */
    private var mIndex = 0

    private var mTradeImagePresenter: ITradeImagePresenter? = null

    private val progressMap = SparseIntArray()

    override fun initView() {
        super.initView()
        view_origin_image.setOnClickListener {
            mTradeImagePresenter?.checkOriginImage(currentPosition)
        }
    }

    override fun initParam(context: Context?) {
        mFrom = TRADE_IMAGE
        mIndex = intent.getIntExtra(EXTRA_INDEX, 0)
        mLoadingView.visibility = View.VISIBLE
        mTradeImagePresenter = TradeImagePresenter(this)
        mTradeImagePresenter?.initData(lifecycleOwner, intent.getStringExtra(EXTRA_IMAGE_PID))
    }

    override fun reportRecentPreview(position: Int) { // 上报交易平台图片于预览行为
        NetdiskStatisticsLogForMutilFields.getInstance()
                .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.PREVIEW_TRADE_PRODUCT_THUMBNAIL)
    }

    override fun createPreviewBeanLoader(loaderParams: PreviewBeanLoaderParams?): BaseImagePreviewBeanLoader? {
        return TradeImagePreviewBeanLoader(mIndex, mUrls)
    }

    override fun onPageSelected(position: Int) {
        updateOriginImageBtn(position)
        super.onPageSelected(position)
    }

    override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
        super.onPageScrolled(position, positionOffset, positionOffsetPixels)
    }

    /**
     * 更新查看原图按钮
     */
    private fun updateOriginImageBtn(position: Int) {
        view_origin_image.visibility = View.VISIBLE
        if (ServerURL.isNetURL(mUrls[position])) {
            val progress = progressMap.get(position, -1)
            if (progress in 0..100) {
                view_origin_image.text = progress.toString() + "%"
                download_progress.progress = progress.toFloat()
                download_progress.visibility = View.VISIBLE
            } else {
                val sizeStr = FileUtils.turnSizeToString(mTradeImagePresenter?.getSize(position)!!)
                view_origin_image.text = resources.getString(R.string.check_origin_image) + "($sizeStr)"
                download_progress.visibility = View.GONE
                download_progress.progress = 0f
            }
        } else {
            view_origin_image.text = resources.getString(R.string.open_image)
            download_progress.visibility = View.GONE
        }
    }

    override fun moveImage(lastPos: Int, destPos: Int) {
    }

    override fun setPresenter(tradeImagePresenter: TradeImagePresenter) {
        mTradeImagePresenter = tradeImagePresenter
    }

    override fun updateProgress(pos: Int, progress: Int) {
        progressMap.put(pos, progress)
        if (pos == currentPosition) {
            updateOriginImageBtn(pos)
        }
    }

    override fun updateUrl(pos: Int, localPath: String?) {
        mUrls[pos] = localPath
        mAdapter.itemList[pos].resourceUrl = localPath
        mAdapter.notifyDataSetChanged()
    }

    override fun onDownloadError() {
    }

    override fun showIntroduceToBuy() {
        DialogBuilder().apply {
            val dialog = buildTipsDialog(this@TradeImagePagerActivityEx, R.string.tips,
                    R.string.trade_tips_content, R.string.buy_now, R.string.think_more)

            setOnDialogCtrListener(object : DialogCtrListener {
                override fun onOkBtnClick() {
                    dialog.dismiss()
                    mTradeImagePresenter?.buy()
                }

                override fun onCancelBtnClick() {
                    dialog.dismiss()
                }
            })
            dialog.show()
        }
    }

    override fun loadDataFinish(isExistData: Boolean, urls: ArrayList<String?>) {
        if (isExistData) {
            mUrls = urls
            updateOriginImageBtn(mIndex)
            super.initParam(context)
        } else {
            finish()
        }
    }

    override fun getActivity(): BaseActivity {
        return this
    }
}