package com.baidu.netdisk.ui.trade

import android.arch.lifecycle.LifecycleOwner
import android.os.Bundle
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.NetDiskApplication
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.trade.external.api.DownloadListener
import com.baidu.netdisk.trade.external.api.ResultCallBack
import com.baidu.netdisk.trade.external.api.TradeCommonApiCompManager
import com.baidu.netdisk.tradeplatform.TradePlatformHelper
import com.baidu.netdisk.ui.preview.trade.AppRecommendOpenFile
import com.baidu.netdisk.ui.webview.hybrid.action.HybridActionDownloadAbility
import com.google.gson.Gson
import org.json.JSONObject
import java.io.File


class TradeImagePresenter(private val tradeImageView: ITradImageView) : ITradeImagePresenter {


    private var imageData: ImageData? = null

    private var isOwner = false

    private var mPid: String? = null

    init {
        tradeImageView.setPresenter(this)
    }

    private val mFinishImage = HashSet<String>()

    override fun initData(lifecycleOwner: LifecycleOwner, pid: String) {
        val bundle = Bundle()
        mPid = pid
        bundle.putString("pid", pid)
        val urls = ArrayList<String?>()
        TradeCommonApiCompManager.getImagePreviewUrl(NetDiskApplication.getNetdiskApplication(),
                lifecycleOwner, bundle,
                object : ResultCallBack {
                    override fun onSuccess(o: Any) {
                        val thumbnailBundle = (o as Bundle)
                        val thumbnailList = thumbnailBundle.getString("data")
                        thumbnailList?.apply {
                            imageData = Gson().fromJson<ImageData>(thumbnailList, ImageData::class.java)
                            isOwner = imageData?.isOwner!!
                            for (thumbnailInfo in imageData?.thumbnailList!!) {
                                urls.add(thumbnailInfo.url)
                            }
                        }
                        tradeImageView.loadDataFinish(true, urls)
                    }

                    override fun onFail(i: Int, s: String) {
                        tradeImageView.loadDataFinish(false, urls)
                    }
                })

    }

    override fun getSize(position: Int): Long {
        return imageData?.thumbnailList?.get(position)?.size!!
    }

    override fun checkOriginImage(currentPosition: Int) {
        if (isOwner) {
            val url = imageData?.thumbnailList?.get(currentPosition)?.url
            if (ServerURL.isNetURL(url)) {
                downloadImage(currentPosition)
            } else {
                openImage(currentPosition)
            }
        } else {
            introduceToBuy()
        }
    }

    private fun downloadImage(pos: Int) {
        val thumbnailInfo = imageData?.thumbnailList?.get(pos)
        TradeCommonApiCompManager.downloadProduct(tradeImageView.getActivity(), tradeImageView.getActivity().lifecycleOwner,
                imageData?.type!!, thumbnailInfo?.size!!, mPid, thumbnailInfo.skuid,
                thumbnailInfo.name, thumbnailInfo.format, object : DownloadListener {
            override fun onDownloadStart(skuid: String?) {
                NetDiskLog.d("DB PIC", "onDownloadStart pos:$pos,skuid:$skuid")
                tradeImageView.updateProgress(pos, 0)

            }

            override fun onDownloadRunning(progerss: Int, skuid: String?) {
                NetDiskLog.d("DB PIC", "onDownloadRunning  pos:$pos,progerss:$progerss,skuid:$skuid")
                tradeImageView.updateProgress(pos, progerss)

            }

            override fun onDownloadFinish(state: Int, skuid: String) {
                when (state) {
                    HybridActionDownloadAbility.CACHE_STATE_FINISHED -> {
                        if (!mFinishImage.contains(skuid)) {
                            mFinishImage.add(skuid)
                            NetDiskLog.d("DB PIC", "onDownloadFinish  pos:$pos,state:$state,skuid:$skuid")
                            tradeImageView.updateUrl(pos,
                                    TradeCommonApiCompManager.hybridActionGetLocalPath(NetDiskApplication.getNetdiskApplication(),
                                            mPid, thumbnailInfo.skuid!!, TradePlatformHelper.TYPE_IMAGE))
                            tradeImageView.updateProgress(pos, 100)
                        }
                    }
                    HybridActionDownloadAbility.CACHE_STATE_PAUSED_BY_NETWORK,
                    HybridActionDownloadAbility.CACHE_STATE_PAUSED_BY_STORAGE,
                    HybridActionDownloadAbility.CACHE_STATE_PAUSED_BY_ERROR -> {
                        // error
                        tradeImageView.onDownloadError()
                    }
                }
            }
        })
    }

    private fun openImage(pos: Int) {
        // 在商城查询该资源是否已下载 并且 返回本地路径
        val localPath = TradeCommonApiCompManager.hybridActionGetLocalPath(NetDiskApplication.getNetdiskApplication(),
                mPid, imageData?.thumbnailList?.get(pos)?.skuid!!, TradePlatformHelper.TYPE_IMAGE)
        localPath?.apply {
            if (File(localPath).exists()) {
                AppRecommendOpenFile().openAppRecommend(
                        BaseApplication.mContext, mPid,
                        localPath, "", imageData?.thumbnailList?.get(pos)?.size!!)
            }
        }
    }

    override fun buy() {
        val param = JSONObject()
        param.put("pid", mPid)
        TradeCommonApiCompManager.buyTradeProduction(NetDiskApplication.getNetdiskApplication(), tradeImageView.getActivity(),
                tradeImageView.getActivity().lifecycleOwner, param.toString(), object : ResultCallBack {
            override fun onSuccess(o: Any) {
//                val intent = Intent()
//                intent.putExtra("result", true)
//                tradeImageView.getActivity().setResult(70, intent)
                tradeImageView.getActivity().finish()
            }

            override fun onFail(i: Int, s: String) {

            }

        })
    }

    private fun introduceToBuy() {
        tradeImageView.showIntroduceToBuy()
    }
}
