package com.baidu.netdisk.ui.verify

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.widget.ImageView
import com.baidu.netdisk.R
import com.baidu.netdisk.base.utils.ActivityUtils
import com.baidu.netdisk.ui.webview.*
import com.baidu.netdisk.ui.webview.hybrid.action.*
import android.net.Uri
import android.os.ResultReceiver
import android.widget.TextView
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.cloudp2p.service.CloudP2PService

/**
 * 验证码校验弹窗
 *
 * @author wangyang34
 * @since 2019/9/2
 */
class VerifyPopupActivity: BaseWebViewActivity() {

    companion object {

        /**
         * 打开弹窗
         * @param context 上下文
         * @param receiver 接收器
         * @param ak mbox的key
         * @param type 弹窗类型
         * @param tips 提示文案
         */
        fun showPopup(context: Context?, receiver: ResultReceiver?, ak: String, type: String, tips: String) {
            val url = Uri.parse(ServerURL.getVerifyH5Url()).buildUpon()
                    .appendQueryParameter("accessKey", ak)
                    .appendQueryParameter("verifyType", type)
                    .toString()
            var bundle = Bundle()
            bundle.putString(BaseWebViewFragment.EXTRA_URL, url)
            var intent = Intent(context, VerifyPopupActivity::class.java)
            intent.putExtras(bundle)
            intent.putExtra(CloudP2PService.EXTRA_VERIFY_DIALOG_TIPS, tips)
            intent.putExtra(BaseExtras.RESULT_RECEIVER, receiver)
            context?.startActivity(intent)
        }
    }

    /** 外部接收者 */
    private var mReceiver: ResultReceiver? = null

    override fun getLayoutId(): Int {
        return R.layout.verify_popup_activity
    }

    override fun initView() {
        ActivityUtils.setSystemBarStatus(this, resources.getColor(R.color.check_huodong_bg_color))
        mReceiver = intent?.getParcelableExtra(BaseExtras.RESULT_RECEIVER)
        var tips = intent?.getStringExtra(CloudP2PService.EXTRA_VERIFY_DIALOG_TIPS)
        var description: TextView? = findViewById(R.id.description)
        description?.text = tips
        var closeView: ImageView? = findViewById(R.id.content_close)
        closeView?.setOnClickListener {
            finish()
            mReceiver?.send(BaseStatus.FAILED, Bundle.EMPTY)
        }
    }

    override fun getActionManager(): IActionManager {
        return HybridActionManager.Builder()
                .setHybridActionForward(HybridActionForward(this))
                .setHybridActionShare(HybridActionShare(this))
                .setHybridActionClientInfo(HybridActionClientInfo(this))
                .setHybridActionClipboard(HybridActionClipboard(this))
                .setHybridActionPayProduct(HybridActionPayProduct(this, this))
                .setHybridActionClose(HybridActionClose(this))
                .setHybridActionPermission(HybridActionPermission(this))
                .setHybridActionThirdApp(HybridActionThirdApp(this))
                .setHybridActionAbilityTask(HybridActionAbilityTask(this))
                .setHybridActionAbilityVerifyCode(HybridActionAbilityVerifyCode(this, mReceiver))
                .build()
    }

    override fun initFragment() {
        val disabledOverrideWebClient = DisabledOverrideWebClient(this, null, actionManager)
        mFragment = WebViewFragmentBuilder()
                .setWebViewClient(disabledOverrideWebClient)
                .setWebChromeClient(VerifyWebChromeClient())
                .setDownloadListener(BaseOnDownloadListener())
                .setUrlLoadable(ImmediatelyUrlLoader(DefaultCookiesSyncer(applicationContext)))
                .build()
        mFragment.arguments = intent.extras
        val ft = supportFragmentManager.beginTransaction()
        ft.add(R.id.content, mFragment, BaseWebViewFragment.TAG)
        ft.commitAllowingStateLoss()
    }
}