package com.baidu.netdisk.ui.vip

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.text.TextUtils
import android.view.View
import android.webkit.JsPromptResult
import android.webkit.JsResult
import android.webkit.WebChromeClient
import android.webkit.WebView
import com.baidu.netdisk.BuildConfig
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.storage.config.ConfigCapacityBar
import com.baidu.netdisk.base.storage.config.ConfigIceBreaking
import com.baidu.netdisk.base.storage.config.ConfigJointVip
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.kotlin.extension.Bundle
import com.baidu.netdisk.payment.viporder.io.model.Order
import com.baidu.netdisk.platform.extension.gone
import com.baidu.netdisk.platform.extension.show
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.webview.*
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import kotlinx.android.synthetic.main.activity_pay_result_page.*
import java.io.UnsupportedEncodingException
import java.net.URLEncoder

class PayResultPageActivity(): BaseWebViewActivity() {

    companion object {
        private const val TAG = "VipOrderResultNewActivity"
        private const val EXTRA_ORDER = "com.baidu.netdisk.extra.EXTRA_ORDER"
        private const val EXTRA_ORDER_STATUS = "com.baidu.netdisk.extra.EXTRA_ORDER_STATUS"
        /** 购买/续费成功  */
        const val ORDER_STATUS_SUCCESS = 0
        /** 购买/续费失败  */
        const val ORDER_STATUS_FAILED = 1
        const val ORDER_STATUS_OPERATING = 2
        private const val DEFAULT_MONEY = 0
        private const val DEFAULT_TIME = 0L

        @JvmStatic
        fun startActivity(activity: Activity, orderStatus: Int, order: Order?, list:
        ArrayList<Pair<String, String>>?) {
            val intent = Intent(activity, PayResultPageActivity::class.java).apply {
                putExtra(EXTRA_ORDER_STATUS, orderStatus)
                putExtra(EXTRA_ORDER, initData(activity, order, list))
            }
            activity.startActivity(intent)
        }

        private fun initData(activity: Activity, order: Order?, list: ArrayList<Pair<String, String>>? = null):
                PayResultBean{
            var tempList = ArrayList<Pair<String, String>>()
            if (list?.isNotEmpty() == true) {
                tempList = list
            } else {
                val nickName = AccountUtils.getInstance().cloudNickName
                val uName = AccountUtils.getInstance().userName
                order?.apply {
                    tempList.apply {
                        add(Pair(activity.getString(R.string.vip_order_result_new_user_name),
                                if (nickName.isNullOrBlank()) uName else nickName))

                        add(Pair(activity.getString(R.string.vip_order_result_new_pay_money), getPayMoney(order)))
                        add(Pair(activity.getString(R.string.vip_order_result_new_product), order.getProductDescription()))
                        add(Pair(activity.getString(R.string.vip_order_result_new_end_time), getEndTime(order)))
                    }
                }
            }

            return PayResultBean(tempList, order?.businessNo, order?.orderNo, order?.productName, order?.phone,
                    order?.phoneMtime, order?.orderType, order?.isAutoSvip, order?.actionUrl, order?.orderOperationDesc)
        }

        private fun getPayMoney(order: Order): String {
            val payMoneyData = try {
                Integer.parseInt(order.getActualPay())
            } catch (e: NumberFormatException) {
                DEFAULT_MONEY
            }

            return if (payMoneyData % 100 == 0) {
                "${payMoneyData / 100}元"
            } else {
                "${payMoneyData.toDouble() / 100}元"
            }
        }

        private fun getEndTime(order: Order): String {
            val endTimeData = try {
                java.lang.Long.parseLong(order.getEndTime())
            } catch (n: NumberFormatException) {
                DEFAULT_TIME
            }

            return DateUtil.formatTimeEndWithDay(endTimeData * 1000)
        }
    }

    private var orderStatus: Int = 0
    private var mOrder: PayResultBean? = null

    private var mJointVipConfig: ConfigJointVip? = null
    private var isJointVip: Boolean = false
    private var mIceBreakingSendSpacePresenter: IceBreakingSendSpacePresenter? = null
    private var mPayResultPagePresenter: PayResultPagePresenter? = null
    private var payResultPageAdapter: PayResultPageAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        mJointVipConfig =
                ConfigJointVip(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.JOINT_VIP))
        initPresenter()
        super.onCreate(savedInstanceState)

    }

    override fun initParams() {
        super.initParams()
        orderStatus = intent.getIntExtra(EXTRA_ORDER_STATUS, -1)
        mOrder = intent.getParcelableExtra(EXTRA_ORDER)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_pay_result_page
    }

    override fun initView() {
        mTitleBar = BaseTitleBar(this).apply {
            setMiddleTitle(R.string.vip_order_result_title_new)
            setBackLayoutVisible(false)
        }

        when (orderStatus) {
            ORDER_STATUS_SUCCESS -> showSuccessView()
            ORDER_STATUS_FAILED -> showFailedView()
            ORDER_STATUS_OPERATING -> showOperatingView()
            else -> finish()
        }
    }

    override fun onRightButtonClicked(view: View) {
        NetDiskLog.d(TAG, "click btn to set phone number")
        when(mOrder?.getProductType()) {
            PayResultBean.ProductType.VIP_PRODUCT, PayResultBean.ProductType.SVIP_PRODUCT -> {
                onClickRightButton(successOption = ::vipOrderSuccessOption, operatingOption = ::vipOrderOperatingOption)
            }
            PayResultBean.ProductType.VOLUME_PRODUCT -> {
                onClickRightButton(successOption = ::volumeOrderSuccessOption, operatingOption = ::volumeOrderOperatingOption)
            }
            else -> finish()
        }
    }

    private fun initPresenter() {
        // 空间破冰活动查询是否首次支付的presenter
        mIceBreakingSendSpacePresenter = IceBreakingSendSpacePresenter(this)
        mPayResultPagePresenter = PayResultPagePresenter(this)
    }

    // 展示支付成功ui
    private fun showSuccessView() {
        mOrder?.let {order ->
            val iceBreakingConfig = ConfigIceBreaking(
                    ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.ICE_BREAKING_CONFIG))
            if (iceBreakingConfig.showFirstPayTip) { // 开关关闭不再弹送空间提示
                mIceBreakingSendSpacePresenter?.querySendSpaceStatus(order.orderNo)
            }
            vip_order_new_result_success_notice_layout.show()
            vip_order_new_result_fail_or_operating_notice_layout.gone()
            vip_order_new_result_success_notice_title.setText(R.string.vip_order_result_new_success)
            vip_order_new_result_success_notice_des.text = mOrder?.orderOperationDesc
            payResultPageAdapter = PayResultPageAdapter()
            success_list.adapter = payResultPageAdapter
            success_list.layoutManager = LinearLayoutManager(activity)
            mOrder?.apply {
                payResultPageAdapter?.data = list
            }

            mTitleBar.setRightLabel(R.string.done)
            mTitleBar.setTopTitleBarClickListener(this)
            // 只有在是购买的联合会员产品并且活动没有下线时才显示入口
            if (!BuildConfig.IS_XIAOMI && mJointVipConfig?.isHideJointEntrance != true && !TextUtils
                            .isEmpty(order.actionUrl)) {
                order.actionUrl?.let { showReceivePrizeEntrance(it) }
                isJointVip = true
            }
        }
    }

    // 展示支付失败ui
    private fun showFailedView() {
        vip_order_new_result_success_notice_layout.gone()
        vip_order_new_result_fail_or_operating_notice_layout.show()
        vip_order_new_result_fail_or_operating_notice_title.apply {
            setText(R.string.vip_order_result_new_fail)
            setCompoundDrawablesWithIntrinsicBounds(resources.getDrawable(R.drawable.pay_result_ic_fail), null, null, null)
            setTextColor(resources.getColor(R.color.bg_dn_common_color_666))
        }

        vip_order_new_result_fail_or_operating_notice_des.setText(R.string.vip_order_result_new_des_fail)
        vip_order_new_result_fail_or_operating_btn.apply {
            setText(R.string.vip_order_result_new_btn_fail_text)
            setOnClickListener {
                failClicked()
            }
        }
    }

    // 展示订单处理中ui
    private fun showOperatingView() {
        vip_order_new_result_success_notice_layout.gone()
        vip_order_new_result_fail_or_operating_notice_layout.show()
        vip_order_new_result_fail_or_operating_notice_title.apply {
            setText(R.string.vip_order_result_new_operating)
            setCompoundDrawablesWithIntrinsicBounds(
                    resources.getDrawable(R.drawable.ic_success), null, null, null)
            setTextColor(resources.getColor(R.color.vip_order_result_new_text))
        }
        vip_order_new_result_fail_or_operating_btn.gone()
        mTitleBar.setRightLabel(R.string.done)
        mTitleBar.setTopTitleBarClickListener(this)
        vip_order_new_result_fail_or_operating_notice_des.text = mOrder?.orderOperationDesc
    }

    // 显示领取爱奇艺会员入口页面
    private fun showReceivePrizeEntrance(url: String) {
        var url = url
        val disabledOverrideWebClient = NoEmptyViewWebClient(this, null, actionManager)
        mFragment = WebViewFragmentBuilder()
                .setWebViewClient(disabledOverrideWebClient)
                .setWebChromeClient(HideProgressWebChromeClient())
                .setDownloadListener(BaseOnDownloadListener())
                .setUrlLoadable(ImmediatelyUrlLoader(DefaultCookiesSyncer(
                        applicationContext)))
                .build()
        mOrder?.apply {
            url = addParam(url, "business_no", businessNo)
            url = addParam(url, "order_no", orderNo)
        }

        val bundle = Bundle {
            BaseWebViewFragment.EXTRA_URL - url
        }
        intent.putExtras(bundle)

        try {
            mFragment.arguments = intent.extras
        } catch (e: Exception) {
            NetDiskLog.e(TAG, e.message, e)
        }

        val ft = supportFragmentManager.beginTransaction()
        ft.add(R.id.content, mFragment, BaseWebViewFragment.TAG)
        ft.commitAllowingStateLoss()
    }


    private fun addParam(url: String, key: String, value: String?): String {
        if (key.isBlank() || value.isNullOrBlank()) {
            return url
        }
        val sb = StringBuilder()
        sb.append(url)
        if (url.contains("?")) {
            sb.append("&")
        } else {
            sb.append("?")
        }
        try {
            sb.append(key).append("=").append(URLEncoder.encode(value, "UTF-8"))
        } catch (e: UnsupportedEncodingException) {
            NetDiskLog.e(TAG, e.message, e)
        }
        return sb.toString()
    }

    override fun initFragment() {
        // do nothing
    }

    private fun onClickRightButton(successOption: () -> Unit, operatingOption: () -> Unit) {
        when (orderStatus) {
            ORDER_STATUS_SUCCESS -> successOption.invoke()
            ORDER_STATUS_OPERATING -> operatingOption.invoke()
            else -> finish()
        }
    }

    /**
     * 会员订单支付成功，点击完成按钮后的操作
     */
    private fun vipOrderSuccessOption() {
        mOrder?.let {
            mPayResultPagePresenter?.showSetPhoneNumberDialog(it)
            if (isJointVip) {
                // 新用户购买自动续费联合会员成功，支付成功页点击 完成 按钮的用户数和次数
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                        NetdiskStatisticsLogForMutilFields.StatisticsKeys.NEW_AUTO_JOINT_VIP_CLICK_DONE)
            }
        }
    }

    /**
     * 会员订单支付完成，处理中，点击完成按钮后的操作
     */
    private fun vipOrderOperatingOption() {
        VipActivity.startActivity(this, VipActivity.FROM_BUY_VIP_NEW_RESULT_PAGE,
                VipActivity.TO_DEFAULT)
        finish()
    }

    /**
     * 空间套餐订单支付成功，点击完成按钮后的操作
     */
    private fun volumeOrderSuccessOption() {
        jumpVolumeActivity()
    }

    /**
     * 空间订单支付完成，处理中，点击完成按钮后的操作
     */
    private fun volumeOrderOperatingOption() {
        jumpVolumeActivity() // 目前成功和处理中操作一致
    }

    /**
     * 跳转容量售卖页面
     */
    private fun jumpVolumeActivity() {
        val capacityBarInfo = ConfigCapacityBar(
                ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.CAPACITY_BAR_CONFIG))
        startActivity(RichMediaActivity.getStartIntent(activity, capacityBarInfo.mManageSpaceUrl))
        finish()
    }

    private fun failClicked() {
        when(mOrder?.getProductType()) {
            PayResultBean.ProductType.VIP_PRODUCT, PayResultBean.ProductType.SVIP_PRODUCT -> {
                VipActivity.startActivity(this, VipActivity.FROM_BUY_VIP_NEW_RESULT_PAGE,
                        VipActivity.TO_DEFAULT)
                finish()
            }
            PayResultBean.ProductType.VOLUME_PRODUCT -> {
                jumpVolumeActivity()
            }
            else -> finish()
        }
    }

    /**
     * 隐藏webview加载进度
     */
    private inner class HideProgressWebChromeClient : WebChromeClient() {
        override fun onProgressChanged(view: WebView, newProgress: Int) {
            super.onProgressChanged(view, newProgress)

            val fragment = (view as NetDiskWebView).webViewFragment ?: return

            if (fragment.textLoading.visibility != View.GONE) {
                fragment.textLoading.visibility = View.GONE
            }
        }

        override fun onJsAlert(view: WebView, url: String, message: String, result: JsResult): Boolean {
            return false
        }

        override fun onJsConfirm(view: WebView, url: String, message: String, result: JsResult): Boolean {
            return false
        }

        override fun onJsPrompt(view: WebView, url: String, message: String, defaultValue: String,
                                result: JsPromptResult): Boolean {
            return false
        }
    }
}
