package com.baidu.netdisk.ui.vip

import android.app.Dialog
import android.support.v4.content.ContextCompat
import android.text.SpannableString
import android.text.Spanned
import android.text.TextUtils
import android.text.style.ForegroundColorSpan
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.view.IBaseView
import java.text.SimpleDateFormat
import java.util.*

class PayResultPagePresenter(private val iView: IBaseView) {

    companion object {
        private const val TAG = "PayResultSetPhoneNumPresenter"
        private const val DEFAULT_TIME = 0L
        private const val DEFAULT_MONEY = 0
        /** 是否是自动续费超级会员  */
        private const val IS_AUTO_VIP = 1
    }

    private val activity = iView.activity

    // 设置手机号提醒弹窗
    fun showSetPhoneNumberDialog(order: PayResultBean) {
        val isAutoSvip = order.isAutoSvip == 1
        val isAlreadySetPhoneNum = !TextUtils.isEmpty(order.phone)
        val dialogBuilder = DialogBuilder()
        var dialog: Dialog? = null
        when {
            isAutoSvip && isAlreadySetPhoneNum -> {
                // 出更改手机号弹窗
                var phoneMTimeData: Long = DEFAULT_TIME
                try {
                    phoneMTimeData = java.lang.Long.parseLong(order.phoneMtime)
                } catch (n: NumberFormatException) {
                    phoneMTimeData = DEFAULT_TIME
                }

                val date = DateUtil.formatTimeEndWithDay(phoneMTimeData * 1000)
                val contentFirst = activity.getString(R.string.auto_buy_vip_set_phone_num_dialog_change_content_first)
                val content = contentFirst + activity.getString(R.string
                        .auto_buy_vip_set_phone_num_dialog_change_content_second,
                        order.phone, date)
                // 提示文案中手机号标蓝
                val spannableString = SpannableString(content)
                spannableString.setSpan(ForegroundColorSpan(ContextCompat.getColor(BaseApplication.getInstance(),
                        R.color.new_dialog_blue_color)),
                        contentFirst.length, contentFirst.length + order.phone!!.length,
                        Spanned.SPAN_INCLUSIVE_INCLUSIVE)

                dialog = dialogBuilder.buildTipsDialog(activity, activity.getString(R.string
                        .auto_buy_vip_set_phone_num_dialog_title),
                        spannableString, null, activity.getString(R.string.auto_buy_vip_set_phone_num_dialog_btn_know),
                        activity.getString(R.string.auto_buy_vip_set_phone_num_dialog_btn_change), true, null,
                        DialogBuilder.DIALOG_CONTENT_TEXT_GRAVITY_LEFT)
            }
            isAutoSvip && !isAlreadySetPhoneNum -> // 出填写手机号弹窗
                dialog = dialogBuilder.buildTipsDialog(activity, activity.getString(R.string
                        .auto_buy_vip_set_phone_num_dialog_title), null,
                        activity.getString(R.string.auto_buy_vip_set_phone_num_dialog_fill_content),
                        activity.getString(R.string.auto_buy_vip_set_phone_num_dialog_btn_fill),
                        activity.getString(R.string.auto_buy_vip_set_phone_num_dialog_btn_ignore), true, null,
                        DialogBuilder.DIALOG_CONTENT_TEXT_GRAVITY_LEFT)
            else -> {
                // 返回会员页刷新
                VipActivity.startActivity(activity, VipActivity.FROM_BUY_VIP_NEW_RESULT_PAGE,
                        VipActivity.TO_DEFAULT)
                activity.finish()
            }
        }

        dialogBuilder.setOnDialogCtrListener(object : DialogCtrListener {
            override fun onCancelBtnClick() {
                if (isAutoSvip && isAlreadySetPhoneNum) {
                    // 更改手机号弹窗--点击更改
                    dialog?.dismiss()
                    // 进入更改手机号页面
                    VipPhoneSetActivity.startActivity(activity,
                            order.phone, true, null)
                    activity.finish()
                } else if (isAutoSvip && !isAlreadySetPhoneNum) {
                    // 填写手机号弹窗--点击忽略
                    dialog?.dismiss()
                    // 返回会员页刷新
                    VipActivity.startActivity(activity, VipActivity.FROM_BUY_VIP_NEW_RESULT_PAGE,
                            VipActivity.TO_DEFAULT)
                    activity.finish()
                }
            }

            override fun onOkBtnClick() {
                if (isAutoSvip && isAlreadySetPhoneNum) {
                    // 更改手机号弹窗--点击知道啦
                    dialog?.dismiss()
                    // 返回会员页刷新
                    VipActivity.startActivity(activity, VipActivity.FROM_BUY_VIP_NEW_RESULT_PAGE,
                            VipActivity.TO_DEFAULT)
                    activity.finish()
                } else if (isAutoSvip && !isAlreadySetPhoneNum) {
                    // 填写手机号弹窗--点击填写手机号
                    dialog?.dismiss()
                    // 进入填写手机号页面
                    VipPhoneSetActivity.startActivity(activity, null, true, null)
                    activity.finish()
                }

            }
        })
    }
}