package com.baidu.netdisk.ui.vip.wallet

import android.content.Intent
import android.text.TextUtils
import com.baidu.location.BDLocation
import com.baidu.location.BDLocationListener
import com.baidu.location.LocationClient
import com.baidu.location.LocationClientOption
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.platform.business.polymerpay.core.IInitCallback
import com.baidu.netdisk.platform.business.polymerpay.core.ILocationCallBack
import com.baidu.netdisk.ui.MainActivity
import com.baidu.netdisk.ui.vip.VipActivity
import java.net.MalformedURLException
import java.net.URL

class WalletListener : IInitCallback {

    companion object {
        private const val TAG = "WalletListener"
    }

    override fun getLocation(callBack: ILocationCallBack) {
        val locationClient = LocationClient(BaseApplication.getInstance())
        locationClient.registerLocationListener(object : BDLocationListener {

            override fun onReceiveLocation(location: BDLocation) {
                locationClient.stop()
                locationClient.unRegisterLocationListener(this)
                // 通知SDK更新定位信息
                val radius = location.radius
                val latitude = location.latitude
                val longitude = location.longitude
                callBack.onGetLocation(radius, latitude, longitude)
            }
        }) // 注册监听函数

        val option = LocationClientOption()
        option.isOpenGps = false // 不开启GPS
        option.locationMode = LocationClientOption.LocationMode.Battery_Saving
        option.setIsNeedAddress(false) // 返回的定位结果不包含地址信息
        option.setCoorType("bd09ll") // 获取定位信息时，坐标系要用百度经纬度坐标系“bd09ll”
        locationClient.locOption = option
        locationClient.start()
        locationClient.requestLocation()
    }

    override fun login() {
        val intent = Intent(BaseApplication.getInstance(), MainActivity::class.java)
        intent.action = MainActivity.ACTION_LOGOUT
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP)
        BaseApplication.getInstance().startActivity(intent)
    }

    override fun startPage(spec: String): Boolean {
        try {
            if (TextUtils.isEmpty(spec)) {
                return false
            }

            val url = URL(spec)
            val path = url.path
            if (TextUtils.isEmpty(path)) {
                // 使用廊桥打开页面
                return false
            }

            NetDiskLog.i(TAG, "startPage spec:$spec")
            val pathVip = URL(ServerURL.getVipWap()).path
            // final String pathGiftPack = new URL(BannerInfoBean.DEFAULT_GIFT_PACK_URL).getPath();

            NetDiskLog.i(TAG, "url : $spec")

            if (path == pathVip) {
                VipActivity.startActivity(BaseApplication.getInstance(), VipActivity.FROM_WALLET,
                        VipActivity.TO_DEFAULT)
            } else if (ServerURL.isBuyPage(spec)) {
                VipActivity.startActivity(BaseApplication.getInstance(), ServerURL.getVipBuyWap())
                // } else if (path.equals(pathGiftPack)) {
                // RichMediaActivity.startRichMediaActivity(NetDiskApplication.getInstance(),
                // BannerInfoBean.DEFAULT_GIFT_PACK_URL);
                //
                // /** 用户进入关于我，在我的钱包中，点击活动banner的用户数/次数 **/
                // NetdiskStatisticsLogForMutilFields.getInstance().updateCount(StatisticsKeys.WALLET_OPEN_HONGBAO_PAGE);
            } else {
                // 使用廊桥打开页面
                return false
            }

            return true
        } catch (e: MalformedURLException) {
            NetDiskLog.e(TAG, "invalid format:$spec")
            return false
        }
    }

}