package com.baidu.netdisk.ui.webview

import android.annotation.TargetApi
import android.app.Activity
import android.graphics.Bitmap
import android.net.Uri
import android.net.http.SslError
import android.os.Message
import android.util.Log
import android.view.KeyEvent
import android.webkit.*
import com.baidu.neidisk.interceptor.IInterceptor
import com.baidu.neidisk.interceptor.TradePackageInterceptor
import com.baidu.netdisk.wap.offlinepackage.OfflinePackageManager

/**
 * 使用拦截器Client，利用本地资源加快页面显示。
 * Todo 后续考虑将BaseClient移如kernal或base，然后将该类移到FastWebview模块中。
 *
 * author guochao02
 * email guochao02@baidu.com
 * created 2019/11/4 11:35
 */

class FastWebViewClient(val activity: Activity, val client: WebViewClient,
                        pkgId: String) : BaseClient(activity, null) {

    private var interceptor: IInterceptor? = null
    private var mWebViewPerformanceStatic: WebViewPerformanceStatic? = null

    constructor(activity: Activity, client: WebViewClient,
                pkgId: String, webViewPerformanceStatic: WebViewPerformanceStatic?) : this(activity, client, pkgId) {
        mWebViewPerformanceStatic = webViewPerformanceStatic
        val pkgVersion = OfflinePackageManager.manifestItemMap[pkgId]?.version
        mWebViewPerformanceStatic?.apply {
            if (pkgVersion != null) {
                this.pkgVersion = pkgVersion
            }
        }
    }

    init {
        val manifest = OfflinePackageManager.manifestItemMap[pkgId]?.tradeManifestItem
        val rootPath = OfflinePackageManager.manifestItemMap[pkgId]?.rootPath
        if (manifest != null && rootPath != null) {
            interceptor = TradePackageInterceptor(activity, rootPath, manifest)
        }
    }

    override fun shouldInterceptRequest(view: WebView, url: String): WebResourceResponse? {
        Log.d("H5Offline", "FastWebViewClient shouldInterceptRequest is $url," +
                " interceptor is $interceptor")
        return if (interceptor == null) {
            client.shouldInterceptRequest(view, url)
        } else {
            interceptor?.intercept(view, url)
        }
    }

    @TargetApi(21)
    override fun shouldInterceptRequest(view: WebView, request: WebResourceRequest): WebResourceResponse? {
        return shouldInterceptRequest(view, request.url.toString())
    }

    override fun onPageFinished(view: WebView?, url: String?) {
        mWebViewPerformanceStatic?.apply {
            pageFinishTime = System.currentTimeMillis()
            view?.originalUrl?.apply {
                urlPath = Uri.parse(this).path
            }
            upload()
        }
        client.onPageFinished(view, url)
    }

    override fun shouldOverrideKeyEvent(view: WebView?, event: KeyEvent?): Boolean {
        return client.shouldOverrideKeyEvent(view, event)
    }

    @TargetApi(27)
    override fun onSafeBrowsingHit(view: WebView?, request: WebResourceRequest?, threatType: Int, callback: SafeBrowsingResponse?) {
        client.onSafeBrowsingHit(view, request, threatType, callback)
    }

    override fun doUpdateVisitedHistory(view: WebView?, url: String?, isReload: Boolean) {
        client.doUpdateVisitedHistory(view, url, isReload)
    }

    override fun onReceivedError(view: WebView?, errorCode: Int, description: String?, failingUrl: String?) {
        mWebViewPerformanceStatic?.apply {
            this.errorCode = errorCode
            this.errorMsg = description
        }
        client.onReceivedError(view, errorCode, description, failingUrl)
    }

    @TargetApi(23)
    override fun onReceivedError(view: WebView?, request: WebResourceRequest?, error: WebResourceError) {
        mWebViewPerformanceStatic?.apply {
            errorCode = error.errorCode
            errorMsg = error.description.toString()
        }
        client.onReceivedError(view, request, error)
    }

    @TargetApi(26)
    override fun onRenderProcessGone(view: WebView?, detail: RenderProcessGoneDetail?): Boolean {
        return client.onRenderProcessGone(view, detail)
    }

    override fun onReceivedLoginRequest(view: WebView?, realm: String?, account: String?, args: String?) {
        client.onReceivedLoginRequest(view, realm, account, args)
    }

    @TargetApi(23)
    override fun onReceivedHttpError(view: WebView?, request: WebResourceRequest?, errorResponse: WebResourceResponse?) {
        client.onReceivedHttpError(view, request, errorResponse)
    }

    override fun onPageStarted(view: WebView?, url: String?, favicon: Bitmap?) {
        mWebViewPerformanceStatic?.apply {
            pageStartTime = System.currentTimeMillis()
        }
        client.onPageStarted(view, url, favicon)
    }

    override fun onScaleChanged(view: WebView?, oldScale: Float, newScale: Float) {
        client.onScaleChanged(view, oldScale, newScale)
    }

    override fun shouldOverrideUrlLoading(view: WebView?, url: String?): Boolean {
        return client.shouldOverrideUrlLoading(view, url)
    }

    @TargetApi(24)
    override fun shouldOverrideUrlLoading(view: WebView?, request: WebResourceRequest?): Boolean {
        return client.shouldOverrideUrlLoading(view, request)
    }

    @TargetApi(23)
    override fun onPageCommitVisible(view: WebView?, url: String?) {
        client.onPageCommitVisible(view, url)
    }

    override fun onUnhandledKeyEvent(view: WebView?, event: KeyEvent?) {
        client.onUnhandledKeyEvent(view, event)
    }

    @TargetApi(21)
    override fun onReceivedClientCertRequest(view: WebView?, request: ClientCertRequest?) {
        client.onReceivedClientCertRequest(view, request)
    }

    override fun onReceivedHttpAuthRequest(view: WebView?, handler: HttpAuthHandler?, host: String?, realm: String?) {
        client.onReceivedHttpAuthRequest(view, handler, host, realm)
    }

    override fun onReceivedSslError(view: WebView?, handler: SslErrorHandler?, error: SslError?) {
        client.onReceivedSslError(view, handler, error)
    }

    override fun onTooManyRedirects(view: WebView?, cancelMsg: Message?, continueMsg: Message?) {
        client.onTooManyRedirects(view, cancelMsg, continueMsg)
    }

    override fun onFormResubmission(view: WebView?, dontResend: Message?, resend: Message?) {
        client.onFormResubmission(view, dontResend, resend)
    }

    override fun onLoadResource(view: WebView?, url: String?) {
        client.onLoadResource(view, url)
    }
}