package com.baidu.netdisk.ui.webview

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.os.Build.VERSION
import android.os.Build.VERSION_CODES
import android.os.Bundle
import android.view.View
import android.view.Window
import com.baidu.neidisk.webview.setTransparent
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.ui.webview.hybrid.action.*


/**
 * 使用H5离线包的WebviewActivity
 *
 * author guochao02
 * email guochao02@baidu.com
 * created 2019/11/4 11:39
 */
class OfflineH5PackageActivity : BaseWebViewActivity() {

    companion object {
        const val EXTRA_PACKAGE_ID = "package_id"
        const val TAG = "OfflineH5PackageActivity"
        const val EXTRA_IMMERSION = "extra_immersion"
        @JvmStatic
        fun getStartIntent(context: Context, url: String, packageId: String, isImmersion: Boolean): Intent {
            val bundle = Bundle()
            bundle.putString(EXTRA_PACKAGE_ID, packageId)
            bundle.putString(BaseWebViewFragment.EXTRA_URL, url)
            bundle.putBoolean(EXTRA_IMMERSION, isImmersion)
            val intent = Intent(context, OfflineH5PackageActivity::class.java)

            if (context !is Activity) {
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
            }

            intent.putExtras(bundle)
            return intent
        }


        @JvmStatic
        fun getStartIntent(context: Context, url: String, packageId: String): Intent {
            return getStartIntent(context, url, packageId, true)
        }
    }

    override fun getActionManager(): IActionManager {
        return HybridActionManager.Builder()
                .setHybridActionForward(HybridActionForward(this))
                .setHybridActionShare(HybridActionShare(this))
                .setHybridActionClientInfo(HybridActionClientInfo(this))
                .setHybridActionClipboard(HybridActionClipboard(this))
                .setHybridActionPayProduct(HybridActionPayProduct(this, this))
                .setHybridActionClose(HybridActionClose(this))
                .setHybridActionPermission(HybridActionPermission(this))
                .setHybridActionThirdApp(HybridActionThirdApp(this))
                .setHybridActionAbilityTask(HybridActionAbilityTask(this))
                .setHybridActionystemDownPath(HybridActionSystemDownPath(this))
                .setHybridActionDownloadAbility(HybridActionDownloadAbility(this, this))
                .setHybridActionCacheState(HybridActionCacheState(this))
                .setHybridActionPlayCore(HybridActionPlayCore(this))
                .setHybridActionFileOperation(HybridActionFileOperation(this, this))
                .setHybridActionNovelShelf(HybridActionNovelShelf(this))
                .build()
    }

    override fun initFragment() {

        val webClient = UniversalHybridWebClient(this, null, actionManager)

        val id = intent.getStringExtra(EXTRA_PACKAGE_ID)
        val viewPerformanceStatic = WebViewPerformanceStatic(id)
        val fastClient = FastWebViewClient(this, webClient, id, viewPerformanceStatic)

        val isImmersion = intent.getBooleanExtra(EXTRA_IMMERSION, true)

        val webview = NetDiskWebView(this)
        webview.setTransparent()
        val builder = WebViewFragmentBuilder()
                .setWebViewClient(fastClient)
                .setWebView(webview)
                .setWebChromeClient(ObserveTitleChangeWebChromeClient(this))
                .setDownloadListener(BaseOnDownloadListener())
                .setPerformanceStatic(viewPerformanceStatic)
                .setUrlLoadable(ImmediatelyUrlLoader(DefaultCookiesSyncer(
                        applicationContext), true))
        if (isImmersion) {
            mTitleBar.setRootViewVisible(false)
            builder.stateCallback = OfflineWebViewStateCallback()
        } else {
            mTitleBar.setRootViewVisible(true)
        }
        mFragment = builder.build()
        try {
            mFragment.arguments = intent.extras
        } catch (e: Exception) {
            NetDiskLog.e(TAG, e.message, e)
        }


        val ft = supportFragmentManager.beginTransaction()
        ft.add(R.id.content, mFragment, BaseWebViewFragment.TAG)
        ft.commitAllowingStateLoss()
    }

    override fun isActivityDark(): Boolean {
        return true
    }

    class OfflineWebViewStateCallback : IStateCallback {

        private var refActivity: Activity? = null

        override fun onViewCreated(activity: Activity?) {
            refActivity = activity
            refActivity?.window?.immerseTransparentStatusBar(false)
        }

        override fun onResume() {
            refActivity?.window?.immerseTransparentStatusBar(false)
        }

        override fun onDestroyView() {
        }

        override fun onPause() {
        }

        override fun onSetUserVisibleHint(fragment: BaseWebViewFragment?, isVisibleToUser: Boolean) {
        }

        /**
         * 状态栏透明沉浸态
         * @param lightColor 是否使用浅色模式，true:此时为浅色模式，此时状态栏上的颜色是黑色。false:反之
         */
        private fun Window.immerseTransparentStatusBar(lightColor: Boolean = false) {
            if (VERSION.SDK_INT >= VERSION_CODES.LOLLIPOP) {
                statusBarColor = Color.TRANSPARENT
                var option = View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN or View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                if (lightColor && VERSION.SDK_INT >= VERSION_CODES.M) {
                    option = option or View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
                }
                decorView.systemUiVisibility = option
            }
        }
    }
}

