package com.baidu.netdisk.ui.webview

import com.baidu.netdisk.kernel.util.encode.MD5Util
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.util.NetworkUtil
import org.json.JSONObject
/**
 * webview性能统计类
 *
 * author guochao02
 * email guochao02@baidu.com
 * created 2019/11/20 15:37
 */
class WebViewPerformanceStatic(private val pkgId: String) {

    // 统计日志串联id
    var mPerformanceLogID: String? = null
    // 创建webview的时间点
    var newWebViewTime: Long = 0
    // 页面启动的时间点
    var pageStartTime: Long = 0
    // 页面加载结束的时间点
    var pageFinishTime: Long = 0
    // 页面错误码
    var errorCode: Int = 0
    // url的path
    var urlPath: String? = null
    // 离线包版本号
    var pkgVersion: Int = 0
    // 页面错误详细信息
    var errorMsg: String? = null

    // 统一webview中页面的加载次数
    var loadCount = 0
    // 首次启动webview所用时间
    private var lastCreateTime:Long = 0

    init {
        mPerformanceLogID = generatePerformanceLogID()
    }

    /**
     * 生成统计日志串联ID
     */
    private fun generatePerformanceLogID(): String {
        val sb = StringBuilder().append(System.currentTimeMillis().toString())
                .append((0..9).random().toString())
        return MD5Util.createMD5WithHex(sb.toString(), "UTF-8")
    }

    /**
     * 上传日志
     */
    fun upload() {
        loadCount++
        val jsonObject = JSONObject()
        jsonObject.put("pkg_id", pkgId)
        jsonObject.put("pkg_version", pkgVersion)
        jsonObject.put("load_count", loadCount)
        if (loadCount == 1) {
            lastCreateTime = pageStartTime - newWebViewTime
        }
        jsonObject.put("create_time", lastCreateTime)
        jsonObject.put("layout_time", pageFinishTime - pageStartTime)
        jsonObject.put("error_code", errorCode)
        errorMsg?.apply {
            jsonObject.put("error_msg", errorMsg)
        }
        NetdiskStatisticsLogForMutilFields.getInstance()
                .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.HYBRID_PERFORMANCE,
                        mPerformanceLogID, urlPath, jsonObject.toString())
    }
}