package com.baidu.netdisk.ui.webview.hybrid.action

import android.app.Activity
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.trade.external.api.StatusListener
import com.baidu.netdisk.trade.external.api.TradeCommonApiCompManager
import com.baidu.netdisk.ui.view.IBaseView
import com.baidu.netdisk.ui.webview.hybrid.HybridConfig
import com.baidu.netdisk.ui.webview.hybrid.param.HybridUrlParam
import org.json.JSONObject

/**
 *
 * @author dnegjie09
 * @description: 为提供h5页面下载能力
 * @date：2019-11-05 20:43
 *
 */
class HybridActionDownloadAbility(val mActivity: Activity?, baseView: IBaseView) : HybridAction(baseView), StatusListener {

    companion object {

        const val TAG = "HybridActionDownloadAbility"

        const val FUNCTION_NAME_GET_DOWNLOAD_ABILITY = "download"

        const val DATA_ERROR = -1
        const val ERROR = -2
        const val TYPE_ERROR = -3

        // 状态和商城保持一致
        const val CACHE_STATE_APPEND = 0
        const val CACHE_STATE_START = 1
        const val CACHE_STATE_FINISHED = 2
        const val CACHE_STATE_PAUSED_BY_USER = 3
        const val CACHE_STATE_PAUSED_BY_NETWORK = 4
        const val CACHE_STATE_PAUSED_BY_STORAGE = 5
        const val CACHE_STATE_PAUSED_BY_ERROR = 6

    }

    // 处理成功多次回调，
    private val mapTask: HashSet<String> = HashSet<String>()

    override fun onAction(param: HybridUrlParam?) {
        NetDiskLog.d(TAG, "param.mParams: " + param?.mParams)

        mHybridParam = param

        if (FUNCTION_NAME_GET_DOWNLOAD_ABILITY == param?.mFuncName) {
            download()
        } else {
            handleRecognizeSchemeError(param)
        }
    }

    /**
     * 调起下载
     */
    private fun download() {

        try {
            val jsonObject = JSONObject(mHybridParam.mParams)

            if (!jsonObject.has("type") || !jsonObject.has("size") ||
            !jsonObject.has("pid") || !jsonObject.has("skuid") ||
            !jsonObject.has("name") || !jsonObject.has("format")) {
                handleFailedCallBack("hiatus params failed")
                return
            }

            NetDiskLog.d(TAG, " type:"  +jsonObject.getInt("type") + " size:" + jsonObject.getLong("size")
                    + " pid: " + jsonObject.getString("pid")+ " skuid: " + jsonObject.getString("skuid")+ "name: " +
                    jsonObject.getString("name")+ "format: " + jsonObject.getString("format"))


            mapTask.add(jsonObject.getString("skuid"))
            NetDiskLog.d(TAG, "add: $jsonObject.getString(\"skuid\")")

            // hybrid 定的类型与原来商城定的类型不一致，不一致表现为 比原来商城类型大1
            if (mActivity == null) {
                handleFailedCallBack("activity error")
                return
            }
            TradeCommonApiCompManager.hybridActionDownload(mActivity,
                    (jsonObject.getInt("type")), jsonObject.getLong("size"),
                    jsonObject.getString("pid"), jsonObject.getString("skuid"),
                    jsonObject.getString("name"), jsonObject.getString("format"),
                    this)
        } catch (e: Exception) {
            NetDiskLog.e(TAG, e.message, e)
        }
    }

    override fun onStatusChanged(status: String, skuId: String) {

        NetDiskLog.d(TAG, "status: $status,skuid:$skuId")
        try {
            when (status.toInt()) {

                TYPE_ERROR -> {
                    handleFailedCallBack("type error")
                }
                DATA_ERROR, ERROR-> {
                    handleFailedCallBack("params error")
                }

                CACHE_STATE_APPEND, CACHE_STATE_START -> {

                }

                CACHE_STATE_FINISHED -> {

                    NetDiskLog.d(TAG, "CACHE_STATE_FINISHED: $skuId")
                    if (mapTask.contains(skuId)) {
                        NetDiskLog.d(TAG, " can report CACHE_STATE_FINISHED: $skuId")
                        mapTask.remove(skuId)
                        val json = JSONObject()
                        json.put("result", "successful")
                        handleHybridCallback(mHybridParam, HybridConfig.CALLBACK_SUCCESS, "", json)
                    }
                }

                CACHE_STATE_PAUSED_BY_NETWORK -> {
                    handleFailedCallBack("network error")
                }
                CACHE_STATE_PAUSED_BY_STORAGE -> {
                    handleFailedCallBack("insufficient memory")
                }

                CACHE_STATE_PAUSED_BY_ERROR -> {
                    handleFailedCallBack("download failed")
                }
            }
        } catch (e: Exception) {

        }
    }

    /**
     * 失败回调
     */
    private fun handleFailedCallBack(msg: String) {
        handleHybridCallback(mHybridParam, HybridConfig.CALLBACK_FAILED,
                msg, JSONObject())
    }


}