package com.baidu.netdisk.ui.webview.hybrid.action

import android.arch.lifecycle.Lifecycle
import android.arch.lifecycle.LifecycleObserver
import android.arch.lifecycle.LifecycleOwner
import android.arch.lifecycle.OnLifecycleEvent
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.trade.external.api.StatusListener
import com.baidu.netdisk.trade.external.api.TradeCommonApiCompManager
import com.baidu.netdisk.tradeplatform.TradePlatformHelper
import com.baidu.netdisk.ui.view.IBaseView
import com.baidu.netdisk.ui.webview.hybrid.HybridConfig
import com.baidu.netdisk.ui.webview.hybrid.call.ICallH5Funtion.FUNCTION_CHANGEPLAYSTATE
import com.baidu.netdisk.ui.webview.hybrid.param.HybridUrlParam
import org.json.JSONObject

/**
 *
 * bdwpin://WPJS_media_play/cb_3_1508399455311/getAudioState?params={""}
 * 调用端上播放能力
 *
 * author guochao02
 * email guochao02@baidu.com
 * created 2019/12/5 15:32
 */
class HybridActionPlayCore(val baseView: IBaseView) : HybridAction(baseView) {

    private val FUNCTION_NAME_GET_AUDIO_STATE = "getAudioState"

    // 监听所属Activity的生命周期
    private var mActivityLifeObserver: ActivityLifeObserver? = null

    // 是否需要清理音频状态监听器
    private var mIsNeedDestructAudioListener = false

    // 音频播放状态
    private var mAudioPlayState: String? = null
    /**
     * 交易平台音频监听器
     */
    private var mTradeAudioListener: StatusListener? = null

    init {
        if (baseView.activity is BaseActivity) {
            val lifecycleOwner = (baseView.activity as BaseActivity)
            mActivityLifeObserver = ActivityLifeObserver(lifecycleOwner) { stopMonitorAudioPlayState() }
            lifecycleOwner.lifecycle.addObserver(mActivityLifeObserver!!)
        }
    }

    override fun onAction(param: HybridUrlParam?) {
        mHybridParam = param
        if (FUNCTION_NAME_GET_AUDIO_STATE == param?.mFuncName) {
            handleGetAudioState(mHybridParam.mParams)
        } else {
            handleRecognizeSchemeError(param)
        }
    }

    /**
     * 启动对交易平台音频播放状态的监听
     */
    private fun startMonitorAudioPlayState() {
        if (mTradeAudioListener == null) {
            mTradeAudioListener = StatusListener { state: String?, _: String? ->
                state?.apply {
                    if (mAudioPlayState != this) {
                        mAudioPlayState = state
                        val jsonObject = JSONObject()
                        jsonObject.put("state", TradePlatformHelper.transformAudioPlayStateToInt(state))
                        callH5Function(FUNCTION_CHANGEPLAYSTATE, 1, "", jsonObject)
                    }
                }
            }
        }
        mIsNeedDestructAudioListener = true
        TradeCommonApiCompManager.registerListener(mTradeAudioListener)
    }

    /**
     * 停止对交易平台音频播放状态的监听
     */
    private fun stopMonitorAudioPlayState() {
        if (mIsNeedDestructAudioListener) {
            TradeCommonApiCompManager.unregisterListener(mTradeAudioListener)
        }
    }

    /**
     * 处理获取音频播放状态的请求
     * state状态值详见TradePlatformHelper
     * 0代表未播放
     * 1代表播放中
     * 2代表暂停
     */
    private fun handleGetAudioState(param: String) {
        val paramJson = JSONObject(param)
        if (paramJson.optString("monitorState") == "1") {
            startMonitorAudioPlayState()
        }
        val json = JSONObject()
        json.put("state", TradePlatformHelper.getTradeAudioPlayState())
        handleHybridCallback(mHybridParam, HybridConfig.CALLBACK_SUCCESS, "", json)
    }

    /**
     * Activity生命周期监听
     * 目前主要用来数据回收
     */
    class ActivityLifeObserver(private val owner: LifecycleOwner, val method: () -> Unit) : LifecycleObserver {

        @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
        fun onDestroy() {
            method()
            owner.lifecycle.removeObserver(this)
        }
    }
}
