package com.baidu.netdisk.ui.webview.hybrid.action

import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.trade.external.api.TradeCommonApiCompManager
import com.baidu.netdisk.ui.view.IBaseView
import com.baidu.netdisk.ui.webview.hybrid.HybridConfig
import com.baidu.netdisk.ui.webview.hybrid.param.HybridUrlParam
import org.json.JSONException
import org.json.JSONObject
import java.lang.NumberFormatException

/**
 *
 * @author dnegjie09
 * @description: H5通过hybird获取下载路径
 * @date：2019-11-05 15:47
 *
 */
class HybridActionSystemDownPath(baseView: IBaseView) : HybridAction(baseView) {

    companion object {

        const val TAG = "HybridActionSystemDownPath"
        const val FUNCTION_NAME_GET_SYSTEM_DOWNPATH = "getDownPath"
    }

    override fun onAction(param: HybridUrlParam?) {

        NetDiskLog.d(TAG, "param.mParams: " + param?.mParams)
        if (FUNCTION_NAME_GET_SYSTEM_DOWNPATH == param?.mFuncName) {
            mHybridParam = param
            handleSystemDownPathFunction(param)
        } else {
            handleRecognizeSchemeError(param)
        }
    }

    /**
     * 封装下载路径并回调
     */
    private fun handleSystemDownPathFunction(param: HybridUrlParam?) {

        try {
            val json = JSONObject()
            val jsonObject = JSONObject(param?.mParams)
            if (!jsonObject.has("type")) {
                json.put("path", "")
                handleHybridCallback(param, HybridConfig.CALLBACK_FAILED, "GET SYSTEM DOWNPATH FAILED", json)
                return
            }
            val type = jsonObject.optString("type","0").toInt()

            val url = TradeCommonApiCompManager.getProductSystenDownPath(BaseApplication.mContext, type)
            json.put("path", url)
            if (!url.isNullOrEmpty()) {
                handleHybridCallback(param, HybridConfig.CALLBACK_SUCCESS, "", json)
            } else {
                handleHybridCallback(param, HybridConfig.CALLBACK_FAILED, "GET SYSTEM DOWNPATH FAILED", json)
            }
        } catch (e: Exception) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: NumberFormatException) {
            NetDiskLog.e(TAG, e.message, e)
        }
    }
}