package com.baidu.netdisk.ui.xpan.classification

import android.app.Activity
import android.app.Dialog
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.cloudfile.io.model.CloudFile
import com.baidu.netdisk.cloudfile.io.model.DirectoryAttribute
import com.baidu.netdisk.cloudimage.ui.propertyalbum.PropertyAlbumActivity
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.xpan.ISmartDevice
import com.baidu.netdisk.xpan.io.parser.model.SmartDevice
import com.baidu.netdisk.xpan.io.parser.model.SmartDuNormalDirResponse

class SmartXAbilityPresenter(var activity: Activity?, private val smartDeviceManager: ISmartDevice) {

    private val mNormalDirReceiver = NormalDirReceiver(this, Handler(), null)

    private var mLoadingDialog: Dialog? = null
    private var mIsDestroy = false

    fun jumpToNormalAlbum(smartDevice: SmartDevice) {
        activity?.takeIf {
            !it.isFinishing
        }?.apply {
            showLoading(this)
            smartDeviceManager.getSmartDuNormalDir(applicationContext, mNormalDirReceiver, AccountUtils.getInstance().bduss, AccountUtils.getInstance().uid,
                    smartDevice.id)
        }
    }

    fun readyJumpNormalAlbum(targetDir: String?) {
        targetDir?.takeIf {
            it.isNotEmpty() && !mIsDestroy && activity != null
        }?.apply {
            val cloudFile = CloudFile(this)
            cloudFile.directoryType = DirectoryAttribute.PHOTO_TYPE
            PropertyAlbumActivity.startActivity(activity, cloudFile, false)
        }
    }

    private fun showLoading(context: Activity) {
        mLoadingDialog = DialogBuilder().buildLoadingDialogWithoutText(context).apply {
            show()
        }
    }

    private fun dismissDialog() {
        mLoadingDialog?.takeIf {
            it.isShowing
        }?.dismiss()
    }

    fun onDestroy() {
        mIsDestroy = true
        activity = null
        dismissDialog()
    }

    inner class NormalDirReceiver(reference: SmartXAbilityPresenter, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<SmartXAbilityPresenter>(reference, handler, resultView) {

        override fun onSuccess(reference: SmartXAbilityPresenter, resultData: Bundle?) {
            val dirResponse = resultData?.getParcelable<SmartDuNormalDirResponse>(BaseExtras.RESULT)
            dismissDialog()
            dirResponse?.takeIf {
                !TextUtils.isEmpty(dirResponse.targetDir)
            }?.apply {
                reference.readyJumpNormalAlbum(this.targetDir)
            } ?: ToastHelper.showToast(R.string.xpan_common_net_error)
        }

        override fun onFailed(reference: SmartXAbilityPresenter, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            dismissDialog()
            ToastHelper.showToast(R.string.xpan_common_net_error)
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}