package com.baidu.netdisk.ui.xpan.classification

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.view.Window
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.xpan.io.parser.model.SmartDevice
import com.baidu.netdisk.xpan.io.parser.model.SmartDeviceCustomizeAbility
import com.baidu.netdisk.xpan.service.Extras

/**
 * 智能硬件独立空间页面
 *
 * @author rendawei
 * @since 2019/10/10
 */
class SmartXCustomizeActivity : BaseActivity() {

    companion object {

        private const val MODE_CUSTOMIZE = 1

        /**
         * 打开看护助手
         */
        fun startSmartXCustomizeActivity(activity: Activity, smartDevice: SmartDevice,
                                         customizeAbility: SmartDeviceCustomizeAbility) {
            Intent(activity, SmartXCustomizeActivity::class.java).apply {
                putExtra(Extras.DEVICE, smartDevice)
                putExtra(Extras.EXTRA_PAGE_MODE, MODE_CUSTOMIZE)
                putExtra(SmartXCustomizeVideoFragment.EXTRA_ABILITY, customizeAbility)
                activity.startActivity(this)
            }
        }
    }

    private var mPageMode = -1
    private var mCurrentFragment: Fragment? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        requestWindowFeature(Window.FEATURE_NO_TITLE)
        super.onCreate(savedInstanceState)
    }

    override fun getLayoutId(): Int {
        return R.layout.activity_smartx_customize
    }

    override fun initParams() {
        intent?.apply {
            mPageMode = getIntExtra(Extras.EXTRA_PAGE_MODE, -1)
        }
    }

    override fun initView() {
        if (mPageMode <= 0) {
            finish()
            return
        }
        initFragment()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        mCurrentFragment?.onActivityResult(requestCode, resultCode, data)
    }

    override fun isActivityDark() = true

    private fun initFragment() {
        mCurrentFragment = when (mPageMode) {
            MODE_CUSTOMIZE -> SmartXCustomizeVideoFragment.newInstance(intent.getParcelableExtra(Extras.DEVICE),
                    intent.getParcelableExtra(SmartXCustomizeVideoFragment.EXTRA_ABILITY))
            else -> null
        }
        mCurrentFragment?.apply {
            val transaction = supportFragmentManager.beginTransaction()
            transaction.replace(R.id.fl_root, this)
            transaction.commitAllowingStateLoss()
        } ?: finish()
    }
}