package com.baidu.netdisk.ui.xpan.classification

import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.FrameLayout
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.base.utils.FileCategory
import com.baidu.netdisk.cloudfile.io.model.CloudFile
import com.baidu.netdisk.cloudfile.storage.db.CloudFileContract
import com.baidu.netdisk.kernel.android.util.deviceinfo.DeviceDisplayUtils
import com.baidu.netdisk.kernel.android.util.file.FileUtils
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.widget.EmptyView
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.ui.widget.titlebar.ITitleBarSelectedModeListener
import com.baidu.netdisk.ui.xpan.classification.adapter.SmartDividerItemDecoration
import com.baidu.netdisk.ui.xpan.classification.adapter.SmartXCustomizeVideoAdapter
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.config.SmartDeviceConfigKey
import com.baidu.netdisk.xpan.io.parser.model.SmartCustomizeVideo
import com.baidu.netdisk.xpan.io.parser.model.SmartDevice
import com.baidu.netdisk.xpan.io.parser.model.SmartDeviceCustomizeAbility
import com.baidu.netdisk.xpan.service.Extras
import com.baidu.netdisk.xpan.statistics.XPanStatisticsKeys
import com.netdisk.themeskin.loader.SkinManager
import kotlinx.android.synthetic.main.fragment_smartx_customize_video.*
import kotlinx.android.synthetic.main.layout_customize_bottom_bar.*
import java.util.*

/**
 * 看护助手视频
 *
 * @author rendawei
 * @since 2019/10/10
 */
class SmartXCustomizeVideoFragment : BaseCustomizeFragment(), ICommonTitleBarClickListener,
        IClassificationLoadView<SmartCustomizeVideo> {

    companion object {

        const val EXTRA_ABILITY = "extra_ability"

        fun newInstance(smartDevice: SmartDevice, customizeAbility: SmartDeviceCustomizeAbility):
                SmartXCustomizeVideoFragment =
                SmartXCustomizeVideoFragment().apply {
                    val bundle = Bundle()
                    bundle.putParcelable(Extras.DEVICE, smartDevice)
                    bundle.putParcelable(EXTRA_ABILITY, customizeAbility)
                    arguments = bundle
                }
    }

    private var mSmartDevice: SmartDevice? = null
    private var mCustomizeAbility: SmartDeviceCustomizeAbility? = null

    private var mCustomizePresenter: SmartXCustomizeVideoPresenter? = null
    private var mAdapter: SmartXCustomizeVideoAdapter? = null

    private var mLoadingMore = false

    private var mBtnDownLoad: Button? = null
    private var mBtnDelete: Button? = null
    private var mBtnRename: Button? = null
    private var mBtnReSave: Button? = null
    private var mFlReSave: FrameLayout? = null
    private var mBtnMore: Button? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        context?.apply {
            mCustomizePresenter = SmartXCustomizeVideoPresenter(getService(BaseActivity.SMART_DEVICE_SERVICE),
                    this@SmartXCustomizeVideoFragment, this@SmartXCustomizeVideoFragment).also {
                it.smartDevice = mSmartDevice
                // 请求vip状态
                it.getDuVipStatus(this, true)
            }
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_smartx_customize_video, container, false)
    }

    override fun onDestroy() {
        super.onDestroy()
        mCustomizePresenter?.onDestroy()
    }

    override fun initView(rootView: View?) {
        initTitle()
        initNotice()
        initList()
        initBottomBar()
    }

    override fun initData() {
        fetchData(refresh = true, loadMore = false)
    }

    override fun validParam(): Boolean {
        return arguments?.let {
            mSmartDevice = it.getParcelable(Extras.DEVICE)
            mCustomizeAbility = it.getParcelable(EXTRA_ABILITY)
            mSmartDevice != null && mCustomizeAbility != null
        } ?: false
    }

    override fun onRightButtonClicked(view: View?) {
        if (!isVideosEmpty()) {
            mBaseTitleBar.switchToEditMode()
            mAdapter?.isChooseMode = true
            refreshSelect()
        }
    }

    override fun onBackButtonClicked() {
        finish()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        mCustomizePresenter?.onActivityResult(requestCode, resultCode, data)
    }

    private fun initTitle() {
        mBaseTitleBar = BaseTitleBar(activity)
        mBaseTitleBar.setLeftLabel(mCustomizeAbility?.title ?: "")
        mBaseTitleBar.leftTextView?.textSize = DeviceDisplayUtils.dip2sp(activity, 17F).toFloat()
        mBaseTitleBar.setRightButtonDrawable(SkinManager.getInstance().getDrawable(R.drawable.bg_dn_common_titlebar_btn_select))
        mBaseTitleBar.setTopTitleBarClickListener(this)
        mBaseTitleBar.setSelectedModeListener(object : ITitleBarSelectedModeListener {
            override fun onCancelClick() {
                cancelEditMode()
            }

            override fun onSelectAllClick() {
                mAdapter?.toggleSelectAll()
                displayBottomBar(true)
                refreshSelect()
            }
        })
    }

    private fun initNotice() {
        // 100版本不显示
        val isShowed = PersonalConfig.getInstance().getBoolean(SmartDeviceConfigKey.SMART_DEVICE_CUSTOMIZE_VIDEO_GUIDE, true)
        if (!isShowed) {
            ll_notice.visibility = View.VISIBLE
            tv_top_title.setText(R.string.classification_customize_video_notice)
            btn_know.setOnClickListener {
                ll_notice.visibility = View.GONE
                PersonalConfig.getInstance().putBoolean(SmartDeviceConfigKey.SMART_DEVICE_CUSTOMIZE_VIDEO_GUIDE, true)
            }
        }
    }

    private fun initList() {
        context?.apply context@{
            val layoutManager = LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false)
            recycler_view.addItemDecoration(SmartDividerItemDecoration(this, layoutManager.orientation).apply {
                setDrawable(SkinManager.getInstance().getDrawable(R.drawable.bg_dn_list_view_divider))
                setShowFirst(false)
                setShowLast(false)
            })
            recycler_view.setLoadMoreEnabled(true)
            recycler_view.layoutManager = layoutManager
            mAdapter = SmartXCustomizeVideoAdapter(this).apply {
                mOnChooseModeListener = {
                    if (it) {
                        mBaseTitleBar?.switchToEditMode()
                        refreshSelect()
                    } else {
                        mBaseTitleBar?.switchToNormalMode()
                    }
                    displayBottomBar(it)
                    // 编辑态禁止刷新
                    recycler_view.setRefreshEnabled(!it)
                }

                mOnItemClickListener = listener@{
                    if (isChooseMode) {
                        toggleItemCheck(it)
                        refreshSelect()
                        displayBottomBar(true)
                        return@listener
                    }
                    // 跳转视频播放
                    mCustomizeVideos[it].runCatching {
                        activity?.takeIf { ctx ->
                            !ctx.isFinishing
                        }?.let { ctx ->
                            mCustomizePresenter?.playSingleVideo(ctx, covertCloudFile(this))
                        }
                        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(XPanStatisticsKeys.XPAN_CUSTOMEIZE_VIDEO_CLICK)
                    }
                }

                mOnItemLongClickListener = listener@{
                    if (!isChooseMode) {
                        isChooseMode = true
                        toggleItemCheck(it)
                        refreshSelect()
                        displayBottomBar(true)
                    }
                    return@listener true
                }
            }
            recycler_view.adapter = mAdapter
            recycler_view.setOnRefreshListener {
                fetchData(refresh = false, loadMore = false)
            }
            recycler_view.setOnLoadMoreListener {
                fetchData(refresh = false, loadMore = true)
            }
        }
    }

    private fun initBottomBar() {
        mCustomizeAbility?.apply {
            mBtnDownLoad = btn_to_download
            mBtnDelete = btn_to_delete
            mBtnRename = btn_to_rename
            mBtnReSave = btn_to_resave
            mFlReSave = fl_resave
            mBtnMore = btn_more
            displayBottomBar(false)
            // 设置点击事件
            btn_to_download?.setOnClickListener {
                onDownLoad()
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(XPanStatisticsKeys.XPAN_CUSTOMEIZE_DOWNLOAD_CLICK)
            }
            btn_to_delete?.setOnClickListener {
                onDelete()
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(XPanStatisticsKeys.XPAN_CUSTOMEIZE_DELETE_CLICK)
            }
            btn_to_rename?.setOnClickListener {
                onRename()
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(XPanStatisticsKeys.XPAN_CUSTOMEIZE_RENAME_CLICK)
            }
            btn_to_resave?.setOnClickListener {
                onReSave()
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(XPanStatisticsKeys.XPAN_CUSTOMEIZE_RESAVE_CLICK)
            }
        }
    }

    private fun displayBottomBar(show: Boolean) {
        if (mCustomizeAbility?.supportBarOpts().isNullOrEmpty() || !show) {
            fl_bottom?.visibility = View.GONE
            return
        }
        fl_bottom?.visibility = View.VISIBLE
        hideOptBtn()
        mCustomizeAbility?.supportBarOpts()?.forEach {
            // 转存按钮特殊处理
            val size = selectedItemsPosition.size
            if (TextUtils.equals(SmartDeviceCustomizeAbility.OPT_COPY, it)) {
                mFlReSave?.visibility = View.VISIBLE
                if (size > 0) {
                    iv_vip?.setImageResource(R.drawable.xpan_ic_du_vip)
                } else {
                    iv_vip?.setImageResource(R.drawable.xpan_ic_du_vip_pressed)
                }
            } else {
                getOptBtn(it)?.visibility = View.VISIBLE
            }
            when {
                size <= 0 -> getOptBtn(it)?.isEnabled = false
                size == 1 -> getOptBtn(it)?.isEnabled = true
                else -> getOptBtn(it)?.isEnabled = !TextUtils.equals(SmartDeviceCustomizeAbility.OPT_RENAME, it)
            }
        }
    }

    private fun hideOptBtn() {
        mBtnDownLoad?.visibility = View.GONE
        mBtnDelete?.visibility = View.GONE
        mBtnRename?.visibility = View.GONE
        mFlReSave?.visibility = View.GONE
        mBtnMore?.visibility = View.GONE
    }

    private fun getOptBtn(optId: String): Button? {
        return when (optId) {
            SmartDeviceCustomizeAbility.OPT_DOWNLOAD -> mBtnDownLoad
            SmartDeviceCustomizeAbility.OPT_DELETE -> mBtnDelete
            SmartDeviceCustomizeAbility.OPT_RENAME -> mBtnRename
            SmartDeviceCustomizeAbility.OPT_COPY -> mBtnReSave
            else -> null
        }
    }

    private fun fetchData(refresh: Boolean, loadMore: Boolean) {
        context?.let {
            mSmartDevice?.takeIf {
                it.id.isNotEmpty()
            }?.apply {
                if (refresh) {
                    showLoading()
                    mCustomizePresenter?.getCustomizeVideos(it, 0, id)
                    return
                }
                if (mLoadingMore) {
                    return
                }
                mLoadingMore = loadMore
                val start = if (loadMore) mAdapter?.itemCount ?: 0 else 0
                mCustomizePresenter?.getCustomizeVideos(it, start, id)
            } ?: finish()
        }
    }

    override fun onLoadSuccess(tags: MutableList<SmartCustomizeVideo>?, actionType: Int, hasMore: Boolean) {
        if (isDestroying) {
            return
        }
        if (mLoadingMore) {
            mAdapter?.appendCustomizeVideos(tags)
        } else {
            mAdapter?.setCustomizeVideos(tags)
        }
        showResultView(false)
        recycler_view?.isRefreshing = false
        recycler_view?.setLoadMoreEnabled(hasMore)
        mLoadingMore = false
    }

    override fun onLoadFail() {
        if (isDestroying) {
            return
        }
        showResultView(true)
        recycler_view?.isRefreshing = false
        mLoadingMore = false
    }

    private fun doOperationSucSync(operation: Int) {
        if (BaseStatus.SUCCESS == operation && !isDestroying) {
            cancelEditMode()
            fetchData(refresh = false, loadMore = false)
        }
    }

    override fun onRenameSuccess(operation: Int) {
        NetDiskLog.d(TAG, "onRenameSuccess operation:$operation")
        doOperationSucSync(operation)
    }

    override fun onDeleteSuccess(operation: Int) {
        NetDiskLog.d(TAG, "onDeleteSuccess operation:$operation")
        doOperationSucSync(operation)
    }

    override fun onMoveFinished(operation: Int) {
        NetDiskLog.d(TAG, "onMoveFinished operation:$operation")
        doOperationSucSync(operation)
    }

    private fun onDownLoad() {
        mNetDiskFilePresenter?.onButtonDownload(null)
    }

    private fun onDelete() {
        mCustomizePresenter?.onButtonDelete()
    }

    private fun onRename() {
        mNetDiskFilePresenter?.onButtonRename()
    }

    private fun onReSave() {
        mCustomizePresenter?.onButtonCopy()
    }

    private fun showLoading() {
        empty_view.setLoading(R.string.loading, resources.getColor(R.color.common_color_666),
                R.drawable.classification_loading_anim)
        rl_content.visibility = View.GONE
    }

    private fun isVideosEmpty(): Boolean = mAdapter?.itemCount ?: 0 <= 0

    private fun showResultView(error: Boolean) {
        if (isVideosEmpty()) {
            empty_view.visibility = View.VISIBLE
            rl_content.visibility = View.GONE
        } else {
            empty_view.visibility = View.GONE
            rl_content.visibility = View.VISIBLE
        }
        if (error) {
            if (!isVideosEmpty()) {
                ToastHelper.showToast(R.string.classification_customize_load_error)
                return
            }
            empty_view.setLoadError(R.string.backup_server_error, R.drawable.xpan_empty_error)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_999))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.VISIBLE)
            empty_view.setRefreshButtonTextColor(resources.getColor(R.color.empty_refresh_text_color))
            empty_view.setRefreshText(R.string.refresh_verify_code_text)
            empty_view.setRefreshListener {
                fetchData(refresh = true, loadMore = false)
            }
            return
        }
        if (isVideosEmpty()) {
            empty_view.setLoadNoData(resources.getString(R.string.classification_customize_video_empty), R.drawable.empty_video)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_666))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.GONE)
        }
    }

    private fun refreshSelect() {
        mAdapter?.apply {
            if (!isChooseMode || isVideosEmpty()) {
                return
            }
            mBaseTitleBar?.setSelectedNum(chooseItems.size, itemCount)
        }
    }

    private fun covertCloudFile(smartCustomizeVideo: SmartCustomizeVideo): CloudFile {
        return CloudFile(smartCustomizeVideo.name
                ?: "", CloudFileContract.Files.IS_FILE, smartCustomizeVideo.size,
                smartCustomizeVideo.path, smartCustomizeVideo.md5, smartCustomizeVideo.fsId).apply {
            parent = CloudFile(FileUtils.getParentPath(smartCustomizeVideo.path))
        }
    }

    override fun getSelectedItemsPosition(): ArrayList<Int> {
        return ArrayList(mAdapter?.chooseItems ?: emptyList<Int>())
    }

    override fun getItem(pos: Int): CloudFile {
        return mAdapter?.mCustomizeVideos?.get(pos)?.let {
            return covertCloudFile(it)
        } ?: CloudFile()
    }

    override fun isViewMode(): Boolean {
        return mAdapter?.isChooseMode == false
    }

    override fun cancelEditMode() {
        mAdapter?.isChooseMode = false
    }

    override fun getAdapterCount(): Int {
        return mAdapter?.itemCount ?: 0
    }

    override fun getEmptyView(): EmptyView {
        return empty_view
    }

    override fun getCurrentCategory(): Int {
        return FileCategory.CATEGORY_VIDEO
    }
}