package com.baidu.netdisk.ui.xpan.classification

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.base.storage.config.ConfigSmartDevice
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.cloudfile.constant.CloudFileConstants
import com.baidu.netdisk.cloudfile.constant.FileManagerExtras
import com.baidu.netdisk.cloudfile.constant.NetdiskErrorCode
import com.baidu.netdisk.cloudfile.constant.PCSErrorCode
import com.baidu.netdisk.cloudfile.io.model.CloudFile
import com.baidu.netdisk.cloudfile.io.model.MoveCopyFile
import com.baidu.netdisk.cloudfile.service.CloudFileService
import com.baidu.netdisk.cloudfile.service.CloudFileServiceHelper
import com.baidu.netdisk.cloudfile.storage.config.CloudFileConfigKey
import com.baidu.netdisk.cloudfile.storage.config.FileManagerConfig
import com.baidu.netdisk.cloudfile.storage.db.CloudFileContract
import com.baidu.netdisk.cloudfile.utils.CloudFileHelper
import com.baidu.netdisk.component.base.CommonConstant
import com.baidu.netdisk.kernel.android.util.file.FileUtils
import com.baidu.netdisk.kernel.architecture.config.GlobalConfig
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.main.caller.FSCommonApiCompManager
import com.baidu.netdisk.main.caller.FSConstantApiCompManager
import com.baidu.netdisk.main.caller.FSLocalFileApiCompManager
import com.baidu.netdisk.main.caller.FSSecondPwdApiCompManager
import com.baidu.netdisk.preview.video.controller.IVideoPluginInstallCallbacker
import com.baidu.netdisk.ui.account.AccountErrorHandler
import com.baidu.netdisk.ui.cloudfile.view.INetdiskFileView
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.preview.OpenFileHelper
import com.baidu.netdisk.ui.preview.OpenVideoFileHelper
import com.baidu.netdisk.ui.widget.LoadingDialog
import com.baidu.netdisk.ui.widget.NewVersionDialog
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.FailedViewManager
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.xpan.ISmartDevice
import com.baidu.netdisk.xpan.config.SmartDeviceConfigKey
import com.baidu.netdisk.xpan.io.parser.model.SmartCustomizeVideo
import com.baidu.netdisk.xpan.io.parser.model.SmartCustomizeVideoResponse
import com.baidu.netdisk.xpan.io.parser.model.SmartDevice
import com.baidu.netdisk.xpan.io.parser.model.SmartDuVipResponse
import java.util.*

class SmartXCustomizeVideoPresenter(private val deviceManager: ISmartDevice?,
                                    private var resultView: IClassificationLoadView<SmartCustomizeVideo>? = null,
                                    private val netDiskFileView: INetdiskFileView) :
        IClassificationLoadView<SmartCustomizeVideo> {

    companion object {
        const val TAG = "SmartXCustomizeVideoPresenter"
    }

    private val mCustomizeResultReceiver = CustomizeResultReceiver(this, Handler(), null)
    private val mDeleteFileResultView: DeleteFileResultView
    private val mDeleteFileResultReceiver: DeleteFileResultReceiver
    private val mCopyFileResultView: CopyResultView
    private val mCopyFileResultReceiver: CopyResultReceiver
    private val mDuVipResultReceiver = object : BaseResultReceiver<SmartXCustomizeVideoPresenter>(this,
            Handler(), null) {
        override fun onSuccess(reference: SmartXCustomizeVideoPresenter, resultData: Bundle?) {
            mRequestVipSuccess = true
        }

        override fun onFailed(reference: SmartXCustomizeVideoPresenter, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            mRequestVipSuccess = false
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    private var mSelectedItemPositions: MutableList<Int>? = null
    private var mTempCopyPositions: List<Int>? = null

    private var mLoadingDialog: Dialog? = null
    private var mRequestVipSuccess = false
    private val mLimitPage: Int

    var smartDevice: SmartDevice? = null

    init {
        mDeleteFileResultView = DeleteFileResultView(netDiskFileView.activity)
        mDeleteFileResultReceiver = DeleteFileResultReceiver(this, Handler(), mDeleteFileResultView)
        mCopyFileResultView = CopyResultView(netDiskFileView.activity)
        mCopyFileResultReceiver = CopyResultReceiver(this, Handler(), mCopyFileResultView)
        ConfigSmartDevice(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.SMART_DEVICE)).also {
            mLimitPage = it.customizeFileLimit
        }
    }

    override fun onLoadSuccess(tags: MutableList<SmartCustomizeVideo>?, actionType: Int, hasMore: Boolean) {
        resultView?.onLoadSuccess(tags, actionType, hasMore)
    }

    override fun onLoadFail() {
        resultView?.onLoadFail()
    }

    /**
     * 获取独立空间的视频列表
     */
    fun getCustomizeVideos(context: Context, start: Int, deviceId: String) {
        deviceManager?.apply {
            getCustomizeVideos(context.applicationContext, mCustomizeResultReceiver, AccountUtils.getInstance().bduss,
                    AccountUtils.getInstance().uid, deviceId, start, mLimitPage)
        }
    }

    /**
     * 获取小度会员状态
     */
    fun getDuVipStatus(context: Context, force: Boolean = false) {
        if (mRequestVipSuccess && !force) {
            return
        }
        deviceManager?.apply {
            smartDevice?.id?.takeIf {
                it.isNotEmpty()
            }?.let {
                getSmartDuVipStatus(context, mDuVipResultReceiver, AccountUtils.getInstance().bduss, AccountUtils.getInstance().uid, it)
            }
        }
    }

    fun isDuVip(context: Context): Boolean {
        getDuVipStatus(context)
        smartDevice?.id?.takeIf {
            it.isNotEmpty()
        }?.let {
            return PersonalConfig.getInstance().getInt(SmartDeviceConfigKey.SMART_DEVICE_DU_VIP + it,
                    -1) == SmartDuVipResponse.STATUS_VIP
        } ?: return false
    }

    fun playSingleVideo(activity: Activity?, cloudFile: CloudFile?) {
        activity?.applicationContext?.let {
            cloudFile ?: return@let
            val videos = arrayListOf(cloudFile)
            with(OpenVideoFileHelper()) {
                openNormalMedia(it, videos, object : IVideoPluginInstallCallbacker {
                    override fun onVideoPluginIgnore() {
                        OpenFileHelper().openIntentActivity(cloudFile, it)
                    }

                    override fun onVideoPluginInstallCancel() {
                        OpenFileHelper().openIntentActivity(cloudFile, it)
                    }

                    override fun onVideoPluginInstallFinish() {
                        openNormalMedia(it, videos, this)
                    }
                })
            }
        }
    }

    private fun doLimitCheck(opCode: Int, operator: ((List<Int>?) -> Unit)?) {
        if (!netDiskFileView.isViewMode) {
            mSelectedItemPositions = netDiskFileView.selectedItemsPosition
            NetDiskLog.d(TAG, "check In Edit Mode:${mSelectedItemPositions?.size}")
        }
        mSelectedItemPositions?.takeIf {
            it.isNotEmpty() && !hasRunningTask()
        }?.let { delPos ->
            val limit = FSCommonApiCompManager.getFileManagerHelperLimit();
//            FileManagerHelper().fileManagerLimit
            if (limit in 1 until delPos.size) {
                FSCommonApiCompManager.showFileManagerHelperLimitDialog(netDiskFileView.activity, opCode, limit,
                        object : DialogCtrListener {

                            override fun onOkBtnClick() {
                                operator?.invoke(delPos.take(limit))
                            }

                            override fun onCancelBtnClick() {
                                mSelectedItemPositions?.clear()
                            }
                        })
            } else {
                operator?.invoke(delPos)
            }
        }
    }

    /**
     * 删除独立空间的文件
     */
    fun onButtonDelete() {
        NetDiskLog.d(TAG, "Delete button clicked")
        doLimitCheck(FSConstantApiCompManager.getFileManagerHelper2OpDelete()) {
            deleteFiles(it)
        }
    }

    private fun deleteFiles(deletePaths: List<Int>?) {
        deletePaths?.apply {
            map {
                val cloudFile = netDiskFileView.getItem(it)
                if (cloudFile.id == 0L) "" else getCurrentPath(cloudFile)
            }.filter {
                it.isNotEmpty()
            }.takeIf {
                it.isNotEmpty()
            }?.let {
                val dialogBuilder = DialogBuilder()
                dialogBuilder.buildTipsDialog(netDiskFileView.activity, R.string.classification_customize_delete_title,
                        R.string.classification_customize_delete_content,
                        R.string.classification_customize_delete_ok, R.string.cancel)
                dialogBuilder.setOnDialogCtrListener(object : DialogCtrListener {
                    override fun onCancelBtnClick() {
                        mSelectedItemPositions?.clear()
                    }

                    override fun onOkBtnClick() {
                        sendDeleteRequestProxy(it)
                    }
                })
            }
        }
    }

    private fun sendDeleteRequestProxy(paths: List<String>) {
        NetDiskLog.d(TAG, "start call delete")
        // 如果删除文件在文件列表界面，传入当前目录；如果删除文件在分类界面，传入null
        CloudFileServiceHelper.delete(netDiskFileView.activity.applicationContext, mDeleteFileResultReceiver, ArrayList(paths),
                null,
                CloudFileService.FILE_MANAGER_AUTO, CloudFileService.ONDUP_FAIL)
        showLoadingView()
    }

    private fun getCurrentPath(item: CloudFile): String {
        val name = item.fileName
        var path = item.filePath
        path = CloudFileHelper.getFilePreferPath(path, name)
        return path
    }

    /**
     * 复制独立空间的文件到网盘
     */
    fun onButtonCopy() {
        NetDiskLog.d(TAG, "Copy button clicked")
        if (!isDuVip(netDiskFileView.activity)) {
            DialogBuilder().buildOneButtonDialog(netDiskFileView.activity, R.string.classification_no_permission,
                    R.string.classification_not_du_vip_content, R.string.classification_not_du_vip_ok).show()
            NetDiskLog.d(TAG, "Copy button clicked no permission")
            return
        }
        doLimitCheck(FSConstantApiCompManager.getFileManagerHelper2OpCopy()) {
            mTempCopyPositions = it
            pickDir()
        }
    }

    fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (resultCode == Activity.RESULT_CANCELED) {
            return
        }
        when (requestCode) {
            CommonConstant.RTN_CODE_PICK_DIRECTORY -> {
                data?.getParcelableExtra<CloudFile>(
                        FSLocalFileApiCompManager.getSelectFolderActivity2SelectPath())?.apply {
                    copyFiles(filePath, directoryType, mTempCopyPositions)
                }
            }
        }
    }

    private fun pickDir() {
//        SelectFolderActivity
//                .startActivityForResult(netDiskFileView.activity, CloudFile(CloudFileConstants.PATH_BAIDUYUN_ROOT),
//                        SelectFolderActivity.COPY_BY_USER_STYLE, NetdiskFileFragment.RTN_CODE_PICK_DIRECTORY,
//                        false)
        FSLocalFileApiCompManager.startSelectFolderActivityForResult(netDiskFileView.activity, CloudFile(CloudFileConstants.PATH_BAIDUYUN_ROOT),
                FSLocalFileApiCompManager.getSelectFolderActivity2CopyByUserStyle(), CommonConstant.RTN_CODE_PICK_DIRECTORY,
                false)
    }

    private fun copyFiles(dest: String?, destProperty: Int, copyPos: List<Int>?) {
        NetDiskLog.d(TAG, "dest:$dest destProperty:$destProperty")
        netDiskFileView.cancelEditMode()
        dest?.let { destDir ->
            if (CloudFileContract.Files.isPropertyAlbum(destProperty)) {
                showOnlyShowImageDialog {
                    if (it) {
                        copyFiles(destDir, copyPos)
                    }
                }
            } else {
                copyFiles(destDir, copyPos)
            }
        } ?: return
    }

    private fun copyFiles(dest: String, copyPos: List<Int>?) {
        copyPos?.apply {
            var copyDirPath = dest
            if (CloudFileConstants.PATH_BAIDUYUN_ROOT != dest && dest.endsWith(FileUtils.PATH_CONNECTOR)) {
                // 如果是新建文件夹，返回时会带"/"
                copyDirPath = dest.substring(0, dest.length - 1)
            }
            map {
                netDiskFileView.getItem(it)
            }.filter {
                it.fileId != 0L && !it.isDir()
            }.map {
                MoveCopyFile(getCurrentPath(it), it.fileName, null, false)
            }.takeIf {
                it.isNotEmpty()
            }?.let {
                CloudFileServiceHelper.copy(netDiskFileView.activity.applicationContext, mCopyFileResultReceiver,
                        ArrayList(it), copyDirPath, null, CloudFileService.FILE_MANAGER_ASYNC)
                showLoadingView()
            }
        }
    }

    private fun hasRunningTask(): Boolean {
        val status = FileManagerConfig().fileManagerStatus
        if (CloudFileContract.FileManagerTasks.TASK_STATUS_RUNNING == status) {
            ToastHelper.showToast(R.string.filemanager_has_task_running)
            return true
        }
        return false
    }

    /**
     * 目标文件夹为属性相册提示框
     * @param callback
     */
    private fun showOnlyShowImageDialog(callback: (Boolean) -> Unit) {
        var contentId = R.string.copy_property_album_only_show_image
        if (PersonalConfig.getInstance().getBoolean(CloudFileConfigKey.ALBUM_VIDEO_DIFF_OPEN)) {
            contentId = R.string.copy_property_album_only_show_media
        }
        DialogBuilder().apply {
            val dialog = buildTipsDialog(netDiskFileView.activity,
                    R.string.dest_dir_is_property_album, contentId,
                    R.string.quick_action_copy, R.string.cancel)
            setOnDialogCtrListener(object : DialogCtrListener {
                override fun onOkBtnClick() {
                    dialog.dismiss()
                    callback.invoke(true)
                }

                override fun onCancelBtnClick() {
                    dialog.dismiss()
                    callback.invoke(false)
                }
            })
            dialog.show()
        }
    }

    private fun showLoadingView() {
        netDiskFileView.activity?.takeIf {
            !it.isFinishing
        }?.apply {
            mLoadingDialog = LoadingDialog.show(this, R.string.xpan_loading_text)
        }
    }

    private fun hideLoadingView() {
        mLoadingDialog?.takeIf {
            it.isShowing
        }?.dismiss()
    }

    fun onDestroy() {
        resultView = null
        hideLoadingView()
    }

    private inner class CustomizeResultReceiver(reference: SmartXCustomizeVideoPresenter, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<SmartXCustomizeVideoPresenter>(reference, handler, resultView) {
        override fun onSuccess(reference: SmartXCustomizeVideoPresenter, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            resultData?.getParcelable<SmartCustomizeVideoResponse>(BaseExtras.RESULT).takeIf {
                it != null
            }?.apply {
                reference.onLoadSuccess(smartCustomizeVideos, -1, hasMore != 0)
            } ?: reference.onLoadSuccess(null, -1, false)
        }

        override fun onFailed(reference: SmartXCustomizeVideoPresenter, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            reference.onLoadFail()
            return true
        }
    }

    private inner class DeleteFileResultView(activity: Activity) : ResultView(activity) {

        override fun initFailedManager(activity: Activity, failedViewManager: FailedViewManager) {
            failedViewManager.setDefaultErrorView(FailedViewManager.FailedViewBean())
            val builder = NewVersionDialog.Builder(activity)
            builder.setSingleConfirmText(R.string.filemanager_i_know)
            failedViewManager.put(PCSErrorCode.ERROR_FILE_MANAGER_OVERFLOW, FailedViewManager.FailedViewBean(builder))

        }

        override fun getFailedMessage(type: ErrorType, errno: Int, resultBundle: Bundle,
                                      activity: Activity): String? {
            if (type == ErrorType.NETWORK_ERROR) {
                return activity.getString(R.string.network_exception_message)
            }
            if (type == ErrorType.SERVER_ERROR) {
                return when (errno) {
                    NetdiskErrorCode.EXCEED_MAX_NUM -> {
                        val numLimit = resultBundle.getInt(BaseExtras.RESULT_FAILED, 2000)
                        String.format(activity.getString(
                                R.string.filemanager_delete_failed_exceed_maxnum), numLimit)
                    }
                    NetdiskErrorCode.FILEMANAGER_TASK_RUNNING -> return activity.getString(R.string.filemanager_has_task_running)
                    NetdiskErrorCode.NO_OPERATE_PERMISSION -> return activity.getString(R.string.filemanager_delete_no_operation_permission)
                    PCSErrorCode.ERROR_FILE_MANAGER_OVERFLOW -> return FSCommonApiCompManager.showFileManagerHelperFailedMsg(
                            activity, FSConstantApiCompManager.getFileManagerHelper2OpDelete())
                    else -> activity.getString(R.string.classification_customize_delete_error)
                }
            }
            return activity.getString(R.string.file_delete_failed)
        }

        override fun closeAllView() {
            hideLoadingView()
        }

        override fun showSuccessView(resultData: Bundle?) {
            super.showSuccessView(resultData)
            closeAllView()
            ToastHelper.showToast(R.string.file_delete_success)
        }

        override fun showOperatingView(resultData: Bundle?) {
            super.showOperatingView(resultData)
            closeAllView()
            // 获取正在处理的action，然后判断是正在处于何种状态，对于新建文件夹的操作来说，区分强刷的操作
            resultData?.let {
                val action = it.getString(BaseExtras.RESULT)
                if (CloudFileServiceHelper.isRefreshAction(action)) {
                    ToastHelper.showToast(R.string.is_refreshing_try_later)
                }
            }
        }
    }

    private inner class DeleteFileResultReceiver(reference: SmartXCustomizeVideoPresenter, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<SmartXCustomizeVideoPresenter>(reference, handler, resultView) {

        override fun onSuccess(reference: SmartXCustomizeVideoPresenter, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            NetDiskLog.d(TAG, "DeleteFileResultReceiver onSuccess")
            reference.netDiskFileView.activity?.takeIf {
                !it.isFinishing
            }?.apply {
                reference.netDiskFileView.onDeleteSuccess(BaseStatus.SUCCESS)
            }
        }

        override fun onFailed(reference: SmartXCustomizeVideoPresenter, errType: ErrorType,
                              errno: Int, resultData: Bundle): Boolean {
            NetDiskLog.d(TAG, "DeleteFileResultReceiver onFailed")
            reference.netDiskFileView.activity?.takeIf {
                !it.isFinishing
            }?.apply {
                reference.netDiskFileView.onDeleteFailed(BaseStatus.FAILED)
            }
            return AccountErrorHandler().commonDoubtHackingErrorHandling(reference.netDiskFileView.activity,
                    errno, CommonConstant.REQUEST_CODE_DOUBT_HACKING_APPEAL_DELETE_FILE) ||
                    FSSecondPwdApiCompManager.handleSecondPwdErrorHandlerError(
                            reference.netDiskFileView.activity, errno)
        }

        override fun onOperating(reference: SmartXCustomizeVideoPresenter, resultData: Bundle?) {
            super.onOperating(reference, resultData)
            NetDiskLog.d(TAG, "DeleteFileResultReceiver onOperating")
            if (resultData == null || resultData.isEmpty) { // 开始异步任务的轮寻操作
                reference.netDiskFileView.activity?.takeIf {
                    !it.isFinishing
                }?.apply {
                    reference.netDiskFileView.onDeleteSuccess(BaseStatus.OPERATING)
                    val intent = Intent(this, FSConstantApiCompManager.getFileManagerProgressActivity())
                    intent.putExtra(FileManagerExtras.EXTRA_FILE_MANAGER_TASK_TYPE,
                            CloudFileContract.FileManagerTasks.DELETE_TASK)
                    startActivity(intent)
                }
            }
        }
    }

    private inner class CopyResultView(activity: Activity) : ResultView(activity) {

        override fun initFailedManager(activity: Activity, failedViewManager: FailedViewManager) {
            failedViewManager.setDefaultErrorView(FailedViewManager.FailedViewBean())
            val builder = NewVersionDialog.Builder(activity)
            builder.setSingleConfirmText(R.string.filemanager_i_know)
            failedViewManager.put(PCSErrorCode.ERROR_FILE_MANAGER_OVERFLOW, FailedViewManager.FailedViewBean(builder))

            val builderCountOver = NewVersionDialog.Builder(activity)
            builderCountOver.setTitle(R.string.copy_link_failed)
                    .setConfirmText(R.string.know_it)
            failedViewManager.put(NetdiskErrorCode.PROPERTY_ALBUM_COUNT_OVER_LIMIT,
                    FailedViewManager.FailedViewBean(builderCountOver))
        }

        override fun closeAllView() {
            hideLoadingView()
        }

        override fun getFailedMessage(type: ErrorType, errno: Int, resultData: Bundle,
                                      activity: Activity): String? {
            if (type == ErrorType.NETWORK_ERROR) {
                return activity.getString(R.string.network_exception_message)
            }
            if (errno != 0) {
                return when (errno) {
                    NetdiskErrorCode.EXCEED_MAX_NUM -> {
                        val numLimit = resultData.getInt(BaseExtras.RESULT_FAILED, 2000)
                        return activity.getString(R.string.filemanager_copy_failed_exceed_maxnum, numLimit)
                    }
                    PCSErrorCode.ERROR_FILE_MANAGER_OVERFLOW -> FSCommonApiCompManager.showFileManagerHelperFailedMsg(
                            activity, FSConstantApiCompManager.getFileManagerHelper2OpCopy())
                    NetdiskErrorCode.FILEMANAGER_TASK_RUNNING -> activity.getString(R.string.filemanager_has_task_running)
                    NetdiskErrorCode.PROPERTY_ALBUM_COUNT_OVER_LIMIT -> {
                        val limit = GlobalConfig.getInstance().getInt(CloudFileConfigKey.ALBUM_CONFIG_MAX_DIR_NUMBER, 500)
                        return activity.getString(R.string.album_number_is_over_flow, limit)
                    }
                    else -> activity.getString(R.string.copy_failed)
                }
            }
            return activity.getString(R.string.copy_failed)
        }

        override fun showSuccessView(resultData: Bundle?) {
            super.showSuccessView(resultData)
            closeAllView()
            ToastHelper.showToast(R.string.copy_success)
        }

        override fun showOperatingView(resultData: Bundle?) {
            super.showOperatingView(resultData)
            closeAllView()
            if (resultData != null && !resultData.isEmpty) {
                // 获取正在处理的action，然后判断是正在处于何种状态，对于新建文件夹的操作来说，区分强刷的操作
                val action = resultData.getString(BaseExtras.RESULT)
                if (CloudFileServiceHelper.isRefreshAction(action)) {
                    ToastHelper.showToast(R.string.is_refreshing_try_later)
                }
            }

        }
    }

    private inner class CopyResultReceiver(reference: SmartXCustomizeVideoPresenter, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<SmartXCustomizeVideoPresenter>(reference, handler, resultView) {

        override fun onFailed(reference: SmartXCustomizeVideoPresenter, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            reference.netDiskFileView.activity.takeIf {
                it != null && !it.isFinishing
            }?.apply {
                reference.netDiskFileView.onMoveFinished(BaseStatus.FAILED)
            }
            return super.onFailed(reference, errType, errno, resultData)
        }

        override fun onSuccess(reference: SmartXCustomizeVideoPresenter, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            reference.netDiskFileView.activity.takeIf {
                it != null && !it.isFinishing
            }?.apply {
                reference.netDiskFileView.onMoveFinished(BaseStatus.SUCCESS)
            }
        }

        override fun onOperating(reference: SmartXCustomizeVideoPresenter, resultData: Bundle?) {
            super.onOperating(reference, resultData)
            if (resultData == null || resultData.isEmpty) { // 开始异步任务的轮寻操作
                reference.netDiskFileView.activity.takeIf {
                    it != null && !it.isFinishing
                }?.apply {
                    reference.netDiskFileView.onMoveFinished(BaseStatus.OPERATING)
                    val intent = Intent(this, FSConstantApiCompManager.getFileManagerProgressActivity())
                    intent.putExtra(FileManagerExtras.EXTRA_FILE_MANAGER_TASK_TYPE,
                            CloudFileContract.FileManagerTasks.COPY_TASK)
                    startActivity(intent)
                }
            }
        }
    }
}