package com.baidu.netdisk.ui.xpan.classification.adapter

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageButton
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.kernel.util.FormatUtils
import com.baidu.netdisk.ui.widget.CheckableItemLayout
import com.baidu.netdisk.ui.xpan.extension.getSecondLeftTime
import com.baidu.netdisk.xpan.io.parser.model.SmartCustomizeVideo
import java.util.concurrent.TimeUnit

class SmartXCustomizeVideoAdapter(context: Context) : BaseCheckableAdapter<SmartXCustomizeVideoAdapter.ViewHolder, Int>() {

    private val mInflater = LayoutInflater.from(context)

    var mCustomizeVideos = mutableListOf<SmartCustomizeVideo>()

    var mOnChooseModeListener: ((Boolean) -> Unit)? = null
    var mOnItemClickListener: ((Int) -> Unit)? = null
    var mOnItemLongClickListener: ((Int) -> Boolean)? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        return ViewHolder(mInflater.inflate(R.layout.item_customize_video, parent, false))
    }

    override fun getItemCount(): Int {
        return mCustomizeVideos.size
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        mCustomizeVideos[position].apply {
            holder.tvName.text = name
            holder.tvTime.text = DateUtil.formatTimeEndWithMin(TimeUnit.SECONDS.toMillis(ctime))
            holder.tvSize.text = FormatUtils.formatFileSize(size)
            holder.tvPeriod.text = period.getSecondLeftTime()
            thumbs?.icon?.takeIf {
                it.isNotEmpty()
            }?.let {
                GlideHelper.getInstance().displayImage(it, holder.ivCover)
            } ?: holder.ivCover.setImageResource(R.drawable.icon_list_large_image_no_shadow)
            if (isChooseMode) {
                holder.checkableLayout.choiceMode = CheckableItemLayout.CHOICE_MODE_MULTIPLE
                holder.checkableLayout.isChecked = isItemChecked(position)
            } else {
                holder.checkableLayout.choiceMode = CheckableItemLayout.CHOICE_MODE_NONE
            }
            holder.checkableLayout.setOnClickListener {
                mOnItemClickListener?.invoke(position)
            }
            holder.checkableLayout.setOnLongClickListener {
                mOnItemLongClickListener?.invoke(position) ?: false
            }
            holder.tvCheck.setOnClickListener {
                isChooseMode = true
                toggleItemCheck(position)
                mOnChooseModeListener?.invoke(isChooseMode)
            }
        }
    }

    override fun onChooseModeChange(chooseMode: Boolean) {
        if (!chooseMode) {
            resetSelectItem()
        }
        mOnChooseModeListener?.invoke(chooseMode)
        notifyDataSetChanged()
    }

    override fun isSelectedAll(): Boolean {
        return chooseItems.size == itemCount && itemCount > 0
    }

    override fun toggleSelectAll() {
        if (isSelectedAll) {
            resetSelectItem()
        } else {
            repeat(itemCount) {
                insertSelectItem(it)
            }
        }
        notifyDataSetChanged()
    }

    override fun toggleItemCheck(position: Int) {
        if (!isChooseMode || position !in 0 until itemCount) {
            return
        }
        if (isItemChecked(position)) {
            chooseItems.remove(position)
        } else {
            insertSelectItem(position)
        }
        notifyItemChanged(position)
    }

    fun setCustomizeVideos(customizeVideos: MutableList<SmartCustomizeVideo>?) {
        mCustomizeVideos.clear()
        customizeVideos?.let {
            mCustomizeVideos.addAll(it)
        }
        notifyDataSetChanged()
    }

    fun appendCustomizeVideos(customizeVideos: MutableList<SmartCustomizeVideo>?) {
        customizeVideos?.let {
            val first = if (mCustomizeVideos.size > 0) mCustomizeVideos.size - 1 else 0
            mCustomizeVideos.addAll(customizeVideos)
            val end = it.size
            notifyItemRangeChanged(first, end)
        }
    }

    inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val ivCover: ImageView = itemView.findViewById(R.id.iv_cover)
        val tvName: TextView = itemView.findViewById(R.id.tv_name)
        val tvTime: TextView = itemView.findViewById(R.id.tv_time)
        val tvSize: TextView = itemView.findViewById(R.id.tv_size)
        val tvPeriod: TextView = itemView.findViewById(R.id.tv_period)
        val checkableLayout: CheckableItemLayout = itemView.findViewById(R.id.cb_check)
        val tvCheck: ImageButton = itemView.findViewById(android.R.id.button1)
    }
}