package com.baidu.netdisk.ui.xpan.device

import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.view.View
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.ui.xpan.classification.IClassificationOperatorView
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.xpan.io.parser.model.SmartDevice
import com.baidu.netdisk.xpan.io.parser.model.SmartDeviceSceneAbility
import com.baidu.netdisk.xpan.service.Extras
import com.baidu.netdisk.xpan.statistics.XPanStatisticsKeys
import kotlinx.android.synthetic.main.activity_device_setting.*

/**
 * 智能设备设置页面
 *
 * @author rendawei
 * @since 2019/09/10
 */
class DeviceSettingActivity : BaseActivity(), ICommonTitleBarClickListener, IClassificationOperatorView {

    private var mSmartDevice: SmartDevice? = null
    private lateinit var mDeviceSettingPresenter: DeviceSettingPresenter

    companion object {
        fun startSettingForResult(activity: Activity, smartDevice: SmartDevice?, requestCode: Int) {
            smartDevice?.let {
                Intent(activity, DeviceSettingActivity::class.java).apply {
                    putExtra(Extras.DEVICE, it)
                    activity.startActivityForResult(this, requestCode)
                }
            }
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        mDeviceSettingPresenter = DeviceSettingPresenter(this)
    }

    override fun onDestroy() {
        super.onDestroy()
        mDeviceSettingPresenter.onDestroy()
    }

    override fun getLayoutId(): Int = R.layout.activity_device_setting

    override fun initParams() {
        mSmartDevice = intent?.getParcelableExtra(Extras.DEVICE)
        if (mSmartDevice == null) {
            finish()
        }
    }

    override fun initView() {
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setMiddleTitle(R.string.settings)
        mTitleBar.setRightMenuButtonVisible(false)
        mTitleBar.setTopTitleBarClickListener(this)
        mSmartDevice?.apply {
            val settingAdapter = DeviceSettingAdapter(this@DeviceSettingActivity, this.sceneAbility.first().settingOption())
            settingAdapter.itemClickListener = { _, setting ->
                when (setting) {
                    SmartDeviceSceneAbility.SETTING_OPTION_UNBIND -> {
                        mDeviceSettingPresenter.operatorUnBindDevice(this, this@DeviceSettingActivity)
                    }
                    SmartDeviceSceneAbility.SETTING_OPTION_RENAME -> {
                        mDeviceSettingPresenter.operatorRenameDevice(this, this@DeviceSettingActivity)
                    }
                    SmartDeviceSceneAbility.SETTING_OPTION_CLOUD_DESC -> {
                        mDeviceSettingPresenter.operatorCloudDesc(this@DeviceSettingActivity)
                    }
                }
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(XPanStatisticsKeys.XPAN_DEVICE_SETTING_CLICK,
                        setting)
            }
            recycler_view.adapter = settingAdapter
            recycler_view.layoutManager = LinearLayoutManager(this@DeviceSettingActivity, LinearLayoutManager.VERTICAL, false)
        }
    }

    override fun onRightButtonClicked(view: View?) {

    }

    override fun onBackButtonClicked() {
        finish()
    }

    override fun onOperateSuccess() {
        setResult(Activity.RESULT_OK)
        finish()
    }

    override fun onOperateFail(errType: ErrorType, errno: Int) {

    }

    override fun isActivityDark() = true
}