package com.baidu.netdisk.ui.xpan.device

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RelativeLayout
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.xpan.io.parser.model.SmartDeviceSceneAbility
import com.baidu.netdisk.xpan.statistics.XPanStatisticsKeys

class DeviceSettingAdapter(context: Context, val settings: List<String>?) :
        RecyclerView.Adapter<DeviceSettingAdapter.ViewHolder>() {

    private val layoutInflater = LayoutInflater.from(context)
    var itemClickListener: ((pos: Int, setting: String) -> Unit)? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = layoutInflater.inflate(R.layout.item_device_setting, parent, false)
        return ViewHolder(view)
    }

    override fun getItemCount(): Int {
        return settings?.size ?: 0
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        settings?.get(position)?.apply {
            holder.settingName.setText(getSettingName(this))
            holder.settingItem.setOnClickListener {
                itemClickListener?.invoke(position, this)
            }
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(XPanStatisticsKeys.XPAN_DEVICE_SETTING_SHOW,
                    this)
        }
    }

    private fun getSettingName(setting: String): Int {
        return when (setting) {
            SmartDeviceSceneAbility.SETTING_OPTION_UNBIND -> R.string.classification_device_unbind
            SmartDeviceSceneAbility.SETTING_OPTION_RENAME -> R.string.classification_device_rename
            SmartDeviceSceneAbility.SETTING_OPTION_CLOUD_DESC -> R.string.classification_device_cloud_desc
            else -> -1
        }
    }

    class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val settingItem: RelativeLayout = itemView.findViewById(R.id.setting_item)
        val settingName: TextView = itemView.findViewById(R.id.setting_name)
    }
}