package com.baidu.netdisk.ui.xpan.device

import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.support.v4.content.LocalBroadcastManager
import android.text.Editable
import android.text.TextUtils
import android.text.TextWatcher
import android.view.Gravity
import android.widget.RelativeLayout
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.kernel.android.util.TextTools
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.webview.CommonWebViewActivity
import com.baidu.netdisk.ui.widget.EditLoadingDialog
import com.baidu.netdisk.ui.xpan.SmartDeviceManager
import com.baidu.netdisk.ui.xpan.classification.IClassificationOperatorView
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.xpan.ISmartDevice
import com.baidu.netdisk.xpan.constants.SmartDeviceScene
import com.baidu.netdisk.xpan.io.ErrorCode
import com.baidu.netdisk.xpan.io.parser.model.SmartDevice
import java.util.regex.Pattern

class DeviceSettingPresenter constructor(private var mOperatorView: IClassificationOperatorView?) : IClassificationOperatorView {

    companion object {
        private const val NICKNAME_MAX_LENGTH = 24
        private const val NICKNAME_MIN_LENGTH = 1
        private const val NICKNAME_REGEX = "^[A-Za-z0-9_\\-\\u4e00-\\u9fa5]+$"

        const val SMART_DEVICE_SETTING_ACTION = "com.baidu.netdisk.xpan.SMART_DEVICE_SETTING"
        const val SMART_DEVICE_SETTING_EXTRA_NAME = "SMART_DEVICE_SETTING_EXTRA_NAME"

        const val DEVICE_DOC_URL = "http://www.baidu.com"
    }

    private val mSmartManager: ISmartDevice
    private val mUnBindReceiver: UnBindReceiver
    private val mRenameReceiver: RenameReceiver

    private var mLoadingDialog: Dialog? = null
    private var mChangeNameDialog: EditLoadingDialog? = null


    private var mNickName: String? = null

    init {
        mSmartManager = SmartDeviceManager()
        mUnBindReceiver = UnBindReceiver(this, Handler(), null)
        mRenameReceiver = RenameReceiver(this, Handler(), null)
    }

    fun operatorUnBindDevice(smartDevice: SmartDevice, activity: Activity) {
        val scene = SmartDeviceScene.getScene(smartDevice.scene)
        val contentId: Int
        contentId = if (scene == SmartDeviceScene.SCENE_NAS) {
            R.string.classification_device_unbind_nas
        } else {
            R.string.classification_device_unbind_consumer
        }
        val dialogBuilder = DialogBuilder()
        dialogBuilder.setOnDialogCtrListener(object : DialogCtrListener {
            override fun onOkBtnClick() {
                mLoadingDialog = DialogBuilder().buildLoadingDialogWithoutText(activity)
                mLoadingDialog?.show()
                unBindDevice(activity.applicationContext, smartDevice)
            }

            override fun onCancelBtnClick() {

            }
        })
        dialogBuilder.buildTipsDialog(activity, R.string.classification_device_unbind_title, contentId,
                R.string.classification_device_unbind, R.string.cancel)
    }

    private fun unBindDevice(context: Context, smartDevice: SmartDevice) {
        mSmartManager.unBindDevice(context, mUnBindReceiver, AccountUtils.getInstance().bduss,
                AccountUtils.getInstance().uid, smartDevice.id)
    }

    fun operatorCloudDesc(activity: Activity) {
        CommonWebViewActivity.startActivity(activity, DEVICE_DOC_URL)
    }

    fun operatorRenameDevice(smartDevice: SmartDevice, activity: Activity) {
        mChangeNameDialog = EditLoadingDialog.build(activity).apply {
            setTitle(R.string.classification_device_rename_title)
            val editName = editText
            editName.setHint(R.string.classification_change_name_hint)
            val hPadding = activity.resources.getDimension(
                    R.dimen.p2pshare_nickname_dialog_text_padding_horizontal).toInt()
            val vPadding = activity.resources.getDimension(
                    R.dimen.p2pshare_nickname_dialog_text_padding_vertical).toInt()
            editName.setPadding(hPadding, vPadding, hPadding, vPadding)
            editName.gravity = Gravity.CENTER_VERTICAL
            editName.textSize = 14f
            editName.addTextChangedListener(object : TextWatcher {
                override fun beforeTextChanged(s: CharSequence, start: Int, count: Int, after: Int) {
                }

                override fun onTextChanged(s: CharSequence, start: Int, before: Int, count: Int) {
                }

                override fun afterTextChanged(s: Editable) {
                    val content = editName.text.toString()
                    if (content.isNotEmpty() && !Pattern.matches(NICKNAME_REGEX, content)) {
                        setDialogError(R.string.classification_change_name_error_format)
                        rightBtn.isEnabled = false
                    } else {
                        setDialogError("")
                    }
                }
            })

            setMaxLength(NICKNAME_MAX_LENGTH)
            setRightBtnText(R.string.ok)

            setDialogErrorEnable(true)
            val inputNumText = inputNumText
            val params = inputNumText.layoutParams
            if (params is RelativeLayout.LayoutParams) {
                params.setMargins(0, 0, hPadding, 0)
                params.addRule(RelativeLayout.CENTER_VERTICAL)
            }

            if (!TextUtils.isEmpty(smartDevice.name)) {
                setEditContent(smartDevice.name)
            }
            setRightBtnOnClickListener {
                renameDevice(editName.text.toString().trim(), smartDevice, activity)
            }
            setLeftBtnOnClickListener {
                editName.setText("")
                dismiss()
            }
            setCanceledOnTouchOutside(true)
            show()
        }
    }

    private fun renameDevice(nickname: String, smartDevice: SmartDevice, activity: Activity) {
        mChangeNameDialog?.apply {
            switch2LoadingMode()
            setDialogError("")
            val length = TextTools.fetchCharNumber(nickname)
            if (length < NICKNAME_MIN_LENGTH || length > NICKNAME_MAX_LENGTH) {
                switch2NormalMode()
                setDialogError(R.string.classification_change_name_error_length)
                rightBtn.isEnabled = false
                return
            }
            if (Pattern.matches(NICKNAME_REGEX, nickname)) {
                mNickName = nickname
                setCanceledOnTouchOutside(false)
                mSmartManager.renameDevice(activity.applicationContext, mRenameReceiver, AccountUtils.getInstance().bduss,
                        AccountUtils.getInstance().uid, smartDevice.id, nickname)
            } else {
                switch2NormalMode()
                setDialogError(R.string.classification_change_name_error_format)
                rightBtn.isEnabled = false
            }
        }
    }

    private fun renameDeviceSuccess() {
        mChangeNameDialog?.dismiss()
        mSmartManager.diffDevices(BaseApplication.getInstance(), null, AccountUtils.getInstance().bduss,
                AccountUtils.getInstance().uid, false)
        Intent().apply {
            action = SMART_DEVICE_SETTING_ACTION
            putExtra(SMART_DEVICE_SETTING_EXTRA_NAME, mNickName)
            LocalBroadcastManager.getInstance(BaseApplication.getInstance()).sendBroadcast(this)
        }
    }

    private fun renameDeviceFail() {
        mChangeNameDialog?.setCanceledOnTouchOutside(true)
        mChangeNameDialog?.switch2NormalMode()
    }

    override fun onOperateSuccess() {
        dismissDialog()
        mOperatorView?.onOperateSuccess()
    }

    override fun onOperateFail(errType: ErrorType, errno: Int) {
        dismissDialog()
        mOperatorView?.onOperateFail(errType, errno)
    }

    private fun dismissDialog() {
        mLoadingDialog?.takeIf {
            mLoadingDialog?.isShowing == true
        }?.dismiss()

        mChangeNameDialog?.takeIf {
            mChangeNameDialog?.isShowing == true
        }?.dismiss()
    }

    fun onDestroy() {
        dismissDialog()
        mOperatorView = null
    }

    private class UnBindReceiver constructor(reference: DeviceSettingPresenter, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<DeviceSettingPresenter>(reference, handler, resultView) {

        override fun onSuccess(reference: DeviceSettingPresenter, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            reference.onOperateSuccess()
            ToastHelper.showToast(R.string.classification_device_unbind_success)
        }

        override fun onFailed(reference: DeviceSettingPresenter, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            reference.onOperateFail(errType, errno)
            if (errType == ErrorType.NETWORK_ERROR) {
                ToastHelper.showToast(R.string.xpan_net_error)
                return true
            }
            if (errno == ErrorCode.DEVICE_UNREGISTER) {
                ToastHelper.showToast(R.string.classification_device_unbind_error)
                return true
            }
            ToastHelper.showToast(R.string.classification_device_unbind_fail)
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    private class RenameReceiver constructor(reference: DeviceSettingPresenter, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<DeviceSettingPresenter>(reference, handler, resultView) {

        override fun onSuccess(reference: DeviceSettingPresenter, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            reference.renameDeviceSuccess()
            ToastHelper.showToast(R.string.classification_device_rename_success)
        }

        override fun onFailed(reference: DeviceSettingPresenter, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            reference.renameDeviceFail()
            if (errType == ErrorType.NETWORK_ERROR) {
                ToastHelper.showToast(R.string.xpan_net_error)
                return true
            }
            if (errno == ErrorCode.DEVICE_UNREGISTER) {
                ToastHelper.showToast(R.string.classification_device_unbind_error)
                return true
            }
            ToastHelper.showToast(R.string.classification_device_rename_fail)
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}
