package com.baidu.netdisk.ui.xpan.transmission

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.util.ToastHelper
import kotlinx.android.synthetic.main.fragment_trans_list_record.*
import kotlinx.android.synthetic.main.layout_xpan_top_notice.view.*

abstract class BaseTransListFragment : BaseLoadingFragment() {

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_trans_list_record, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initView(view)
        initData()
    }

    open fun initView(view: View) {}

    open fun initData() {}

    abstract fun isDataEmpty(): Boolean
    abstract fun fetchData(refresh: Boolean, loadMore: Boolean)

    fun showResultView(error: Boolean, emptyRes: Int = R.string.trans_file_empty,
                       emptyBtnRes: Int = R.string.trans_go_collect_file, emptyCall: ((View) -> Unit)? = null) {
        if (isDataEmpty()) {
            empty_view.visibility = View.VISIBLE
            ll_content.visibility = View.GONE
        } else {
            empty_view.visibility = View.GONE
            ll_content.visibility = View.VISIBLE
        }
        if (error) {
            if (!isDataEmpty()) {
                ToastHelper.showToast(R.string.classification_customize_load_error)
                return
            }
            empty_view.setLoadError(R.string.backup_server_error, R.drawable.xpan_empty_error)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_999))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.VISIBLE)
            empty_view.setRefreshButtonTextColor(resources.getColor(R.color.empty_refresh_text_color))
            empty_view.setRefreshText(R.string.refresh_verify_code_text)
            empty_view.setRefreshListener {
                fetchData(refresh = true, loadMore = false)
            }
            return
        }
        if (isDataEmpty()) {
            empty_view.setLoadNoData(resources.getString(emptyRes), R.drawable.empty_folder)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_666))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.GONE)
            if (emptyCall != null) {
                empty_view.setRefreshVisibility(View.VISIBLE)
                empty_view.setRefreshText(emptyBtnRes)
                empty_view.setRefreshListener {
                    emptyCall.invoke(it)
                }
            }
        }
    }

    fun showEmptyLoading() {
        empty_view.setLoading(R.string.loading, resources.getColor(R.color.common_color_666),
                R.drawable.classification_loading_anim)
        ll_content.visibility = View.GONE
    }

    fun showNoticeView(key: String, strId: Int) {
        val showed = PersonalConfig.getInstance().getBoolean(key, false)
        if (showed) return
        in_notice.visibility = View.VISIBLE
        in_notice.tv_top_title.setText(strId)
        in_notice.btn_know.setOnClickListener {
            hideNoticeView(key)
        }
    }

    fun hideNoticeView(key: String) {
        in_notice.visibility = View.GONE
        PersonalConfig.getInstance().putBoolean(key, true)
    }
}