package com.baidu.netdisk.ui.xpan.transmission

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.support.v4.app.FragmentTransaction
import android.view.Gravity
import android.view.View
import android.widget.RelativeLayout
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.android.util.deviceinfo.DeviceDisplayUtils
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.sns.util.Utility
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.webview.CommonWebViewActivity
import com.baidu.netdisk.ui.widget.PopupMenuItem
import com.baidu.netdisk.ui.widget.TextGuidePopMenu
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.ui.widget.titlebar.TitleBarPopupMenu
import com.baidu.netdisk.ui.widget.titlebar.TitleBarWithPopupMenu
import com.baidu.netdisk.ui.widget.titlebar.TransAssistantTitleBar
import com.baidu.netdisk.ui.xpan.transmission.share.TransShareViewModel
import com.baidu.netdisk.ui.xpan.transmission.viewmodel.TransAssistantViewModel
import com.baidu.netdisk.xpan.config.TransAssistantConfigKey
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys

/**
 * 传输助手(传输助手首页/收文件页面)
 *
 * @author rendawei
 * @since 2019/11/12
 */
class TransAssistantActivity : BaseTransActivity(), ICommonTitleBarClickListener {

    companion object {
        const val TAG = "TransAssistantActivity"

        private const val EXTRA_PAGE_TYPE = "extra_page_type"
        private const val EXTRA_COME_FROM = "extra_come_from"

        const val PAGE_TYPE_HOME = 0
        const val PAGE_TYPE_COLLECT = 1

        private const val ID_DOC = 1

        fun getIntent(activity: Activity, comeFrom: Int, pageType: Int = PAGE_TYPE_HOME) =
                Intent(activity, TransAssistantActivity::class.java).apply {
                    putExtra(EXTRA_PAGE_TYPE, pageType)
                    putExtra(EXTRA_COME_FROM, comeFrom)
                }
    }

    private var mPageType: Int = PAGE_TYPE_HOME
    private var mComeFrom: Int = -1
    private var mTitle: Int = -1

    /** 引导弹窗  */
    private var mTextGuidePopMenu: TextGuidePopMenu? = null
    private var mPopupMenu: TitleBarPopupMenu? = null

    private var mViewModel: TransAssistantViewModel? = null

    override fun getLayoutId(): Int = R.layout.activity_trans_assistant

    override fun onWindowFocusChanged(hasFocus: Boolean) {
        super.onWindowFocusChanged(hasFocus)
        initTips((mTitleBar as TransAssistantTitleBar).fileListTitleRightTwo)
    }

    override fun onResume() {
        super.onResume()
        mViewModel?.getMsgTotalCount()
        displayRedIcon()
    }

    override fun initParams() {
        mPageType = intent.getIntExtra(EXTRA_PAGE_TYPE, PAGE_TYPE_HOME)
        mComeFrom = intent.getIntExtra(EXTRA_COME_FROM, PAGE_TYPE_HOME)
        mTitle = if (mPageType == PAGE_TYPE_COLLECT) R.string.trans_assistant_collect else R.string.trans_assistant
        ViewModelProviders.of(this).get(TransAssistantViewModel::class.java).apply {
            mTransAssManager = getService(TRANS_ASSISTANT_SERVICE)
            mMsgLiveData.observe(this@TransAssistantActivity, Observer {
                if (it?.isSuccess == true) {
                    displayRedIcon()
                }
            })
        }.also {
            mViewModel = it
            it.getMsgTotalCount()
        }
    }

    override fun initView() {
        initTitle()
        initFragment()
    }

    private fun initTitle() {
        TransAssistantTitleBar(this).apply {
            setTopTitleBarClickListener(this@TransAssistantActivity)
            setMiddleTitle(mTitle)
            if (mPageType == PAGE_TYPE_COLLECT) {
                setRightTwoOnClickListener(R.drawable.ic_trans_titlebar_assistant) {
                    goToTransRecordList(this)
                }
            } else {
                setRightOneOnClickListener(R.drawable.common_titlebar_icon_more) {
                    showMoreMenu(it)
                }
                setRightTwoOnClickListener(R.drawable.ic_trans_titlebar_assistant) {
                    goToTransRecordList(this)
                }
            }
            displayRedIcon()
        }.also {
            mTitleBar = it
        }
    }

    private fun displayRedIcon() {
        (titleBar as? TransAssistantTitleBar)?.let {
            val cnt = PersonalConfig.getInstance().getInt(TransAssistantConfigKey.TRANS_RECORDS_UNREAD_CNT, 0)
            it.showFileListTwoLabel(cnt)
        }
    }

    private fun goToTransRecordList(titleBar: TransAssistantTitleBar) {
        titleBar.fileListTitleRightTwo?.hideIndicator()
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_GO_LIST_CLICK,
                mPageType.toString())
        TransListRecordActivity.getIntent(this).apply {
            startActivity(this)
        }
    }

    private fun initTips(view: View?) {
        val showed = PersonalConfig.getInstance().getBoolean(TransAssistantConfigKey.TRANS_ASSISTANT_HOME_TIPS, false)
        if (showed || mPageType != PAGE_TYPE_HOME) return
        mTextGuidePopMenu ?: TextGuidePopMenu(this, true, null).apply {
            rootView?.let {
                val rootPointView = it.findViewById<View>(R.id.point_root_layout)
                val ls = rootPointView.layoutParams as RelativeLayout.LayoutParams
                ls.rightMargin = Utility.UIUtility.dip2px(this@TransAssistantActivity, 48f)
                rootPointView.layoutParams = ls
                it.findViewById<TextView>(R.id.point_text_1)?.setText(R.string.trans_assistant_tips)
            }
            setPopupWindowDismissListener {
                PersonalConfig.getInstance().putBoolean(TransAssistantConfigKey.TRANS_ASSISTANT_HOME_TIPS, true)
            }
        }.also {
            mTextGuidePopMenu = it
            view ?: return
            it.showAsDropDown(view, 0, 0)
        }
    }

    private fun showMoreMenu(view: View) {
        mPopupMenu ?: TitleBarPopupMenu(this).apply {
            setItemLayoutGravity(Gravity.CENTER_VERTICAL or Gravity.START)
            setItemWidth(DeviceDisplayUtils.getPx(TitleBarWithPopupMenu.POPUP_MENU_ITEM_WIDTH_DEFAULT_DP))
            setOffset(DeviceDisplayUtils.getPx(TitleBarWithPopupMenu.POPUP_MENU_OFFSET_X_DP), 30)
            addItem(PopupMenuItem(ID_DOC, getString(R.string.trans_assistant_pop_doc)))
            setOnMenuItemClickListener {
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_DOC_CLICK)
                CommonWebViewActivity.startActivity(this@TransAssistantActivity, TransShareViewModel.MINI_URl)
            }
        }.also {
            mPopupMenu = it
        }
        mPopupMenu?.show(view)
    }

    private fun initFragment() {
        var transaction: FragmentTransaction? = null
        try {
            transaction = supportFragmentManager.beginTransaction()
            val fragment = when (mPageType) {
                PAGE_TYPE_HOME -> TransAssistantFragment()
                PAGE_TYPE_COLLECT -> TransAssistantCollectFragment()
                else -> null
            }
            fragment ?: return
            transaction.replace(R.id.fl_content, fragment)
        } catch (e: Exception) {
            NetDiskLog.d(TAG, e.message)
        } finally {
            transaction?.commitAllowingStateLoss()
        }
    }

    override fun onRightButtonClicked(view: View?) {

    }

    override fun onBackButtonClicked() {
        finish()
    }
}