package com.baidu.netdisk.ui.xpan.transmission

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.graphics.drawable.Drawable
import android.net.Uri
import android.os.Bundle
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.xpan.transmission.share.TransShareBottomSheet
import com.baidu.netdisk.ui.xpan.transmission.share.TransShareViewModel
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.config.TransAssistantConfigKey
import com.baidu.netdisk.xpan.io.parser.transmission.model.CreateTransTaskBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransTaskResponse
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import com.netdisk.glide.Glide
import com.netdisk.glide.load.DataSource
import com.netdisk.glide.load.engine.GlideException
import com.netdisk.glide.request.RequestListener
import com.netdisk.glide.request.target.Target
import kotlinx.android.synthetic.main.fragment_trans_assistant_collect.*
import java.io.File

/**
 * 传输助手-收文件
 *
 * @author rendawei
 * @since 2019/11/12
 */
class TransAssistantCollectFragment : BaseTransAssistantFragment() {

    private var mViewModel: TransShareViewModel? = null

    /**
     * 当前是否有可用显示的二维码
     */
    private var mShowedQRCode = false

    private var mCreateParam: CreateTransTaskBean? = null
    private var mTransTaskResponse: TransTaskResponse? = null
    private var mEditEnable: Boolean = true

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_trans_assistant_collect, container, false)
    }

    override fun initView(view: View) {
        // 调起分享面板
        rl_share.setOnClickListener {
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_COLLECT_SHARE_CLICK)
            TransShareBottomSheet.getTransShareDialog(mCreateParam, mEditEnable, mTransTaskResponse).apply {
                mShareListener = { p0, _, task ->
                    mCreateParam = p0
                    mTransTaskResponse = task
                    mEditEnable = false
                }
            }.show(childFragmentManager, "ShareDialog")
        }
        // 重新刷新默认二维码
        tv_reload.setOnClickListener {
            refreshQRCode()
        }
        // 保存默认的二维码
        tv_save.setOnClickListener {
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SAVE_DEFAULT_CLICK)
            activity?.let {
                mViewModel?.saveDefaultQRCode(it)
            }
        }

        showDefaultQRCode()
    }

    override fun initData() {
        ViewModelProviders.of(this).get(TransShareViewModel::class.java).apply {
            mTransAssManager = getService(BaseActivity.TRANS_ASSISTANT_SERVICE)
            // 处理请求默认二维码的回调
            mCreateDefaultTaskLiveData.observe(this@TransAssistantCollectFragment, Observer {
                dismissLoading()
                it?.takeIf {
                    it.isSuccess && !TextUtils.isEmpty(it.getOrNull()?.qrcodeUrl)
                }?.apply {
                    // 处理默认不带背景的二维码带上type
                    val url = it.getOrNull()!!.qrcodeUrl
                    val picUrl: String
                    val result = String.runCatching {
                        var uri = Uri.parse(url)
                        if (!uri.queryParameterNames.contains("type")) {
                            uri = uri.buildUpon().appendQueryParameter("type", "1").build()
                        }
                        uri.toString()
                    }
                    picUrl = result.getOrDefault(url)
                    showQRCodeSuccess(picUrl)
                } ?: showDefaultQRCode()
            })
            // 保存默认二维码
            mSaveQRCodeLiveData.observe(this@TransAssistantCollectFragment, Observer {
                runCatching {
                    val uri = Uri.fromFile(File(it?.getOrNull()))
                    NetDiskLog.i("TransAssistantCollectFragment", "file uri ：$uri")
                    // 发送广播通知更新图库，这样系统图库可以找到这张图片
                    activity?.applicationContext?.sendBroadcast(Intent(Intent.ACTION_MEDIA_SCANNER_SCAN_FILE, uri))
                }
                ToastHelper.showToast(R.string.trans_assistant_save_code_success)
            })
            // 统一的错误处理
            mShareErrorLiveData.observe(this@TransAssistantCollectFragment, Observer {
                dismissLoading()
                it?.getFailureOrNull()?.let { failure ->
                    when (failure.opCode) {
                        CreateTransTaskBean.OP_DEFAULT -> {
                            showDefaultQRCode()
                        }
                        CreateTransTaskBean.OP_SAVE_DEFAULT -> {
                            failure.data?.getString(BaseExtras.RESULT)?.takeIf { path ->
                                path.isNotEmpty()
                            }?.let {
                                // 文件存在了，直接提示成功
                                ToastHelper.showToast(R.string.trans_assistant_save_code_success)
                            } ?: ToastHelper.showToast(R.string.trans_assistant_save_code_error)
                        }
                    }
                }
            })
        }.also {
            it.createShareTransTask(activity!!)
            mViewModel = it
        }
    }

    private fun showQRCodeSuccess(url: String) {
        Glide.with(this).load(url).listener(object : RequestListener<Drawable> {
            override fun onLoadFailed(p0: GlideException?, p1: Any?, p2: Target<Drawable>?, p3: Boolean): Boolean {
                showQRCodeStatus(success = false)
                mShowedQRCode = false
                return true
            }

            override fun onResourceReady(p0: Drawable?, p1: Any?, p2: Target<Drawable>?, p3: DataSource?, p4: Boolean): Boolean {
                mShowedQRCode = true
                iv_qrcode?.setImageDrawable(p0)
                showQRCodeStatus(success = true, qrUrl = url)
                return true
            }
        }).submit()
    }

    private fun showDefaultQRCode() {
        val qrUrl = PersonalConfig.getInstance().getString(TransAssistantConfigKey.TRANS_ASSISTANT_DEFAULT_QRCODE)
        showQRCodeSuccess(qrUrl)
    }

    private fun showQRCodeStatus(success: Boolean, qrUrl: String = "") {
        if (success) {
            tv_sub_title?.visibility = View.VISIBLE
            tv_qrcode_desc?.visibility = View.VISIBLE
            tv_save?.visibility = View.VISIBLE
            tv_reload?.visibility = View.INVISIBLE
            PersonalConfig.getInstance().putString(TransAssistantConfigKey.TRANS_ASSISTANT_DEFAULT_QRCODE, qrUrl)
        } else {
            tv_sub_title?.visibility = View.INVISIBLE
            tv_qrcode_desc?.visibility = View.INVISIBLE
            tv_save?.visibility = View.INVISIBLE
            tv_reload?.visibility = View.VISIBLE
            iv_qrcode?.setImageResource(R.drawable.ic_trans_error_qrcode)
        }
    }

    private fun refreshQRCode() {
        showLoading()
        mViewModel?.createShareTransTask(activity!!)
    }

}