package com.baidu.netdisk.ui.xpan.transmission

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v4.view.ViewPager
import android.view.View
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.main.caller.FSConstantApiCompManager
import com.baidu.netdisk.main.caller.FSCreateObjectApiCompManager
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.ConversationActivity
import com.baidu.netdisk.ui.cloudp2p.ICloudP2PNetdiskBackView
import com.baidu.netdisk.ui.cloudp2p.ITransFilePick
import com.baidu.netdisk.ui.widget.CachedFragmentPagerAdapter
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.ui.xpan.transmission.viewmodel.TransFileSelectViewModel
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import kotlinx.android.synthetic.main.activity_trans_file_select.*
import java.util.*

/**
 * 传输助手文件选择页面
 *
 * @author rendawei
 * @since 2019/11/04
 */
class TransFileSelectActivity : BaseTransActivity(), ICommonTitleBarClickListener, ViewPager.OnPageChangeListener,
        ITransFilePick {

    companion object {
        private const val TAG = "TransFileSelectActivity"

        private const val INDEX_NETDISK_FILE_TAB = 0
        private const val INDEX_LOCAL_FILE_TAB = 1

        const val FROM_SCHEME = 0
        const val FROM_PCODE_DIALOG = 1
        const val FROM_CONVERSATION = 2
        const val FROM_QRCODE = 3

        /**
         * 当前页面的选择文件来源种类
         */
        private const val COUNT = 1
        private const val EXTRA_TRANS_TASK = "extra_trans_task"
        private const val EXTRA_COME_FROM = "extra_come_from"

        fun getIntent(activity: Activity, surl: String?, comeFrom: Int = FROM_QRCODE) =
                Intent(activity, TransFileSelectActivity::class.java).apply {
                    putExtra(EXTRA_TRANS_TASK, surl)
                    putExtra(EXTRA_COME_FROM, comeFrom)
                }
    }

    private var mViewPager: ViewPager? = null
    private var mPageAdapter: FileCategoryPageAdapter? = null
    private var mCurrentIndex: Int = 0

    private var mTransViewModel: TransFileSelectViewModel? = null
    private var mSurl: String? = null
    private var mComeFrom: Int? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        mSurl?.let {
            ViewModelProviders.of(this).get(TransFileSelectViewModel::class.java).apply {
                mTransManager = getService(TRANS_ASSISTANT_SERVICE)
                this.mSurl = it
                mCheckLiveData.observe(this@TransFileSelectActivity, Observer {
                    dismissLoading()
                    if (it?.isFailure == true) {
                        finish()
                    }
                })
                mSendFileLiveData.observe(this@TransFileSelectActivity, Observer {
                    dismissLoading()
                    if (it?.isSuccess == true) {
                        TransListRecordActivity.getIntent(this@TransFileSelectActivity, TransListRecordActivity.PAGE_MY_SEND).apply {
                            startActivity(this)
                        }.also {
                            finish()
                            ToastHelper.showToast(R.string.trans_file_send_success)
                        }
                        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SEND_FILE_SUCCESS,
                                mComeFrom?.toString() ?: "-1")
                    }
                })
            }.also { pre ->
                mTransViewModel = pre
                showLoading(-1) {
                    finish()
                }
                // 校验收文件的二维码是否过期
                pre.checkLinkValid(this)
            }
        } ?: finish()
    }

    override fun getLayoutId(): Int = R.layout.activity_trans_file_select

    override fun initParams() {
        intent?.apply {
            mSurl = getStringExtra(EXTRA_TRANS_TASK)
            mComeFrom = getIntExtra(EXTRA_COME_FROM, -1)
        }

        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SEND_FILE_SHOW, mComeFrom?.toString()
                ?: "-1")
    }

    override fun initView() {
        initTitle()
        initPagers()
    }

    private fun initTitle() {
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setMiddleTitle(R.string.trans_file_select_title)
        mTitleBar.setTopTitleBarClickListener(this)
    }

    private fun initPagers() {
        mViewPager = fragment_container
        mPageAdapter = FileCategoryPageAdapter(supportFragmentManager, Bundle().apply {
            val key = FSConstantApiCompManager.getFileCategoryActivity2ExtraFromPage()
            val value = FSConstantApiCompManager.getFileCategoryActivity2FormPageTransAssist()
            putInt(key, value)
        })
        mViewPager?.adapter = mPageAdapter

        pager_tab_strip.setViewPager(mViewPager)
        pager_tab_strip.setOnPageChangeListener(this)
        // 一期只做云端，隐藏strip
        if (COUNT <= 1) {
            pager_tab_strip.visibility = View.GONE
        }
    }

    override fun onPageScrollStateChanged(state: Int) {
    }

    override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
    }

    override fun onPageSelected(position: Int) {
        mCurrentIndex = position
        if (mCurrentIndex != INDEX_NETDISK_FILE_TAB) {
            mPageAdapter?.getFragment(INDEX_NETDISK_FILE_TAB)?.takeIf {
                it is ICloudP2PNetdiskBackView
            }?.let {
                (it as ICloudP2PNetdiskBackView).cancelChooseMode()
            }
        }
    }

    override fun onRightButtonClicked(view: View?) {
    }

    override fun onBackPressed() {
        if (mCurrentIndex == INDEX_NETDISK_FILE_TAB) {
            val fragment = mPageAdapter?.getFragment(mCurrentIndex) as? ICloudP2PNetdiskBackView
            fragment?.goBack() ?: finish()
        } else {
            super.onBackPressed()
        }
    }

    override fun onBackButtonClicked() {
        backOrFinish()
    }

    private fun backOrFinish() {
        if (mCurrentIndex == INDEX_NETDISK_FILE_TAB) {
            val fragment = mPageAdapter?.getFragment(mCurrentIndex) as? ICloudP2PNetdiskBackView
            fragment?.goBack() ?: finish()
        } else {
            finish()
        }
    }

    override fun onPickFiles(fromType: Int, data: Bundle?) {
        NetDiskLog.d(TAG, "onPickFiles formType:$fromType")
        when (fromType) {
            ITransFilePick.FROM_CLOUD_TYPE -> processCloudSend(data)
            ITransFilePick.FROM_LOCAL_TYPE -> {
                // 10.0.120不做本地，预留
            }
        }
    }

    override fun onPickCloudImg(data: Intent?) {
        NetDiskLog.d(TAG, "onPickFiles cloud img")
        data?.apply {
            processCloudSend(extras)
        }
    }

    private val processCloudSend = { data: Bundle? ->
        val selectIds = data?.getSerializable(ConversationActivity.EXTRA_FILES) as? ArrayList<Long>
        selectIds?.takeIf {
            it.isNotEmpty()
        }?.apply {
            showLoading(R.string.trans_file_sending)
            mTransViewModel?.sendCloudFile(this@TransFileSelectActivity, this)
        }
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SEND_FILE_BTN_CLICK, mComeFrom?.toString()
                ?: "-1")
    }

    public override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        mPageAdapter?.getFragment(mCurrentIndex)?.onActivityResult(requestCode, resultCode, data)
    }

    private inner class FileCategoryPageAdapter(fm: FragmentManager, val bundle: Bundle?) :
            CachedFragmentPagerAdapter(fm) {

        override fun getItem(index: Int): Fragment {
            return createFragment(index)
        }

        private fun createFragment(index: Int): Fragment {
            return if (index == INDEX_NETDISK_FILE_TAB) {
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                        TransAssistStatisticsKeys.TRANS_ASSIST_SEND_FILE_BTN_SHOW)
                FSCreateObjectApiCompManager.createCloudP2PNetdiskFileFragment(bundle)
            } else {
                FSCreateObjectApiCompManager.createLocalFileCategoryFragment(bundle)
            }
        }

        override fun getCount(): Int {
            return COUNT
        }

        override fun getPageTitle(position: Int): CharSequence? {
            return when (position) {
                INDEX_NETDISK_FILE_TAB -> getString(R.string.cloudp2p_netdisk_file_tab)
                INDEX_LOCAL_FILE_TAB -> getString(R.string.cloudp2p_local_file_tab)
                else -> throw IllegalArgumentException("页面位置 $position 不在正确的范围内")
            }
        }
    }
}