package com.baidu.netdisk.ui.xpan.transmission

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.text.TextUtils
import android.view.View
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.ui.xpan.classification.adapter.SmartDividerItemDecoration
import com.baidu.netdisk.ui.xpan.transmission.adapter.TransItemDetailAdapter
import com.baidu.netdisk.ui.xpan.transmission.viewmodel.TransItemDetailViewModel
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransItemDetailResponse
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import com.netdisk.themeskin.loader.SkinManager
import kotlinx.android.synthetic.main.activity_trans_list_detail.*
import kotlinx.android.synthetic.main.fragment_trans_list_record.empty_view
import kotlinx.android.synthetic.main.fragment_trans_list_record.recycler_view
import kotlinx.android.synthetic.main.layout_trans_item_detail_header.*
import java.util.concurrent.TimeUnit

class TransItemDetailActivity : BaseTransActivity() {

    companion object {

        private const val EXTRA_TID = "extra_tid"
        private const val EXTRA_UUID = "extra_uuid"
        private const val EXTRA_PATH = "extra_path"
        private const val EXTRA_TYPE = "extra_type"
        private const val EXTRA_UK = "extra_uk"
        private const val EXTRA_COME_FROM = "extra_come_from"

        const val TYPE_RECEIVER = 0
        const val TYPE_SENDER = 1

        fun startActivity(activity: Activity?, uuid: String?, tid: String?, fsId: String? = "", type: Int, uk: String?) {
            getIntent(activity, uuid, tid, fsId, type, uk)?.apply {
                putExtra(EXTRA_COME_FROM, 1)
                activity?.startActivity(this)
            }
        }

        fun getIntent(activity: Activity?, uuid: String?, tid: String?, fsId: String? = "",
                      type: Int = TYPE_RECEIVER, uk: String?): Intent? {
            if (TextUtils.isEmpty(uuid) || TextUtils.isEmpty(tid) || activity == null) {
                return null
            }
            return Intent(activity, TransItemDetailActivity::class.java).apply {
                putExtra(EXTRA_UUID, uuid)
                putExtra(EXTRA_TID, tid)
                putExtra(EXTRA_PATH, fsId)
                putExtra(EXTRA_TYPE, type)
                putExtra(EXTRA_UK, uk)
            }
        }
    }

    private var mUUId: String? = null
    private var mTId: String? = null
    private var mFsId: String? = null
    private var mUK: String = ""
    private var mType: Int = TYPE_RECEIVER
    private var mComeFrom: Int = 1

    private var mViewModel: TransItemDetailViewModel? = null
    private var mAdapter: TransItemDetailAdapter? = null
    private var mLoadingMore = false

    private var mSavedInstanceState: Bundle? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        mSavedInstanceState = savedInstanceState
        super.onCreate(savedInstanceState)
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        outState?.apply {
            putString(EXTRA_UK, mUK)
            putString(EXTRA_TID, mTId)
            putString(EXTRA_UUID, mUUId)
            putString(EXTRA_PATH, mFsId)
            putInt(EXTRA_COME_FROM, mComeFrom)
            putInt(EXTRA_TYPE, mType)
        }
    }

    override fun getLayoutId(): Int = R.layout.activity_trans_item_detail

    override fun initParams() {
        intent?.extras?.let {
            opData(it)
        }
    }

    override fun validateParams(savedInstanceState: Bundle?): Boolean {
        savedInstanceState?.let {
            opData(it)
        }
        return !TextUtils.isEmpty(mUUId) && !TextUtils.isEmpty(mTId)
    }

    private fun opData(bundle: Bundle) {
        with(bundle) {
            mTId = getString(EXTRA_TID)
            mUUId = getString(EXTRA_UUID)
            mFsId = getString(EXTRA_PATH)
            mType = getInt(EXTRA_TYPE, TYPE_RECEIVER)
            mComeFrom = getInt(EXTRA_COME_FROM, 1)
            mUK = getString(EXTRA_UK) ?: ""
        }
    }

    override fun initView() {
        if (!validateParams(mSavedInstanceState)) {
            finish()
            return
        }
        initTitle()
        initDetailList()
        initData()
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_ITEM_DETAIL_SHOW, mComeFrom.toString())
    }

    private fun initTitle() {
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setMiddleTitle(R.string.trans_item_detail)
        mTitleBar.setTopTitleBarClickListener(object : ICommonTitleBarClickListener {
            override fun onRightButtonClicked(view: View?) {
            }

            override fun onBackButtonClicked() {
                finish()
            }
        })
    }

    private fun initDetailList() {
        TransItemDetailAdapter(this).apply {
            mItemClickListener = { pos, _ ->
                val item = mDtas[pos]
                if (item.isFile) {
                    val res = if (mType == TYPE_RECEIVER) R.string.trans_save_netdisk_toast else R.string.trans_previewer_no_support
                    ToastHelper.showToast(res)
                } else {
                    startActivity(this@TransItemDetailActivity, mUUId, mTId, item.fsId, mType, mUK)
                }
            }
        }.also {
            mAdapter = it
        }
        val layoutManager = LinearLayoutManager(context!!, LinearLayoutManager.VERTICAL, false)
        recycler_view.addItemDecoration(SmartDividerItemDecoration(context!!, layoutManager.orientation).apply {
            setDrawable(SkinManager.getInstance().getDrawable(R.drawable.bg_dn_list_view_divider))
            setShowFirst(false)
            setShowLast(false)
        })
        recycler_view.setLoadMoreEnabled(true)
        recycler_view.layoutManager = layoutManager
        recycler_view.adapter = mAdapter

        recycler_view.setOnRefreshListener {
            fetchData(refresh = false, loadMore = false)
        }
        recycler_view.setOnLoadMoreListener {
            fetchData(refresh = false, loadMore = true)
        }
    }

    private fun initData() {
        ViewModelProviders.of(this).get(TransItemDetailViewModel::class.java).apply {
            transManager = getService(TRANS_ASSISTANT_SERVICE)
            transListLiveData.observe(this@TransItemDetailActivity, Observer {
                it?.getOrNull()?.apply {
                    if (mLoadingMore) {
                        mLoadingMore = false
                        mAdapter?.appendData(this.tasks)
                    } else {
                        mAdapter?.addData(this.tasks)
                    }
                    showHeaderView(this)
                }
                showResultView(false)
                recycler_view?.isRefreshing = false
            })
            transErrorLiveData.observe(this@TransItemDetailActivity, Observer {
                showResultView(true)
                recycler_view?.isRefreshing = false
            })
        }.also {
            mViewModel = it
            fetchData(refresh = true, loadMore = false)
        }
    }

    private fun showHeaderView(detail: TransItemDetailResponse?) {
        detail?.tasks?.takeIf {
            it.isNotEmpty()
        }?.apply {
            val item = this[0]
            if (mType == TYPE_RECEIVER) {
                nickname.text = "${detail.senderName}的分享"
                sharetime.text = DateUtil.formatTimeEndWithMin(TimeUnit.SECONDS.toMillis(item.ctime))
                GlideHelper.getInstance().displayImage(detail.senderAvatarUrl, avatar_image)
            } else {
                nickname.text = "${AccountUtils.getInstance().cloudName}的分享"
                sharetime.text = DateUtil.formatTimeEndWithMin(TimeUnit.SECONDS.toMillis(item.ctime))
                GlideHelper.getInstance().displayImage(AccountUtils.getInstance().cloudAvatarURL, avatar_image)
            }
        }
    }

    private fun showLoading() {
        empty_view.setLoading(R.string.loading, resources.getColor(R.color.common_color_666),
                R.drawable.classification_loading_anim)
        rl_content.visibility = View.GONE
    }

    private fun showResultView(error: Boolean) {
        if (isDataEmpty()) {
            empty_view.visibility = View.VISIBLE
            rl_content.visibility = View.GONE
        } else {
            empty_view.visibility = View.GONE
            rl_content.visibility = View.VISIBLE
        }
        if (error) {
            if (!isDataEmpty()) {
                ToastHelper.showToast(R.string.classification_customize_load_error)
                return
            }
            empty_view.setLoadError(R.string.backup_server_error, R.drawable.xpan_empty_error)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_999))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.VISIBLE)
            empty_view.setRefreshButtonTextColor(resources.getColor(R.color.empty_refresh_text_color))
            empty_view.setRefreshText(R.string.refresh_verify_code_text)
            empty_view.setRefreshListener {
                fetchData(refresh = true, loadMore = false)
            }
            return
        }
        if (isDataEmpty()) {
            empty_view.setLoadNoData(resources.getString(R.string.trans_item_detail_empty), R.drawable.empty_folder)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_666))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.GONE)
        }
    }

    private fun isDataEmpty(): Boolean = mAdapter?.itemCount ?: 0 <= 0

    override fun fetchData(refresh: Boolean, loadMore: Boolean) {
        // 页面第一次进入的整体Loading
        if (refresh) {
            showLoading()
            mViewModel?.getTransListDetail(0, mUUId!!, mTId!!, mFsId, mType, mUK)
            return
        }
        if (mLoadingMore && loadMore) {
            return
        }
        mLoadingMore = loadMore
        val start = if (mLoadingMore) mAdapter?.itemCount ?: 0 else 0
        mViewModel?.getTransListDetail(start, mUUId!!, mTId!!, mFsId, mType, mUK)
    }
}