package com.baidu.netdisk.ui.xpan.transmission

import android.app.Activity
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.content.Intent
import android.graphics.Typeface
import android.os.Bundle
import android.support.v7.widget.LinearLayoutManager
import android.text.TextUtils
import android.view.View
import android.widget.Button
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.android.util.deviceinfo.DeviceDisplayUtils
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.ui.widget.titlebar.ITitleBarSelectedModeListener
import com.baidu.netdisk.ui.xpan.classification.adapter.SmartDividerItemDecoration
import com.baidu.netdisk.ui.xpan.transmission.adapter.TransListDetailAdapter
import com.baidu.netdisk.ui.xpan.transmission.viewmodel.TransListDetailViewModel
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import com.netdisk.themeskin.loader.SkinManager
import kotlinx.android.synthetic.main.activity_trans_list_detail.*
import kotlinx.android.synthetic.main.layout_trans_list_detail_bottom_bar.*

/**
 * 传输记录-列表详情页
 *
 * @author rendawei
 * @since 2019/11/11
 */
class TransListDetailActivity : BaseTransActivity(), ICommonTitleBarClickListener {

    companion object {

        private const val EXTRA_UUID = "extra_uuid"
        private const val EXTRA_TITLE = "extra_title"
        private const val EXTRA_TYPE = "extra_type"

        fun startActivity(activity: Activity?, uuid: String?, title: String? = "", type: Int) {
            uuid ?: return
            activity?.takeIf {
                !it.isDestroyed && !it.isFinishing
            }?.let {
                Intent(it, TransListDetailActivity::class.java).apply {
                    putExtra(EXTRA_UUID, uuid)
                    putExtra(EXTRA_TITLE, title)
                    putExtra(EXTRA_TYPE, type)
                    it.startActivity(this)
                }
            }
        }

        fun getIntent(activity: Activity?, uuid: String?, title: String? = "", type: Int): Intent? {
            uuid ?: return null
            activity?.takeIf {
                !it.isDestroyed && !it.isFinishing
            }?.let {
                return Intent(it, TransListDetailActivity::class.java).apply {
                    putExtra(EXTRA_UUID, uuid)
                    putExtra(EXTRA_TITLE, title)
                    putExtra(EXTRA_TYPE, type)
                }
            } ?: return null
        }
    }

    private var mUUId: String? = null
    private var mTitle: String? = null
    private var mType: Int = TransItemDetailActivity.TYPE_RECEIVER

    private var mAdapter: TransListDetailAdapter? = null
    private var mViewModel: TransListDetailViewModel? = null

    private var mLoadingMore = false

    private var mSavedInstanceState: Bundle? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        mSavedInstanceState = savedInstanceState
        super.onCreate(savedInstanceState)
    }

    override fun onSaveInstanceState(outState: Bundle?) {
        super.onSaveInstanceState(outState)
        outState?.apply {
            putString(EXTRA_UUID, mUUId)
            putString(EXTRA_TITLE, mTitle)
            putInt(EXTRA_TYPE, mType)
        }
    }

    override fun getLayoutId(): Int = R.layout.activity_trans_list_detail

    override fun initParams() {
        intent?.extras?.let {
            opData(it)
        }
    }

    private fun opData(bundle: Bundle) {
        with(bundle) {
            mUUId = getString(EXTRA_UUID)
            mTitle = getString(EXTRA_TITLE)
            mType = getInt(EXTRA_TYPE, TransItemDetailActivity.TYPE_RECEIVER)
        }
    }

    override fun validateParams(savedInstanceState: Bundle?): Boolean {
        savedInstanceState?.let {
            opData(it)
        }
        return !TextUtils.isEmpty(mUUId)
    }

    override fun initView() {
        if (!validateParams()) {
            finish()
            return
        }
        initTitle()
        initBottom()
        initDetailList()
        initData()
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        mViewModel?.onActivityResult(requestCode, resultCode, data)
    }

    private fun initTitle() {
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setLeftLabel(mTitle)
        mTitleBar.setTopTitleBarClickListener(this)
        mTitleBar.setRightLabel(R.string.trans_save_other_all)
        displayRight()
        mTitleBar.leftTextView?.textSize = DeviceDisplayUtils.dip2sp(this, 17F).toFloat()
        (mTitleBar.rightButtonView as? Button)?.apply {
            setTextColor(resources.getColor(R.color.trans_text_color_06A7FF))
            textSize = DeviceDisplayUtils.dip2sp(this@TransListDetailActivity, 16F).toFloat()
            typeface = Typeface.defaultFromStyle(Typeface.BOLD)
        }
        mTitleBar.setSelectedModeListener(object : ITitleBarSelectedModeListener {
            override fun onCancelClick() {
                mAdapter?.isChooseMode = false
            }

            override fun onSelectAllClick() {

            }
        })
        mTitleBar.setSelectedAllButtonVisible(false)
    }

    private fun displayRight() {
        mTitleBar?.setRightLayoutVisible(!isDataEmpty())
    }

    private fun initBottom() {
        btn_to_save_other.setOnClickListener {
            val results = mAdapter?.let { ad ->
                ad.chooseItems.map {
                    ad.mTransListDetails[it]
                }.toMutableList()
            }
            // 开始转存操作
            results?.apply {
                mViewModel?.copyTransTask(this@TransListDetailActivity, this)
            }
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_RECEIVE_COPY_CLICK)
        }
    }

    private fun initDetailList() {
        val layoutManager = LinearLayoutManager(this, LinearLayoutManager.VERTICAL, false)
        recycler_view.addItemDecoration(SmartDividerItemDecoration(this, layoutManager.orientation).apply {
            setDrawable(SkinManager.getInstance().getDrawable(R.drawable.bg_dn_list_view_divider))
            setShowFirst(false)
            setShowLast(false)
        })
        recycler_view.setLoadMoreEnabled(true)
        recycler_view.layoutManager = layoutManager
        mAdapter = TransListDetailAdapter(this).apply {
            mOnChooseModeListener = {
                if (it) {
                    mTitleBar?.switchToEditMode()
                    refreshSelect()
                } else {
                    mTitleBar?.switchToNormalMode()
                }
                displayBottomBar(it)
                // 编辑态禁止刷新
                recycler_view.setRefreshEnabled(!it)
            }

            mOnItemClickListener = listener@{
                if (isChooseMode) {
                    toggleItemCheck(it)
                    refreshSelect()
                    displayBottomBar(true)
                    return@listener
                }
                val item = mTransListDetails[it]
                if (!item.isFile) {
                    TransItemDetailActivity.startActivity(this@TransListDetailActivity, item.uuid, item.tid, item.fsId, mType, item.senderId)
                } else {
                    ToastHelper.showToast(R.string.trans_save_netdisk_toast)
                }
            }

            mOnItemLongClickListener = listener@{
                if (!isChooseMode) {
                    isChooseMode = true
                    toggleItemCheck(it)
                    refreshSelect()
                    displayBottomBar(true)
                }
                return@listener true
            }
        }
        recycler_view.adapter = mAdapter
        recycler_view.setOnRefreshListener {
            fetchData(refresh = false, loadMore = false)
        }
        recycler_view.setOnLoadMoreListener {
            fetchData(refresh = false, loadMore = true)
        }
    }

    private fun initData() {
        ViewModelProviders.of(this).get(TransListDetailViewModel::class.java).apply {
            transManager = getService(TRANS_ASSISTANT_SERVICE)
            transListLiveData.observe(this@TransListDetailActivity, Observer {
                it?.getOrNull()?.apply {
                    if (mLoadingMore) {
                        mLoadingMore = false
                        mAdapter?.appendTransList(this.tasks)
                    } else {
                        mAdapter?.setTransList(this.tasks)
                    }
                }
                showResultView(false)
                recycler_view?.isRefreshing = false
                displayRight()
            })
            transErrorLiveData.observe(this@TransListDetailActivity, Observer {
                showResultView(true)
                recycler_view?.isRefreshing = false
            })
            transCopyLiveData.observe(this@TransListDetailActivity, Observer {
                if (it?.isSuccess == true) {
                    mAdapter?.isChooseMode = false
                }
                if (it?.getOrNull() == BaseStatus.SUCCESS) {
                    // 成功刷新列表，成功会有已转存标志
                    recycler_view?.isRefreshing = true
                    fetchData(refresh = true, loadMore = false)
                }
            })
        }.also {
            mViewModel = it
            fetchData(refresh = true, loadMore = false)
        }
    }

    private fun refreshSelect() {
        mAdapter?.apply {
            if (!isChooseMode || isDataEmpty()) {
                return
            }
            mTitleBar?.setSelectedNum(chooseItems.size, itemCount)
        }
    }

    private fun displayBottomBar(show: Boolean) {
        fl_bottom?.visibility = if (show) View.VISIBLE else View.GONE
        btn_to_save_other.isEnabled = mAdapter?.chooseItems?.size ?: 0 > 0
    }

    private fun showLoading() {
        empty_view.setLoading(R.string.loading, resources.getColor(R.color.common_color_666),
                R.drawable.classification_loading_anim)
        rl_content.visibility = View.GONE
    }

    private fun showResultView(error: Boolean) {
        if (isDataEmpty()) {
            empty_view.visibility = View.VISIBLE
            rl_content.visibility = View.GONE
        } else {
            empty_view.visibility = View.GONE
            rl_content.visibility = View.VISIBLE
        }
        if (error) {
            if (!isDataEmpty()) {
                ToastHelper.showToast(R.string.classification_customize_load_error)
                return
            }
            empty_view.setLoadError(R.string.backup_server_error, R.drawable.xpan_empty_error)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_999))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.VISIBLE)
            empty_view.setRefreshButtonTextColor(resources.getColor(R.color.empty_refresh_text_color))
            empty_view.setRefreshText(R.string.refresh_verify_code_text)
            empty_view.setRefreshListener {
                fetchData(refresh = true, loadMore = false)
            }
            return
        }
        if (isDataEmpty()) {
            empty_view.setLoadNoData(resources.getString(R.string.trans_list_detail_empty), R.drawable.empty_folder)
            empty_view.setEmptyTextColor(resources.getColor(R.color.common_color_666))
            empty_view.setEmptyTextSize(15f)
            empty_view.setRefreshVisibility(View.GONE)
        }
    }

    private fun isDataEmpty(): Boolean = mAdapter?.itemCount ?: 0 <= 0

    override fun fetchData(refresh: Boolean, loadMore: Boolean) {
        // 页面第一次进入的整体Loading
        if (refresh) {
            showLoading()
            mViewModel?.getTransListDetail(0, mUUId!!)
            return
        }
        if (mLoadingMore && loadMore) {
            return
        }
        mLoadingMore = loadMore
        val start = if (loadMore) mAdapter?.itemCount ?: 0 else 0
        mViewModel?.getTransListDetail(start, mUUId!!)
    }

    override fun onRightButtonClicked(view: View?) {
        if (isDataEmpty()) return
        mAdapter?.isChooseMode = true
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_RECEIVE_TOP_COPY_CLICK)
    }

    override fun onBackButtonClicked() {
        backOrFinish()
    }

    override fun onBackPressed() {
        if (mAdapter?.isChooseMode == true) {
            mAdapter?.isChooseMode = false
            return
        }
        super.onBackPressed()
    }

    private fun backOrFinish() {
        if (mAdapter?.isChooseMode == true) {
            mAdapter?.isChooseMode = false
            return
        }
        finish()
    }
}