package com.baidu.netdisk.ui.xpan.transmission

import android.app.Activity
import android.content.Intent
import android.support.v4.app.Fragment
import android.support.v4.view.ViewPager
import android.view.View
import com.baidu.netdisk.R
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.widget.titlebar.BaseTitleBar
import com.baidu.netdisk.ui.widget.titlebar.ICommonTitleBarClickListener
import com.baidu.netdisk.ui.xpan.transmission.adapter.TransFragmentAdapter
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import kotlinx.android.synthetic.main.activity_trans_list_record.*

/**
 * 传输列表记录页面
 * @author rendawei
 * @since 2019/11/06
 */
class TransListRecordActivity : BaseTransActivity(), ICommonTitleBarClickListener {

    companion object {
        private const val EXTRA_PAGE = "extra_page"
        const val PAGE_MY_SEND = 1
        fun getIntent(activity: Activity, currentPage: Int = 0): Intent =
                Intent(activity, TransListRecordActivity::class.java).apply {
                    putExtra(EXTRA_PAGE, currentPage)
                }
    }

    private var mCurrentPage: Int = 0

    override fun getLayoutId(): Int = R.layout.activity_trans_list_record

    override fun initView() {
        initParam()
        initTitle()
        initPagers()
    }

    private fun initParam() {
        intent?.getIntExtra(EXTRA_PAGE, 0)?.takeIf {
            // 确保在adapter的count范围内
            it in 0..1
        }?.let {
            mCurrentPage = it
        }
    }

    private fun initTitle() {
        mTitleBar = BaseTitleBar(this)
        mTitleBar.setMiddleTitle(R.string.trans_list_record_title)
        mTitleBar.setTopTitleBarClickListener(this)
    }

    private fun initPagers() {
        val titles = mutableListOf<String>(getString(R.string.trans_list_record_receiver),
                getString(R.string.trans_list_record_send))
        val fragments = mutableListOf<Fragment>(TransMyReceiverFragment(), TransMySendFragment())
        val mPagerAdapter = TransFragmentAdapter(supportFragmentManager, titles, fragments)
        fragment_container.adapter = mPagerAdapter

        pager_tab_strip.setViewPager(fragment_container)
        pager_tab_strip.setOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrollStateChanged(state: Int) {
            }

            override fun onPageScrolled(position: Int, positionOffset: Float, positionOffsetPixels: Int) {
            }

            override fun onPageSelected(position: Int) {
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_RECORD_TAB_CLICK,
                        position.toString())
            }
        })
        pager_tab_strip.initTabPosition(mCurrentPage)
    }

    override fun onRightButtonClicked(view: View?) {
    }

    override fun onBackButtonClicked() {
        finish()
    }
}