package com.baidu.netdisk.ui.xpan.transmission

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.support.v7.widget.LinearLayoutManager
import android.view.View
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.xpan.classification.adapter.SmartDividerItemDecoration
import com.baidu.netdisk.ui.xpan.transmission.adapter.TransMyReceiverAdapter
import com.baidu.netdisk.ui.xpan.transmission.share.TransShareBottomSheet
import com.baidu.netdisk.ui.xpan.transmission.viewmodel.TransMyReceiverViewModel
import com.baidu.netdisk.xpan.config.TransAssistantConfigKey
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransTaskResponse
import com.baidu.netdisk.xpan.service.transmission.GetTransListRecordJob
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import kotlinx.android.synthetic.main.fragment_trans_list_record.*

/**
 * 传输记录-我接收的
 *
 * @author rednawei
 * @since 2019/11/06
 */
class TransMyReceiverFragment : BaseTransListFragment() {

    companion object {
        const val TAG = "TransMyReceiverFragment"
    }

    private var mLoadingMore = false

    private var mReceiverAdapter: TransMyReceiverAdapter? = null
    private var mLayoutManager: LinearLayoutManager? = null
    private var mReceiverViewModel: TransMyReceiverViewModel? = null

    override fun initView(view: View) {
        initListView()
    }

    override fun onResume() {
        super.onResume()
        freshItem()
    }

    override fun initData() {
        ViewModelProviders.of(this).get(TransMyReceiverViewModel::class.java).apply {
            transManager = getService(BaseActivity.TRANS_ASSISTANT_SERVICE)
            transListLiveData.observe(this@TransMyReceiverFragment, Observer {
                it?.getOrNull()?.apply {
                    if (mLoadingMore) {
                        mLoadingMore = false
                        mReceiverAdapter?.appendData(this.tasks)
                    } else {
                        mReceiverAdapter?.addData(this.tasks)
                    }
                }
                if (!isDataEmpty()) {
                    showNoticeView(TransAssistantConfigKey.TRANS_LIST_DETAIL_NOTICE, R.string.trans_list_detail_notice)
                }
                changeDisplayView(false)
                recycler_view?.isRefreshing = false
            })
            transRefreshLiveData.observe(this@TransMyReceiverFragment, Observer {
                NetDiskLog.d(TAG, "transRefreshLiveData receiver")
                it?.getOrNull()?.apply {
                    // 刷新列表
                    mReceiverAdapter?.refreshData(first, second.tasks)
                }
                recycler_view?.isRefreshing = false
            })
            transErrorLiveData.observe(this@TransMyReceiverFragment, Observer {
                changeDisplayView(true)
                recycler_view?.isRefreshing = false
            })
        }.also {
            mReceiverViewModel = it
            fetchData(refresh = true, loadMore = false)
        }
    }

    private fun initListView() {
        context ?: return
        TransMyReceiverAdapter(context!!).apply {
            mItemClickListener = { pos, _ ->
                val item = mDtas[pos]
                item.msgCnt = 0
                notifyItemChanged(pos)
                TransListDetailActivity.startActivity(activity, item.uuid, item.title, TransItemDetailActivity.TYPE_RECEIVER)
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_RECEIVE_PREVIEW_CLICK)
            }
            mItemShareClickListener = { pos, _ ->
                val item = mDtas[pos]
                TransTaskResponse().apply {
                    title = item.title
                    qrcodeUrl = item.qrcodeUrl
                    surl = item.surl
                    expireTime = item.endTime
                }.also {
                    TransShareBottomSheet.getTransShareDialog(createTransTaskBean = null, editable = false,
                            transTaskResponse = it, showBottomBar = false).show(childFragmentManager, "TransMyReceiverFragment_SHARE")
                }
                NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_RECORD_SHARE_CLICK)
            }
        }.also {
            mReceiverAdapter = it
        }
        mLayoutManager = LinearLayoutManager(context!!, LinearLayoutManager.VERTICAL, false)
        recycler_view.addItemDecoration(SmartDividerItemDecoration(context!!, LinearLayoutManager.VERTICAL).apply {
            setDrawable(resources.getDrawable(R.drawable.bg_dn_list_view_divider))
            setShowFirst(false)
            setShowLast(false)
        })
        recycler_view.setLoadMoreEnabled(true)
        recycler_view.layoutManager = mLayoutManager
        recycler_view.adapter = mReceiverAdapter

        recycler_view.setOnRefreshListener {
            fetchData(refresh = false, loadMore = false)
        }
        recycler_view.setOnLoadMoreListener {
            fetchData(refresh = false, loadMore = true)
        }
    }

    override fun isDataEmpty(): Boolean {
        return mReceiverAdapter?.itemCount == 0
    }

    override fun fetchData(refresh: Boolean, loadMore: Boolean) {
        // 页面第一次进入的整体Loading
        if (refresh) {
            showEmptyLoading()
            mReceiverViewModel?.getTransReceiverList(0, GetTransListRecordJob.FETCH_TYPE_REQUEST)
            return
        }
        if (mLoadingMore && loadMore) {
            return
        }
        mLoadingMore = loadMore
        val start = if (mLoadingMore) mReceiverAdapter?.itemCount ?: 0 else 0
        mReceiverViewModel?.getTransReceiverList(start, GetTransListRecordJob.FETCH_TYPE_REQUEST)
    }

    private fun changeDisplayView(error: Boolean) {
        showResultView(error, emptyRes = R.string.trans_file_empty, emptyBtnRes = R.string.trans_go_collect_file) {
            activity?.let { act ->
                val intent = TransAssistantActivity.getIntent(act, -1, pageType = TransAssistantActivity.PAGE_TYPE_COLLECT)
                act.startActivity(intent)
            }
        }
    }

    private fun freshItem() {
        if (recycler_view?.isRefreshing == true || mReceiverAdapter?.itemCount == 0
                || mLayoutManager?.isSmoothScrolling == true) {
            NetDiskLog.d(TAG, "transRefreshLiveData refuse")
            return
        }
        val position = mLayoutManager?.findFirstVisibleItemPosition()
        NetDiskLog.d(TAG, "transRefreshLiveData fresh:$position")
        mReceiverViewModel?.getTransReceiverList(position
                ?: 0, GetTransListRecordJob.FETCH_TYPE_REFRESH)
    }
}