package com.baidu.netdisk.ui.xpan.transmission

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.support.v7.widget.LinearLayoutManager
import android.view.View
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.ui.xpan.classification.adapter.SmartDividerItemDecoration
import com.baidu.netdisk.ui.xpan.transmission.adapter.TransMySendAdapter
import com.baidu.netdisk.ui.xpan.transmission.viewmodel.TransMySendViewModel
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.config.TransAssistantConfigKey
import kotlinx.android.synthetic.main.fragment_trans_list_record.*

/**
 * 传输记录-我发送的
 * @author rednawei
 * @since 2019/11/06
 */
class TransMySendFragment : BaseTransListFragment() {

    private var mSendAdapter: TransMySendAdapter? = null
    private var mLoadingMore = false

    private var mSendViewModel: TransMySendViewModel? = null

    override fun initView(view: View) {
        initListView()
    }

    override fun initData() {
        ViewModelProviders.of(this).get(TransMySendViewModel::class.java).apply {
            transManager = getService(BaseActivity.TRANS_ASSISTANT_SERVICE)
            transListLiveData.observe(this@TransMySendFragment, Observer {
                it?.getOrNull()?.apply {
                    if (mLoadingMore) {
                        mLoadingMore = false
                        mSendAdapter?.appendData(this.tasks)
                    } else {
                        mSendAdapter?.addData(this.tasks)
                    }
                }
                if (!isDataEmpty()) {
                    showNoticeView(TransAssistantConfigKey.TRANS_RECORDS_SEND_NOTICE, R.string.trans_list_top_notice)
                }
                showResultView(false, R.string.trans_file_send_empty)
                recycler_view?.isRefreshing = false
            })
            transErrorLiveData.observe(this@TransMySendFragment, Observer {
                showResultView(true, R.string.trans_file_send_empty)
                recycler_view?.isRefreshing = false
            })
        }.also {
            mSendViewModel = it
            fetchData(refresh = true, loadMore = false)
        }
    }

    private fun initListView() {
        context ?: return
        TransMySendAdapter(context!!).apply {
            mItemClickListener = { pos, _ ->
                val item = mDtas[pos]
                if (item.isFile) {
                    ToastHelper.showToast(R.string.trans_previewer_no_support)
                } else {
                    TransItemDetailActivity.startActivity(activity, item.uuid, item.tid, item.fsId,
                            TransItemDetailActivity.TYPE_SENDER, item.receiverId)
                }
            }
        }.also {
            mSendAdapter = it
        }
        val layoutManager = LinearLayoutManager(context!!, LinearLayoutManager.VERTICAL, false)
        recycler_view.addItemDecoration(SmartDividerItemDecoration(context!!, LinearLayoutManager.VERTICAL).apply {
            setDrawable(resources.getDrawable(R.drawable.bg_dn_list_view_divider))
            setShowFirst(false)
            setShowLast(false)
        })
        recycler_view.setLoadMoreEnabled(true)
        recycler_view.layoutManager = layoutManager
        recycler_view.adapter = mSendAdapter
        recycler_view.setOnRefreshListener {
            fetchData(refresh = false, loadMore = false)
        }
        recycler_view.setOnLoadMoreListener {
            fetchData(refresh = false, loadMore = true)
        }
    }

    override fun isDataEmpty(): Boolean {
        return mSendAdapter?.itemCount == 0
    }

    override fun fetchData(refresh: Boolean, loadMore: Boolean) {
        if (refresh) {
            showEmptyLoading()
            mSendViewModel?.getTransSendList(0)
            return
        }
        if (mLoadingMore && loadMore) {
            return
        }
        mLoadingMore = loadMore
        val start = if (mLoadingMore) mSendAdapter?.itemCount ?: 0 else 0
        mSendViewModel?.getTransSendList(start)
    }
}