package com.baidu.netdisk.ui.xpan.transmission.adapter

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.cloudfile.utils.CloudFileHelper
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.kernel.util.FormatUtils
import com.baidu.netdisk.ui.widget.CheckableItemLayout
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransListBaseBean
import java.util.concurrent.TimeUnit

class TransItemDetailAdapter(context: Context) : RecyclerView.Adapter<TransItemDetailAdapter.ViewHolder>() {

    private val mInflater: LayoutInflater = LayoutInflater.from(context)
    val mDtas = mutableListOf<TransListBaseBean>()

    var mItemClickListener: ((Int, View) -> Unit)? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val itemView = mInflater.inflate(R.layout.item_trans_item_detail, parent, false)
        return ViewHolder(itemView)
    }

    override fun getItemCount(): Int = mDtas.size

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        with(mDtas[position]) {
            val isDir = fileType == TransListBaseBean.FILE_TYPE_DIR
            val icon = CloudFileHelper.getFileIcon(fileName, isDir)
            if (isDir) {
                holder.tvSize?.visibility = View.GONE
            } else {
                holder.tvSize?.visibility = View.VISIBLE
                holder.tvSize?.text = FormatUtils.formatFileSize(size)
            }
            holder.ivIcon?.setImageResource(icon)
            holder.tvName?.text = fileName
            holder.tvTime?.text = DateUtil.formatTimeEndWithMin(TimeUnit.SECONDS.toMillis(ctime))
            holder.rlRoot?.setOnClickListener {
                mItemClickListener?.invoke(position, it)
            }
        }
    }

    fun addData(newData: MutableList<TransListBaseBean>?) {
        mDtas.clear()
        newData?.takeIf {
            it.isNotEmpty()
        }?.let {
            mDtas.addAll(newData)
        }
        notifyDataSetChanged()
    }

    fun appendData(newData: MutableList<TransListBaseBean>?) {
        newData?.takeIf {
            it.isNotEmpty()
        }?.let {
            mDtas.addAll(newData)
            notifyDataSetChanged()
        }
    }

    inner class ViewHolder(val rootView: View) : RecyclerView.ViewHolder(rootView) {
        val rlRoot: CheckableItemLayout? = rootView.findViewById(R.id.cb_root)
        val tvName: TextView? = rootView.findViewById(R.id.tv_file_name)
        val tvTime: TextView? = rootView.findViewById(R.id.tv_file_time)
        val tvSize: TextView? = rootView.findViewById(R.id.tv_file_size)
        val ivIcon: ImageView? = rootView.findViewById(R.id.iv_file_icon)
    }
}