package com.baidu.netdisk.ui.xpan.transmission.adapter

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageButton
import android.widget.ImageView
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.cloudfile.utils.CloudFileHelper
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.kernel.util.FormatUtils
import com.baidu.netdisk.ui.widget.CheckableItemLayout
import com.baidu.netdisk.ui.xpan.classification.adapter.BaseCheckableAdapter
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransListBaseBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransReceiverListDetailBean
import java.util.concurrent.TimeUnit

class TransListDetailAdapter(context: Context) : BaseCheckableAdapter<TransListDetailAdapter.ViewHolder, Int>() {

    private val mInflater = LayoutInflater.from(context)
    private val mComeFrom = context.getString(R.string.trans_come_from)

    var mTransListDetails = mutableListOf<TransReceiverListDetailBean>()

    var mOnChooseModeListener: ((Boolean) -> Unit)? = null
    var mOnItemClickListener: ((Int) -> Unit)? = null
    var mOnItemLongClickListener: ((Int) -> Boolean)? = null

    var mLimitCheckNum: Int = 1

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        return ViewHolder(mInflater.inflate(R.layout.item_trans_list_detail, parent, false))
    }

    override fun getItemCount(): Int {
        return mTransListDetails.size
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        mTransListDetails[position].apply {
            holder.tvName.text = fileName
            holder.tvTime.text = DateUtil.formatTimeEndWithMin(TimeUnit.SECONDS.toMillis(ctime))
            holder.tvReceiver.text = mComeFrom.format(senderName)
            if (isChooseMode) {
                holder.checkableLayout.choiceMode = CheckableItemLayout.CHOICE_MODE_MULTIPLE
                val checked = isItemChecked(position)
                holder.checkableLayout.isChecked = checked
                holder.tvCheck.isEnabled = !(!checked && chooseItems.size == mLimitCheckNum)
            } else {
                holder.tvCheck.isEnabled = true
                holder.checkableLayout.choiceMode = CheckableItemLayout.CHOICE_MODE_NONE
            }
            holder.checkableLayout.setOnClickListener {
                mOnItemClickListener?.invoke(position)
            }
            holder.checkableLayout.setOnLongClickListener {
                mOnItemLongClickListener?.invoke(position) ?: false
            }
            holder.tvCheck.setOnClickListener {
                isChooseMode = true
                toggleItemCheck(position)
                mOnChooseModeListener?.invoke(isChooseMode)
            }
            holder.tvLabel.visibility = if (status == TransListBaseBean.SAVE_STATUS_YES) View.VISIBLE else View.GONE
            val icon = CloudFileHelper.getFileIcon(fileName, !isFile)
            if (isFile) {
                holder.tvSize.visibility = View.VISIBLE
                holder.tvSize.text = FormatUtils.formatFileSize(size)
            } else {
                holder.tvSize.visibility = View.GONE
            }
            holder.ivCover.setImageResource(icon)
        }
    }

    override fun onChooseModeChange(chooseMode: Boolean) {
        if (!chooseMode) {
            resetSelectItem()
        }
        mOnChooseModeListener?.invoke(chooseMode)
        notifyDataSetChanged()
    }

    override fun isSelectedAll(): Boolean {
        return chooseItems.size == itemCount && itemCount > 0 || chooseItems.size == mLimitCheckNum
    }

    override fun toggleSelectAll() {
        if (isSelectedAll) {
            resetSelectItem()
        } else {
            resetSelectItem()
            val total = if (itemCount < mLimitCheckNum) {
                itemCount
            } else {
                mLimitCheckNum
            }
            repeat(total) {
                insertSelectItem(it)
            }
        }
        notifyDataSetChanged()
    }

    override fun toggleItemCheck(position: Int) {
        if (!isChooseMode || position !in 0 until itemCount) {
            return
        }
        if (isItemChecked(position)) {
            chooseItems.remove(position)
        } else {
            if (chooseItems.size >= mLimitCheckNum) {
                ToastHelper.showToast(R.string.trans_list_detail_save_limit)
                return
            }
            insertSelectItem(position)
        }
        notifyItemChanged(position)
    }

    fun setTransList(details: MutableList<TransReceiverListDetailBean>?) {
        mTransListDetails.clear()
        details?.let {
            mTransListDetails.addAll(it)
        }
        notifyDataSetChanged()
    }

    fun appendTransList(details: MutableList<TransReceiverListDetailBean>?) {
        details?.let {
            val first = if (mTransListDetails.size > 0) mTransListDetails.size - 1 else 0
            mTransListDetails.addAll(details)
            val end = it.size
            notifyItemRangeChanged(first, end)
        }
    }

    inner class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val ivCover: ImageView = itemView.findViewById(R.id.iv_file_icon)
        val tvName: TextView = itemView.findViewById(R.id.tv_name)
        val tvLabel: TextView = itemView.findViewById(R.id.tv_label)
        val tvSize: TextView = itemView.findViewById(R.id.tv_size)
        val tvTime: TextView = itemView.findViewById(R.id.tv_time)
        val tvReceiver: TextView = itemView.findViewById(R.id.tv_receiver)
        val checkableLayout: CheckableItemLayout = itemView.findViewById(R.id.cb_check)
        val tvCheck: ImageButton = itemView.findViewById(android.R.id.button1)
    }
}