package com.baidu.netdisk.ui.xpan.transmission.adapter

import android.content.Context
import android.support.v7.widget.RecyclerView
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RelativeLayout
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.ui.xpan.extension.getSecondLeftTime
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransReceiverListBean
import java.util.concurrent.TimeUnit

class TransMyReceiverAdapter(context: Context) : RecyclerView.Adapter<TransMyReceiverAdapter.ReceiverHolder>() {

    private val mInflater: LayoutInflater = LayoutInflater.from(context)
    val mDtas = mutableListOf<TransReceiverListBean>()

    var mItemClickListener: ((Int, View) -> Unit)? = null
    var mItemShareClickListener: ((Int, View) -> Unit)? = null

    private val mDateLabel = context.getString(R.string.trans_receiver_expired)
    private val mReceiverFormat = context.getString(R.string.trans_receiver_uid_format)
    private val mStopLabel = context.getString(R.string.trans_stop_receiver)
    private val mNormalColor = context.resources.getColor(R.color.trans_text_color_999999)
    private val mExpireColor = context.resources.getColor(R.color.trans_text_color_ED3F3F)


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ReceiverHolder {
        val itemView = mInflater.inflate(R.layout.item_trans_my_receiver, parent, false)
        return ReceiverHolder(itemView)
    }

    override fun getItemCount(): Int = mDtas.size

    override fun onBindViewHolder(holder: ReceiverHolder, position: Int) {
        with(mDtas[position]) {
            holder.tvName?.text = title
            holder.tvSendCnt?.text = String.format(mReceiverFormat, uidCnt, uidLimit)
            holder.tvTime?.text = DateUtil.formatTimeEndWithMin(TimeUnit.SECONDS.toMillis(ctime))
            holder.tvExpire?.text = if (leftTime > 0) leftTime.getSecondLeftTime(mDateLabel) else mStopLabel
            if (leftTime in 1 until 24 * 60 * 60) {
                holder.tvExpire?.setTextColor(mExpireColor)
            } else {
                holder.tvExpire?.setTextColor(mNormalColor)
            }
            when {
                msgCnt <= 0 -> holder.tvMsgCnt?.visibility = View.INVISIBLE
                msgCnt in 1..99 -> {
                    holder.tvMsgCnt?.visibility = View.VISIBLE
                    holder.tvMsgCnt?.text = msgCnt.toString()
                }
                else -> {
                    holder.tvMsgCnt?.visibility = View.VISIBLE
                    holder.tvMsgCnt?.text = "99+"
                }
            }
            holder.rlRoot?.setOnClickListener {
                mItemClickListener?.invoke(position, it)
            }
            holder.tvShare?.isEnabled = leftTime > 0
            holder.tvShare?.setOnClickListener {
                mItemShareClickListener?.invoke(position, it)
            }
        }
    }

    fun addData(newData: MutableList<TransReceiverListBean>?) {
        mDtas.clear()
        newData?.takeIf {
            it.isNotEmpty()
        }?.let {
            mDtas.addAll(newData)
        }
        notifyDataSetChanged()
    }

    fun appendData(newData: MutableList<TransReceiverListBean>?) {
        newData?.takeIf {
            it.isNotEmpty()
        }?.let {
            mDtas.addAll(newData)
            notifyDataSetChanged()
        }
    }

    fun refreshData(start: Int, newData: MutableList<TransReceiverListBean>?) {
        newData?.takeIf {
            it.isNotEmpty() && mDtas.isNotEmpty() && start < mDtas.size
        }?.let {
            var first = -1
            var last = 0
            it.forEachIndexed { index, bean ->
                val pos = start + index
                if (pos in 0 until mDtas.size && TextUtils.equals(bean.uuid, mDtas[pos].uuid)) {
                    if (first == -1) {
                        first = pos
                    }
                    last = pos
                    mDtas[pos] = bean
                }
            }
            if (start >= 0) notifyItemRangeChanged(first, last - first)
        }
    }

    inner class ReceiverHolder(val rootView: View) : RecyclerView.ViewHolder(rootView) {
        val rlRoot: RelativeLayout? = rootView.findViewById(R.id.rl_root)
        val tvName: TextView? = rootView.findViewById(R.id.tv_file_name)
        val tvSendCnt: TextView? = rootView.findViewById(R.id.tv_send_cnt)
        val tvExpire: TextView? = rootView.findViewById(R.id.tv_expire)
        val tvTime: TextView? = rootView.findViewById(R.id.tv_file_time)
        val tvMsgCnt: TextView? = rootView.findViewById(R.id.tv_msg_cnt)
        val tvShare: TextView? = rootView.findViewById(R.id.tv_share)
    }
}