package com.baidu.netdisk.ui.xpan.transmission.share

import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.os.Bundle
import android.support.annotation.Nullable
import android.support.v4.app.DialogFragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.R
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.xpan.io.parser.transmission.model.CreateTransTaskBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransTaskResponse
import com.baidu.netdisk.xpan.statistics.TransAssistStatisticsKeys
import kotlinx.android.synthetic.main.layout_trans_share_cover.*
import java.io.File

/**
 * 传输助手二维码展示页面
 *
 * @author rendawei
 * @since 2019/11/18
 */
class TransShareQRCodeDialog : DialogFragment() {

    companion object {
        private const val EXTRA_SHARE_PARAM = "extra_share_param"

        fun getCoverShareFragment(transTaskResponse: TransTaskResponse): TransShareQRCodeDialog {
            return TransShareQRCodeDialog().apply {
                arguments = Bundle().apply {
                    putParcelable(EXTRA_SHARE_PARAM, transTaskResponse)
                }
            }
        }
    }

    private var mShareParams: TransTaskResponse? = null
    private lateinit var mShareViewModel: TransShareViewModel
    private var mLocalPath: String? = null

    override fun onCreate(@Nullable savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setStyle(STYLE_NORMAL, R.style.NetDisk_TransShareFullScreen)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.layout_trans_share_cover, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        showContentView(false)
        initParams()
        initItemListener()
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        activity?.let { ctx ->
            if (!mShareViewModel.saveQRCode(ctx, mShareParams!!)) {
                dismiss()
            }
        }
    }

    private fun initParams() {
        arguments?.apply {
            mShareParams = getParcelable(EXTRA_SHARE_PARAM)
        }
        ViewModelProviders.of(this).get(TransShareViewModel::class.java).apply {
            mTransAssManager = (activity as? BaseActivity)?.getService(BaseActivity.TRANS_ASSISTANT_SERVICE)
            mSaveQRCodeLiveData.observe(this@TransShareQRCodeDialog, Observer {
                it?.getOrNull()?.let { path ->
                    mLocalPath = path
                    GlideHelper.getInstance().displayImage(File(path), share_view_content)
                    showContentView(true)
                } ?: run {
                    ToastHelper.showToast(R.string.trans_create_qrcode_fail)
                    dismiss()
                }
            })
            mShareErrorLiveData.observe(this@TransShareQRCodeDialog, Observer {
                ToastHelper.showToast(R.string.trans_create_qrcode_fail)
                dismiss()
            })
            mShareResultLiveData.observe(this@TransShareQRCodeDialog, Observer {
                it?.getOrNull()?.let {
                    dismiss()
                }
            })
        }.also {
            mShareViewModel = it
        }
    }

    private fun showContentView(show: Boolean) {
        fl_content?.visibility = if (show) View.VISIBLE else View.INVISIBLE
    }

    private fun initItemListener() {
        close_btn.setOnClickListener {
            dismiss()
        }
        qr_code_download.setOnClickListener {
            val content = String.format(getString(R.string.trans_qrcode_save_format), mLocalPath)
            ToastHelper.showToast(content)
            dismiss()
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_QR_CHANNEL_CLICK,
                    "0")
        }
        qr_code_wx_share.setOnClickListener {
            mShareViewModel.shareToWX(share_view_content, false)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_QR_CHANNEL_CLICK,
                    CreateTransTaskBean.CHANNEL_WX.toString())
        }
        qr_code_wx_quan_share.setOnClickListener {
            mShareViewModel.shareToWX(share_view_content, true)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_QR_CHANNEL_CLICK,
                    CreateTransTaskBean.CHANNEL_WX_QUAN.toString())
        }
        qr_code_qq_share.setOnClickListener {
            mLocalPath?.let {
                mShareViewModel.shareToQQ(activity!!, it, false)
            }
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_QR_CHANNEL_CLICK,
                    CreateTransTaskBean.CHANNEL_QQ.toString())
        }
        qr_code_sina_share.setOnClickListener {
            mShareViewModel.shareToSina(activity!!, share_view_content)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_QR_CHANNEL_CLICK,
                    CreateTransTaskBean.CHANNEL_SINA.toString())
        }
        qr_code_qzone_share.setOnClickListener {
            mLocalPath?.let {
                mShareViewModel.shareToQQ(activity!!, it, true)
            }
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(TransAssistStatisticsKeys.TRANS_ASSIST_SHARE_QR_CHANNEL_CLICK,
                    CreateTransTaskBean.CHANNEL_QZONE.toString())
        }
    }
}