package com.baidu.netdisk.ui.xpan.transmission.share

import android.app.Activity
import android.app.Application
import android.app.Dialog
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.MutableLiveData
import android.graphics.Bitmap
import android.os.Bundle
import android.os.Handler
import android.support.annotation.StringRes
import android.support.v4.app.FragmentActivity
import android.text.TextUtils
import android.view.View
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.NetdiskCommonConfig
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.storage.config.Setting
import com.baidu.netdisk.cloudp2p.network.model.MsgRichTextBean
import com.baidu.netdisk.kernel.android.util.TextTools
import com.baidu.netdisk.kernel.android.util.image.ImageUtil
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.util.DateUtil
import com.baidu.netdisk.main.caller.ShareCommonApiCompManager
import com.baidu.netdisk.permission.IPermission
import com.baidu.netdisk.ui.cloudp2p.ConversationActivity
import com.baidu.netdisk.ui.cloudp2p.PickShareListActivity
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.permission.PermissionContentHelper
import com.baidu.netdisk.ui.permission.presenter.PermissionPresenter
import com.baidu.netdisk.ui.xpan.extension.doSafeLifeCallback
import com.baidu.netdisk.ui.xpan.extension.getBDuss
import com.baidu.netdisk.ui.xpan.extension.getClouName
import com.baidu.netdisk.ui.xpan.extension.getUid
import com.baidu.netdisk.ui.xpan.extension.model.CustomResult
import com.baidu.netdisk.ui.xpan.transmission.TransFileSelectActivity
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.expansion.qq.QqApiUtils
import com.baidu.netdisk.util.expansion.sina.SinaApiUtils
import com.baidu.netdisk.util.expansion.wx.WXApiUtils
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.wap.launch.IACTIONS
import com.baidu.netdisk.xpan.ITransAssistant
import com.baidu.netdisk.xpan.io.parser.transmission.model.CreateTransTaskBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.PCodeDataBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransPDataBean
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransTaskResponse
import com.baidu.netdisk.xpan.service.Extras
import com.google.gson.Gson
import com.sina.weibo.sdk.WbSdk
import com.sina.weibo.sdk.auth.AuthInfo
import com.sina.weibo.sdk.share.WbShareHandler
import com.tencent.tauth.IUiListener
import com.tencent.tauth.UiError
import java.text.SimpleDateFormat
import java.util.*
import java.util.regex.Pattern

class TransShareViewModel(application: Application) : AndroidViewModel(application) {

    companion object {
        const val TAG = "TransShareViewModel"
        const val DEFAULT_PERIOD = 1
        const val DEFAULT_UID_LIMIT = 500

        const val NICKNAME_MAX_LENGTH = 28
        private const val NICKNAME_MIN_LENGTH = 1
        private const val NICKNAME_REGEX = "^[A-Za-z0-9_\\-\\u4e00-\\u9fa5]+$"

        private const val FRIEND_SHARE_ICON = "https://issuecdn.baidupcs.com/issue/netdisk/tu/chuanshuzhushou/chuanshuzhushou.png"

        const val MINI_URl = "https://pan.baidu.com/union/smartProgram#/transOperating"

        fun getDefaultTaskName(): String {
            val originName = AccountUtils.getInstance().cloudName
            var name:String
            name = if (TextUtils.isEmpty(originName)) {
                AccountUtils.getInstance().uid ?: ""
            } else {
                originName
            }
            // 长度超过五个汉子的进行切割
            name = TextTools.cutCharByNumber(name, 10)
            val date = SimpleDateFormat("yyyyMMdd", Locale.getDefault()).format(Date())
            return "${name}的文件收集$date"
        }
    }

    var mTransAssManager: ITransAssistant? = null

    private var mLoadingDialog: Dialog? = null

    val mShareResultLiveData = MutableLiveData<CustomResult<Int>>()
    val mShareErrorLiveData = MutableLiveData<CustomResult<Int>>()
    val mSaveQRCodeLiveData = MutableLiveData<CustomResult<String?>>()
    val mCreatePCodeLiveData = MutableLiveData<CustomResult<Pair<Int, PCodeDataBean>>>()
    val mCreateDefaultTaskLiveData = MutableLiveData<CustomResult<TransTaskResponse?>>()
    val mCreateTaskLiveData = MutableLiveData<CustomResult<Pair<Int, TransTaskResponse>>>()

    private val mCreateTaskResultReceiver = CreateTaskResultReceiver(this, Handler(), null)
    private val mGeneratePCodeReceiver = GeneratePCodeReceiver(this, Handler(), null)
    private val mSaveQRCodeReceiver = SaveQRCodeReceiver(this, Handler(), null)

    /**
     * 用于分享已有任务
     */
    var mCurrentTaskResponse: TransTaskResponse? = null
    private var mCurrentCreateBean: CreateTransTaskBean? = null
    private var mShareChannel: Int? = null


    /**
     * 保存默认的传输二维码
     */
    fun saveDefaultQRCode(activity: Activity) {
        // 校验存储权限
        val permissionPresenter = PermissionPresenter(activity)
        if (!permissionPresenter.isAllPermissionGranted(IPermission.STORAGE_GROUP_PERMISSION)) {
            permissionPresenter.requestPermissions(IPermission.STORAGE_GROUP_PERMISSION,
                    PermissionContentHelper.STORGE_PERMISSION)
            return
        }
        showShareLoading(activity, R.string.trans_assistant_save_code_loading)
        CreateTransTaskBean(CreateTransTaskBean.CREATE_TYPE_SHARE).apply {
            this.opCode = CreateTransTaskBean.OP_SAVE_DEFAULT
            this.type = CreateTransTaskBean.CREATE_TYPE_DEFAULT
            this.title = getDefaultTaskName()
            this.period = DEFAULT_PERIOD
            this.uidLimit = DEFAULT_UID_LIMIT
            this.channel = CreateTransTaskBean.CHANNEL_QRCODE
            this.fileName = "$title.png"
            this.destPath = Setting.getDefaultSaveDir(getApplication())
        }.also {
            createShareTransTask(activity, it)
        }
    }

    fun saveQRCode(activity: Activity, transTaskResponse: TransTaskResponse): Boolean {
        // 校验存储权限
        val permissionPresenter = PermissionPresenter(activity)
        if (!permissionPresenter.isAllPermissionGranted(IPermission.STORAGE_GROUP_PERMISSION)) {
            permissionPresenter.requestPermissions(IPermission.STORAGE_GROUP_PERMISSION,
                    PermissionContentHelper.STORGE_PERMISSION)
            return false
        }
        showShareLoading(activity, R.string.trans_share_qrcode_loading)
        CreateTransTaskBean(CreateTransTaskBean.CREATE_TYPE_SHARE).apply {
            this.title = transTaskResponse.title
            this.fileName = "${title}_${DateUtil.formatTimeEndWithSeconds(System.currentTimeMillis())}.png"
            this.destPath = Setting.getDefaultSaveDir(getApplication())
        }.also {
            mTransAssManager?.saveQRCode(getApplication(), mSaveQRCodeReceiver, getBDuss(), getUid(), transTaskResponse, it)
        }
        return true
    }

    fun createShareTransTask(activity: Activity, createTaskBean: CreateTransTaskBean = CreateTransTaskBean(CreateTransTaskBean.CREATE_TYPE_DEFAULT)) {
        NetDiskLog.d(TAG, "createShareTransTask:$createTaskBean")
        mCurrentCreateBean = createTaskBean
        // 复用已有任务不去创建新的
        mCurrentTaskResponse?.takeIf {
            // 排除默认任务和同名任务
            createTaskBean.opCode != CreateTransTaskBean.OP_SAVE_DEFAULT &&
                    createTaskBean.opCode != CreateTransTaskBean.OP_DEFAULT &&
                    createTaskBean.ondup != CreateTransTaskBean.DUP_OVERRIDE
        }?.let {
            Bundle().apply {
                putInt(Extras.EXTRA_OP_TYPE, createTaskBean.opCode)
                putInt(Extras.EXTRA_SHARE_CHANNEL, createTaskBean.channel)
                putParcelable(BaseExtras.RESULT, mCurrentTaskResponse)
            }.also {
                // 复用上次生成的分享任务
                dismissShareLoading()
                processSameNameTask(activity, CreateTransTaskBean.DUP_NORMAL, createTaskBean.opCode, it)
                NetDiskLog.d(TAG, "createShareTransTask reuse task response:$mCurrentTaskResponse")
            }
            return
        }
        createTaskBean.userName = getClouName()
        mTransAssManager?.createTransTask(getApplication(), mCreateTaskResultReceiver, getBDuss(), getUid(), createTaskBean)
    }

    /**
     * 处理同名任务错误,处理复用Task的情况
     */
    fun processSameNameTask(activity: Activity, dup: Int, opCode: Int, resultData: Bundle?) {
        val taskResponse = resultData?.getParcelable<TransTaskResponse>(BaseExtras.RESULT)
        val shareChannel = resultData?.getInt(Extras.EXTRA_SHARE_CHANNEL)
                ?: CreateTransTaskBean.CHANNEL_WX
        NetDiskLog.d(TAG, "processSameNameTask channel:$shareChannel taskResponse:$taskResponse opCode:$opCode")
        when (opCode) {
            CreateTransTaskBean.OP_MINI, CreateTransTaskBean.OP_QRCODE, CreateTransTaskBean.OP_FRIEND -> {
                if (dup == CreateTransTaskBean.DUP_OVERRIDE) {
                    mCurrentCreateBean?.let {
                        it.ondup = CreateTransTaskBean.DUP_OVERRIDE
                        showShareLoading(activity)
                        createShareTransTask(activity, it)
                    }
                } else {
                    taskResponse?.let {
                        processCreateTaskSuccess(activity, shareChannel, it)
                    }
                }
            }
            CreateTransTaskBean.OP_PCODE -> {
                if (dup == CreateTransTaskBean.DUP_OVERRIDE) {
                    mCurrentCreateBean?.let {
                        it.ondup = CreateTransTaskBean.DUP_OVERRIDE
                        generateTaskAndPCode(activity, it.title, it.period, it.uidLimit, it.channel, it.ondup)
                    }
                } else {
                    taskResponse?.let {
                        val pd = TransPDataBean(it.title, it.surl)
                        pd.userName = getClouName()
                        mShareChannel = shareChannel
                        generatePCode(activity, Gson().toJson(pd))
                    }
                }
            }
        }
    }

    fun handleShare(activity: Activity, channel: Int, tranTaskBean: CreateTransTaskBean) {
        // 校验分享参数是否合法
        if (!checkParam(tranTaskBean)) {
            return
        }
        // 校验分享APP是否安装
        when (channel) {
            CreateTransTaskBean.CHANNEL_WX -> {
                if (!WXApiUtils.getInstance().isWXInstalled) {
                    ToastHelper.showToast(R.string.story_share_wechat_notinstall)
                    return
                }
                // 分享小程序
                showShareLoading(activity)
                createShareTransTask(activity, tranTaskBean)
                return
            }
            CreateTransTaskBean.CHANNEL_QRCODE, CreateTransTaskBean.CHANNEL_FRIEND -> {
                // 分享二维码/朋友
                showShareLoading(activity)
                createShareTransTask(activity, tranTaskBean)
                return
            }
            CreateTransTaskBean.CHANNEL_WX_QUAN -> {
                if (!WXApiUtils.getInstance().isWXInstalled) {
                    ToastHelper.showToast(R.string.story_share_wechat_notinstall)
                    return
                }
            }
            CreateTransTaskBean.CHANNEL_QQ, CreateTransTaskBean.CHANNEL_QZONE -> {
                if (!QqApiUtils.isQQClientAvailable(activity)) {
                    ToastHelper.showToast(R.string.story_share_qq_notinstall)
                    return
                }
            }
            CreateTransTaskBean.CHANNEL_SINA -> {
                if (!SinaApiUtils.isWeiboInstalled(getApplication())) {
                    ToastHelper.showToast(R.string.story_share_sina_notinstall)
                    return
                }
            }
        }
        // 请求生成盘口令
        generateTaskAndPCode(activity, tranTaskBean.title, tranTaskBean.period, tranTaskBean.uidLimit, channel)
    }

    fun processPCodeSuccess(activity: Activity, channel: Int, pCodeDataBean: PCodeDataBean) {
        when (channel) {
            CreateTransTaskBean.CHANNEL_QQ -> {
                shareToQQ(activity, pCodeDataBean)
            }
            CreateTransTaskBean.CHANNEL_QZONE -> {
                shareToQZone(activity, pCodeDataBean)
            }
            CreateTransTaskBean.CHANNEL_SINA -> {
                shareToSina(activity, pCodeDataBean)
            }
            CreateTransTaskBean.CHANNEL_WX_QUAN -> {
                shareToWXQuan(pCodeDataBean)
            }
            CreateTransTaskBean.CHANNEL_PCODE -> {
                showPCodeDialog(activity, pCodeDataBean)
            }
        }
    }

    fun processCreateTaskSuccess(activity: Activity, channel: Int, transTaskResponse: TransTaskResponse) {
        when (channel) {
            CreateTransTaskBean.CHANNEL_WX -> {
                shareToWXMini(transTaskResponse)
            }
            CreateTransTaskBean.CHANNEL_FRIEND -> {
                shareToFriend(activity, transTaskResponse)
            }
            CreateTransTaskBean.CHANNEL_QRCODE -> {
                mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_QRCODE))
                if (activity is FragmentActivity) {
                    TransShareQRCodeDialog.getCoverShareFragment(transTaskResponse).show(activity.supportFragmentManager, "Cover")
                }
            }
        }
    }

    private fun checkParam(tranTaskBean: CreateTransTaskBean): Boolean {
        val length = TextTools.fetchCharNumber(tranTaskBean.title)
        if (length !in (NICKNAME_MIN_LENGTH..NICKNAME_MAX_LENGTH)) {
            ToastHelper.showToast(R.string.classification_change_name_error_length)
            return false
        }
        if (!Pattern.matches(NICKNAME_REGEX, tranTaskBean.title)) {
            ToastHelper.showToast(R.string.classification_change_name_error_format)
            return false
        }
        return true
    }

    /**
     * 获得View的bitmap
     *
     * @param view
     * @return Bitmap
     */
    private fun convertViewToBitmap(view: View): Bitmap {
        NetDiskLog.d(TAG, "convert View To Bitmap")
        view.buildDrawingCache()
        return view.drawingCache
    }

    private fun shareToWXMini(trans: TransTaskResponse) {
        val params = "surl=${trans.surl}&title=${trans.title}&expire_time=${trans.expireTime}&username=${getClouName()}"
        val miniPath = "/pages/netdisk_transAssistant/index?$params"
        NetDiskLog.d(TAG, "mini path:$miniPath")
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_WX))
        WXApiUtils.getInstance().sendMiniProgramToWx(trans.title, trans.title, MINI_URl, miniPath, R.drawable.ic_trans_wx_mini)
    }

    fun shareToWX(view: View, timeLine: Boolean) {
        if (!WXApiUtils.getInstance().isWXInstalled) {
            ToastHelper.showToast(R.string.story_share_wechat_notinstall)
            return
        }
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_WX))
        WXApiUtils.getInstance().sendBitmapMsgToWX(convertViewToBitmap(view), timeLine)
    }

    fun shareToQQ(activity: Activity, localPath: String, timeLine: Boolean) {
        if (!QqApiUtils.isQQClientAvailable(activity)) {
            ToastHelper.showToast(R.string.story_share_qq_notinstall)
            return
        }
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_QQ))
        val listener = object : IUiListener {
            override fun onComplete(p0: Any?) {
                ToastHelper.showToast(activity.resources.getString(R.string.share_qq_success))
            }

            override fun onCancel() {
                ToastHelper.showToast(activity.resources.getString(R.string.share_qq_failed))
            }

            override fun onError(p0: UiError?) {
                ToastHelper.showToast(activity.resources.getString(R.string.share_qq_canceld))
            }
        }
        if (timeLine) {
            QqApiUtils().shareToQzone(activity, arrayListOf(localPath), listener)
        } else {
            QqApiUtils().shareQQ(activity, localPath, listener)
        }
    }

    fun shareToSina(activity: Activity, view: View) {
        if (!SinaApiUtils.isWeiboInstalled(getApplication())) {
            ToastHelper.showToast(R.string.story_share_sina_notinstall)
            return
        }
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_SINA))
        val mAuthInfo = AuthInfo(activity, NetdiskCommonConfig.NETDISK_THIRD_SDK_ID.sinaAppid,
                SinaApiUtils.REDIRECT_URL, SinaApiUtils.SCOPE)
        WbSdk.install(activity, mAuthInfo)
        val mShareHandler = WbShareHandler(activity)
        mShareHandler.registerApp()
        SinaApiUtils().shareWeiBoImage(mShareHandler, activity,
                ImageUtil.bmpToByteArray(convertViewToBitmap(view), true))
    }

    private fun shareToSina(activity: Activity, pCodeDataBean: PCodeDataBean) {
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_SINA))
        ShareCommonApiCompManager.sinaShare(activity, mCurrentCreateBean?.title, pCodeDataBean.msg,
                "", "",
                "", "")
    }

    private fun shareToQQ(activity: Activity, pCodeDataBean: PCodeDataBean) {
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_QQ))
        QqApiUtils.shareQQText(activity, pCodeDataBean.msg)
    }

    private fun shareToQZone(activity: Activity, pCodeDataBean: PCodeDataBean) {
        val listener = object : IUiListener {
            override fun onComplete(p0: Any?) {
                ToastHelper.showToast(activity.resources.getString(R.string.share_qq_success))
            }

            override fun onCancel() {
                ToastHelper.showToast(activity.resources.getString(R.string.share_qq_failed))
            }

            override fun onError(p0: UiError?) {
                ToastHelper.showToast(activity.resources.getString(R.string.share_qq_canceld))
            }
        }
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_QZONE))
        QqApiUtils().publishToQzone(activity, pCodeDataBean.msg, listener)
    }

    private fun shareToWXQuan(pCodeDataBean: PCodeDataBean) {
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_WX_QUAN))
        WXApiUtils.getInstance().sendTextToWX(pCodeDataBean.msg, true)
    }

    private fun shareToFriend(activity: Activity, transTaskResponse: TransTaskResponse) {
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_FRIEND))
        val bundle = Bundle()
        val richTextBean = MsgRichTextBean()
        richTextBean.mTitle = transTaskResponse.title
        richTextBean.mThumbUrl = FRIEND_SHARE_ICON
        richTextBean.mContent = activity.resources.getString(R.string.trans_share_friend_subtitle)
        richTextBean.mUrl = "bdnetdisk://n/${IACTIONS.TRANSFER_ASSISTANT}?page=send&surl=${transTaskResponse.surl}" +
                "&comeFrom=${TransFileSelectActivity.FROM_CONVERSATION}&m_n_v=10.0.120&params=%7bcheck%3a1%7d"
        bundle.putParcelable(ConversationActivity.EXTRA_RICH_TEXT, richTextBean)
        PickShareListActivity.startPickShareListActivity(activity, bundle, 0)
    }

    private fun showPCodeDialog(activity: Activity, pCodeDataBean: PCodeDataBean) {
        mShareResultLiveData.postValue(CustomResult.success(CreateTransTaskBean.CHANNEL_PCODE))
        DialogBuilder().apply {
            val dialog = buildCustomViewDialog(activity, R.string.trans_share_pcode_title,
                    R.string.trans_share_pcode_confirm, DialogBuilder.DISABLE_DIALOG_BUTTON, R.layout.layout_trans_generate_pcode)
            val content = dialog.findViewById<TextView>(R.id.txt_pcode)
            content.text = pCodeDataBean.msg
            setOnDialogCtrListener(object : DialogCtrListener {
                override fun onOkBtnClick() {
                    TextTools.setCurrentClipData(pCodeDataBean.msg, activity)
                }

                override fun onCancelBtnClick() {
                }
            })
            dialog.show()
        }
    }

    /**
     * 生成传输助手的盘口令(聚合的操作，先生成任务，在生成盘口令)
     */
    private fun generateTaskAndPCode(activity: Activity, title: String, period: Int, uidLimit: Int, channel: Int,
                                     dup: Int = CreateTransTaskBean.DUP_NORMAL) {
        showShareLoading(activity)
        CreateTransTaskBean(CreateTransTaskBean.CREATE_TYPE_SHARE).apply {
            this.opCode = CreateTransTaskBean.OP_PCODE
            this.title = title
            this.period = period
            this.uidLimit = uidLimit
            this.channel = channel
            this.ondup = dup
            this.userName = getClouName()
        }.also {
            createShareTransTask(activity, it)
        }
    }

    /**
     * 只生成盘口令
     */
    private fun generatePCode(activity: Activity, pData: String) {
        showShareLoading(activity)
        mTransAssManager?.generatePCode(getApplication(), mGeneratePCodeReceiver, getBDuss(), getUid(), pData)
    }

    fun showShareLoading(activity: Activity, @StringRes resId: Int = R.string.trans_share_pcode_loading, cancelCB: (() -> Unit)? = null) {
        activity.doSafeLifeCallback { ctx ->
            resId.takeIf {
                it > 0
            }?.let {
                DialogBuilder().buildLoadingDialog(ctx, DialogBuilder.DISABLE_DIALOG_BUTTON,
                        it, DialogBuilder.DISABLE_DIALOG_BUTTON)?.apply {
                    setCanceledOnTouchOutside(false)
                    setOnCancelListener {
                        cancelCB?.invoke()
                    }
                }.also { d ->
                    mLoadingDialog = d
                }
            } ?: DialogBuilder().buildLoadingDialogWithoutText(ctx)?.apply {
                setCanceledOnTouchOutside(false)
                setOnCancelListener {
                    cancelCB?.invoke()
                }
                show()
            }.also {
                mLoadingDialog = it
            }
        }
    }

    fun dismissShareLoading() {
        mLoadingDialog?.takeIf {
            it.isShowing
        }?.apply {
            dismiss()
        }
    }

    fun showSameNameNotice(activity: Activity, listener: DialogCtrListener? = null) {
        DialogBuilder().apply {
            buildTipsDialog(activity, R.string.trans_error_same_name_title, R.string.trans_error_same_name_content,
                    R.string.trans_error_same_name_sure, R.string.trans_error_same_name_cancel)
            this.setOnDialogCtrListener(object : DialogCtrListener {
                override fun onOkBtnClick() {
                    listener?.onOkBtnClick()
                }

                override fun onCancelBtnClick() {
                    listener?.onCancelBtnClick()
                }
            })
        }
    }

    /**
     * 处理服务器返回
     *
     * @return 是否真的成功了
     */
    private fun handSuccessResult(opCode: Int, resultData: Bundle?): Boolean {
        return when (opCode) {
            CreateTransTaskBean.OP_DEFAULT -> {
                resultData?.getParcelable<TransTaskResponse>(BaseExtras.RESULT)?.takeIf {
                    !TextUtils.isEmpty(it.qrcodeUrl)
                }?.let {
                    mCreateDefaultTaskLiveData.postValue(CustomResult.success(it))
                    return true
                } ?: false
            }
            CreateTransTaskBean.OP_SAVE_DEFAULT -> {
                val path = resultData?.getString(BaseExtras.RESULT, null)
                path?.let {
                    mSaveQRCodeLiveData.postValue(CustomResult.success(it))
                    return true
                } ?: false
            }
            CreateTransTaskBean.OP_PCODE -> {
                resultData?.getParcelable<PCodeDataBean>(BaseExtras.RESULT)?.takeIf {
                    !TextUtils.isEmpty(it.msg)
                }?.let {
                    val shareChannel = resultData.getInt(Extras.EXTRA_SHARE_CHANNEL)
                    mCreatePCodeLiveData.postValue(CustomResult.success(Pair(shareChannel, it)))
                    return true
                } ?: false
            }
            CreateTransTaskBean.OP_QRCODE, CreateTransTaskBean.OP_MINI, CreateTransTaskBean.OP_FRIEND -> {
                resultData?.getParcelable<TransTaskResponse>(BaseExtras.RESULT)?.takeIf {
                    !TextUtils.isEmpty(it.qrcodeUrl)
                }?.let {
                    val shareChannel = resultData.getInt(Extras.EXTRA_SHARE_CHANNEL)
                    mCreateTaskLiveData.postValue(CustomResult.success(Pair(shareChannel, it)))
                    return true
                } ?: false
            }
            else -> false
        }
    }

    private inner class GeneratePCodeReceiver(reference: TransShareViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransShareViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransShareViewModel, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            mShareChannel?.let {
                resultData?.putInt(Extras.EXTRA_SHARE_CHANNEL, it)
            }
            if (!handSuccessResult(CreateTransTaskBean.OP_PCODE, resultData)) {
                mShareErrorLiveData.postValue(CustomResult.failure(null, -1, CreateTransTaskBean.OP_PCODE))
            }
            dismissShareLoading()
        }

        override fun onFailed(reference: TransShareViewModel, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            val opCode = resultData.getInt(Extras.EXTRA_OP_TYPE, -1)
            mShareErrorLiveData.postValue(CustomResult.failure(errType, errno, opCode, resultData))
            dismissShareLoading()
            return true
        }
    }

    private inner class SaveQRCodeReceiver(reference: TransShareViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransShareViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransShareViewModel, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            val path = resultData?.getString(BaseExtras.RESULT)
            path?.takeIf {
                !TextUtils.isEmpty(it)
            }?.let {
                mSaveQRCodeLiveData.postValue(CustomResult.success(it))
            } ?: mSaveQRCodeLiveData.postValue(CustomResult.failure(null, -1))
            dismissShareLoading()
        }

        override fun onFailed(reference: TransShareViewModel, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            mShareErrorLiveData.postValue(CustomResult.failure(errType, errno, -1, resultData))
            dismissShareLoading()
            return true
        }
    }

    private inner class CreateTaskResultReceiver(reference: TransShareViewModel, handler: Handler,
                                                 resultView: ResultView?) :
            BaseResultReceiver<TransShareViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransShareViewModel, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            val opCode = resultData?.getInt(Extras.EXTRA_OP_TYPE, -1) ?: -1
            keepTaskResponse(opCode, resultData)
            if (!handSuccessResult(opCode, resultData)) {
                mShareErrorLiveData.postValue(CustomResult.failure(null, -1, opCode))
            }
            dismissShareLoading()
        }

        override fun onFailed(reference: TransShareViewModel, errType: ErrorType, errno: Int, resultData: Bundle): Boolean {
            val opCode = resultData.getInt(Extras.EXTRA_OP_TYPE, -1)
            keepTaskResponse(opCode, resultData)
            mShareErrorLiveData.postValue(CustomResult.failure(errType, errno, opCode, resultData))
            dismissShareLoading()
            return true
        }
    }

    private fun keepTaskResponse(opCode: Int, resultData: Bundle?) {
        resultData?.getParcelable<TransTaskResponse>(Extras.EXTRA_TRANS_TASK)?.takeIf {
            NetDiskLog.d(TAG, "keep task response:$mCurrentTaskResponse opCode:$opCode")
            opCode != CreateTransTaskBean.OP_DEFAULT && opCode != CreateTransTaskBean.OP_SAVE_DEFAULT
        }?.let {
            mCurrentTaskResponse = it
        }
    }

    override fun onCleared() {
        super.onCleared()
        dismissShareLoading()
    }
}