package com.baidu.netdisk.ui.xpan.transmission.viewmodel

import android.app.Activity
import android.app.Application
import android.content.Intent
import android.text.TextUtils
import com.baidu.netdisk.BaseActivity
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.network.ServerURL
import com.baidu.netdisk.cloudfile.constant.CloudFileConstants
import com.baidu.netdisk.cloudfile.io.model.CloudFile
import com.baidu.netdisk.component.base.CommonConstant
import com.baidu.netdisk.io.ErrorCode
import com.baidu.netdisk.kernel.android.util.file.FileUtils
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.main.caller.FSCommonApiCompManager
import com.baidu.netdisk.main.caller.FSConstantApiCompManager
import com.baidu.netdisk.main.caller.FSLocalFileApiCompManager
import com.baidu.netdisk.onlineactivity.OnlineActivityContract
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.task.OnlineTaskManager
import com.baidu.netdisk.ui.SaveFileManager
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogCategory
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogCtrListener
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogFactory
import com.baidu.netdisk.ui.dialog.configdialog.ConfigDialogStyle
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.vip.VipActivity
import com.baidu.netdisk.ui.webview.CommonWebViewActivity
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransReceiverListDetailBean

abstract class BaseFileOperateViewModel(application: Application) : BaseXPanViewModel(application) {

    companion object {
        const val TAG = "BaseFileOperateViewModel"

        const val DEFAULT_LIMIT = 500
    }

    private var mTempCopyFiles: List<TransReceiverListDetailBean>? = null

    protected fun <E> doLimitCheck(activity: Activity, opCode: Int, limit: Int = FSCommonApiCompManager.getFileManagerHelperLimit(),
                                   originData: List<E>?, operator: ((List<E>?) -> Unit)?) {
        originData?.takeIf {
            NetDiskLog.d(TAG, "doLimitCheck data is empty")
            it.isNotEmpty()
        }?.let { delPos ->
            if (limit in 1 until delPos.size) {
                FSCommonApiCompManager.showFileManagerHelperLimitDialog(activity, opCode, limit,
                        object : DialogCtrListener {
                            override fun onOkBtnClick() {
                                operator?.invoke(delPos.take(limit))
                            }

                            override fun onCancelBtnClick() {

                            }
                        })
            } else {
                operator?.invoke(delPos)
            }
        }
    }

    fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        if (resultCode == Activity.RESULT_CANCELED) {
            return
        }
        when (requestCode) {
            CommonConstant.RTN_CODE_PICK_DIRECTORY -> {
                data?.getParcelableExtra<CloudFile>(FSLocalFileApiCompManager.getSelectFolderActivity2SelectPath())?.apply {
                    onPickDirectoryResult(this)
                }
            }
        }
    }

    private fun pickDir(activity: Activity) {
        FSLocalFileApiCompManager.startSelectFolderActivityForResult(activity, CloudFile(CloudFileConstants.PATH_BAIDUYUN_ROOT),
                FSLocalFileApiCompManager.getSelectFolderActivity2CopyByUserStyle(), CommonConstant.RTN_CODE_PICK_DIRECTORY,
                false)
    }

    private fun onPickDirectoryResult(cloudFile: CloudFile) {
        val dest = cloudFile.filePath
        var copyDirPath = dest
        if (CloudFileConstants.PATH_BAIDUYUN_ROOT != dest && dest.endsWith(FileUtils.PATH_CONNECTOR)) {
            // 如果是新建文件夹，返回时会带"/"
            copyDirPath = dest.substring(0, dest.length - 1)
        }
        mTempCopyFiles?.takeIf {
            it.isNotEmpty()
        }?.let {
            copyTransTask(copyDirPath, it)
        }
    }

    fun copyTransTask(activity: Activity, fsIds: List<TransReceiverListDetailBean>) {
        setWeakActivity(activity)
        doLimitCheck(activity, FSConstantApiCompManager.getFileManagerHelper2OpCopy(), DEFAULT_LIMIT, fsIds) {
            pickDir(activity)
            mTempCopyFiles = it
        }
    }

    /**
     * 展示超出空间限制提示
     */
    fun processNoSpaceError(activity: BaseActivity, errno: Int): Boolean {
        val saveFileManager = SaveFileManager()
        if (isNoSpaceError(errno)) {
            if (AccountUtils.getInstance().isSVip) { // 弹出空间超限的提示
                saveFileManager.showNoPermissionDialog(errno, activity)
            } else {
                ConfigDialogFactory.buildDialog(BaseActivity.getTopAvailableActivity(),
                        ConfigDialogStyle.DIALOG_STYLE_ONE_BUTTON,
                        ConfigDialogCategory.DIALOG_CATEGORY_SPACE,
                        ConfigDialogCategory.DIALOG_SPACE_ACTION_SAVE, object : ConfigDialogCtrListener {
                    override fun onOkBtnClick(link: String) {
                        if (TextUtils.isEmpty(link)) {
                            CommonWebViewActivity.startActivity(
                                    BaseActivity.getTopAvailableActivity(), ServerURL.getVipWap())
                        } else {
                            if (link.contains(ServerURL.getVipWap())) {
                                VipActivity.startActivity(BaseActivity.getTopAvailableActivity(),
                                        VipActivity.FROM_SHARE_TOKEN_SAVE_FILE_VIP,
                                        VipActivity.TO_SVIP)
                            } else {
                                CommonWebViewActivity.startActivity(
                                        BaseActivity.getTopAvailableActivity(), link)
                            }
                        }
                        // 统计，分享口令，保存文件时容量不足的弹窗按钮点击
                        NetdiskStatisticsLogForMutilFields.getInstance()
                                .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.SHARE_URL_NOT_ENOUGH_STORAGE_DIALOG_BTN_CLICKED)
                    }

                    override fun onCancelBtnClick() {
                        OnlineTaskManager.getInstance().handleActivity(
                                BaseApplication.getInstance().applicationContext,
                                OnlineActivityContract.ActivityType.NO_SPACE_ACTIVITY_TYPE,
                                OnlineTaskManager.FORM_POP)
                    }
                }).show()
                // 统计，分享口令，保存文件时容量不足的弹窗展现
                NetdiskStatisticsLogForMutilFields.getInstance()
                        .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.SHARE_URL_NOT_ENOUGH_STORAGE_DIALOG_DISPLAY)
            }
            return true
        } else {
            if (saveFileManager.isGuide(false, errno)) {
                saveFileManager.showNoPermissionDialog(errno, activity)
                return true
            }
            return false
        }
    }

    /**
     * 是否是空间不足错误码
     * @return true 是
     */
    private fun isNoSpaceError(errno: Int): Boolean {
        return (errno == ErrorCode.ERROR_TRANSFER_NO_MORE_STORAGE
                || errno == ErrorCode.ERROR_PAN_TRANSFER_NO_MORE_STORAGE)
    }

    protected open fun copyTransTask(dest: String, copyFiles: List<TransReceiverListDetailBean>) {}

}