package com.baidu.netdisk.ui.xpan.transmission.viewmodel

import android.app.Activity
import android.app.Application
import android.app.Dialog
import android.arch.lifecycle.AndroidViewModel
import android.support.annotation.StringRes
import com.baidu.netdisk.R
import com.baidu.netdisk.ui.manager.DialogBuilder
import com.baidu.netdisk.ui.xpan.extension.doSafeLifeCallback
import java.lang.ref.WeakReference

abstract class BaseXPanViewModel(application: Application) : AndroidViewModel(application) {

    private var mLoadingDialog: Dialog? = null

    private var mRefActivity: WeakReference<Activity>? = null

    fun showVMLoading(activity: Activity, @StringRes resId: Int = R.string.xpan_loading,
                         cancelCB: (() -> Unit)? = null) {
        activity.doSafeLifeCallback { ctx ->
            resId.takeIf {
                it > 0
            }?.let {
                DialogBuilder().buildLoadingDialog(ctx, DialogBuilder.DISABLE_DIALOG_BUTTON,
                        it, DialogBuilder.DISABLE_DIALOG_BUTTON)?.apply {
                    setCanceledOnTouchOutside(false)
                    setOnCancelListener {
                        cancelCB?.invoke()
                    }
                }.also { d ->
                    mLoadingDialog = d
                }
            } ?: DialogBuilder().buildLoadingDialogWithoutText(ctx)?.apply {
                setCanceledOnTouchOutside(false)
                setOnCancelListener {
                    cancelCB?.invoke()
                }
                show()
            }.also {
                mLoadingDialog = it
            }
        }
    }

    fun dismissVMLoading() {
        mLoadingDialog?.takeIf {
            it.isShowing
        }?.apply {
            dismiss()
        }
    }

    fun setWeakActivity(activity: Activity?) {
        activity?.takeIf {
            !it.isFinishing
        }?.let {
            mRefActivity = WeakReference(it)
        }
    }

    fun getWeakActivity(): Activity? {
        return mRefActivity?.get()?.takeIf {
            !it.isFinishing
        }
    }
}