package com.baidu.netdisk.ui.xpan.transmission.viewmodel

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.MutableLiveData
import android.content.Context
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.kernel.util.CollectionUtils
import com.baidu.netdisk.ui.xpan.extension.getBDuss
import com.baidu.netdisk.ui.xpan.extension.getUid
import com.baidu.netdisk.ui.xpan.extension.model.CustomResult
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.xpan.ITransAssistant
import com.baidu.netdisk.xpan.io.ErrorCode
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransSendFileResponse
import com.baidu.netdisk.xpan.service.Extras
import com.google.gson.Gson
import java.util.*

class TransFileSelectViewModel(application: Application) : AndroidViewModel(application) {

    var mTransManager: ITransAssistant? = null
    var mSurl: String? = null

    private val mGsonInstance = Gson()

    val mCheckLiveData = MutableLiveData<CustomResult<Any>>()
    val mSendFileLiveData = MutableLiveData<CustomResult<Any>>()

    private val mCheckLinkResultReceiver = CheckLinkResultReceiver(this, Handler(), null)
    private val mSendCloudResultReceiver = SendFileResultReceiver(this, Handler(), null)

    private var mLastIds: String? = null
    private var mLastTid: String? = null

    /**
     * 校验分享的链接是否合法
     */
    fun checkLinkValid(context: Context) {
        mSurl?.takeIf {
            it.isNotEmpty()
        }?.let {
            mTransManager?.verifyTransTask(context, mCheckLinkResultReceiver, getBDuss(), getUid(), it)
        }
    }

    /**
     * 发送云端文件
     */
    fun sendCloudFile(context: Context, ids: ArrayList<Long>?) {
        if (CollectionUtils.isEmpty(ids)) {
            return
        }
        mSurl?.takeIf {
            it.isNotEmpty()
        }?.let {
            var tid = ""
            val idArray = mGsonInstance.toJson(ids)
            if (!TextUtils.isEmpty(mLastTid) && TextUtils.equals(idArray, mLastIds)) {
                tid = mLastIds!!
                mLastTid = null
                mLastIds = null
            }
            mTransManager?.sendTransFiles(context, mSendCloudResultReceiver, getBDuss(), getUid(), it, idArray, tid)
        }
    }

    private inner class CheckLinkResultReceiver(reference: TransFileSelectViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransFileSelectViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransFileSelectViewModel, resultData: Bundle?) {
            mCheckLiveData.postValue(CustomResult.success(Any()))
        }

        override fun onFailed(reference: TransFileSelectViewModel, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            handleCommonError(errno)
            mCheckLiveData.postValue(CustomResult.failure(errType, errno))
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    private inner class SendFileResultReceiver(reference: TransFileSelectViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransFileSelectViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransFileSelectViewModel, resultData: Bundle?) {
            mSendFileLiveData.postValue(CustomResult.success(Any()))
        }

        override fun onFailed(reference: TransFileSelectViewModel, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            handleCommonError(errno)
            // 发送失败如果参数不变需要带上上次的发送ID
            if (errno == ErrorCode.TRANS_SEND_FILE_FAILURE) {
                val respo = resultData.getParcelable<TransSendFileResponse>(BaseExtras.RESULT)
                val ids = resultData.getString(Extras.EXTRA_FILE)
                mLastTid = respo?.tid
                mLastIds = ids
            }
            mSendFileLiveData.postValue(CustomResult.failure(errType, errno))
            return super.onFailed(reference, errType, errno, resultData)
        }
    }

    private fun handleCommonError(errno: Int) {
        val msgRes = when (errno) {
            ErrorCode.TRANS_SEND_FILE_SELF -> R.string.trans_file_send_self
            ErrorCode.TRANS_SEND_FILE_UID_LIMIT -> R.string.trans_file_send_uid_limit
            ErrorCode.TRANS_SEND_FILE_EXPIRED -> R.string.trans_file_send_code_expired
            ErrorCode.TRANS_SEND_FILE_FAILURE -> R.string.trans_file_send_fail
            ErrorCode.TRANS_SEND_FILE_OVERFLOW -> R.string.trans_file_send_overflow
            ErrorCode.TRANS_SEND_FILE_ILLEGAL -> R.string.trans_file_send_fail
            ErrorCode.TRANS_FILE_NOT_EXIST -> R.string.trans_file_not_exist
            ErrorCode.TRANS_ID_NOT_EXIST -> R.string.trans_file_send_code_expired
            else -> R.string.xpan_common_net_error
        }
        ToastHelper.showToast(msgRes)
    }
}