package com.baidu.netdisk.ui.xpan.transmission.viewmodel

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.MutableLiveData
import android.os.Bundle
import android.os.Handler
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.storage.config.ConfigSmartDevice
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.ui.xpan.extension.getBDuss
import com.baidu.netdisk.ui.xpan.extension.getUid
import com.baidu.netdisk.ui.xpan.extension.model.CustomResult
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.xpan.ITransAssistant
import com.baidu.netdisk.xpan.io.ErrorCode
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransItemDetailResponse

class TransItemDetailViewModel(application: Application) : AndroidViewModel(application) {

    var transManager: ITransAssistant? = null

    val transListLiveData = MutableLiveData<CustomResult<TransItemDetailResponse>>()
    val transErrorLiveData = MutableLiveData<CustomResult<Any>>()

    private val transListResultReceiver = TransListResultReceiver(this, Handler(), null)
    private var limit: Int = 50

    init {
        ConfigSmartDevice(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.SMART_DEVICE)).also {
            limit = it.customizeFileLimit
        }
    }

    fun getTransListDetail(start: Int, uuid: String, tid: String, fsId: String?, type: Int, uk: String) {
        transManager?.getTransItemDetail(getApplication(), transListResultReceiver, getBDuss(), getUid(),
                start, limit, uuid, tid, fsId, type, uk)
    }

    private inner class TransListResultReceiver(reference: TransItemDetailViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransItemDetailViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransItemDetailViewModel, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            resultData?.getParcelable<TransItemDetailResponse>(BaseExtras.RESULT)?.takeIf {
                it.tasks != null
            }?.apply {
                transListLiveData.postValue(CustomResult.success(this))
            } ?: transErrorLiveData.postValue(CustomResult.failure(null, -1))
        }

        override fun onFailed(reference: TransItemDetailViewModel, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            if (errno == ErrorCode.TRANS_FILE_NOT_EXIST) {
                ToastHelper.showToast(R.string.trans_copy_file_not_exist)
            }
            transErrorLiveData.postValue(CustomResult.failure(errType, errno))
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}