package com.baidu.netdisk.ui.xpan.transmission.viewmodel

import android.app.Application
import android.arch.lifecycle.AndroidViewModel
import android.arch.lifecycle.MutableLiveData
import android.os.Bundle
import android.os.Handler
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.storage.config.ConfigSmartDevice
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.ui.xpan.extension.getBDuss
import com.baidu.netdisk.ui.xpan.extension.getUid
import com.baidu.netdisk.ui.xpan.extension.model.CustomResult
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView
import com.baidu.netdisk.xpan.ITransAssistant
import com.baidu.netdisk.xpan.io.parser.transmission.model.TransReceiverListResponse
import com.baidu.netdisk.xpan.service.Extras
import com.baidu.netdisk.xpan.service.transmission.GetTransListRecordJob

class TransMyReceiverViewModel(application: Application) : AndroidViewModel(application) {

    var transManager: ITransAssistant? = null

    val transListLiveData = MutableLiveData<CustomResult<TransReceiverListResponse>>()
    val transRefreshLiveData = MutableLiveData<CustomResult<Pair<Int, TransReceiverListResponse>>>()
    val transErrorLiveData = MutableLiveData<CustomResult<Any>>()

    private val transListResultReceiver = TransListResultReceiver(this, Handler(), null)
    private var limit: Int = 50

    init {
        ConfigSmartDevice(ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.SMART_DEVICE)).also {
            limit = it.customizeFileLimit
        }
    }

    fun getTransReceiverList(start: Int, fetchType: Int = GetTransListRecordJob.FETCH_TYPE_REQUEST) {
        transManager ?: return
        transManager!!.getTransList(getApplication(), transListResultReceiver, getBDuss(), getUid(), start, limit, fetchType)
    }

    private inner class TransListResultReceiver(reference: TransMyReceiverViewModel, handler: Handler, resultView: ResultView?) :
            BaseResultReceiver<TransMyReceiverViewModel>(reference, handler, resultView) {
        override fun onSuccess(reference: TransMyReceiverViewModel, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            val requestType = resultData?.getInt(Extras.EXTRA_OP_TYPE, GetTransListRecordJob.FETCH_TYPE_REQUEST)
            val start = resultData?.getInt(Extras.START, GetTransListRecordJob.FETCH_TYPE_REQUEST)
            val response = resultData?.getParcelable<TransReceiverListResponse>(BaseExtras.RESULT)
            if (requestType == GetTransListRecordJob.FETCH_TYPE_REFRESH) {
                response?.takeIf {
                    it.tasks != null
                }?.apply {
                    transRefreshLiveData.postValue(CustomResult.success(Pair(start ?: 0, this)))
                }
                return
            }
            response?.takeIf {
                it.tasks != null
            }?.apply {
                transListLiveData.postValue(CustomResult.success(this))
            } ?: transErrorLiveData.postValue(CustomResult.failure(null, -1))
        }

        override fun onFailed(reference: TransMyReceiverViewModel, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            transErrorLiveData.postValue(CustomResult.failure(errType, errno))
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}