package com.baidu.netdisk.ui.youaguide

import android.app.Activity
import android.text.TextUtils
import android.view.View
import android.widget.*
import com.baidu.netdisk.R
import com.baidu.netdisk.base.imageloader.GlideHelper
import com.baidu.netdisk.base.storage.config.ConfigYouaGuide
import com.baidu.netdisk.base.storage.config.ServerConfigKey
import com.baidu.netdisk.kernel.architecture.config.PersonalConfig
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.RichMediaActivity
import com.baidu.netdisk.ui.transfer.ITransferYoua


/**
 *
 * @author dengjie09
 * @description: 一刻相册引导 view
 * @date：2019-12-08 16:22
 *
 */
class YouaGuide(private val activity: Activity) : ITransferYoua {

    private val TAG = "YouaGuide"

    /**
     * youa引导是否被用户手动关闭
     */
    private val YOUA_GUIDE_CLOSED_BY_USER = "youa_guide_closed_by_user"

    /**
     * 被用户关闭的日期
     */
    private val YOUA_GUIDE_CLOSED_DATE = "youa_guide_closed_date"

    /**
     * 一刻相册 应用包名
     */
    private val YOUA_PACKAGE_NAME = "com.baidu.youavideo"

    // 一刻相册引导
    private lateinit var youaGuide: LinearLayout

    // 一刻相册引导框按钮
    private lateinit var youaGuideView: RelativeLayout

    // 一刻相册引导 图片内容
    private lateinit var youaGuideLogo: ImageView

    // 一刻相册引导 关闭按钮
    private lateinit var youaGuideClose: ImageView

    // 一刻相册引导 文案title
    private lateinit var youaGuideTitle: TextView

    // 一刻相册引导 文案context
    private lateinit var youaGuideContext: TextView

    // 一刻相册引导 链接跳转按钮
    private lateinit var youaGuideLinkButton: Button

    // 一刻相册引导 是否可以点击
    private  var isCanClick: Boolean = true

    // 一刻相册引导 跳转链接
    private val URL = "?from=xcA3"

    /**
     *
     * 初始化一刻相册引导
     */
    fun initYouaGuideView(view: View) {

        val config = ConfigYouaGuide(
                ServerConfigKey.getCfgConfigBodyByType(ServerConfigKey.ConfigType.YOUA_GUIDE))

        // 一刻相册引导
//        youaGuide = view.findViewById(R.id.youa_guide)
        youaGuide = view as LinearLayout
//        youaGuide.visibility = View.GONE

        // 一刻相册引导框按钮
        youaGuideView = view.findViewById(R.id.youa_guide_view)

        // 一刻相册引导 图片内容
        youaGuideLogo = view.findViewById(R.id.youa_logo)

        // 一刻相册引导 关闭按钮
        youaGuideClose = view.findViewById(R.id.youa_guide_close)

        // 一刻相册引导 文案title
        youaGuideTitle = view.findViewById(R.id.youa_title_guide)

        // 一刻相册引导 文案context
        youaGuideContext = view.findViewById(R.id.youa_context_guide)

        // 一刻相册引导 链接跳转按钮
        youaGuideLinkButton = view.findViewById(R.id.youa_guide_button)

        if (!TextUtils.isEmpty(config.transferListPicture)) {
            GlideHelper.getInstance().displayImage(config.transferListPicture,
                    R.drawable.youa_logo, youaGuideLogo, false)
        }

        if (!config.showTransferListCloseButton) {
            youaGuideClose.visibility = View.GONE
        } else {
            youaGuideClose.setOnClickListener { v ->
                NetDiskLog.d(TAG, "关闭引导")
                if (isCanClick) {
                    NetdiskStatisticsLogForMutilFields.getInstance()
                            .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.TRANSFER_LIST_YOUA_GUIDE_COLOSE)
                    youaGuide.visibility = View.GONE
                    PersonalConfig.getInstance().putBoolean(YOUA_GUIDE_CLOSED_BY_USER, true)
                    PersonalConfig.getInstance().putLong(YOUA_GUIDE_CLOSED_DATE, System.currentTimeMillis())
                    PersonalConfig.getInstance().asyncCommit()
                }
            }
        }

        if (!TextUtils.isEmpty(config.transferListGuideTitle)) {
            youaGuideTitle.text = config.transferListGuideTitle
        }

        if (!TextUtils.isEmpty(config.transferListGuideContext)) {
            youaGuideContext.text = config.transferListGuideContext
        }


        youaGuideLinkButton.setOnClickListener {
            NetDiskLog.d(TAG, "跳转链接")
            if (isCanClick) {
                val intent = RichMediaActivity.getStartIntent(activity, config.youaGuideLoadURL + URL)
                if (intent != null) {
                    activity.startActivity(intent)
                }
            }
        }

        youaGuideView.setOnClickListener {
            NetDiskLog.d(TAG, "跳转链接")
            if (isCanClick) {
                val intent = RichMediaActivity.getStartIntent(activity, config.youaGuideLoadURL + URL)
                if (intent != null) {
                    activity.startActivity(intent)
                }
            }
        }

        if (shouldShowYouaGuide(config)) {
            NetdiskStatisticsLogForMutilFields.getInstance()
                    .updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.TRANSFER_LIST_YOUA_GUIDE_SHOW)
            youaGuide.visibility = View.VISIBLE
        }
    }

    /**
     *
     * 是否展示一刻相册引导
     * @return true: 展示； false: 不展示
     */
    private fun shouldShowYouaGuide(config: ConfigYouaGuide): Boolean {

        if (!config.showTransferListGuide) {
            return false
        }

        val timeShowYouaGuideAgainDays = if (installedYoua())
            config.showTransferListPictureAfterNDays
        else
            config.showTransferListPictureAfterNDaysWithUninstall

        // 0 表示不展示
        if ( timeShowYouaGuideAgainDays == 0 ) {
            return false
        }

        // 如果被用户手动关闭 则判断时间
        if (PersonalConfig.getInstance().getBoolean(YOUA_GUIDE_CLOSED_BY_USER, false)) {
            return shouldShowYouaGuideAgainByTime(timeShowYouaGuideAgainDays)
        }

        return true
    }

    /**
     *
     * 被关闭后，按照时间判断是都再次开启
     * @return 是否因该再次开启一刻相册引导
     */
    private fun shouldShowYouaGuideAgainByTime(timeShowYouaGuideAgainDays: Int): Boolean {

        val closedTime = PersonalConfig.getInstance().getLong(YOUA_GUIDE_CLOSED_DATE, 0)
        val durationTimeMillis = System.currentTimeMillis() - closedTime

        val timeShowYouaGuideAgainMillis: Long = calculateMillisFromDays(timeShowYouaGuideAgainDays)
        // 关闭时间已经超出 配置期限 则再次开启
        if (durationTimeMillis > timeShowYouaGuideAgainMillis) {
            PersonalConfig.getInstance().putBoolean(YOUA_GUIDE_CLOSED_BY_USER, false)
            PersonalConfig.getInstance().putLong(YOUA_GUIDE_CLOSED_DATE, 0)
            PersonalConfig.getInstance().asyncCommit()
            return true
        }
        return false
    }

    /**
     *
     * 用户是否已经安装一刻相册
     */
    private fun installedYoua(): Boolean {

        if (TextUtils.isEmpty(YOUA_PACKAGE_NAME)) {
            return false
        }

        val packageManager = activity.packageManager
        val info = packageManager.getInstalledPackages(0)

        if (info == null || info.isEmpty()) {
            return false
        }

        for (info1 in info) {
            if (info1.packageName == YOUA_PACKAGE_NAME) {
                return true
            }
        }
        return false
    }

    /**
     *
     * 将天数转换为 毫秒
     * @return 毫秒
     */
    private fun calculateMillisFromDays(days: Int): Long {
//        if (NetDiskLog.isDebug()) {
//            return (days*1000).toLong()
//        }
        return (days * 24 * 60 * 60 * 1000).toLong()
    }

    override fun cannelYouaGuideView() {
        isCanClick = false
    }

    override fun showYouaGuideView() {
        isCanClick = true
    }
}