package com.baidu.netdisk.uiframe.cardimpl

import android.app.Activity
import android.app.Dialog
import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.support.v4.content.LocalBroadcastManager
import android.text.TextUtils
import android.view.KeyEvent
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.cloudp2p.network.api.ErrorCode
import com.baidu.netdisk.cloudp2p.provider.CloudP2PProviderHelper
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.architecture.net.exception.RemoteExceptionInfo
import com.baidu.netdisk.ui.account.AccountErrorHandler
import com.baidu.netdisk.ui.cloudp2p.presenter.AddFollowPresenter
import com.baidu.netdisk.ui.cloudp2p.presenter.IAddFollowView
import com.baidu.netdisk.ui.manager.BaseDialogBuilder
import com.baidu.netdisk.ui.manager.DialogCtrListener
import com.baidu.netdisk.ui.widget.LoadingDialog
import com.baidu.netdisk.uiframe.UIFrameConstans
import com.baidu.netdisk.util.ToastHelper

/**
 * 添加好友帮助类
 *
 * @author xujing31
 * @since 2019/12/4
 */
class AddFollowHelper(val mActivity: Activity) : IAddFollowView {

    /** loading弹窗 **/
    private var mProgressDialog: Dialog? = null

    /** 加好友逻辑处理类 **/
    private val mAddFollowPresenter = AddFollowPresenter(this)

    /**
     * 黑名单添加好友展示dialog
     * @param error
     */
    fun showAddBlackDialog(error: Int) {
        val dlg: Dialog
        val builder = BaseDialogBuilder()
        if (error == ErrorCode.ADD_BLACK_FRIEND_ERROR) {
            dlg = builder.buildOneButtonDialog(mActivity, R.string.add_error_title,
                    R.string.black_dialog_error_content, R.string.ok)
        } else {
            dlg = builder.buildOneButtonDialog(mActivity, R.string.add_error_title,
                    R.string.black_dialog_error_content_b, R.string.ok)
        }
        builder.setOnDialogCtrListener(object : DialogCtrListener {
            override fun onOkBtnClick() {
                dlg.dismiss()
            }

            override fun onCancelBtnClick() {}
        })
        dlg.setCanceledOnTouchOutside(false)
        dlg.setCancelable(false)
    }


    /**
     * 添加好友
     */
    fun addFollow(uk: Long) {
        showProgressDialog(R.string.adding_follow)
        var followType = CloudP2PServiceHelper.FOLLOW_TYPE_NORMAL
        mAddFollowPresenter.addFollow(uk, followType, null, null, null)
//        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(NetdiskStatisticsLogForMutilFields.StatisticsKeys.ADD_FOLLOW_BUTTON_CLICK)
    }

    /**
     * 显示loading弹窗
     * @param message 文案
     */
    private fun showProgressDialog(message: Int) {
        mProgressDialog = LoadingDialog.show(mActivity, message)
        mProgressDialog?.setOnKeyListener(DialogInterface.OnKeyListener { dialog, keyCode, event ->
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                dismissProgressDialog()
            }
            false
        })
    }

    /* 取消loading弹窗
    * @return 是否成功
    */
    private fun dismissProgressDialog(): Boolean {
        mProgressDialog?.apply {
            if (this.isShowing && !mActivity.isFinishing()) {
                this.dismiss()
                return true
            }
        }
        return false
    }

    override fun getActivity(): Activity {
        return mActivity
    }

    override fun onAddFollowFinished(resultCode: Int, resultData: Bundle?, uk: Long) {
        dismissProgressDialog()
        if (resultCode == BaseStatus.SUCCESS) {
            ToastHelper.showToast(R.string.add_follow_success)
            // 发广播通知添加好友成功
            val broadcastManager = LocalBroadcastManager.getInstance(BaseApplication.getInstance())
            val intent = Intent(UIFrameConstans.ACTION_USER_INFO_PAGE_ADD_SUCCEED)
            intent.putExtra(UIFrameConstans.DATA_FRIEND_UK, uk)
            broadcastManager.sendBroadcast(intent)
        } else if (!CloudP2PServiceHelper.isNetWorkError(resultData)) {
            if (resultData == null) {
                return
            }
            val errorInfo: RemoteExceptionInfo? = resultData.getParcelable(BaseExtras.ERROR_INFO)
            if (AccountErrorHandler().commonServerBanErrorHandling(mActivity, errorInfo)) {
                return
            }
            val errorNo = resultData.getInt(BaseExtras.ERROR)
            val alartMessage = resultData.getString(BaseExtras.SERVER_ALERT_MESSAGE)
            var toastMessage: String? = null
            if (errorNo == ErrorCode.USER_NOT_EXIST) {
                toastMessage = BaseApplication.getInstance().getResources().getString(R.string.error_user_not_exist)
            } else if (errorNo == ErrorCode.FOLLOWLIST_EXCEED_LIMIT) {
                toastMessage = BaseApplication.getInstance().getResources().getString(R.string.error_followlist_exceed_limit)
            } else if (errorNo == ErrorCode.ADD_FOLLOW_REFUSE) {
                toastMessage = BaseApplication.getInstance().getResources().getString(R.string.add_follow_refuse)
            } else if (errorNo == ErrorCode.NICKNAME_MODIFY_FREQUENT) {
                toastMessage = BaseApplication.getInstance().getResources().getString(R.string.add_follow_frequent)
            } else if (errorNo == ErrorCode.IS_FOLLOW_ALREADY) {
                toastMessage = BaseApplication.getInstance().getResources().getString(R.string.add_friend_already_friend)
            } else if (errorNo == ErrorCode.ERROR_CODE_ADD_FRIEND_REFUSE_ANY) {
                toastMessage = BaseApplication.getInstance().getResources().getString(R.string.add_friend_refuse_any)
            } else if (errorNo == ErrorCode.BLACK_ADD_FRIEND_ERROR) {
                showAddBlackDialog(ErrorCode.BLACK_ADD_FRIEND_ERROR)
            } else if (errorNo == ErrorCode.ERROR_CODE_ADD_FRIEND_NEED_VERIFY
                    || errorNo == ErrorCode.ERROR_CODE_VERIFICATION) {
                return
            } else {
                toastMessage = BaseApplication.getInstance().getResources().getString(R.string.add_friend_failed)
            }
            if (errorNo == ErrorCode.BLACK_ADD_FRIEND_ERROR) {
                return
            }
            // 有server下发的提示文案，则优先使用server下发的文案
            toastMessage = if (!TextUtils.isEmpty(alartMessage)) alartMessage else toastMessage
            if (!TextUtils.isEmpty(toastMessage)) {
                ToastHelper.showToast(toastMessage)
            }
        }
    }

    override fun onPassFollowFinished(resultCode: Int, resultData: Bundle?, type: Int, uk: Long) {
        CloudP2PProviderHelper(AccountUtils.getInstance().bduss).updateNewFriendVerifyStatus(BaseApplication.getInstance(), type, uk)
    }

    override fun onAddFollowCancel(uk: Long) {
        dismissProgressDialog()
        ToastHelper.showToast(R.string.add_follow_failed)
    }

}