package com.baidu.netdisk.uiframe.cardimpl

import android.view.View
import com.airbnb.lottie.LottieComposition
import com.baidu.netdisk.R
import com.baidu.netdisk.ui.lottie.NetdiskLottieView
import com.baidu.netdisk.uiframe.card.BaseCardCreator
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs.MESSAGE_COMMON_LIST_LOAD_MORE
import com.baidu.netdisk.uiframe.message.MessageIDs.MESSAGE_COMMON_LIST_LOAD_RETRY

/**
 * 分页加载用的footer
 *
 * @author wangyang34
 * @since 2019/11/7
 */
class ListFooterCreator : BaseCardCreator() {

    companion object {

        /** 没有更多了 */
        const val LIST_FOOTER_STATE_NO_MORE = 0

        /** 重试态 */
        const val LIST_FOOTER_STATE_RETRY = 1

        /** 加载态 */
        const val LIST_FOOTER_STATE_LOADING = 2
    }

    /** 没有更多了 */
    private var mNoMore: View? = null

    /** 重试 */
    private var mRetry: View? = null

    /** loading的视图 */
    private var mLoading: NetdiskLottieView? = null

    override fun layout(): Int {
        return R.layout.list_footer_layout
    }

    override fun onCreateView(convertView: View) {
        mNoMore = convertView.findViewById(R.id.no_more_tips)
        mRetry = convertView.findViewById(R.id.retry_tips)
        mLoading = convertView.findViewById(R.id.loading_more)
        mLoading?.let {
            it.setImageDrawable(context.resources.getDrawable(R.drawable.loading_red))
            LottieComposition.Factory.fromAssetFileName(context, "pullToRefresh.json") { composition ->
                it.setComposition(composition!!)
                it.scale = 0.6f
                if (!it.isAnimating) {
                    it.playAnimation()
                }
            }
        }
    }

    override fun onBindView(cardInfo: Any?, position: Int) {
        mNoMore?.visibility = View.GONE
        mRetry?.visibility = View.GONE
        mLoading?.visibility = View.GONE
        mRetry?.setOnClickListener {
            val event = CommonEvent()
            event.what = MESSAGE_COMMON_LIST_LOAD_RETRY
            container.fragment.postEventToRootContainer(event)
        }
        val info: ListFooterInfo = cardInfo as ListFooterInfo
        when (info.state) {
            LIST_FOOTER_STATE_NO_MORE -> {
                mNoMore?.visibility = View.VISIBLE
            }
            LIST_FOOTER_STATE_RETRY -> {
                mRetry?.visibility = View.VISIBLE
            }
            LIST_FOOTER_STATE_LOADING -> {
                mLoading?.visibility = View.VISIBLE
                if (!info.hasPost) {
                    info.hasPost = true
                    val event = CommonEvent()
                    event.what = MESSAGE_COMMON_LIST_LOAD_MORE
                    container.fragment.postEventToRootContainer(event)
                }
            }
        }
    }
}