package com.baidu.netdisk.uiframe.cardimpl

import android.app.Activity
import android.net.Uri
import android.text.TextUtils
import android.view.View
import android.widget.Button
import android.widget.TextView
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.cloudp2p.network.api.ErrorCode
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.label.entity.CharacterLabel
import com.baidu.netdisk.label.view.LabelListView
import com.baidu.netdisk.label.view.base.BaseLabelListView.Companion.TYPE_TWO_LINE
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.ConversationActivity
import com.baidu.netdisk.ui.cloudp2p.GroupConversationDetailActivity
import com.baidu.netdisk.ui.cloudp2p.GroupConversationDetailActivity.EXTRA_FROM_SEARCH
import com.baidu.netdisk.ui.cloudp2p.GroupInfoActivity
import com.baidu.netdisk.ui.cloudp2p.UserInfoActivity
import com.baidu.netdisk.ui.widget.CircleImageView
import com.baidu.netdisk.ui.widget.RoundedGroupImageView
import com.baidu.netdisk.uiframe.containerimpl.FriendGroupRequestContainer.Companion.FROM_SEARCH
import com.baidu.netdisk.util.imageloader.AvatarLoaderHelper
import com.netdisk.themeskin.loader.SkinManager
import java.util.*

/**
 * author : linwentao
 * date : On 2019-11-06
 *
 * 好友搜索的卡片数据绑定帮助类
 */
class SearchCardHelper {

    companion object{

        /** 用户在群里 **/
        private const val INSIDE_GROUP = 1

        /** 用户不在群里 **/
        private const val NO_INSIDE_GROUP = 0

        /** 好友关系 **/
        private const val IS_FRIEND = 1

        /** 非好友关系 **/
        private const val IS_NOT_FRIEND = 0

        /** 双向好友关系 是双向好友 **/
        private const val TWO_WAY_FRIEND = 10

        /** 在黑名单中 **/
        private const val IS_BLACK = 1

        /**
         * 加群的schemal协议的基础URI
         */
        val BASE_JOIN_URI = Uri.parse("bdnetdisk://n/action.ADD_GROUP")
    }

    private var colorArray = BaseApplication.getInstance().resources.getIntArray(R.array.label_colors);

    /** 好友头像 **/
    lateinit var friendImage: CircleImageView

    /** 群头像 **/
    lateinit var groupImage: RoundedGroupImageView

    /** 名称 **/
    lateinit var title: TextView

    /** 添加/发送按钮 **/
    lateinit var button: Button

    /** 签名 **/
    lateinit var signature: TextView

    /** 标签 **/
    lateinit var lable: LabelListView

    /** 整个卡片view **/
    lateinit var rootView: View

    /** 加好友帮助类 **/
    var mAddFollowHelper: AddFollowHelper? = null

    /**
     * 无参构造
     */
    constructor()

    /**
     * 好友搜索view绑定
     */
    fun onCreateFriendView(view: View) {
        rootView = view.findViewById(R.id.root_layout)
        friendImage = view.findViewById(R.id.friend_image)
        title = view.findViewById(R.id.name_text)
        button = view.findViewById(R.id.add_send_button)
        signature = view.findViewById(R.id.signature_text)
        lable = view.findViewById(R.id.label_tag)
    }

    /**
     * 群号搜索view绑定
     */
    fun onCreateGroupView(view: View) {
        rootView = view.findViewById(R.id.root_layout)
        groupImage = view.findViewById(R.id.group_image)
        title = view.findViewById(R.id.name_text)
        button = view.findViewById(R.id.add_send_button)
        signature = view.findViewById(R.id.signature_text)
    }

    /**
     * 绑定好友卡片view
     */
    fun onBindFriendView(activity: Activity, cardInfo: Any, pageName: String) {
        if (cardInfo !is FriendSearchInfo) {
            return
        }
        mAddFollowHelper = AddFollowHelper(activity)
        title.text = getUserName(cardInfo)
        if (!TextUtils.isEmpty(cardInfo.avatarUrl)) {
            AvatarLoaderHelper.getInstance().displayAvatar(cardInfo.avatarUrl, R.drawable.default_user_head_icon, friendImage)
        }

        if(TextUtils.isEmpty(cardInfo.intro)) {
            signature.visibility = View.GONE
        } else {
            signature.visibility = View.VISIBLE
            signature.text = cardInfo.intro
        }
        var isFriend = false
        button.text = "添加好友"
        if (cardInfo.rel > 0) {
            if (cardInfo.rel == TWO_WAY_FRIEND) {
                button.text = "发送消息"
                isFriend = true
            }
        } else if(cardInfo.followFlag == IS_FRIEND) {
            button.text = "发送消息"
            isFriend = true
        }
        button.setOnClickListener {
            onFriendCardClick(activity, cardInfo, isFriend, pageName)
        }
        lable.setSize(12)
        lable.setBoldStyle(true)
        lable.setData(getLabels(cardInfo.tags?.toList()), TYPE_TWO_LINE)

        rootView.setOnClickListener {
            UserInfoActivity.startUserInfoActivity(activity, cardInfo.uk, null)
        }
    }

    /**
     * 好友卡片和按钮的点击事件
     * @param activity
     * @param cardInfo 用户信息
     * @param page 页面
     */
    private fun onFriendCardClick(activity: Activity, cardInfo: FriendSearchInfo, isFriend: Boolean, page: String){
        if(isFriend) {
            val intent = ConversationActivity.getStartIntent(activity,
                    CloudP2PContract.People.buildDetailUri(cardInfo.uk, AccountUtils.getInstance().bduss),
                    "", cardInfo.avatarUrl, false, false, null)
            activity.startActivity(intent)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_RESULT_FRIEND_SEND_MESSAGE_CLICK, page)
        } else {
            if (cardInfo.blackFlag == IS_BLACK) {
                mAddFollowHelper?.showAddBlackDialog(ErrorCode.ADD_BLACK_FRIEND_ERROR)
            } else {
                mAddFollowHelper?.addFollow(cardInfo.uk)
            }

            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_RESULT_ADD_FRIEND_CLICK, page)
        }
    }

    /**
     * 获取个人标签
     * @param tags 下发的标签内容
     */
    private fun getLabels(tags: List<String>?): List<CharacterLabel> {
        val list = ArrayList<CharacterLabel>()
        if (tags == null || tags.isEmpty()) {
            return list
        }
        for (tag in tags) {
            val label = CharacterLabel()
            val backColor = SkinManager.getInstance()
                    .getColor(R.color.bg_dn_button_gray)
            label.backgroudColor = backColor
            label.textColor = colorArray[tags.indexOf(tag) % colorArray.size]
            label.name = tag
            list.add(label)
        }
        return list
    }

    /**
     * 绑定群卡片view
     */
    fun onBindGroupView(activity: Activity, cardInfo: Any) {
        if (cardInfo !is GroupSearchInfo) {
            return
        }
        title.text = cardInfo.name
        groupImage.setIsGroup(true)
        groupImage.setSmallImageSize(20f)

        if (cardInfo.photoAvatar.size == 1) {
            groupImage.setImageUrls(cardInfo.photoAvatar.toList() as ArrayList<String>)
        } else {
            groupImage.setImageUrls((cardInfo.photoAvatar.toList() as ArrayList<String>).take(4) as ArrayList<String>)
        }

        if(TextUtils.isEmpty(cardInfo.gdesc)) {
            signature.visibility = View.GONE
        } else {
            signature.visibility = View.VISIBLE
            signature.text = cardInfo.gdesc
        }

        if(cardInfo.inside == INSIDE_GROUP) {
            button.text = "进入群聊"
        }
        button.setOnClickListener {
            onGroupCardClick(activity, cardInfo)
        }

        rootView.setOnClickListener {
            if(cardInfo.inside == INSIDE_GROUP) {
                // 群会话详情
                GroupConversationDetailActivity.startGroupConversationDetailActivity(activity,
                        CloudP2PContract.Groups.buildGroupUri(cardInfo.gid, AccountUtils.getInstance().bduss),
                        cardInfo.name, EXTRA_FROM_SEARCH)
            } else {
                val uri = BASE_JOIN_URI.buildUpon()
                        .appendQueryParameter("gid", cardInfo.gid.toString())
                        .appendQueryParameter("sign", cardInfo.sign)
                        .appendQueryParameter("invite_uk", cardInfo.inviteUk.toString())
                        .appendQueryParameter("invite_time", cardInfo.inviteTime.toString())
                        .appendQueryParameter("from", FROM_SEARCH)
                GroupInfoActivity().openPage(activity, uri.toString())
            }
        }
    }

    /**
     * 群组卡片和按钮的点击事件
     * @param activity
     * @param cardInfo 用户信息
     */
    private fun onGroupCardClick(activity: Activity, cardInfo: GroupSearchInfo){
        if(cardInfo.inside == INSIDE_GROUP) {
            val intent = ConversationActivity.getStartIntent(activity,
                    CloudP2PContract.Groups.buildGroupUri(cardInfo.gid, AccountUtils.getInstance().bduss),
                    cardInfo.name, null, false, false, null)
            activity.startActivity(intent)

            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_RESULT_GROUP_SEND_MESSAGE_CLICK)
        } else {
            val uri = BASE_JOIN_URI.buildUpon()
                    .appendQueryParameter("gid", cardInfo.gid.toString())
                    .appendQueryParameter("sign", cardInfo.sign)
                    .appendQueryParameter("invite_uk", cardInfo.inviteUk.toString())
                    .appendQueryParameter("invite_time", cardInfo.inviteTime.toString())
                    .appendQueryParameter("from", FROM_SEARCH)
            GroupInfoActivity().openPage(activity, uri.toString())

            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.SEARCH_RESULT_JOIN_GROUP_CLICK)
        }
    }


    /**
     * 获取展示名字
     *
     * @param info 下发的信息
     */
    fun getUserName(info: FriendSearchInfo): String {
        return when {
            !info.name.isNullOrEmpty() -> info.name
            info.remark.isNotEmpty() -> info.remark
            info.nickName.isNotEmpty() -> info.nickName
            info.displayName.isNotEmpty() -> info.displayName
            info.uname.isNotEmpty() -> info.uname
            else -> info.uname
        }
    }
}