package com.baidu.netdisk.uiframe.containerimpl

import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.R
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.cloudp2p.network.model.GroupInfoBean
import com.baidu.netdisk.cloudp2p.network.model.JoinGroupResponse
import com.baidu.netdisk.cloudp2p.provider.CloudP2PContract
import com.baidu.netdisk.cloudp2p.provider.CloudP2PProviderHelper
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.net.Response
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.cloudp2p.ConversationActivity
import com.baidu.netdisk.ui.widget.RoundedGroupImageView
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.containerimpl.FriendGroupRequestContainer.Companion.FROM_SACN_JOIN
import com.baidu.netdisk.uiframe.containerimpl.FriendGroupRequestContainer.Companion.FROM_SEARCH
import com.baidu.netdisk.uiframe.containerimpl.FriendGroupRequestContainer.Companion.FROM_SEARCH_JOIN
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs
import com.baidu.netdisk.util.LoadingDialogHelper
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import kotlinx.android.synthetic.main.cloudp2p_group_info_layout.view.*

class FriendGroupContainer : BaseContainer() {

    companion object {
        /**
         * 可以加入群组
         */
        const val CAN_JOIN_GROUP = 0

        /**
         * 邀请者已经进群
         */
        const val JOIN_GROUP_OWNER_ALREADY_JOIN = 2100

        /**
         * 邀请者不在该群
         */
        const val JOIN_GROUP_OWNER_NOT_IN = 2103

        /**
         * 已关闭群成员邀请
         */
        const val JOIN_GROUP_CLOSED = 2107

        /**
         * 群成员已满
         */
        const val JOIN_GROUP_MEMBER_LIMIT = 2119


        /**
         * 	群组不存在
         */
        const val JOIN_GROUP_NOT_EXIST = 2102

        /**
         * 超过时间点，链接失效
         */
        const val JOIN_GROUP_LINK_OVER_TIME = 112

        /**
         * 签名错误
         */
        const val JOIN_GROUP_SIGNATURE_WRONG = 113
    }

    /** 加群的uri */
    lateinit var mUri: Uri

    /** 头像组view  */
    private lateinit var mGroupAvatarView : RoundedGroupImageView

    /** 群组名称view  */
    private lateinit var mGroupNameTextView : TextView

    /** 群组成员数量view  */
    private lateinit var mGroupNumTextView : TextView

    /** 群号view  */
    private lateinit var mGroupNumberTextView : TextView

    /** 群组描述view  */
    private lateinit var mGroupDescTextView : TextView

    /** 加入群组按钮view  */
    private lateinit var mGroupJoinBtn : TextView

    /** 群组无法加入原因view  */
    private lateinit var mGroupReasonTextView : TextView

    /** 群组数据  */
    private var mGroupInfo: GroupInfoBean? = null

    /** 群组数据  */
    private var mResponse: JoinGroupResponse? = null

    /** loading  */
    private lateinit var mLoadingDialogHelper: LoadingDialogHelper

    /** 来源  */
    private var mFrom: String? = null

    override fun onCreateView(): View {
        val root = initView()
        mUri = Uri.parse(mInfo.data as String)
        mFrom = mUri.getQueryParameter("from")
        mLoadingDialogHelper = LoadingDialogHelper(activity)
        mLoadingDialogHelper.showLoading(R.string.loading)
        return root
    }

    override fun onCommonEvent(event: CommonEvent): Boolean {
        super.onCommonEvent(event)
        when (event.what) {
           MessageIDs.MESSAGE_FRIEND_GROUP_PRE_JOIN_INFO -> {
               mLoadingDialogHelper.dismisssLoading()

               var response: JoinGroupResponse? = null
               if (event.obj != null && event.obj is JoinGroupResponse) {
                   response = event.obj as JoinGroupResponse
               }
               onJoinGroupResponse(response)

           }
        }
        return false
    }

    /**
     * 当收到预加群组的返回时的处理
     * @param joinGroupResponse 返回加群组response
     */
    private fun onJoinGroupResponse(joinGroupResponse: JoinGroupResponse?) {
        mResponse = joinGroupResponse
        initData()
    }


    /**
     * 初始化view
     * @return 根部局
     */
    private fun initView() : View {
        val inflater = LayoutInflater.from(activity)
        val root = inflater.inflate(R.layout.cloudp2p_group_info_layout, null)
        mGroupAvatarView = root.group_avatar
        mGroupAvatarView.setSmallImageSize(40f)
        mGroupNameTextView = root.group_name
        mGroupNumTextView = root.group_member_limit
        mGroupDescTextView = root.group_desc
        mGroupJoinBtn = root.group_join_btn
        mGroupReasonTextView = root.group_reason
        mGroupNumberTextView = root.group_number
        return root
    }

    /**
     * 初始化页面数据
     * 
     */
    private fun initData() {
        if (mResponse == null) {
            dealFinishActivity(mResponse, R.string.scan_join_group_cant_join_other_reason)
            activity.finish()
            return
        }
        mGroupInfo = mResponse?.mGroupInfo
        if (mGroupInfo == null) {
            dealFinishActivity(mResponse, R.string.scan_join_group_cant_join_other_reason)
            activity.finish()
            return
        }
        // 1.设置群组员头像图片（最多4个）
        val avatarUrls = ArrayList<String>()
        val photoList = mGroupInfo?.mPhotoInfo
        if (photoList != null && photoList.size > 0) {
            for (i in 0 until photoList.size) {
                avatarUrls.add(photoList.get(i).mPhoto)
                if (avatarUrls.size == 4) {
                    break
                }
            }
        }
        mGroupAvatarView.setIsGroup(true)
        mGroupAvatarView.setImageUrls(avatarUrls)

        // 2.设置名称
        mGroupNameTextView.text = mGroupInfo?.mGroupName

        // 3.设置成员数量
        mGroupNumTextView.text = activity.resources.getString(R.string.p2p_group_qrcode_num,
                mGroupInfo?.mUserCount, mGroupInfo?.mMaxCount)

        // 设置群号
        mGroupNumberTextView.text =  activity.resources.getString(R.string.p2p_group_number, mGroupInfo?.mGroupNumber)

        // 4.设置群描述
        mGroupDescTextView.text = mGroupInfo?.mGroupDesc

        NetdiskStatisticsLogForMutilFields.getInstance()
                .updateCount(NetdiskStatisticsLogForMutilFields
                        .StatisticsKeys.GROUP_HOME_PAGE_SHOW)

        // 5.设置加入群组按钮及提示文案
        when (mResponse?.errno) {
            CAN_JOIN_GROUP -> {
                mRoot.visibility = View.VISIBLE
                mGroupJoinBtn.isEnabled = true
                mGroupJoinBtn.setOnClickListener {
                    NetDiskLog.d("FriendGroupContainer", "点击加入群组")
                    mLoadingDialogHelper.showLoading(R.string.loading)
                    joinToGroup()
                    NetdiskStatisticsLogForMutilFields.getInstance()
                            .updateCount(NetdiskStatisticsLogForMutilFields
                                    .StatisticsKeys.CLOUDP2P_CLICK_JOIN_GROUP_BTN)
                }

                // 来源如果是搜索，直接申请加群操作
                val from = mUri.getQueryParameter("from")
                if (FROM_SEARCH == from) {
                    mLoadingDialogHelper.showLoading(R.string.loading_text_join_group)
                    joinToGroup()
                }
            }
            JOIN_GROUP_OWNER_ALREADY_JOIN -> openGroup(context.getString(R.string.scan_join_group_already_in))
            JOIN_GROUP_OWNER_NOT_IN -> dealCantJoin(R.string.scan_join_group_owner_not_in)
            JOIN_GROUP_CLOSED -> {
                if (FROM_SEARCH == mFrom) {
                    dealCantJoin(R.string.search_join_group_closed)
                } else {
                    dealCantJoin(R.string.scan_join_group_closed)
                }

            }
            JOIN_GROUP_MEMBER_LIMIT -> dealCantJoin(R.string.scan_join_group_member_limit)
            JOIN_GROUP_NOT_EXIST -> dealCantJoin(R.string.scan_join_group_not_exist)
            JOIN_GROUP_LINK_OVER_TIME -> dealCantJoin(R.string.scan_join_group_link_over_time)
            JOIN_GROUP_SIGNATURE_WRONG -> dealCantJoin(R.string.scan_join_group_signature_wrong)
            else -> {
                dealFinishActivity(mResponse, R.string.scan_join_group_cant_join_other_reason)
                activity.finish()
            }
        }
    }

    /**
     * 处理不可加入状态
     * @param stringId 展示的不可加入的原因文案的id
     */
    private fun dealCantJoin(stringId: Int) {
        mRoot.visibility = View.VISIBLE
        mGroupJoinBtn.isEnabled = false
        mGroupReasonTextView.visibility = View.VISIBLE
        mGroupReasonTextView.text = context.getString(stringId)
    }

    /**
     * 处理失败状态
     * @param stringId 提示失败的toast文案的id
     */
    private fun dealFinishActivity(response: Response?, stringId: Int) {
        var toastString : String? = null
        if (stringId > 0) {
            toastString = context.getString(stringId)
        }
        // 若server下发错误文案，则以server文案优先提示
        if (response != null && !TextUtils.isEmpty(response?.alertmsg)) {
            toastString = response?.alertmsg
        }
        if (!TextUtils.isEmpty(toastString)) {
            ToastHelper.showToast(toastString)
        }
    }

    /**
     * 加入群组
     */
    private fun joinToGroup() {
        val gid = mUri.getQueryParameter("gid")
        val inviteUK = mUri.getQueryParameter("invite_uk")
        val inviteTime = mUri.getQueryParameter("invite_time")
        val sign = mUri.getQueryParameter("sign")
        if (FROM_SEARCH == mFrom) {
            CloudP2PServiceHelper.joinToGroup(BaseApplication.getInstance(), JoinToGroupReceiver(this, Handler()),
                    gid.toLong(), inviteTime.toLong(), inviteUK.toLong(), sign, FROM_SEARCH_JOIN, false)
        } else {
            CloudP2PServiceHelper.joinToGroup(BaseApplication.getInstance(), JoinToGroupReceiver(this, Handler()),
                    gid.toLong(), inviteTime.toLong(), inviteUK.toLong(), sign, FROM_SACN_JOIN, false)
        }
    }

    /**
     * 打开群组
     */
    private fun openGroup(message : String?) {
        // 在对话框页面内插入一条系统提示消息
        if (!TextUtils.isEmpty(message)) {
            val helper = CloudP2PProviderHelper(AccountUtils.getInstance().bduss)
            val resolver = BaseApplication.getInstance().contentResolver
            val gid = mUri.getQueryParameter("gid")
            helper.addGroupTextMessage(resolver, gid!!.toLong(), message, null, 0, "", "",
                    CloudP2PContract.Messages.TYPE_GROUP_SYSTEM, false)
        }
        // 跳转进入群组对话框页面
        mResponse?.apply {
            val intent = ConversationActivity.getStartIntent(activity,
                    CloudP2PContract.Groups.buildGroupUri(this.mGroupInfo.mGroupId, AccountUtils.getInstance().bduss),
                    this.mGroupInfo.mGroupName, null, false, false, null)
            activity.startActivity(intent)
        }
        activity.finish()
    }

    /**
     * 加群组的接收器
     */
    private class JoinToGroupReceiver

    /**
     * 构造
     * @param reference 宿主引用
     * @param handler Handler
     */
    internal constructor(reference: FriendGroupContainer, handler: Handler) :
            BaseResultReceiver<FriendGroupContainer>(reference, handler, null) {

        override fun onSuccess(reference: FriendGroupContainer, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            if (reference == null) {
                return
            }
            reference.mLoadingDialogHelper.dismisssLoading()
            reference.openGroup(null)
        }

        override fun onFailed(reference: FriendGroupContainer, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            reference.mLoadingDialogHelper.dismisssLoading()
            var toastId : Int = 0
            if (errType == ErrorType.NETWORK_ERROR) {
                toastId = R.string.network_error_msg
            } else {
                when (errno) {
                    JOIN_GROUP_OWNER_NOT_IN -> toastId = R.string.scan_join_group_owner_not_in
                    JOIN_GROUP_CLOSED -> toastId = R.string.scan_join_group_closed
                    JOIN_GROUP_MEMBER_LIMIT -> toastId = R.string.scan_join_group_member_limit
                    JOIN_GROUP_NOT_EXIST -> toastId = R.string.scan_join_group_not_exist
                    JOIN_GROUP_LINK_OVER_TIME -> toastId = R.string.scan_join_group_link_over_time
                    JOIN_GROUP_SIGNATURE_WRONG -> toastId = R.string.scan_join_group_signature_wrong
                }
                // 如果有以上的错误，则需要刷新当前页面的按钮状态及提示文案等界面信息
                if (toastId > 0) {
                    reference.dealCantJoin(toastId)
                }
            }
            // 弹toast提示
            reference.dealFinishActivity(resultData.getParcelable<JoinGroupResponse>(BaseExtras.RESULT), toastId)
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}