package com.baidu.netdisk.uiframe.containerimpl

import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.view.View
import com.baidu.netdisk.BaseApplication
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.cloudp2p.network.model.JoinGroupResponse
import com.baidu.netdisk.cloudp2p.service.CloudP2PServiceHelper
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType

/**
 * 预加群组的无页面的container
 *
 * @author wangyang34
 * @since 2019年9月14日
 */
class FriendGroupRequestContainer  : BaseContainer() {

    companion object {

        /** 来自搜索 **/
        const val FROM_SEARCH = "from_search"

        /** 来自扫码的预取和加群 **/
        const val FROM_SACN_JOIN = 1

        /** 来自搜索的预取和加群 **/
        const val FROM_SEARCH_JOIN = 2

    }

    override fun onCreateView(): View? {
        val uri = Uri.parse(mInfo.data as String)
        val gid = uri.getQueryParameter("gid")
        val inviteUK = uri.getQueryParameter("invite_uk")
        val inviteTime = uri.getQueryParameter("invite_time")
        val sign = uri.getQueryParameter("sign")
        val from = uri.getQueryParameter("from")
        if (FROM_SEARCH == from) {
            CloudP2PServiceHelper.joinToGroup(BaseApplication.getInstance(), JoinToGroupReceiver(this, Handler()),
                    gid.toLong(), inviteTime.toLong(), inviteUK.toLong(), sign, FROM_SEARCH_JOIN, true)
        } else {
            CloudP2PServiceHelper.joinToGroup(BaseApplication.getInstance(), JoinToGroupReceiver(this, Handler()),
                    gid.toLong(), inviteTime.toLong(), inviteUK.toLong(), sign, FROM_SACN_JOIN, true)
        }
        return null
    }

    /**
     * 加群组的接收器
     */
    private class JoinToGroupReceiver

    /**
     * 构造
     * @param reference 宿主引用
     * @param handler Handler
     */
    internal constructor(reference: FriendGroupRequestContainer, handler: Handler) :
            BaseResultReceiver<FriendGroupRequestContainer>(reference, handler, null) {

        override fun onSuccess(reference: FriendGroupRequestContainer, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            postEvent(reference, resultData)
        }

        override fun onFailed(reference: FriendGroupRequestContainer, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            postEvent(reference, resultData)
            return super.onFailed(reference, errType, errno, resultData)
        }

        /**
         * 上报加群event
         * @param reference 宿主引用
         * @param resultData 序列化数据
         */
        fun postEvent(reference: FriendGroupRequestContainer, resultData: Bundle?) {
            if (reference == null) {
                return
            }
            var joinGroupResponse = resultData?.getParcelable<JoinGroupResponse>(BaseExtras.RESULT)
            val event = CommonEvent()
            event.what = MessageIDs.MESSAGE_FRIEND_GROUP_PRE_JOIN_INFO
            event.obj = joinGroupResponse
            reference.postEventToPage(event)
        }
    }
}