package com.baidu.netdisk.uiframe.containerimpl.list

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.ResultReceiver
import android.text.TextUtils
import com.baidu.netdisk.base.service.BaseServiceHelper
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.architecture.net.exception.RemoteException
import com.baidu.netdisk.statistics.BaseReportJob
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListService.Companion.EXTRA_COMMON_LIST_URL
import org.json.JSONException
import java.io.IOException
import java.io.UnsupportedEncodingException
import java.security.KeyManagementException
import java.security.KeyStoreException
import java.security.NoSuchAlgorithmException
import java.security.UnrecoverableKeyException

/**
 * 通用列表请求
 *
 * @author wangyang34
 * @since 2019/11/6
 */
class CommonListJob(val context: Context?, val intent: Intent?, val receiver: ResultReceiver?,
                    val bduss: String?, val uid: String?) : BaseReportJob(NAME) {

    companion object {

        /** job名字 */
        const val NAME = "CommonListJob"
    }

    override fun performExecute() {
        if (receiver == null || intent == null) {
            return
        }
        val info = intent.getParcelableExtra<CommonRequestInfo>(EXTRA_COMMON_LIST_URL)
        if (info == null) {
            receiver.send(BaseStatus.FAILED, Bundle.EMPTY)
        }
        try {
            val response = sendRequest(info)
            if (response == null) {
                receiver.send(BaseStatus.FAILED, Bundle.EMPTY)
                return
            }
            var bundle = Bundle()
            bundle.putSerializable(BaseExtras.RESULT, response)
            if (response.errno == 0) {
                receiver.send(BaseStatus.SUCCESS, bundle)
            } else {
                receiver.send(BaseStatus.FAILED, bundle)
            }
        } catch (e: RemoteException) {
            BaseServiceHelper.handleRemoteException(e, receiver)
        } catch (e: IOException) {
            BaseServiceHelper.handleIOException(e, receiver)
        }
    }

    /**
     * 发出请求
     *
     * @param info 请求信息
     * @return 结果
     */
    private fun sendRequest(info: CommonRequestInfo): CommonListResponse? {
        return try {
            ListRequestApi(bduss, uid).getList(info)
        } catch (e: KeyManagementException) {
            null
        } catch (e: UnrecoverableKeyException) {
            null
        } catch (e: NoSuchAlgorithmException) {
            null
        } catch (e: KeyStoreException) {
            null
        } catch (e: UnsupportedEncodingException) {
            null
        } catch (e: JSONException) {
            null
        }
    }
}