package com.baidu.netdisk.uiframe.containerimpl.list

import android.content.IntentFilter
import android.os.Bundle
import android.os.Handler
import android.support.v4.content.LocalBroadcastManager
import android.view.View
import com.baidu.netdisk.R
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.kotlin.extension.Tag
import com.baidu.netdisk.kotlin.extension.d
import com.baidu.netdisk.uiframe.card.CardStoreCardID.CARD_ITEM_TYPE_LIST_FOOTER_ITEM
import com.baidu.netdisk.uiframe.card.CommonCardInfo
import com.baidu.netdisk.uiframe.cardimpl.ListFooterCreator.Companion.LIST_FOOTER_STATE_LOADING
import com.baidu.netdisk.uiframe.cardimpl.ListFooterCreator.Companion.LIST_FOOTER_STATE_NO_MORE
import com.baidu.netdisk.uiframe.cardimpl.ListFooterCreator.Companion.LIST_FOOTER_STATE_RETRY
import com.baidu.netdisk.uiframe.cardimpl.ListFooterInfo
import com.baidu.netdisk.uiframe.container.BaseContainer
import com.baidu.netdisk.uiframe.containerimpl.list.ListStateContainer.LIST_STATE_FINISH
import com.baidu.netdisk.uiframe.containerimpl.list.ListStateContainer.LIST_STATE_LOADING
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.CommonListAdapter
import com.baidu.netdisk.uiframe.containerimpl.list.adapter.MemoryListAdapterProxy
import com.baidu.netdisk.uiframe.message.CommonEvent
import com.baidu.netdisk.uiframe.message.MessageIDs.*
import com.baidu.netdisk.util.ToastHelper
import com.baidu.netdisk.util.receiver.BaseResultReceiver
import com.baidu.netdisk.util.receiver.ErrorType
import com.baidu.netdisk.util.receiver.ResultView

/**
 * 通用列表请求容器
 *
 * @author wangyang34
 * @since 2019/11/6
 */
@Tag("ListRequestContainer")
class ListRequestContainer : BaseContainer() {

    /** 搜索结果返回值  */
    private var mResultReceiver: ListResultReceiver = ListResultReceiver(this, Handler(), null)

    /** 列表容器  */
    private lateinit var mListContainer: CommonListContainer

    override fun onCreateView(): View? {
        mListContainer = parent as CommonListContainer
        return null
    }

    override fun onDestroyView() {
        ListRequestHelper().cancelGetList(context)
    }

    override fun onCommonEvent(ev: CommonEvent): Boolean {
        when (ev.what) {
            MESSAGE_COMMON_LIST_REQUEST -> {
                "onCommonEvent $ev".d()
                val info = ev.obj as CommonRequestInfo
                if (info.first) {
                    val adapter = mListContainer.recyclerView.adapter as CommonListAdapter
                    val adapterProxy = adapter.adapterProxy as MemoryListAdapterProxy
                    adapterProxy.clear()
                    postLoadingEvent()
                }
                ListRequestHelper().getList(context, mResultReceiver, info)
            }
        }
        return false
    }

    /**
     * 数据成功
     * @param cardList 列表数据
     * @param hasMore 还有更多
     */
    private fun onDataSucceed(cardList: ArrayList<CommonCardInfo>, hasMore: Boolean) {
        val adapter = mListContainer.recyclerView.adapter as CommonListAdapter
        val adapterProxy = adapter.adapterProxy as MemoryListAdapterProxy
        var existList = adapterProxy.listData as ArrayList<CommonCardInfo>
        val firstAdd = existList == null || existList.isEmpty()
        if (firstAdd && cardList.isEmpty()) {
            postFinishEvent(0)
            return
        }
        if (!firstAdd) {
            removeFooter(adapterProxy)
        }
        adapterProxy.addAll(cardList)
        if (hasMore) {
            addFooter(adapterProxy, LIST_FOOTER_STATE_LOADING)
        } else {
            addFooter(adapterProxy, LIST_FOOTER_STATE_NO_MORE)
        }
        postFinishEvent(adapterProxy.itemCount)
    }

    /**
     * 数据失败
     */
    private fun onDataFailed() {
        val adapter = mListContainer.recyclerView.adapter as CommonListAdapter
        val adapterProxy = adapter.adapterProxy as MemoryListAdapterProxy
        var existList = adapterProxy.listData as ArrayList<CommonCardInfo>
        val firstAdd = existList == null || existList.isEmpty()
        if (firstAdd) {
            postFinishEvent(0)
            return
        }
        removeFooter(adapterProxy)
        addFooter(adapterProxy, LIST_FOOTER_STATE_RETRY)
        postFinishEvent(adapterProxy.itemCount)
    }

    /**
     * 添加footer
     * @param adapter 适配器
     * @param state footer状态
     */
    private fun addFooter(adapter: MemoryListAdapterProxy, state: Int) {
        val info = mInfo.data as ListRequestInfo
        if (!info.needFooter) {
            return
        }
        val footer = CommonCardInfo(CARD_ITEM_TYPE_LIST_FOOTER_ITEM, ListFooterInfo(state))
        adapter.add(footer)
    }

    /**
     * 删除footer
     * @param adapter 适配器
     */
    private fun removeFooter(adapter: MemoryListAdapterProxy) {
        val info = mInfo.data as ListRequestInfo
        if (!info.needFooter) {
            return
        }
        var existList = adapter.listData as ArrayList<CommonCardInfo>
        adapter.remove(existList.size - 1)
    }

    /**
     * post数据loading事件
     */
    private fun postLoadingEvent() {
        val event = CommonEvent()
        event.what = MESSAGE_STATE_CHANGE
        event.arg1 = LIST_STATE_LOADING
        fragment.postEventToRootContainer(event)
    }

    /**
     * post数据完成事件
     */
    private fun postFinishEvent(count: Int) {
        val event = CommonEvent()
        event.what = MESSAGE_STATE_CHANGE
        event.arg1 = LIST_STATE_FINISH
        event.arg2 = count
        event.from = containerId
        postEventToPage(event)
    }

    /**
     * 列表结果接收器
     */
    private class ListResultReceiver internal constructor(reference: ListRequestContainer,
                                                          handler: Handler, view: ResultView?)
        : BaseResultReceiver<ListRequestContainer>(reference, handler, view) {

        override fun onInterceptResult(reference: ListRequestContainer, resultCode: Int,
                                       resultData: Bundle?): Boolean {
            return if (reference.activity == null || reference.activity.isFinishing) {
                !super.onInterceptResult(reference, resultCode, resultData)
            } else {
                super.onInterceptResult(reference, resultCode, resultData)
            }
        }

        override fun onSuccess(reference: ListRequestContainer, resultData: Bundle?) {
            super.onSuccess(reference, resultData)
            if (reference == null || resultData == null) {
                return
            }
            val response = resultData.get(BaseExtras.RESULT)
            if (response !is CommonListResponse) {
                return
            }
            val event = CommonEvent()
            event.what = MESSAGE_COMMON_LIST_REQUEST_RESPONSE
            event.obj = response
            reference.postEventToPage(event)
            var cardList = ArrayList<CommonCardInfo>()
            response.mCardList?.let {
                cardList.addAll(it)
            }
            reference.onDataSucceed(cardList, response.mHasMore)
        }

        override fun onFailed(reference: ListRequestContainer, errType: ErrorType, errno: Int,
                              resultData: Bundle): Boolean {
            if (resultData.getBoolean(BaseExtras.ERROR_NETWORK)) {
                ToastHelper.showToast(R.string.network_error)
            }
            val response = resultData.get(BaseExtras.RESULT)
            val event = CommonEvent()
            event.what = MESSAGE_COMMON_LIST_REQUEST_RESPONSE
            event.obj = response
            reference.postEventToPage(event)
            reference.onDataFailed()
            return super.onFailed(reference, errType, errno, resultData)
        }
    }
}