package com.baidu.netdisk.uiframe.containerimpl.list

import android.content.Context
import android.os.ResultReceiver
import android.text.TextUtils
import com.baidu.netdisk.account.AccountUtils
import com.baidu.netdisk.base.service.BaseServiceHelper
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.ServiceTypes

/**
 * 通用列表请求帮助类
 *
 * @author wangyang34
 * @since 2019/11/6
 */
class ListRequestHelper : BaseServiceHelper() {

    /**
     * 获取列表
     *
     * @param context 上下文
     * @param resultReceiver 用于返回结果
     * @param url 地址
     */
    fun getList(context: Context, resultReceiver: ResultReceiver, info: CommonRequestInfo) {
        if (!isNetWorkAvailable(context, resultReceiver)) {
            return
        }
        val bduss = AccountUtils.getInstance().bduss
        if (TextUtils.isEmpty(bduss)) {
            return
        }
        val uid = AccountUtils.getInstance().uid
        if (TextUtils.isEmpty(uid)) {
            return
        }
        startTargetVersionService(context, buildIntent(context, bduss, uid, resultReceiver)
                .putExtra(BaseExtras.EXTRA_SERVICE_TYPE, ServiceTypes.COMMON_LIST)
                .setAction(CommonListService.ACTION_COMMON_LIST)
                .putExtra(CommonListService.EXTRA_COMMON_LIST_URL, info))
    }

    /**
     * 取消获取列表
     *
     * @param context 上下文
     * @param resultReceiver 用于返回结果
     *
     */
    fun cancelGetList(context: Context) {
        if (!isNetWorkAvailable(context, null)) {
            return
        }
        val bduss = AccountUtils.getInstance().bduss
        if (TextUtils.isEmpty(bduss)) {
            return
        }
        val uid = AccountUtils.getInstance().uid
        if (TextUtils.isEmpty(uid)) {
            return
        }
        startTargetVersionService(context, buildIntent(context, bduss, uid, null)
                .putExtra(BaseExtras.EXTRA_SERVICE_TYPE, ServiceTypes.COMMON_LIST)
                .setAction(CommonListService.ACTION_COMMON_LIST_CANCEL))
    }
}