package com.baidu.netdisk.uiframe.containerimpl.sug

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.os.ResultReceiver
import android.text.TextUtils
import com.baidu.netdisk.base.service.BaseServiceHelper
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.architecture.net.exception.RemoteException
import com.baidu.netdisk.statistics.BaseReportJob
import com.baidu.netdisk.uiframe.containerimpl.list.CommonListJob
import org.json.JSONException
import java.io.IOException
import java.io.UnsupportedEncodingException
import java.security.KeyManagementException
import java.security.KeyStoreException
import java.security.NoSuchAlgorithmException
import java.security.UnrecoverableKeyException

/**
 * 获取sug请求
 *
 * @author xujing31
 * @since 2019/11/7
 */
class RequestSugJob (val context: Context?, val intent: Intent?, val receiver: ResultReceiver?,
                     val bduss: String?, val uid: String?) : BaseReportJob(CommonListJob.NAME) {

    companion object {

        /** job名字 */
        const val NAME = "RequestSugJob"
    }

    override fun performExecute() {
        if (receiver == null || intent == null) {
            return
        }
        val inputText = intent.getStringExtra(SugService.EXTRA_INPUT_TEXT)
        if (TextUtils.isEmpty(inputText)) {
            receiver.send(BaseStatus.FAILED, Bundle.EMPTY)
        }
        try {
            val response = sendRequest(inputText)
            if (response == null) {
                receiver.send(BaseStatus.FAILED, Bundle.EMPTY)
                return
            }
            var bundle = Bundle()
            bundle.putSerializable(BaseExtras.RESULT, response)
            if (!isCancelled()) {
                receiver.send(BaseStatus.SUCCESS, bundle)
            }
        } catch (e: RemoteException) {
            BaseServiceHelper.handleRemoteException(e, receiver)
        } catch (e: IOException) {
            BaseServiceHelper.handleIOException(e, receiver)
        }
    }

    /**
     * 发出请求
     *
     * @param inputText 用户输入的文本
     * @return 结果
     */
    private fun sendRequest(inputText: String): SugResponse? {
        return try {
            SugApi(bduss, uid).getSug(inputText)
        } catch (e: KeyManagementException) {
            null
        } catch (e: UnrecoverableKeyException) {
            null
        } catch (e: NoSuchAlgorithmException) {
            null
        } catch (e: KeyStoreException) {
            null
        } catch (e: UnsupportedEncodingException) {
            null
        } catch (e: JSONException) {
            null
        }
    }
}