package com.baidu.netdisk.uiframe.containerimpl.sug

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import com.baidu.netdisk.R

/**
 * sug item项
 *
 * @author xujing31
 * @since 2019/11/8
 */
class SugItem (val context: Context?, val sugText: String) {

    /** item的view  */
    private lateinit var mRootView: View

    /** item的textView  */
    private lateinit var mTitleTextView: TextView

    /** 点击监听  */
    private var mListener: SugItemClickListener? = null

    /**
     * 初始化
     *
     */
    init{
        mRootView = LayoutInflater.from(context).inflate(R.layout.label_search_frame_sug_item, null);
        mTitleTextView = mRootView.findViewById<View>(R.id.sug_item_text) as TextView
        mTitleTextView.setText(sugText)
        mRootView.setOnClickListener(View.OnClickListener {
            mListener?.onClickSug(mTitleTextView.text.toString())
        })
    }

    /**
     * 设置文字距离左边的距离
     * @param marginLeft 间距
     */
    fun setTextMarginLeft(marginLeft: Int) {
        var layoutParams: ViewGroup.LayoutParams? = mTitleTextView.layoutParams
        (layoutParams as ViewGroup.MarginLayoutParams).leftMargin = marginLeft
        mTitleTextView.layoutParams = layoutParams
    }

    /**
     * 获取view
     *
     */
    fun getView() : View? {
        return mRootView
    }

    /**
     * 设置点击监听
     * @param listener 点击监听
     */
    fun setOnClickListener(listener: SugItemClickListener) {
        mListener = listener
    }

    /**
     * sug点击监听接口
     *
     */
    interface SugItemClickListener {
        /**
         * 点击某个sug
         * @param sug 文案
         */
        fun onClickSug(sug: String?)
    }

}