package com.baidu.netdisk.videoservice.service

import android.content.Context
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.BaseJobService
import com.baidu.netdisk.base.service.BaseServiceHelper
import com.baidu.netdisk.base.service.BaseServiceHelper.isNetWorkAvailable
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.ServiceTypes

/**
 * Created by linwentao on 2019/04/24.
 */
class VideoRecentManager : IVideoRecent {

    /**
     * 最近观看list接口
     *
     * @param context Context
     * @param receiver ResultReceiver
     * @param bduss String
     * @param uid String
     */
    override fun recentList(context: Context, receiver: ResultReceiver, bduss: String, uid: String) {
        if (!isNetWorkAvailable(context, receiver)) {
            return
        }
        val intent = BaseServiceHelper.buildIntent(context, bduss, uid, receiver)
                .putExtra(BaseExtras.EXTRA_SERVICE_TYPE, ServiceTypes.VIDEO_RECENT)
                .setAction(VideoRecentAction.VIDEO_RECENT_LIST_ACTION)

        BaseServiceHelper.startTargetVersionService(BaseJobService.VIDEO_RECENT_SERVICE,
                context, intent)
    }

    /**
     * 最近观看清除接口
     *
     * @param context Context
     * @param receiver ResultReceiver
     * @param bduss String
     * @param uid String
     * @param fsid Array<String>
     * @param clearAll Int 为1表示清除所有，默认为0
     */
    override fun recentClear(context: Context, receiver: ResultReceiver, bduss: String,
                             uid: String, fsids: Array<String>, clearAll: Int) {
        if (!isNetWorkAvailable(context, receiver)) {
            return
        }
        val intent = BaseServiceHelper.buildIntent(context, bduss, uid, receiver)
                .putExtra(BaseExtras.EXTRA_SERVICE_TYPE, ServiceTypes.VIDEO_RECENT)
                .putExtra(VideoRecentAction.VIDEO_FSID, fsids)
                .putExtra(VideoRecentAction.VIDEO_CLEAR_FLAG, clearAll)
                .setAction(VideoRecentAction.VIDEO_RECENT_CLEAR_ACTION)

        BaseServiceHelper.startTargetVersionService(BaseJobService.VIDEO_RECENT_SERVICE,
                context, intent)
    }
}