package com.baidu.netdisk.videoservice.service

import android.content.Context
import android.content.Intent
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.ISchedulerService
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.kernel.architecture.job.PriorityScheduler
import com.baidu.netdisk.videoservice.service.clear.VideoRecentClearJob
import com.baidu.netdisk.videoservice.service.list.VideoRecentListJob

/**
 * Created by linwentao on 2019/4/24.
 */
class VideoRecentService(
        /**
         * 优先级队列调度器
         */
        private var mScheduler: PriorityScheduler
) : ISchedulerService, IVideoRecent {

    /**
     * 最近list操作
     *
     * @param context Context
     * @param receiver ResultReceiver
     * @param bduss String
     * @param uid String
     */
    override fun recentList(context: Context, receiver: ResultReceiver, bduss: String, uid: String) {
        mScheduler.addJobWithHigh(VideoRecentListJob(context, receiver, bduss, uid))
    }

    /**
     * 最近clear操作
     *
     * @param context Context
     * @param receiver ResultReceiver
     * @param bduss String
     * @param uid String
     * @param fsids Array<String>
     * @param clearAll Int
     */
    override fun recentClear(context: Context, receiver: ResultReceiver, bduss: String, uid: String,
                             fsids: Array<String>, clearAll: Int) {
        mScheduler.addJobWithHigh(VideoRecentClearJob(receiver, bduss, uid, fsids, clearAll))
    }

    override fun onHandleIntent(intent: Intent, context: Context) {
        val bduss = intent.getStringExtra(BaseExtras.BDUSS)
        val action = intent.action
        val receiver = intent.getParcelableExtra<ResultReceiver>(BaseExtras.RESULT_RECEIVER)
        val uid = intent.getStringExtra(BaseExtras.UID)

        when (action) {
            // 拉取list数据
            VideoRecentAction.VIDEO_RECENT_LIST_ACTION -> {
                recentList(context, receiver, bduss, uid)
            }
            // 处理清除操作
            VideoRecentAction.VIDEO_RECENT_CLEAR_ACTION -> {
                val fsids = intent.getStringArrayExtra(VideoRecentAction.VIDEO_FSID)
                val clear = intent.getIntExtra(VideoRecentAction.VIDEO_CLEAR_FLAG, 0)
                recentClear(context, receiver, bduss, uid, fsids, clear)
            }
            else ->
                // 抛出异常方便发现问题
                throw IllegalArgumentException("$action unhandled") as Throwable
        }
    }
}