package com.baidu.netdisk.videoservice.service.clear

import android.os.Build
import android.os.ResultReceiver
import android.support.annotation.RequiresApi
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.net.exception.RemoteException
import com.baidu.netdisk.statistics.BaseReportJob
import org.json.JSONException
import java.io.IOException
import java.security.KeyManagementException
import java.security.KeyStoreException
import java.security.NoSuchAlgorithmException
import java.security.UnrecoverableKeyException

/**
 * Created by linwentao on 2019/4/24.
 */
class VideoRecentClearJob(
        private val mReceiver: ResultReceiver?,
        private val mBduss: String,
        private val mUid: String,
        private val mFsids: Array<String>,
        private val mClear: Int
) : BaseReportJob("VideoRecentClearJob") {

    private var TAG = "VideoRecentClearJob"

    @RequiresApi(Build.VERSION_CODES.KITKAT)
    @Throws(Exception::class)
    override fun performExecute() {
        NetDiskLog.d(TAG, "performExecute VideoRecentClearJob")
        var videoRecentclearResponse: VideoRecentDeleteResponse? = null
        try {
            videoRecentclearResponse = VideoRecentClearApi(mBduss, mUid).clearRecentData(mFsids, mClear)
            NetDiskLog.d(TAG, " videoRecentClearResponse " + videoRecentclearResponse?.toString())
        } catch (e: KeyManagementException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: UnrecoverableKeyException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: NoSuchAlgorithmException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: KeyStoreException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: JSONException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: IOException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: RemoteException) {
            NetDiskLog.e(TAG, e.message, e)
        }

        if (videoRecentclearResponse?.errno != 0) {
            mReceiver?.send(BaseStatus.FAILED, null)
        } else {
            mReceiver?.send(BaseStatus.SUCCESS, null)
        }
    }
}