package com.baidu.netdisk.videoservice.service.list

import android.content.Context
import android.os.Bundle
import android.os.ResultReceiver
import com.baidu.netdisk.base.service.constant.BaseExtras
import com.baidu.netdisk.base.service.constant.BaseStatus
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.kernel.architecture.net.exception.RemoteException
import com.baidu.netdisk.statistics.BaseReportJob
import com.baidu.netdisk.videoservice.VideoRecentHelper
import com.baidu.netdisk.videoservice.VideoRecentFileInfo
import org.json.JSONException
import java.io.IOException
import java.security.*
import java.util.ArrayList

/**
 * Created by linwentao on 2019/4/24.
 */
class VideoRecentListJob(
        private val mContext: Context,
        private val mReceiver: ResultReceiver?,
        private val mBduss: String,
        private val mUid: String
) : BaseReportJob("VideoRecentJob") {

    private var TAG = "VideoRecentJob"

    @Throws(Exception::class)
    override fun performExecute() {
        NetDiskLog.d(TAG, "performExecute VideoRecentJob")
        var videoRecentListResponse: VideoRecentListResponse? = null
        try {
            videoRecentListResponse = VideoRecentListApi(mBduss, mUid).getRecentListData()
        } catch (e: KeyManagementException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: UnrecoverableKeyException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: NoSuchAlgorithmException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: KeyStoreException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: JSONException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: IOException) {
            NetDiskLog.e(TAG, e.message, e)
        } catch (e: RemoteException) {
            NetDiskLog.e(TAG, e.message, e)
        }

        val bundle = Bundle()
        if (videoRecentListResponse?.errno != 0) {
            bundle.putParcelable(BaseExtras.RESULT, videoRecentListResponse)
            mReceiver?.send(BaseStatus.FAILED, bundle)
        } else {
            bundle.putParcelable(BaseExtras.RESULT, videoRecentListResponse)
            mReceiver?.send(BaseStatus.SUCCESS, bundle)

            val fsidList = videoRecentListResponse.fsidList
            if (fsidList == null || fsidList.isEmpty()) {
                return
            }
            val recordList = videoRecentListResponse.viewTime
            val contentList = ArrayList<VideoRecentFileInfo>()
            for (i in fsidList.indices) {
                val videoRecentContent = VideoRecentFileInfo()
                videoRecentContent.setId(java.lang.Long.valueOf(fsidList[i]))
                videoRecentContent.record = recordList[i].toLong()
                contentList.add(videoRecentContent)
            }
            // 结果插入数据库
            VideoRecentHelper().insertRecentData(mContext, contentList)
        }
    }
}