package com.baidu.netdisk.wap.launch.extopen

import android.app.Activity
import android.app.Dialog
import android.content.Intent
import android.os.Bundle
import android.view.KeyEvent
import android.view.View
import android.widget.LinearLayout
import android.widget.TextView
import com.baidu.netdisk.R
import com.baidu.netdisk.backup.ui.CommonBackupSettingActivity
import com.baidu.netdisk.statistics.NetdiskStatisticsLogForMutilFields
import com.baidu.netdisk.ui.MainActivity
import com.baidu.netdisk.ui.manager.DialogBuilder

/**
 * 第三方跳转至备份页
 *
 * @author wangyang34
 * @since 2019年10月10日
 */
class BackUpCenterActivityEntry(val bundle: Bundle?) : ExtEntry() {

    override fun openTargetPage(activity: MainActivity) {
        bundle?.let {
            if (it.getBoolean(IExtPage.KEY_EXT_FROM_CLIPBOARD, false)) {
                showDialog(activity)
                return
            }
        }
        enterBackUpCenterActivity(activity)
    }

    /**
     * 显示对话框
     *
     * @param activity 页面
     */
    fun showDialog(activity: Activity) {
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                NetdiskStatisticsLogForMutilFields.StatisticsKeys.ENTER_BACKUP_SETTING_VERIFY_DIALOG)

        val dialog: Dialog = DialogBuilder().buildCustomViewDialog(activity, DialogBuilder.DISABLE_DIALOG_BUTTON,
                DialogBuilder.DISABLE_DIALOG_BUTTON, DialogBuilder.DISABLE_DIALOG_BUTTON,
                R.layout.dialog_open_backup_setting)
        dialog.findViewById<View>(R.id.dialog_footer).visibility = View.GONE
        val contentLayout = dialog.findViewById<View>(R.id.contentLayout)
        val lp = contentLayout.layoutParams as LinearLayout.LayoutParams
        lp.setMargins(lp.leftMargin, 0, lp.rightMargin, lp.bottomMargin)
        contentLayout.layoutParams = lp
        val pcode = dialog.findViewById<TextView>(R.id.dialog_button_blue)
        pcode.setText(R.string.go_and_open)
        pcode.visibility = View.VISIBLE
        pcode.setOnClickListener {
            dialog.dismiss()
            enterBackUpCenterActivity(activity)
            NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                    NetdiskStatisticsLogForMutilFields.StatisticsKeys.ENTER_BACKUP_SETTING_VERIFY_DIALOG_CLICK_OK)
        }
        val cancel = dialog.findViewById<View>(R.id.dialog_top_cancel)
        cancel.visibility = View.VISIBLE
        cancel.setOnClickListener {
            dialog.dismiss()
        }
        dialog.setOnKeyListener { dialog, keyCode, event ->
            if (keyCode == KeyEvent.KEYCODE_BACK) {
                dialog.dismiss()
            }
            false
        }
        dialog.show()
    }

    /**
     * 进入备份设置页
     *
     * @param activity
     */
    private fun enterBackUpCenterActivity(activity: Activity) {
        val intent = Intent(activity, CommonBackupSettingActivity::class.java)
        intent.addFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP)
        activity.startActivity(intent)
        NetdiskStatisticsLogForMutilFields.getInstance().updateCount(
                NetdiskStatisticsLogForMutilFields.StatisticsKeys.ENTER_BACKUP_SETTING)
    }
}