package com.baidu.netdisk.wap.offlinepackage

import android.os.Handler
import com.baidu.netdisk.dynamic.DynamicPluginConstant
import com.baidu.netdisk.dynamic.SyncPluginListener
import com.baidu.netdisk.dynamic.base.model.DynamicPlugin
import com.baidu.netdisk.dynamic.offlineh5.OfflinePackageStaticsUtil
import com.baidu.netdisk.kernel.architecture.debug.NetDiskLog
import com.baidu.netdisk.statistics.ThreadJob
import java.util.ArrayList

/**
 * 插件同步监听器
 */
class OfflinePackageSyncListener(reference: Any?, handler: Handler?) : SyncPluginListener(reference, handler) {


    private var dynamicPluginList = ArrayList<DynamicPlugin>()

    override fun onSyncListFinish(dynamicPlugins: ArrayList<DynamicPlugin>?) {
        dynamicPlugins?.apply {
            dynamicPluginList = dynamicPlugins
        }
    }

    override fun onStart() {

    }

    /**
     * 离线包结束后，进行拦截映射的初始化
     */
    override fun onFinish(state: Int, failedReason: Int) {
        NetDiskLog.d("H5Offline","onFinish state:$state")
        if (state == DynamicPluginConstant.Status.SUCCESS) {
            object : ThreadJob("readH5Package") {
                override fun performExecute() {
                    OfflinePackageManager().initManifestItemMap()
                    NetDiskLog.d("H5Offline","initManifestItemMap")
                }
            }.start()
        }
    }

    override fun onDownloadStart(pluginId: String?) {

    }

    override fun onDownloadFinish(pluginId: String?, state: Int, reason: Int) {
        NetDiskLog.d("H5Offline", "onDownloadFinish:$pluginId, state:$state, reason:$reason")
        if (state == DynamicPluginConstant.Status.FAILED) {
            for (dynamicPlugin in dynamicPluginList) {
                if (dynamicPlugin.id == pluginId
                        && dynamicPlugin.type == DynamicPluginConstant.PluginTypes.OFFLINE_H5_PACKAGE) {
                    OfflinePackageStaticsUtil.updateError(dynamicPlugin.id,
                            dynamicPlugin.version, OfflinePackageStaticsUtil.DOWNLOAD_FAIL,
                            reason.toString())
                }
            }
        }
    }

    override fun onDownloadRate(pluginId: String?, size: Long, offSize: Long, rate: Long) {

    }

    override fun onInstallStart(pluginId: String?) {
    }

    override fun onInstallFinish(pluginId: String?, state: Int, reason: Int) {
        if (state == DynamicPluginConstant.Status.FAILED
                && reason == DynamicPluginConstant.FailedReason.ILLEGAL_FILE_MD5_TO_INSTALL_PLUGIN) {
            for (dynamicPlugin in dynamicPluginList) {
                if (dynamicPlugin.id == pluginId
                        && dynamicPlugin.type == DynamicPluginConstant.PluginTypes.OFFLINE_H5_PACKAGE) {
                    OfflinePackageStaticsUtil.updateError(dynamicPlugin.id, dynamicPlugin.version,
                            OfflinePackageStaticsUtil.CHECK_FAIL)
                }
            }
        }
    }
}